/**
 * Copyright (c) 2012 Andrew Prock. All rights reserved.
 * $Id: PokerEvaluationTables.h 2649 2012-06-30 04:53:24Z prock $
 */
#ifndef PEVAL_POKEREVALUATIONTABLES_H_
#define PEVAL_POKEREVALUATIONTABLES_H_

#include <boost/cstdint.hpp>

namespace pokerstove
{

/* the tables defined in this file:
 *
 *
 * const int8_t topRankTable[]
 * 2^13 lookup for the top rank
 *
 * const int8_t botRankTable[]
 *
 * const int bottomRankMask[]
 *   A table that returns the bitmask with
 *   only the bottom bit set.
 *
 * const int8_t straightTable[]
 *  -3 runner runner
 *  -2 open-ended
 *  -1 gutshot
 *   0
 *   N N-high straight
 *
 * const uint8_t nRanksTable[]
 *
 * const int8_t flushTable[]
 *
 * const uint16_t topFiveRanksTable[]
 *   A bitmask of the top five ranks.
 *   Useful for flushes, high cards (in high)
 *
 * const uint16_t topThreeRanksTable[]
 *   A bitmask for the top three ranks.
 *   Useful for a pair + three kickers
 *
 * const int lowballA5Ranks[]
 */


const int8_t topRankTable[] =
{
    -1,    0,    1,    1,    2,    2,    2,    2,    3,    3,    3,    3,
    3,    3,    3,    3,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    4,    5,    5,    5,    5,
    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
    5,    5,    5,    5,    6,    6,    6,    6,    6,    6,    6,    6,
    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,
    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,
    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,
    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,
    6,    6,    6,    6,    6,    6,    6,    6,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
    7,    7,    7,    7,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
    9,    9,    9,    9,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,   10,   10,   10,   10,   10,   10,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
    11,   11,   11,   11,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
    12,   12,   12,   12,   12,   12,   12,   12,
};

const int8_t botRankTable[] =
{
    -1, //    0,  00000000 00000000 00000000 00000000
    0, //    1,  10000000 00000000 00000000 00000000
    1, //    2,  01000000 00000000 00000000 00000000
    0, //    3,  11000000 00000000 00000000 00000000
    2, //    4,  00100000 00000000 00000000 00000000
    0, //    5,  10100000 00000000 00000000 00000000
    1, //    6,  01100000 00000000 00000000 00000000
    0, //    7,  11100000 00000000 00000000 00000000
    3, //    8,  00010000 00000000 00000000 00000000
    0, //    9,  10010000 00000000 00000000 00000000
    1, //   10,  01010000 00000000 00000000 00000000
    0, //   11,  11010000 00000000 00000000 00000000
    2, //   12,  00110000 00000000 00000000 00000000
    0, //   13,  10110000 00000000 00000000 00000000
    1, //   14,  01110000 00000000 00000000 00000000
    0, //   15,  11110000 00000000 00000000 00000000
    4, //   16,  00001000 00000000 00000000 00000000
    0, //   17,  10001000 00000000 00000000 00000000
    1, //   18,  01001000 00000000 00000000 00000000
    0, //   19,  11001000 00000000 00000000 00000000
    2, //   20,  00101000 00000000 00000000 00000000
    0, //   21,  10101000 00000000 00000000 00000000
    1, //   22,  01101000 00000000 00000000 00000000
    0, //   23,  11101000 00000000 00000000 00000000
    3, //   24,  00011000 00000000 00000000 00000000
    0, //   25,  10011000 00000000 00000000 00000000
    1, //   26,  01011000 00000000 00000000 00000000
    0, //   27,  11011000 00000000 00000000 00000000
    2, //   28,  00111000 00000000 00000000 00000000
    0, //   29,  10111000 00000000 00000000 00000000
    1, //   30,  01111000 00000000 00000000 00000000
    0, //   31,  11111000 00000000 00000000 00000000
    5, //   32,  00000100 00000000 00000000 00000000
    0, //   33,  10000100 00000000 00000000 00000000
    1, //   34,  01000100 00000000 00000000 00000000
    0, //   35,  11000100 00000000 00000000 00000000
    2, //   36,  00100100 00000000 00000000 00000000
    0, //   37,  10100100 00000000 00000000 00000000
    1, //   38,  01100100 00000000 00000000 00000000
    0, //   39,  11100100 00000000 00000000 00000000
    3, //   40,  00010100 00000000 00000000 00000000
    0, //   41,  10010100 00000000 00000000 00000000
    1, //   42,  01010100 00000000 00000000 00000000
    0, //   43,  11010100 00000000 00000000 00000000
    2, //   44,  00110100 00000000 00000000 00000000
    0, //   45,  10110100 00000000 00000000 00000000
    1, //   46,  01110100 00000000 00000000 00000000
    0, //   47,  11110100 00000000 00000000 00000000
    4, //   48,  00001100 00000000 00000000 00000000
    0, //   49,  10001100 00000000 00000000 00000000
    1, //   50,  01001100 00000000 00000000 00000000
    0, //   51,  11001100 00000000 00000000 00000000
    2, //   52,  00101100 00000000 00000000 00000000
    0, //   53,  10101100 00000000 00000000 00000000
    1, //   54,  01101100 00000000 00000000 00000000
    0, //   55,  11101100 00000000 00000000 00000000
    3, //   56,  00011100 00000000 00000000 00000000
    0, //   57,  10011100 00000000 00000000 00000000
    1, //   58,  01011100 00000000 00000000 00000000
    0, //   59,  11011100 00000000 00000000 00000000
    2, //   60,  00111100 00000000 00000000 00000000
    0, //   61,  10111100 00000000 00000000 00000000
    1, //   62,  01111100 00000000 00000000 00000000
    0, //   63,  11111100 00000000 00000000 00000000
    6, //   64,  00000010 00000000 00000000 00000000
    0, //   65,  10000010 00000000 00000000 00000000
    1, //   66,  01000010 00000000 00000000 00000000
    0, //   67,  11000010 00000000 00000000 00000000
    2, //   68,  00100010 00000000 00000000 00000000
    0, //   69,  10100010 00000000 00000000 00000000
    1, //   70,  01100010 00000000 00000000 00000000
    0, //   71,  11100010 00000000 00000000 00000000
    3, //   72,  00010010 00000000 00000000 00000000
    0, //   73,  10010010 00000000 00000000 00000000
    1, //   74,  01010010 00000000 00000000 00000000
    0, //   75,  11010010 00000000 00000000 00000000
    2, //   76,  00110010 00000000 00000000 00000000
    0, //   77,  10110010 00000000 00000000 00000000
    1, //   78,  01110010 00000000 00000000 00000000
    0, //   79,  11110010 00000000 00000000 00000000
    4, //   80,  00001010 00000000 00000000 00000000
    0, //   81,  10001010 00000000 00000000 00000000
    1, //   82,  01001010 00000000 00000000 00000000
    0, //   83,  11001010 00000000 00000000 00000000
    2, //   84,  00101010 00000000 00000000 00000000
    0, //   85,  10101010 00000000 00000000 00000000
    1, //   86,  01101010 00000000 00000000 00000000
    0, //   87,  11101010 00000000 00000000 00000000
    3, //   88,  00011010 00000000 00000000 00000000
    0, //   89,  10011010 00000000 00000000 00000000
    1, //   90,  01011010 00000000 00000000 00000000
    0, //   91,  11011010 00000000 00000000 00000000
    2, //   92,  00111010 00000000 00000000 00000000
    0, //   93,  10111010 00000000 00000000 00000000
    1, //   94,  01111010 00000000 00000000 00000000
    0, //   95,  11111010 00000000 00000000 00000000
    5, //   96,  00000110 00000000 00000000 00000000
    0, //   97,  10000110 00000000 00000000 00000000
    1, //   98,  01000110 00000000 00000000 00000000
    0, //   99,  11000110 00000000 00000000 00000000
    2, //  100,  00100110 00000000 00000000 00000000
    0, //  101,  10100110 00000000 00000000 00000000
    1, //  102,  01100110 00000000 00000000 00000000
    0, //  103,  11100110 00000000 00000000 00000000
    3, //  104,  00010110 00000000 00000000 00000000
    0, //  105,  10010110 00000000 00000000 00000000
    1, //  106,  01010110 00000000 00000000 00000000
    0, //  107,  11010110 00000000 00000000 00000000
    2, //  108,  00110110 00000000 00000000 00000000
    0, //  109,  10110110 00000000 00000000 00000000
    1, //  110,  01110110 00000000 00000000 00000000
    0, //  111,  11110110 00000000 00000000 00000000
    4, //  112,  00001110 00000000 00000000 00000000
    0, //  113,  10001110 00000000 00000000 00000000
    1, //  114,  01001110 00000000 00000000 00000000
    0, //  115,  11001110 00000000 00000000 00000000
    2, //  116,  00101110 00000000 00000000 00000000
    0, //  117,  10101110 00000000 00000000 00000000
    1, //  118,  01101110 00000000 00000000 00000000
    0, //  119,  11101110 00000000 00000000 00000000
    3, //  120,  00011110 00000000 00000000 00000000
    0, //  121,  10011110 00000000 00000000 00000000
    1, //  122,  01011110 00000000 00000000 00000000
    0, //  123,  11011110 00000000 00000000 00000000
    2, //  124,  00111110 00000000 00000000 00000000
    0, //  125,  10111110 00000000 00000000 00000000
    1, //  126,  01111110 00000000 00000000 00000000
    0, //  127,  11111110 00000000 00000000 00000000
    7, //  128,  00000001 00000000 00000000 00000000
    0, //  129,  10000001 00000000 00000000 00000000
    1, //  130,  01000001 00000000 00000000 00000000
    0, //  131,  11000001 00000000 00000000 00000000
    2, //  132,  00100001 00000000 00000000 00000000
    0, //  133,  10100001 00000000 00000000 00000000
    1, //  134,  01100001 00000000 00000000 00000000
    0, //  135,  11100001 00000000 00000000 00000000
    3, //  136,  00010001 00000000 00000000 00000000
    0, //  137,  10010001 00000000 00000000 00000000
    1, //  138,  01010001 00000000 00000000 00000000
    0, //  139,  11010001 00000000 00000000 00000000
    2, //  140,  00110001 00000000 00000000 00000000
    0, //  141,  10110001 00000000 00000000 00000000
    1, //  142,  01110001 00000000 00000000 00000000
    0, //  143,  11110001 00000000 00000000 00000000
    4, //  144,  00001001 00000000 00000000 00000000
    0, //  145,  10001001 00000000 00000000 00000000
    1, //  146,  01001001 00000000 00000000 00000000
    0, //  147,  11001001 00000000 00000000 00000000
    2, //  148,  00101001 00000000 00000000 00000000
    0, //  149,  10101001 00000000 00000000 00000000
    1, //  150,  01101001 00000000 00000000 00000000
    0, //  151,  11101001 00000000 00000000 00000000
    3, //  152,  00011001 00000000 00000000 00000000
    0, //  153,  10011001 00000000 00000000 00000000
    1, //  154,  01011001 00000000 00000000 00000000
    0, //  155,  11011001 00000000 00000000 00000000
    2, //  156,  00111001 00000000 00000000 00000000
    0, //  157,  10111001 00000000 00000000 00000000
    1, //  158,  01111001 00000000 00000000 00000000
    0, //  159,  11111001 00000000 00000000 00000000
    5, //  160,  00000101 00000000 00000000 00000000
    0, //  161,  10000101 00000000 00000000 00000000
    1, //  162,  01000101 00000000 00000000 00000000
    0, //  163,  11000101 00000000 00000000 00000000
    2, //  164,  00100101 00000000 00000000 00000000
    0, //  165,  10100101 00000000 00000000 00000000
    1, //  166,  01100101 00000000 00000000 00000000
    0, //  167,  11100101 00000000 00000000 00000000
    3, //  168,  00010101 00000000 00000000 00000000
    0, //  169,  10010101 00000000 00000000 00000000
    1, //  170,  01010101 00000000 00000000 00000000
    0, //  171,  11010101 00000000 00000000 00000000
    2, //  172,  00110101 00000000 00000000 00000000
    0, //  173,  10110101 00000000 00000000 00000000
    1, //  174,  01110101 00000000 00000000 00000000
    0, //  175,  11110101 00000000 00000000 00000000
    4, //  176,  00001101 00000000 00000000 00000000
    0, //  177,  10001101 00000000 00000000 00000000
    1, //  178,  01001101 00000000 00000000 00000000
    0, //  179,  11001101 00000000 00000000 00000000
    2, //  180,  00101101 00000000 00000000 00000000
    0, //  181,  10101101 00000000 00000000 00000000
    1, //  182,  01101101 00000000 00000000 00000000
    0, //  183,  11101101 00000000 00000000 00000000
    3, //  184,  00011101 00000000 00000000 00000000
    0, //  185,  10011101 00000000 00000000 00000000
    1, //  186,  01011101 00000000 00000000 00000000
    0, //  187,  11011101 00000000 00000000 00000000
    2, //  188,  00111101 00000000 00000000 00000000
    0, //  189,  10111101 00000000 00000000 00000000
    1, //  190,  01111101 00000000 00000000 00000000
    0, //  191,  11111101 00000000 00000000 00000000
    6, //  192,  00000011 00000000 00000000 00000000
    0, //  193,  10000011 00000000 00000000 00000000
    1, //  194,  01000011 00000000 00000000 00000000
    0, //  195,  11000011 00000000 00000000 00000000
    2, //  196,  00100011 00000000 00000000 00000000
    0, //  197,  10100011 00000000 00000000 00000000
    1, //  198,  01100011 00000000 00000000 00000000
    0, //  199,  11100011 00000000 00000000 00000000
    3, //  200,  00010011 00000000 00000000 00000000
    0, //  201,  10010011 00000000 00000000 00000000
    1, //  202,  01010011 00000000 00000000 00000000
    0, //  203,  11010011 00000000 00000000 00000000
    2, //  204,  00110011 00000000 00000000 00000000
    0, //  205,  10110011 00000000 00000000 00000000
    1, //  206,  01110011 00000000 00000000 00000000
    0, //  207,  11110011 00000000 00000000 00000000
    4, //  208,  00001011 00000000 00000000 00000000
    0, //  209,  10001011 00000000 00000000 00000000
    1, //  210,  01001011 00000000 00000000 00000000
    0, //  211,  11001011 00000000 00000000 00000000
    2, //  212,  00101011 00000000 00000000 00000000
    0, //  213,  10101011 00000000 00000000 00000000
    1, //  214,  01101011 00000000 00000000 00000000
    0, //  215,  11101011 00000000 00000000 00000000
    3, //  216,  00011011 00000000 00000000 00000000
    0, //  217,  10011011 00000000 00000000 00000000
    1, //  218,  01011011 00000000 00000000 00000000
    0, //  219,  11011011 00000000 00000000 00000000
    2, //  220,  00111011 00000000 00000000 00000000
    0, //  221,  10111011 00000000 00000000 00000000
    1, //  222,  01111011 00000000 00000000 00000000
    0, //  223,  11111011 00000000 00000000 00000000
    5, //  224,  00000111 00000000 00000000 00000000
    0, //  225,  10000111 00000000 00000000 00000000
    1, //  226,  01000111 00000000 00000000 00000000
    0, //  227,  11000111 00000000 00000000 00000000
    2, //  228,  00100111 00000000 00000000 00000000
    0, //  229,  10100111 00000000 00000000 00000000
    1, //  230,  01100111 00000000 00000000 00000000
    0, //  231,  11100111 00000000 00000000 00000000
    3, //  232,  00010111 00000000 00000000 00000000
    0, //  233,  10010111 00000000 00000000 00000000
    1, //  234,  01010111 00000000 00000000 00000000
    0, //  235,  11010111 00000000 00000000 00000000
    2, //  236,  00110111 00000000 00000000 00000000
    0, //  237,  10110111 00000000 00000000 00000000
    1, //  238,  01110111 00000000 00000000 00000000
    0, //  239,  11110111 00000000 00000000 00000000
    4, //  240,  00001111 00000000 00000000 00000000
    0, //  241,  10001111 00000000 00000000 00000000
    1, //  242,  01001111 00000000 00000000 00000000
    0, //  243,  11001111 00000000 00000000 00000000
    2, //  244,  00101111 00000000 00000000 00000000
    0, //  245,  10101111 00000000 00000000 00000000
    1, //  246,  01101111 00000000 00000000 00000000
    0, //  247,  11101111 00000000 00000000 00000000
    3, //  248,  00011111 00000000 00000000 00000000
    0, //  249,  10011111 00000000 00000000 00000000
    1, //  250,  01011111 00000000 00000000 00000000
    0, //  251,  11011111 00000000 00000000 00000000
    2, //  252,  00111111 00000000 00000000 00000000
    0, //  253,  10111111 00000000 00000000 00000000
    1, //  254,  01111111 00000000 00000000 00000000
    0, //  255,  11111111 00000000 00000000 00000000
    8, //  256,  00000000 10000000 00000000 00000000
    0, //  257,  10000000 10000000 00000000 00000000
    1, //  258,  01000000 10000000 00000000 00000000
    0, //  259,  11000000 10000000 00000000 00000000
    2, //  260,  00100000 10000000 00000000 00000000
    0, //  261,  10100000 10000000 00000000 00000000
    1, //  262,  01100000 10000000 00000000 00000000
    0, //  263,  11100000 10000000 00000000 00000000
    3, //  264,  00010000 10000000 00000000 00000000
    0, //  265,  10010000 10000000 00000000 00000000
    1, //  266,  01010000 10000000 00000000 00000000
    0, //  267,  11010000 10000000 00000000 00000000
    2, //  268,  00110000 10000000 00000000 00000000
    0, //  269,  10110000 10000000 00000000 00000000
    1, //  270,  01110000 10000000 00000000 00000000
    0, //  271,  11110000 10000000 00000000 00000000
    4, //  272,  00001000 10000000 00000000 00000000
    0, //  273,  10001000 10000000 00000000 00000000
    1, //  274,  01001000 10000000 00000000 00000000
    0, //  275,  11001000 10000000 00000000 00000000
    2, //  276,  00101000 10000000 00000000 00000000
    0, //  277,  10101000 10000000 00000000 00000000
    1, //  278,  01101000 10000000 00000000 00000000
    0, //  279,  11101000 10000000 00000000 00000000
    3, //  280,  00011000 10000000 00000000 00000000
    0, //  281,  10011000 10000000 00000000 00000000
    1, //  282,  01011000 10000000 00000000 00000000
    0, //  283,  11011000 10000000 00000000 00000000
    2, //  284,  00111000 10000000 00000000 00000000
    0, //  285,  10111000 10000000 00000000 00000000
    1, //  286,  01111000 10000000 00000000 00000000
    0, //  287,  11111000 10000000 00000000 00000000
    5, //  288,  00000100 10000000 00000000 00000000
    0, //  289,  10000100 10000000 00000000 00000000
    1, //  290,  01000100 10000000 00000000 00000000
    0, //  291,  11000100 10000000 00000000 00000000
    2, //  292,  00100100 10000000 00000000 00000000
    0, //  293,  10100100 10000000 00000000 00000000
    1, //  294,  01100100 10000000 00000000 00000000
    0, //  295,  11100100 10000000 00000000 00000000
    3, //  296,  00010100 10000000 00000000 00000000
    0, //  297,  10010100 10000000 00000000 00000000
    1, //  298,  01010100 10000000 00000000 00000000
    0, //  299,  11010100 10000000 00000000 00000000
    2, //  300,  00110100 10000000 00000000 00000000
    0, //  301,  10110100 10000000 00000000 00000000
    1, //  302,  01110100 10000000 00000000 00000000
    0, //  303,  11110100 10000000 00000000 00000000
    4, //  304,  00001100 10000000 00000000 00000000
    0, //  305,  10001100 10000000 00000000 00000000
    1, //  306,  01001100 10000000 00000000 00000000
    0, //  307,  11001100 10000000 00000000 00000000
    2, //  308,  00101100 10000000 00000000 00000000
    0, //  309,  10101100 10000000 00000000 00000000
    1, //  310,  01101100 10000000 00000000 00000000
    0, //  311,  11101100 10000000 00000000 00000000
    3, //  312,  00011100 10000000 00000000 00000000
    0, //  313,  10011100 10000000 00000000 00000000
    1, //  314,  01011100 10000000 00000000 00000000
    0, //  315,  11011100 10000000 00000000 00000000
    2, //  316,  00111100 10000000 00000000 00000000
    0, //  317,  10111100 10000000 00000000 00000000
    1, //  318,  01111100 10000000 00000000 00000000
    0, //  319,  11111100 10000000 00000000 00000000
    6, //  320,  00000010 10000000 00000000 00000000
    0, //  321,  10000010 10000000 00000000 00000000
    1, //  322,  01000010 10000000 00000000 00000000
    0, //  323,  11000010 10000000 00000000 00000000
    2, //  324,  00100010 10000000 00000000 00000000
    0, //  325,  10100010 10000000 00000000 00000000
    1, //  326,  01100010 10000000 00000000 00000000
    0, //  327,  11100010 10000000 00000000 00000000
    3, //  328,  00010010 10000000 00000000 00000000
    0, //  329,  10010010 10000000 00000000 00000000
    1, //  330,  01010010 10000000 00000000 00000000
    0, //  331,  11010010 10000000 00000000 00000000
    2, //  332,  00110010 10000000 00000000 00000000
    0, //  333,  10110010 10000000 00000000 00000000
    1, //  334,  01110010 10000000 00000000 00000000
    0, //  335,  11110010 10000000 00000000 00000000
    4, //  336,  00001010 10000000 00000000 00000000
    0, //  337,  10001010 10000000 00000000 00000000
    1, //  338,  01001010 10000000 00000000 00000000
    0, //  339,  11001010 10000000 00000000 00000000
    2, //  340,  00101010 10000000 00000000 00000000
    0, //  341,  10101010 10000000 00000000 00000000
    1, //  342,  01101010 10000000 00000000 00000000
    0, //  343,  11101010 10000000 00000000 00000000
    3, //  344,  00011010 10000000 00000000 00000000
    0, //  345,  10011010 10000000 00000000 00000000
    1, //  346,  01011010 10000000 00000000 00000000
    0, //  347,  11011010 10000000 00000000 00000000
    2, //  348,  00111010 10000000 00000000 00000000
    0, //  349,  10111010 10000000 00000000 00000000
    1, //  350,  01111010 10000000 00000000 00000000
    0, //  351,  11111010 10000000 00000000 00000000
    5, //  352,  00000110 10000000 00000000 00000000
    0, //  353,  10000110 10000000 00000000 00000000
    1, //  354,  01000110 10000000 00000000 00000000
    0, //  355,  11000110 10000000 00000000 00000000
    2, //  356,  00100110 10000000 00000000 00000000
    0, //  357,  10100110 10000000 00000000 00000000
    1, //  358,  01100110 10000000 00000000 00000000
    0, //  359,  11100110 10000000 00000000 00000000
    3, //  360,  00010110 10000000 00000000 00000000
    0, //  361,  10010110 10000000 00000000 00000000
    1, //  362,  01010110 10000000 00000000 00000000
    0, //  363,  11010110 10000000 00000000 00000000
    2, //  364,  00110110 10000000 00000000 00000000
    0, //  365,  10110110 10000000 00000000 00000000
    1, //  366,  01110110 10000000 00000000 00000000
    0, //  367,  11110110 10000000 00000000 00000000
    4, //  368,  00001110 10000000 00000000 00000000
    0, //  369,  10001110 10000000 00000000 00000000
    1, //  370,  01001110 10000000 00000000 00000000
    0, //  371,  11001110 10000000 00000000 00000000
    2, //  372,  00101110 10000000 00000000 00000000
    0, //  373,  10101110 10000000 00000000 00000000
    1, //  374,  01101110 10000000 00000000 00000000
    0, //  375,  11101110 10000000 00000000 00000000
    3, //  376,  00011110 10000000 00000000 00000000
    0, //  377,  10011110 10000000 00000000 00000000
    1, //  378,  01011110 10000000 00000000 00000000
    0, //  379,  11011110 10000000 00000000 00000000
    2, //  380,  00111110 10000000 00000000 00000000
    0, //  381,  10111110 10000000 00000000 00000000
    1, //  382,  01111110 10000000 00000000 00000000
    0, //  383,  11111110 10000000 00000000 00000000
    7, //  384,  00000001 10000000 00000000 00000000
    0, //  385,  10000001 10000000 00000000 00000000
    1, //  386,  01000001 10000000 00000000 00000000
    0, //  387,  11000001 10000000 00000000 00000000
    2, //  388,  00100001 10000000 00000000 00000000
    0, //  389,  10100001 10000000 00000000 00000000
    1, //  390,  01100001 10000000 00000000 00000000
    0, //  391,  11100001 10000000 00000000 00000000
    3, //  392,  00010001 10000000 00000000 00000000
    0, //  393,  10010001 10000000 00000000 00000000
    1, //  394,  01010001 10000000 00000000 00000000
    0, //  395,  11010001 10000000 00000000 00000000
    2, //  396,  00110001 10000000 00000000 00000000
    0, //  397,  10110001 10000000 00000000 00000000
    1, //  398,  01110001 10000000 00000000 00000000
    0, //  399,  11110001 10000000 00000000 00000000
    4, //  400,  00001001 10000000 00000000 00000000
    0, //  401,  10001001 10000000 00000000 00000000
    1, //  402,  01001001 10000000 00000000 00000000
    0, //  403,  11001001 10000000 00000000 00000000
    2, //  404,  00101001 10000000 00000000 00000000
    0, //  405,  10101001 10000000 00000000 00000000
    1, //  406,  01101001 10000000 00000000 00000000
    0, //  407,  11101001 10000000 00000000 00000000
    3, //  408,  00011001 10000000 00000000 00000000
    0, //  409,  10011001 10000000 00000000 00000000
    1, //  410,  01011001 10000000 00000000 00000000
    0, //  411,  11011001 10000000 00000000 00000000
    2, //  412,  00111001 10000000 00000000 00000000
    0, //  413,  10111001 10000000 00000000 00000000
    1, //  414,  01111001 10000000 00000000 00000000
    0, //  415,  11111001 10000000 00000000 00000000
    5, //  416,  00000101 10000000 00000000 00000000
    0, //  417,  10000101 10000000 00000000 00000000
    1, //  418,  01000101 10000000 00000000 00000000
    0, //  419,  11000101 10000000 00000000 00000000
    2, //  420,  00100101 10000000 00000000 00000000
    0, //  421,  10100101 10000000 00000000 00000000
    1, //  422,  01100101 10000000 00000000 00000000
    0, //  423,  11100101 10000000 00000000 00000000
    3, //  424,  00010101 10000000 00000000 00000000
    0, //  425,  10010101 10000000 00000000 00000000
    1, //  426,  01010101 10000000 00000000 00000000
    0, //  427,  11010101 10000000 00000000 00000000
    2, //  428,  00110101 10000000 00000000 00000000
    0, //  429,  10110101 10000000 00000000 00000000
    1, //  430,  01110101 10000000 00000000 00000000
    0, //  431,  11110101 10000000 00000000 00000000
    4, //  432,  00001101 10000000 00000000 00000000
    0, //  433,  10001101 10000000 00000000 00000000
    1, //  434,  01001101 10000000 00000000 00000000
    0, //  435,  11001101 10000000 00000000 00000000
    2, //  436,  00101101 10000000 00000000 00000000
    0, //  437,  10101101 10000000 00000000 00000000
    1, //  438,  01101101 10000000 00000000 00000000
    0, //  439,  11101101 10000000 00000000 00000000
    3, //  440,  00011101 10000000 00000000 00000000
    0, //  441,  10011101 10000000 00000000 00000000
    1, //  442,  01011101 10000000 00000000 00000000
    0, //  443,  11011101 10000000 00000000 00000000
    2, //  444,  00111101 10000000 00000000 00000000
    0, //  445,  10111101 10000000 00000000 00000000
    1, //  446,  01111101 10000000 00000000 00000000
    0, //  447,  11111101 10000000 00000000 00000000
    6, //  448,  00000011 10000000 00000000 00000000
    0, //  449,  10000011 10000000 00000000 00000000
    1, //  450,  01000011 10000000 00000000 00000000
    0, //  451,  11000011 10000000 00000000 00000000
    2, //  452,  00100011 10000000 00000000 00000000
    0, //  453,  10100011 10000000 00000000 00000000
    1, //  454,  01100011 10000000 00000000 00000000
    0, //  455,  11100011 10000000 00000000 00000000
    3, //  456,  00010011 10000000 00000000 00000000
    0, //  457,  10010011 10000000 00000000 00000000
    1, //  458,  01010011 10000000 00000000 00000000
    0, //  459,  11010011 10000000 00000000 00000000
    2, //  460,  00110011 10000000 00000000 00000000
    0, //  461,  10110011 10000000 00000000 00000000
    1, //  462,  01110011 10000000 00000000 00000000
    0, //  463,  11110011 10000000 00000000 00000000
    4, //  464,  00001011 10000000 00000000 00000000
    0, //  465,  10001011 10000000 00000000 00000000
    1, //  466,  01001011 10000000 00000000 00000000
    0, //  467,  11001011 10000000 00000000 00000000
    2, //  468,  00101011 10000000 00000000 00000000
    0, //  469,  10101011 10000000 00000000 00000000
    1, //  470,  01101011 10000000 00000000 00000000
    0, //  471,  11101011 10000000 00000000 00000000
    3, //  472,  00011011 10000000 00000000 00000000
    0, //  473,  10011011 10000000 00000000 00000000
    1, //  474,  01011011 10000000 00000000 00000000
    0, //  475,  11011011 10000000 00000000 00000000
    2, //  476,  00111011 10000000 00000000 00000000
    0, //  477,  10111011 10000000 00000000 00000000
    1, //  478,  01111011 10000000 00000000 00000000
    0, //  479,  11111011 10000000 00000000 00000000
    5, //  480,  00000111 10000000 00000000 00000000
    0, //  481,  10000111 10000000 00000000 00000000
    1, //  482,  01000111 10000000 00000000 00000000
    0, //  483,  11000111 10000000 00000000 00000000
    2, //  484,  00100111 10000000 00000000 00000000
    0, //  485,  10100111 10000000 00000000 00000000
    1, //  486,  01100111 10000000 00000000 00000000
    0, //  487,  11100111 10000000 00000000 00000000
    3, //  488,  00010111 10000000 00000000 00000000
    0, //  489,  10010111 10000000 00000000 00000000
    1, //  490,  01010111 10000000 00000000 00000000
    0, //  491,  11010111 10000000 00000000 00000000
    2, //  492,  00110111 10000000 00000000 00000000
    0, //  493,  10110111 10000000 00000000 00000000
    1, //  494,  01110111 10000000 00000000 00000000
    0, //  495,  11110111 10000000 00000000 00000000
    4, //  496,  00001111 10000000 00000000 00000000
    0, //  497,  10001111 10000000 00000000 00000000
    1, //  498,  01001111 10000000 00000000 00000000
    0, //  499,  11001111 10000000 00000000 00000000
    2, //  500,  00101111 10000000 00000000 00000000
    0, //  501,  10101111 10000000 00000000 00000000
    1, //  502,  01101111 10000000 00000000 00000000
    0, //  503,  11101111 10000000 00000000 00000000
    3, //  504,  00011111 10000000 00000000 00000000
    0, //  505,  10011111 10000000 00000000 00000000
    1, //  506,  01011111 10000000 00000000 00000000
    0, //  507,  11011111 10000000 00000000 00000000
    2, //  508,  00111111 10000000 00000000 00000000
    0, //  509,  10111111 10000000 00000000 00000000
    1, //  510,  01111111 10000000 00000000 00000000
    0, //  511,  11111111 10000000 00000000 00000000
    9, //  512,  00000000 01000000 00000000 00000000
    0, //  513,  10000000 01000000 00000000 00000000
    1, //  514,  01000000 01000000 00000000 00000000
    0, //  515,  11000000 01000000 00000000 00000000
    2, //  516,  00100000 01000000 00000000 00000000
    0, //  517,  10100000 01000000 00000000 00000000
    1, //  518,  01100000 01000000 00000000 00000000
    0, //  519,  11100000 01000000 00000000 00000000
    3, //  520,  00010000 01000000 00000000 00000000
    0, //  521,  10010000 01000000 00000000 00000000
    1, //  522,  01010000 01000000 00000000 00000000
    0, //  523,  11010000 01000000 00000000 00000000
    2, //  524,  00110000 01000000 00000000 00000000
    0, //  525,  10110000 01000000 00000000 00000000
    1, //  526,  01110000 01000000 00000000 00000000
    0, //  527,  11110000 01000000 00000000 00000000
    4, //  528,  00001000 01000000 00000000 00000000
    0, //  529,  10001000 01000000 00000000 00000000
    1, //  530,  01001000 01000000 00000000 00000000
    0, //  531,  11001000 01000000 00000000 00000000
    2, //  532,  00101000 01000000 00000000 00000000
    0, //  533,  10101000 01000000 00000000 00000000
    1, //  534,  01101000 01000000 00000000 00000000
    0, //  535,  11101000 01000000 00000000 00000000
    3, //  536,  00011000 01000000 00000000 00000000
    0, //  537,  10011000 01000000 00000000 00000000
    1, //  538,  01011000 01000000 00000000 00000000
    0, //  539,  11011000 01000000 00000000 00000000
    2, //  540,  00111000 01000000 00000000 00000000
    0, //  541,  10111000 01000000 00000000 00000000
    1, //  542,  01111000 01000000 00000000 00000000
    0, //  543,  11111000 01000000 00000000 00000000
    5, //  544,  00000100 01000000 00000000 00000000
    0, //  545,  10000100 01000000 00000000 00000000
    1, //  546,  01000100 01000000 00000000 00000000
    0, //  547,  11000100 01000000 00000000 00000000
    2, //  548,  00100100 01000000 00000000 00000000
    0, //  549,  10100100 01000000 00000000 00000000
    1, //  550,  01100100 01000000 00000000 00000000
    0, //  551,  11100100 01000000 00000000 00000000
    3, //  552,  00010100 01000000 00000000 00000000
    0, //  553,  10010100 01000000 00000000 00000000
    1, //  554,  01010100 01000000 00000000 00000000
    0, //  555,  11010100 01000000 00000000 00000000
    2, //  556,  00110100 01000000 00000000 00000000
    0, //  557,  10110100 01000000 00000000 00000000
    1, //  558,  01110100 01000000 00000000 00000000
    0, //  559,  11110100 01000000 00000000 00000000
    4, //  560,  00001100 01000000 00000000 00000000
    0, //  561,  10001100 01000000 00000000 00000000
    1, //  562,  01001100 01000000 00000000 00000000
    0, //  563,  11001100 01000000 00000000 00000000
    2, //  564,  00101100 01000000 00000000 00000000
    0, //  565,  10101100 01000000 00000000 00000000
    1, //  566,  01101100 01000000 00000000 00000000
    0, //  567,  11101100 01000000 00000000 00000000
    3, //  568,  00011100 01000000 00000000 00000000
    0, //  569,  10011100 01000000 00000000 00000000
    1, //  570,  01011100 01000000 00000000 00000000
    0, //  571,  11011100 01000000 00000000 00000000
    2, //  572,  00111100 01000000 00000000 00000000
    0, //  573,  10111100 01000000 00000000 00000000
    1, //  574,  01111100 01000000 00000000 00000000
    0, //  575,  11111100 01000000 00000000 00000000
    6, //  576,  00000010 01000000 00000000 00000000
    0, //  577,  10000010 01000000 00000000 00000000
    1, //  578,  01000010 01000000 00000000 00000000
    0, //  579,  11000010 01000000 00000000 00000000
    2, //  580,  00100010 01000000 00000000 00000000
    0, //  581,  10100010 01000000 00000000 00000000
    1, //  582,  01100010 01000000 00000000 00000000
    0, //  583,  11100010 01000000 00000000 00000000
    3, //  584,  00010010 01000000 00000000 00000000
    0, //  585,  10010010 01000000 00000000 00000000
    1, //  586,  01010010 01000000 00000000 00000000
    0, //  587,  11010010 01000000 00000000 00000000
    2, //  588,  00110010 01000000 00000000 00000000
    0, //  589,  10110010 01000000 00000000 00000000
    1, //  590,  01110010 01000000 00000000 00000000
    0, //  591,  11110010 01000000 00000000 00000000
    4, //  592,  00001010 01000000 00000000 00000000
    0, //  593,  10001010 01000000 00000000 00000000
    1, //  594,  01001010 01000000 00000000 00000000
    0, //  595,  11001010 01000000 00000000 00000000
    2, //  596,  00101010 01000000 00000000 00000000
    0, //  597,  10101010 01000000 00000000 00000000
    1, //  598,  01101010 01000000 00000000 00000000
    0, //  599,  11101010 01000000 00000000 00000000
    3, //  600,  00011010 01000000 00000000 00000000
    0, //  601,  10011010 01000000 00000000 00000000
    1, //  602,  01011010 01000000 00000000 00000000
    0, //  603,  11011010 01000000 00000000 00000000
    2, //  604,  00111010 01000000 00000000 00000000
    0, //  605,  10111010 01000000 00000000 00000000
    1, //  606,  01111010 01000000 00000000 00000000
    0, //  607,  11111010 01000000 00000000 00000000
    5, //  608,  00000110 01000000 00000000 00000000
    0, //  609,  10000110 01000000 00000000 00000000
    1, //  610,  01000110 01000000 00000000 00000000
    0, //  611,  11000110 01000000 00000000 00000000
    2, //  612,  00100110 01000000 00000000 00000000
    0, //  613,  10100110 01000000 00000000 00000000
    1, //  614,  01100110 01000000 00000000 00000000
    0, //  615,  11100110 01000000 00000000 00000000
    3, //  616,  00010110 01000000 00000000 00000000
    0, //  617,  10010110 01000000 00000000 00000000
    1, //  618,  01010110 01000000 00000000 00000000
    0, //  619,  11010110 01000000 00000000 00000000
    2, //  620,  00110110 01000000 00000000 00000000
    0, //  621,  10110110 01000000 00000000 00000000
    1, //  622,  01110110 01000000 00000000 00000000
    0, //  623,  11110110 01000000 00000000 00000000
    4, //  624,  00001110 01000000 00000000 00000000
    0, //  625,  10001110 01000000 00000000 00000000
    1, //  626,  01001110 01000000 00000000 00000000
    0, //  627,  11001110 01000000 00000000 00000000
    2, //  628,  00101110 01000000 00000000 00000000
    0, //  629,  10101110 01000000 00000000 00000000
    1, //  630,  01101110 01000000 00000000 00000000
    0, //  631,  11101110 01000000 00000000 00000000
    3, //  632,  00011110 01000000 00000000 00000000
    0, //  633,  10011110 01000000 00000000 00000000
    1, //  634,  01011110 01000000 00000000 00000000
    0, //  635,  11011110 01000000 00000000 00000000
    2, //  636,  00111110 01000000 00000000 00000000
    0, //  637,  10111110 01000000 00000000 00000000
    1, //  638,  01111110 01000000 00000000 00000000
    0, //  639,  11111110 01000000 00000000 00000000
    7, //  640,  00000001 01000000 00000000 00000000
    0, //  641,  10000001 01000000 00000000 00000000
    1, //  642,  01000001 01000000 00000000 00000000
    0, //  643,  11000001 01000000 00000000 00000000
    2, //  644,  00100001 01000000 00000000 00000000
    0, //  645,  10100001 01000000 00000000 00000000
    1, //  646,  01100001 01000000 00000000 00000000
    0, //  647,  11100001 01000000 00000000 00000000
    3, //  648,  00010001 01000000 00000000 00000000
    0, //  649,  10010001 01000000 00000000 00000000
    1, //  650,  01010001 01000000 00000000 00000000
    0, //  651,  11010001 01000000 00000000 00000000
    2, //  652,  00110001 01000000 00000000 00000000
    0, //  653,  10110001 01000000 00000000 00000000
    1, //  654,  01110001 01000000 00000000 00000000
    0, //  655,  11110001 01000000 00000000 00000000
    4, //  656,  00001001 01000000 00000000 00000000
    0, //  657,  10001001 01000000 00000000 00000000
    1, //  658,  01001001 01000000 00000000 00000000
    0, //  659,  11001001 01000000 00000000 00000000
    2, //  660,  00101001 01000000 00000000 00000000
    0, //  661,  10101001 01000000 00000000 00000000
    1, //  662,  01101001 01000000 00000000 00000000
    0, //  663,  11101001 01000000 00000000 00000000
    3, //  664,  00011001 01000000 00000000 00000000
    0, //  665,  10011001 01000000 00000000 00000000
    1, //  666,  01011001 01000000 00000000 00000000
    0, //  667,  11011001 01000000 00000000 00000000
    2, //  668,  00111001 01000000 00000000 00000000
    0, //  669,  10111001 01000000 00000000 00000000
    1, //  670,  01111001 01000000 00000000 00000000
    0, //  671,  11111001 01000000 00000000 00000000
    5, //  672,  00000101 01000000 00000000 00000000
    0, //  673,  10000101 01000000 00000000 00000000
    1, //  674,  01000101 01000000 00000000 00000000
    0, //  675,  11000101 01000000 00000000 00000000
    2, //  676,  00100101 01000000 00000000 00000000
    0, //  677,  10100101 01000000 00000000 00000000
    1, //  678,  01100101 01000000 00000000 00000000
    0, //  679,  11100101 01000000 00000000 00000000
    3, //  680,  00010101 01000000 00000000 00000000
    0, //  681,  10010101 01000000 00000000 00000000
    1, //  682,  01010101 01000000 00000000 00000000
    0, //  683,  11010101 01000000 00000000 00000000
    2, //  684,  00110101 01000000 00000000 00000000
    0, //  685,  10110101 01000000 00000000 00000000
    1, //  686,  01110101 01000000 00000000 00000000
    0, //  687,  11110101 01000000 00000000 00000000
    4, //  688,  00001101 01000000 00000000 00000000
    0, //  689,  10001101 01000000 00000000 00000000
    1, //  690,  01001101 01000000 00000000 00000000
    0, //  691,  11001101 01000000 00000000 00000000
    2, //  692,  00101101 01000000 00000000 00000000
    0, //  693,  10101101 01000000 00000000 00000000
    1, //  694,  01101101 01000000 00000000 00000000
    0, //  695,  11101101 01000000 00000000 00000000
    3, //  696,  00011101 01000000 00000000 00000000
    0, //  697,  10011101 01000000 00000000 00000000
    1, //  698,  01011101 01000000 00000000 00000000
    0, //  699,  11011101 01000000 00000000 00000000
    2, //  700,  00111101 01000000 00000000 00000000
    0, //  701,  10111101 01000000 00000000 00000000
    1, //  702,  01111101 01000000 00000000 00000000
    0, //  703,  11111101 01000000 00000000 00000000
    6, //  704,  00000011 01000000 00000000 00000000
    0, //  705,  10000011 01000000 00000000 00000000
    1, //  706,  01000011 01000000 00000000 00000000
    0, //  707,  11000011 01000000 00000000 00000000
    2, //  708,  00100011 01000000 00000000 00000000
    0, //  709,  10100011 01000000 00000000 00000000
    1, //  710,  01100011 01000000 00000000 00000000
    0, //  711,  11100011 01000000 00000000 00000000
    3, //  712,  00010011 01000000 00000000 00000000
    0, //  713,  10010011 01000000 00000000 00000000
    1, //  714,  01010011 01000000 00000000 00000000
    0, //  715,  11010011 01000000 00000000 00000000
    2, //  716,  00110011 01000000 00000000 00000000
    0, //  717,  10110011 01000000 00000000 00000000
    1, //  718,  01110011 01000000 00000000 00000000
    0, //  719,  11110011 01000000 00000000 00000000
    4, //  720,  00001011 01000000 00000000 00000000
    0, //  721,  10001011 01000000 00000000 00000000
    1, //  722,  01001011 01000000 00000000 00000000
    0, //  723,  11001011 01000000 00000000 00000000
    2, //  724,  00101011 01000000 00000000 00000000
    0, //  725,  10101011 01000000 00000000 00000000
    1, //  726,  01101011 01000000 00000000 00000000
    0, //  727,  11101011 01000000 00000000 00000000
    3, //  728,  00011011 01000000 00000000 00000000
    0, //  729,  10011011 01000000 00000000 00000000
    1, //  730,  01011011 01000000 00000000 00000000
    0, //  731,  11011011 01000000 00000000 00000000
    2, //  732,  00111011 01000000 00000000 00000000
    0, //  733,  10111011 01000000 00000000 00000000
    1, //  734,  01111011 01000000 00000000 00000000
    0, //  735,  11111011 01000000 00000000 00000000
    5, //  736,  00000111 01000000 00000000 00000000
    0, //  737,  10000111 01000000 00000000 00000000
    1, //  738,  01000111 01000000 00000000 00000000
    0, //  739,  11000111 01000000 00000000 00000000
    2, //  740,  00100111 01000000 00000000 00000000
    0, //  741,  10100111 01000000 00000000 00000000
    1, //  742,  01100111 01000000 00000000 00000000
    0, //  743,  11100111 01000000 00000000 00000000
    3, //  744,  00010111 01000000 00000000 00000000
    0, //  745,  10010111 01000000 00000000 00000000
    1, //  746,  01010111 01000000 00000000 00000000
    0, //  747,  11010111 01000000 00000000 00000000
    2, //  748,  00110111 01000000 00000000 00000000
    0, //  749,  10110111 01000000 00000000 00000000
    1, //  750,  01110111 01000000 00000000 00000000
    0, //  751,  11110111 01000000 00000000 00000000
    4, //  752,  00001111 01000000 00000000 00000000
    0, //  753,  10001111 01000000 00000000 00000000
    1, //  754,  01001111 01000000 00000000 00000000
    0, //  755,  11001111 01000000 00000000 00000000
    2, //  756,  00101111 01000000 00000000 00000000
    0, //  757,  10101111 01000000 00000000 00000000
    1, //  758,  01101111 01000000 00000000 00000000
    0, //  759,  11101111 01000000 00000000 00000000
    3, //  760,  00011111 01000000 00000000 00000000
    0, //  761,  10011111 01000000 00000000 00000000
    1, //  762,  01011111 01000000 00000000 00000000
    0, //  763,  11011111 01000000 00000000 00000000
    2, //  764,  00111111 01000000 00000000 00000000
    0, //  765,  10111111 01000000 00000000 00000000
    1, //  766,  01111111 01000000 00000000 00000000
    0, //  767,  11111111 01000000 00000000 00000000
    8, //  768,  00000000 11000000 00000000 00000000
    0, //  769,  10000000 11000000 00000000 00000000
    1, //  770,  01000000 11000000 00000000 00000000
    0, //  771,  11000000 11000000 00000000 00000000
    2, //  772,  00100000 11000000 00000000 00000000
    0, //  773,  10100000 11000000 00000000 00000000
    1, //  774,  01100000 11000000 00000000 00000000
    0, //  775,  11100000 11000000 00000000 00000000
    3, //  776,  00010000 11000000 00000000 00000000
    0, //  777,  10010000 11000000 00000000 00000000
    1, //  778,  01010000 11000000 00000000 00000000
    0, //  779,  11010000 11000000 00000000 00000000
    2, //  780,  00110000 11000000 00000000 00000000
    0, //  781,  10110000 11000000 00000000 00000000
    1, //  782,  01110000 11000000 00000000 00000000
    0, //  783,  11110000 11000000 00000000 00000000
    4, //  784,  00001000 11000000 00000000 00000000
    0, //  785,  10001000 11000000 00000000 00000000
    1, //  786,  01001000 11000000 00000000 00000000
    0, //  787,  11001000 11000000 00000000 00000000
    2, //  788,  00101000 11000000 00000000 00000000
    0, //  789,  10101000 11000000 00000000 00000000
    1, //  790,  01101000 11000000 00000000 00000000
    0, //  791,  11101000 11000000 00000000 00000000
    3, //  792,  00011000 11000000 00000000 00000000
    0, //  793,  10011000 11000000 00000000 00000000
    1, //  794,  01011000 11000000 00000000 00000000
    0, //  795,  11011000 11000000 00000000 00000000
    2, //  796,  00111000 11000000 00000000 00000000
    0, //  797,  10111000 11000000 00000000 00000000
    1, //  798,  01111000 11000000 00000000 00000000
    0, //  799,  11111000 11000000 00000000 00000000
    5, //  800,  00000100 11000000 00000000 00000000
    0, //  801,  10000100 11000000 00000000 00000000
    1, //  802,  01000100 11000000 00000000 00000000
    0, //  803,  11000100 11000000 00000000 00000000
    2, //  804,  00100100 11000000 00000000 00000000
    0, //  805,  10100100 11000000 00000000 00000000
    1, //  806,  01100100 11000000 00000000 00000000
    0, //  807,  11100100 11000000 00000000 00000000
    3, //  808,  00010100 11000000 00000000 00000000
    0, //  809,  10010100 11000000 00000000 00000000
    1, //  810,  01010100 11000000 00000000 00000000
    0, //  811,  11010100 11000000 00000000 00000000
    2, //  812,  00110100 11000000 00000000 00000000
    0, //  813,  10110100 11000000 00000000 00000000
    1, //  814,  01110100 11000000 00000000 00000000
    0, //  815,  11110100 11000000 00000000 00000000
    4, //  816,  00001100 11000000 00000000 00000000
    0, //  817,  10001100 11000000 00000000 00000000
    1, //  818,  01001100 11000000 00000000 00000000
    0, //  819,  11001100 11000000 00000000 00000000
    2, //  820,  00101100 11000000 00000000 00000000
    0, //  821,  10101100 11000000 00000000 00000000
    1, //  822,  01101100 11000000 00000000 00000000
    0, //  823,  11101100 11000000 00000000 00000000
    3, //  824,  00011100 11000000 00000000 00000000
    0, //  825,  10011100 11000000 00000000 00000000
    1, //  826,  01011100 11000000 00000000 00000000
    0, //  827,  11011100 11000000 00000000 00000000
    2, //  828,  00111100 11000000 00000000 00000000
    0, //  829,  10111100 11000000 00000000 00000000
    1, //  830,  01111100 11000000 00000000 00000000
    0, //  831,  11111100 11000000 00000000 00000000
    6, //  832,  00000010 11000000 00000000 00000000
    0, //  833,  10000010 11000000 00000000 00000000
    1, //  834,  01000010 11000000 00000000 00000000
    0, //  835,  11000010 11000000 00000000 00000000
    2, //  836,  00100010 11000000 00000000 00000000
    0, //  837,  10100010 11000000 00000000 00000000
    1, //  838,  01100010 11000000 00000000 00000000
    0, //  839,  11100010 11000000 00000000 00000000
    3, //  840,  00010010 11000000 00000000 00000000
    0, //  841,  10010010 11000000 00000000 00000000
    1, //  842,  01010010 11000000 00000000 00000000
    0, //  843,  11010010 11000000 00000000 00000000
    2, //  844,  00110010 11000000 00000000 00000000
    0, //  845,  10110010 11000000 00000000 00000000
    1, //  846,  01110010 11000000 00000000 00000000
    0, //  847,  11110010 11000000 00000000 00000000
    4, //  848,  00001010 11000000 00000000 00000000
    0, //  849,  10001010 11000000 00000000 00000000
    1, //  850,  01001010 11000000 00000000 00000000
    0, //  851,  11001010 11000000 00000000 00000000
    2, //  852,  00101010 11000000 00000000 00000000
    0, //  853,  10101010 11000000 00000000 00000000
    1, //  854,  01101010 11000000 00000000 00000000
    0, //  855,  11101010 11000000 00000000 00000000
    3, //  856,  00011010 11000000 00000000 00000000
    0, //  857,  10011010 11000000 00000000 00000000
    1, //  858,  01011010 11000000 00000000 00000000
    0, //  859,  11011010 11000000 00000000 00000000
    2, //  860,  00111010 11000000 00000000 00000000
    0, //  861,  10111010 11000000 00000000 00000000
    1, //  862,  01111010 11000000 00000000 00000000
    0, //  863,  11111010 11000000 00000000 00000000
    5, //  864,  00000110 11000000 00000000 00000000
    0, //  865,  10000110 11000000 00000000 00000000
    1, //  866,  01000110 11000000 00000000 00000000
    0, //  867,  11000110 11000000 00000000 00000000
    2, //  868,  00100110 11000000 00000000 00000000
    0, //  869,  10100110 11000000 00000000 00000000
    1, //  870,  01100110 11000000 00000000 00000000
    0, //  871,  11100110 11000000 00000000 00000000
    3, //  872,  00010110 11000000 00000000 00000000
    0, //  873,  10010110 11000000 00000000 00000000
    1, //  874,  01010110 11000000 00000000 00000000
    0, //  875,  11010110 11000000 00000000 00000000
    2, //  876,  00110110 11000000 00000000 00000000
    0, //  877,  10110110 11000000 00000000 00000000
    1, //  878,  01110110 11000000 00000000 00000000
    0, //  879,  11110110 11000000 00000000 00000000
    4, //  880,  00001110 11000000 00000000 00000000
    0, //  881,  10001110 11000000 00000000 00000000
    1, //  882,  01001110 11000000 00000000 00000000
    0, //  883,  11001110 11000000 00000000 00000000
    2, //  884,  00101110 11000000 00000000 00000000
    0, //  885,  10101110 11000000 00000000 00000000
    1, //  886,  01101110 11000000 00000000 00000000
    0, //  887,  11101110 11000000 00000000 00000000
    3, //  888,  00011110 11000000 00000000 00000000
    0, //  889,  10011110 11000000 00000000 00000000
    1, //  890,  01011110 11000000 00000000 00000000
    0, //  891,  11011110 11000000 00000000 00000000
    2, //  892,  00111110 11000000 00000000 00000000
    0, //  893,  10111110 11000000 00000000 00000000
    1, //  894,  01111110 11000000 00000000 00000000
    0, //  895,  11111110 11000000 00000000 00000000
    7, //  896,  00000001 11000000 00000000 00000000
    0, //  897,  10000001 11000000 00000000 00000000
    1, //  898,  01000001 11000000 00000000 00000000
    0, //  899,  11000001 11000000 00000000 00000000
    2, //  900,  00100001 11000000 00000000 00000000
    0, //  901,  10100001 11000000 00000000 00000000
    1, //  902,  01100001 11000000 00000000 00000000
    0, //  903,  11100001 11000000 00000000 00000000
    3, //  904,  00010001 11000000 00000000 00000000
    0, //  905,  10010001 11000000 00000000 00000000
    1, //  906,  01010001 11000000 00000000 00000000
    0, //  907,  11010001 11000000 00000000 00000000
    2, //  908,  00110001 11000000 00000000 00000000
    0, //  909,  10110001 11000000 00000000 00000000
    1, //  910,  01110001 11000000 00000000 00000000
    0, //  911,  11110001 11000000 00000000 00000000
    4, //  912,  00001001 11000000 00000000 00000000
    0, //  913,  10001001 11000000 00000000 00000000
    1, //  914,  01001001 11000000 00000000 00000000
    0, //  915,  11001001 11000000 00000000 00000000
    2, //  916,  00101001 11000000 00000000 00000000
    0, //  917,  10101001 11000000 00000000 00000000
    1, //  918,  01101001 11000000 00000000 00000000
    0, //  919,  11101001 11000000 00000000 00000000
    3, //  920,  00011001 11000000 00000000 00000000
    0, //  921,  10011001 11000000 00000000 00000000
    1, //  922,  01011001 11000000 00000000 00000000
    0, //  923,  11011001 11000000 00000000 00000000
    2, //  924,  00111001 11000000 00000000 00000000
    0, //  925,  10111001 11000000 00000000 00000000
    1, //  926,  01111001 11000000 00000000 00000000
    0, //  927,  11111001 11000000 00000000 00000000
    5, //  928,  00000101 11000000 00000000 00000000
    0, //  929,  10000101 11000000 00000000 00000000
    1, //  930,  01000101 11000000 00000000 00000000
    0, //  931,  11000101 11000000 00000000 00000000
    2, //  932,  00100101 11000000 00000000 00000000
    0, //  933,  10100101 11000000 00000000 00000000
    1, //  934,  01100101 11000000 00000000 00000000
    0, //  935,  11100101 11000000 00000000 00000000
    3, //  936,  00010101 11000000 00000000 00000000
    0, //  937,  10010101 11000000 00000000 00000000
    1, //  938,  01010101 11000000 00000000 00000000
    0, //  939,  11010101 11000000 00000000 00000000
    2, //  940,  00110101 11000000 00000000 00000000
    0, //  941,  10110101 11000000 00000000 00000000
    1, //  942,  01110101 11000000 00000000 00000000
    0, //  943,  11110101 11000000 00000000 00000000
    4, //  944,  00001101 11000000 00000000 00000000
    0, //  945,  10001101 11000000 00000000 00000000
    1, //  946,  01001101 11000000 00000000 00000000
    0, //  947,  11001101 11000000 00000000 00000000
    2, //  948,  00101101 11000000 00000000 00000000
    0, //  949,  10101101 11000000 00000000 00000000
    1, //  950,  01101101 11000000 00000000 00000000
    0, //  951,  11101101 11000000 00000000 00000000
    3, //  952,  00011101 11000000 00000000 00000000
    0, //  953,  10011101 11000000 00000000 00000000
    1, //  954,  01011101 11000000 00000000 00000000
    0, //  955,  11011101 11000000 00000000 00000000
    2, //  956,  00111101 11000000 00000000 00000000
    0, //  957,  10111101 11000000 00000000 00000000
    1, //  958,  01111101 11000000 00000000 00000000
    0, //  959,  11111101 11000000 00000000 00000000
    6, //  960,  00000011 11000000 00000000 00000000
    0, //  961,  10000011 11000000 00000000 00000000
    1, //  962,  01000011 11000000 00000000 00000000
    0, //  963,  11000011 11000000 00000000 00000000
    2, //  964,  00100011 11000000 00000000 00000000
    0, //  965,  10100011 11000000 00000000 00000000
    1, //  966,  01100011 11000000 00000000 00000000
    0, //  967,  11100011 11000000 00000000 00000000
    3, //  968,  00010011 11000000 00000000 00000000
    0, //  969,  10010011 11000000 00000000 00000000
    1, //  970,  01010011 11000000 00000000 00000000
    0, //  971,  11010011 11000000 00000000 00000000
    2, //  972,  00110011 11000000 00000000 00000000
    0, //  973,  10110011 11000000 00000000 00000000
    1, //  974,  01110011 11000000 00000000 00000000
    0, //  975,  11110011 11000000 00000000 00000000
    4, //  976,  00001011 11000000 00000000 00000000
    0, //  977,  10001011 11000000 00000000 00000000
    1, //  978,  01001011 11000000 00000000 00000000
    0, //  979,  11001011 11000000 00000000 00000000
    2, //  980,  00101011 11000000 00000000 00000000
    0, //  981,  10101011 11000000 00000000 00000000
    1, //  982,  01101011 11000000 00000000 00000000
    0, //  983,  11101011 11000000 00000000 00000000
    3, //  984,  00011011 11000000 00000000 00000000
    0, //  985,  10011011 11000000 00000000 00000000
    1, //  986,  01011011 11000000 00000000 00000000
    0, //  987,  11011011 11000000 00000000 00000000
    2, //  988,  00111011 11000000 00000000 00000000
    0, //  989,  10111011 11000000 00000000 00000000
    1, //  990,  01111011 11000000 00000000 00000000
    0, //  991,  11111011 11000000 00000000 00000000
    5, //  992,  00000111 11000000 00000000 00000000
    0, //  993,  10000111 11000000 00000000 00000000
    1, //  994,  01000111 11000000 00000000 00000000
    0, //  995,  11000111 11000000 00000000 00000000
    2, //  996,  00100111 11000000 00000000 00000000
    0, //  997,  10100111 11000000 00000000 00000000
    1, //  998,  01100111 11000000 00000000 00000000
    0, //  999,  11100111 11000000 00000000 00000000
    3, // 1000,  00010111 11000000 00000000 00000000
    0, // 1001,  10010111 11000000 00000000 00000000
    1, // 1002,  01010111 11000000 00000000 00000000
    0, // 1003,  11010111 11000000 00000000 00000000
    2, // 1004,  00110111 11000000 00000000 00000000
    0, // 1005,  10110111 11000000 00000000 00000000
    1, // 1006,  01110111 11000000 00000000 00000000
    0, // 1007,  11110111 11000000 00000000 00000000
    4, // 1008,  00001111 11000000 00000000 00000000
    0, // 1009,  10001111 11000000 00000000 00000000
    1, // 1010,  01001111 11000000 00000000 00000000
    0, // 1011,  11001111 11000000 00000000 00000000
    2, // 1012,  00101111 11000000 00000000 00000000
    0, // 1013,  10101111 11000000 00000000 00000000
    1, // 1014,  01101111 11000000 00000000 00000000
    0, // 1015,  11101111 11000000 00000000 00000000
    3, // 1016,  00011111 11000000 00000000 00000000
    0, // 1017,  10011111 11000000 00000000 00000000
    1, // 1018,  01011111 11000000 00000000 00000000
    0, // 1019,  11011111 11000000 00000000 00000000
    2, // 1020,  00111111 11000000 00000000 00000000
    0, // 1021,  10111111 11000000 00000000 00000000
    1, // 1022,  01111111 11000000 00000000 00000000
    0, // 1023,  11111111 11000000 00000000 00000000
    10, // 1024,  00000000 00100000 00000000 00000000
    0, // 1025,  10000000 00100000 00000000 00000000
    1, // 1026,  01000000 00100000 00000000 00000000
    0, // 1027,  11000000 00100000 00000000 00000000
    2, // 1028,  00100000 00100000 00000000 00000000
    0, // 1029,  10100000 00100000 00000000 00000000
    1, // 1030,  01100000 00100000 00000000 00000000
    0, // 1031,  11100000 00100000 00000000 00000000
    3, // 1032,  00010000 00100000 00000000 00000000
    0, // 1033,  10010000 00100000 00000000 00000000
    1, // 1034,  01010000 00100000 00000000 00000000
    0, // 1035,  11010000 00100000 00000000 00000000
    2, // 1036,  00110000 00100000 00000000 00000000
    0, // 1037,  10110000 00100000 00000000 00000000
    1, // 1038,  01110000 00100000 00000000 00000000
    0, // 1039,  11110000 00100000 00000000 00000000
    4, // 1040,  00001000 00100000 00000000 00000000
    0, // 1041,  10001000 00100000 00000000 00000000
    1, // 1042,  01001000 00100000 00000000 00000000
    0, // 1043,  11001000 00100000 00000000 00000000
    2, // 1044,  00101000 00100000 00000000 00000000
    0, // 1045,  10101000 00100000 00000000 00000000
    1, // 1046,  01101000 00100000 00000000 00000000
    0, // 1047,  11101000 00100000 00000000 00000000
    3, // 1048,  00011000 00100000 00000000 00000000
    0, // 1049,  10011000 00100000 00000000 00000000
    1, // 1050,  01011000 00100000 00000000 00000000
    0, // 1051,  11011000 00100000 00000000 00000000
    2, // 1052,  00111000 00100000 00000000 00000000
    0, // 1053,  10111000 00100000 00000000 00000000
    1, // 1054,  01111000 00100000 00000000 00000000
    0, // 1055,  11111000 00100000 00000000 00000000
    5, // 1056,  00000100 00100000 00000000 00000000
    0, // 1057,  10000100 00100000 00000000 00000000
    1, // 1058,  01000100 00100000 00000000 00000000
    0, // 1059,  11000100 00100000 00000000 00000000
    2, // 1060,  00100100 00100000 00000000 00000000
    0, // 1061,  10100100 00100000 00000000 00000000
    1, // 1062,  01100100 00100000 00000000 00000000
    0, // 1063,  11100100 00100000 00000000 00000000
    3, // 1064,  00010100 00100000 00000000 00000000
    0, // 1065,  10010100 00100000 00000000 00000000
    1, // 1066,  01010100 00100000 00000000 00000000
    0, // 1067,  11010100 00100000 00000000 00000000
    2, // 1068,  00110100 00100000 00000000 00000000
    0, // 1069,  10110100 00100000 00000000 00000000
    1, // 1070,  01110100 00100000 00000000 00000000
    0, // 1071,  11110100 00100000 00000000 00000000
    4, // 1072,  00001100 00100000 00000000 00000000
    0, // 1073,  10001100 00100000 00000000 00000000
    1, // 1074,  01001100 00100000 00000000 00000000
    0, // 1075,  11001100 00100000 00000000 00000000
    2, // 1076,  00101100 00100000 00000000 00000000
    0, // 1077,  10101100 00100000 00000000 00000000
    1, // 1078,  01101100 00100000 00000000 00000000
    0, // 1079,  11101100 00100000 00000000 00000000
    3, // 1080,  00011100 00100000 00000000 00000000
    0, // 1081,  10011100 00100000 00000000 00000000
    1, // 1082,  01011100 00100000 00000000 00000000
    0, // 1083,  11011100 00100000 00000000 00000000
    2, // 1084,  00111100 00100000 00000000 00000000
    0, // 1085,  10111100 00100000 00000000 00000000
    1, // 1086,  01111100 00100000 00000000 00000000
    0, // 1087,  11111100 00100000 00000000 00000000
    6, // 1088,  00000010 00100000 00000000 00000000
    0, // 1089,  10000010 00100000 00000000 00000000
    1, // 1090,  01000010 00100000 00000000 00000000
    0, // 1091,  11000010 00100000 00000000 00000000
    2, // 1092,  00100010 00100000 00000000 00000000
    0, // 1093,  10100010 00100000 00000000 00000000
    1, // 1094,  01100010 00100000 00000000 00000000
    0, // 1095,  11100010 00100000 00000000 00000000
    3, // 1096,  00010010 00100000 00000000 00000000
    0, // 1097,  10010010 00100000 00000000 00000000
    1, // 1098,  01010010 00100000 00000000 00000000
    0, // 1099,  11010010 00100000 00000000 00000000
    2, // 1100,  00110010 00100000 00000000 00000000
    0, // 1101,  10110010 00100000 00000000 00000000
    1, // 1102,  01110010 00100000 00000000 00000000
    0, // 1103,  11110010 00100000 00000000 00000000
    4, // 1104,  00001010 00100000 00000000 00000000
    0, // 1105,  10001010 00100000 00000000 00000000
    1, // 1106,  01001010 00100000 00000000 00000000
    0, // 1107,  11001010 00100000 00000000 00000000
    2, // 1108,  00101010 00100000 00000000 00000000
    0, // 1109,  10101010 00100000 00000000 00000000
    1, // 1110,  01101010 00100000 00000000 00000000
    0, // 1111,  11101010 00100000 00000000 00000000
    3, // 1112,  00011010 00100000 00000000 00000000
    0, // 1113,  10011010 00100000 00000000 00000000
    1, // 1114,  01011010 00100000 00000000 00000000
    0, // 1115,  11011010 00100000 00000000 00000000
    2, // 1116,  00111010 00100000 00000000 00000000
    0, // 1117,  10111010 00100000 00000000 00000000
    1, // 1118,  01111010 00100000 00000000 00000000
    0, // 1119,  11111010 00100000 00000000 00000000
    5, // 1120,  00000110 00100000 00000000 00000000
    0, // 1121,  10000110 00100000 00000000 00000000
    1, // 1122,  01000110 00100000 00000000 00000000
    0, // 1123,  11000110 00100000 00000000 00000000
    2, // 1124,  00100110 00100000 00000000 00000000
    0, // 1125,  10100110 00100000 00000000 00000000
    1, // 1126,  01100110 00100000 00000000 00000000
    0, // 1127,  11100110 00100000 00000000 00000000
    3, // 1128,  00010110 00100000 00000000 00000000
    0, // 1129,  10010110 00100000 00000000 00000000
    1, // 1130,  01010110 00100000 00000000 00000000
    0, // 1131,  11010110 00100000 00000000 00000000
    2, // 1132,  00110110 00100000 00000000 00000000
    0, // 1133,  10110110 00100000 00000000 00000000
    1, // 1134,  01110110 00100000 00000000 00000000
    0, // 1135,  11110110 00100000 00000000 00000000
    4, // 1136,  00001110 00100000 00000000 00000000
    0, // 1137,  10001110 00100000 00000000 00000000
    1, // 1138,  01001110 00100000 00000000 00000000
    0, // 1139,  11001110 00100000 00000000 00000000
    2, // 1140,  00101110 00100000 00000000 00000000
    0, // 1141,  10101110 00100000 00000000 00000000
    1, // 1142,  01101110 00100000 00000000 00000000
    0, // 1143,  11101110 00100000 00000000 00000000
    3, // 1144,  00011110 00100000 00000000 00000000
    0, // 1145,  10011110 00100000 00000000 00000000
    1, // 1146,  01011110 00100000 00000000 00000000
    0, // 1147,  11011110 00100000 00000000 00000000
    2, // 1148,  00111110 00100000 00000000 00000000
    0, // 1149,  10111110 00100000 00000000 00000000
    1, // 1150,  01111110 00100000 00000000 00000000
    0, // 1151,  11111110 00100000 00000000 00000000
    7, // 1152,  00000001 00100000 00000000 00000000
    0, // 1153,  10000001 00100000 00000000 00000000
    1, // 1154,  01000001 00100000 00000000 00000000
    0, // 1155,  11000001 00100000 00000000 00000000
    2, // 1156,  00100001 00100000 00000000 00000000
    0, // 1157,  10100001 00100000 00000000 00000000
    1, // 1158,  01100001 00100000 00000000 00000000
    0, // 1159,  11100001 00100000 00000000 00000000
    3, // 1160,  00010001 00100000 00000000 00000000
    0, // 1161,  10010001 00100000 00000000 00000000
    1, // 1162,  01010001 00100000 00000000 00000000
    0, // 1163,  11010001 00100000 00000000 00000000
    2, // 1164,  00110001 00100000 00000000 00000000
    0, // 1165,  10110001 00100000 00000000 00000000
    1, // 1166,  01110001 00100000 00000000 00000000
    0, // 1167,  11110001 00100000 00000000 00000000
    4, // 1168,  00001001 00100000 00000000 00000000
    0, // 1169,  10001001 00100000 00000000 00000000
    1, // 1170,  01001001 00100000 00000000 00000000
    0, // 1171,  11001001 00100000 00000000 00000000
    2, // 1172,  00101001 00100000 00000000 00000000
    0, // 1173,  10101001 00100000 00000000 00000000
    1, // 1174,  01101001 00100000 00000000 00000000
    0, // 1175,  11101001 00100000 00000000 00000000
    3, // 1176,  00011001 00100000 00000000 00000000
    0, // 1177,  10011001 00100000 00000000 00000000
    1, // 1178,  01011001 00100000 00000000 00000000
    0, // 1179,  11011001 00100000 00000000 00000000
    2, // 1180,  00111001 00100000 00000000 00000000
    0, // 1181,  10111001 00100000 00000000 00000000
    1, // 1182,  01111001 00100000 00000000 00000000
    0, // 1183,  11111001 00100000 00000000 00000000
    5, // 1184,  00000101 00100000 00000000 00000000
    0, // 1185,  10000101 00100000 00000000 00000000
    1, // 1186,  01000101 00100000 00000000 00000000
    0, // 1187,  11000101 00100000 00000000 00000000
    2, // 1188,  00100101 00100000 00000000 00000000
    0, // 1189,  10100101 00100000 00000000 00000000
    1, // 1190,  01100101 00100000 00000000 00000000
    0, // 1191,  11100101 00100000 00000000 00000000
    3, // 1192,  00010101 00100000 00000000 00000000
    0, // 1193,  10010101 00100000 00000000 00000000
    1, // 1194,  01010101 00100000 00000000 00000000
    0, // 1195,  11010101 00100000 00000000 00000000
    2, // 1196,  00110101 00100000 00000000 00000000
    0, // 1197,  10110101 00100000 00000000 00000000
    1, // 1198,  01110101 00100000 00000000 00000000
    0, // 1199,  11110101 00100000 00000000 00000000
    4, // 1200,  00001101 00100000 00000000 00000000
    0, // 1201,  10001101 00100000 00000000 00000000
    1, // 1202,  01001101 00100000 00000000 00000000
    0, // 1203,  11001101 00100000 00000000 00000000
    2, // 1204,  00101101 00100000 00000000 00000000
    0, // 1205,  10101101 00100000 00000000 00000000
    1, // 1206,  01101101 00100000 00000000 00000000
    0, // 1207,  11101101 00100000 00000000 00000000
    3, // 1208,  00011101 00100000 00000000 00000000
    0, // 1209,  10011101 00100000 00000000 00000000
    1, // 1210,  01011101 00100000 00000000 00000000
    0, // 1211,  11011101 00100000 00000000 00000000
    2, // 1212,  00111101 00100000 00000000 00000000
    0, // 1213,  10111101 00100000 00000000 00000000
    1, // 1214,  01111101 00100000 00000000 00000000
    0, // 1215,  11111101 00100000 00000000 00000000
    6, // 1216,  00000011 00100000 00000000 00000000
    0, // 1217,  10000011 00100000 00000000 00000000
    1, // 1218,  01000011 00100000 00000000 00000000
    0, // 1219,  11000011 00100000 00000000 00000000
    2, // 1220,  00100011 00100000 00000000 00000000
    0, // 1221,  10100011 00100000 00000000 00000000
    1, // 1222,  01100011 00100000 00000000 00000000
    0, // 1223,  11100011 00100000 00000000 00000000
    3, // 1224,  00010011 00100000 00000000 00000000
    0, // 1225,  10010011 00100000 00000000 00000000
    1, // 1226,  01010011 00100000 00000000 00000000
    0, // 1227,  11010011 00100000 00000000 00000000
    2, // 1228,  00110011 00100000 00000000 00000000
    0, // 1229,  10110011 00100000 00000000 00000000
    1, // 1230,  01110011 00100000 00000000 00000000
    0, // 1231,  11110011 00100000 00000000 00000000
    4, // 1232,  00001011 00100000 00000000 00000000
    0, // 1233,  10001011 00100000 00000000 00000000
    1, // 1234,  01001011 00100000 00000000 00000000
    0, // 1235,  11001011 00100000 00000000 00000000
    2, // 1236,  00101011 00100000 00000000 00000000
    0, // 1237,  10101011 00100000 00000000 00000000
    1, // 1238,  01101011 00100000 00000000 00000000
    0, // 1239,  11101011 00100000 00000000 00000000
    3, // 1240,  00011011 00100000 00000000 00000000
    0, // 1241,  10011011 00100000 00000000 00000000
    1, // 1242,  01011011 00100000 00000000 00000000
    0, // 1243,  11011011 00100000 00000000 00000000
    2, // 1244,  00111011 00100000 00000000 00000000
    0, // 1245,  10111011 00100000 00000000 00000000
    1, // 1246,  01111011 00100000 00000000 00000000
    0, // 1247,  11111011 00100000 00000000 00000000
    5, // 1248,  00000111 00100000 00000000 00000000
    0, // 1249,  10000111 00100000 00000000 00000000
    1, // 1250,  01000111 00100000 00000000 00000000
    0, // 1251,  11000111 00100000 00000000 00000000
    2, // 1252,  00100111 00100000 00000000 00000000
    0, // 1253,  10100111 00100000 00000000 00000000
    1, // 1254,  01100111 00100000 00000000 00000000
    0, // 1255,  11100111 00100000 00000000 00000000
    3, // 1256,  00010111 00100000 00000000 00000000
    0, // 1257,  10010111 00100000 00000000 00000000
    1, // 1258,  01010111 00100000 00000000 00000000
    0, // 1259,  11010111 00100000 00000000 00000000
    2, // 1260,  00110111 00100000 00000000 00000000
    0, // 1261,  10110111 00100000 00000000 00000000
    1, // 1262,  01110111 00100000 00000000 00000000
    0, // 1263,  11110111 00100000 00000000 00000000
    4, // 1264,  00001111 00100000 00000000 00000000
    0, // 1265,  10001111 00100000 00000000 00000000
    1, // 1266,  01001111 00100000 00000000 00000000
    0, // 1267,  11001111 00100000 00000000 00000000
    2, // 1268,  00101111 00100000 00000000 00000000
    0, // 1269,  10101111 00100000 00000000 00000000
    1, // 1270,  01101111 00100000 00000000 00000000
    0, // 1271,  11101111 00100000 00000000 00000000
    3, // 1272,  00011111 00100000 00000000 00000000
    0, // 1273,  10011111 00100000 00000000 00000000
    1, // 1274,  01011111 00100000 00000000 00000000
    0, // 1275,  11011111 00100000 00000000 00000000
    2, // 1276,  00111111 00100000 00000000 00000000
    0, // 1277,  10111111 00100000 00000000 00000000
    1, // 1278,  01111111 00100000 00000000 00000000
    0, // 1279,  11111111 00100000 00000000 00000000
    8, // 1280,  00000000 10100000 00000000 00000000
    0, // 1281,  10000000 10100000 00000000 00000000
    1, // 1282,  01000000 10100000 00000000 00000000
    0, // 1283,  11000000 10100000 00000000 00000000
    2, // 1284,  00100000 10100000 00000000 00000000
    0, // 1285,  10100000 10100000 00000000 00000000
    1, // 1286,  01100000 10100000 00000000 00000000
    0, // 1287,  11100000 10100000 00000000 00000000
    3, // 1288,  00010000 10100000 00000000 00000000
    0, // 1289,  10010000 10100000 00000000 00000000
    1, // 1290,  01010000 10100000 00000000 00000000
    0, // 1291,  11010000 10100000 00000000 00000000
    2, // 1292,  00110000 10100000 00000000 00000000
    0, // 1293,  10110000 10100000 00000000 00000000
    1, // 1294,  01110000 10100000 00000000 00000000
    0, // 1295,  11110000 10100000 00000000 00000000
    4, // 1296,  00001000 10100000 00000000 00000000
    0, // 1297,  10001000 10100000 00000000 00000000
    1, // 1298,  01001000 10100000 00000000 00000000
    0, // 1299,  11001000 10100000 00000000 00000000
    2, // 1300,  00101000 10100000 00000000 00000000
    0, // 1301,  10101000 10100000 00000000 00000000
    1, // 1302,  01101000 10100000 00000000 00000000
    0, // 1303,  11101000 10100000 00000000 00000000
    3, // 1304,  00011000 10100000 00000000 00000000
    0, // 1305,  10011000 10100000 00000000 00000000
    1, // 1306,  01011000 10100000 00000000 00000000
    0, // 1307,  11011000 10100000 00000000 00000000
    2, // 1308,  00111000 10100000 00000000 00000000
    0, // 1309,  10111000 10100000 00000000 00000000
    1, // 1310,  01111000 10100000 00000000 00000000
    0, // 1311,  11111000 10100000 00000000 00000000
    5, // 1312,  00000100 10100000 00000000 00000000
    0, // 1313,  10000100 10100000 00000000 00000000
    1, // 1314,  01000100 10100000 00000000 00000000
    0, // 1315,  11000100 10100000 00000000 00000000
    2, // 1316,  00100100 10100000 00000000 00000000
    0, // 1317,  10100100 10100000 00000000 00000000
    1, // 1318,  01100100 10100000 00000000 00000000
    0, // 1319,  11100100 10100000 00000000 00000000
    3, // 1320,  00010100 10100000 00000000 00000000
    0, // 1321,  10010100 10100000 00000000 00000000
    1, // 1322,  01010100 10100000 00000000 00000000
    0, // 1323,  11010100 10100000 00000000 00000000
    2, // 1324,  00110100 10100000 00000000 00000000
    0, // 1325,  10110100 10100000 00000000 00000000
    1, // 1326,  01110100 10100000 00000000 00000000
    0, // 1327,  11110100 10100000 00000000 00000000
    4, // 1328,  00001100 10100000 00000000 00000000
    0, // 1329,  10001100 10100000 00000000 00000000
    1, // 1330,  01001100 10100000 00000000 00000000
    0, // 1331,  11001100 10100000 00000000 00000000
    2, // 1332,  00101100 10100000 00000000 00000000
    0, // 1333,  10101100 10100000 00000000 00000000
    1, // 1334,  01101100 10100000 00000000 00000000
    0, // 1335,  11101100 10100000 00000000 00000000
    3, // 1336,  00011100 10100000 00000000 00000000
    0, // 1337,  10011100 10100000 00000000 00000000
    1, // 1338,  01011100 10100000 00000000 00000000
    0, // 1339,  11011100 10100000 00000000 00000000
    2, // 1340,  00111100 10100000 00000000 00000000
    0, // 1341,  10111100 10100000 00000000 00000000
    1, // 1342,  01111100 10100000 00000000 00000000
    0, // 1343,  11111100 10100000 00000000 00000000
    6, // 1344,  00000010 10100000 00000000 00000000
    0, // 1345,  10000010 10100000 00000000 00000000
    1, // 1346,  01000010 10100000 00000000 00000000
    0, // 1347,  11000010 10100000 00000000 00000000
    2, // 1348,  00100010 10100000 00000000 00000000
    0, // 1349,  10100010 10100000 00000000 00000000
    1, // 1350,  01100010 10100000 00000000 00000000
    0, // 1351,  11100010 10100000 00000000 00000000
    3, // 1352,  00010010 10100000 00000000 00000000
    0, // 1353,  10010010 10100000 00000000 00000000
    1, // 1354,  01010010 10100000 00000000 00000000
    0, // 1355,  11010010 10100000 00000000 00000000
    2, // 1356,  00110010 10100000 00000000 00000000
    0, // 1357,  10110010 10100000 00000000 00000000
    1, // 1358,  01110010 10100000 00000000 00000000
    0, // 1359,  11110010 10100000 00000000 00000000
    4, // 1360,  00001010 10100000 00000000 00000000
    0, // 1361,  10001010 10100000 00000000 00000000
    1, // 1362,  01001010 10100000 00000000 00000000
    0, // 1363,  11001010 10100000 00000000 00000000
    2, // 1364,  00101010 10100000 00000000 00000000
    0, // 1365,  10101010 10100000 00000000 00000000
    1, // 1366,  01101010 10100000 00000000 00000000
    0, // 1367,  11101010 10100000 00000000 00000000
    3, // 1368,  00011010 10100000 00000000 00000000
    0, // 1369,  10011010 10100000 00000000 00000000
    1, // 1370,  01011010 10100000 00000000 00000000
    0, // 1371,  11011010 10100000 00000000 00000000
    2, // 1372,  00111010 10100000 00000000 00000000
    0, // 1373,  10111010 10100000 00000000 00000000
    1, // 1374,  01111010 10100000 00000000 00000000
    0, // 1375,  11111010 10100000 00000000 00000000
    5, // 1376,  00000110 10100000 00000000 00000000
    0, // 1377,  10000110 10100000 00000000 00000000
    1, // 1378,  01000110 10100000 00000000 00000000
    0, // 1379,  11000110 10100000 00000000 00000000
    2, // 1380,  00100110 10100000 00000000 00000000
    0, // 1381,  10100110 10100000 00000000 00000000
    1, // 1382,  01100110 10100000 00000000 00000000
    0, // 1383,  11100110 10100000 00000000 00000000
    3, // 1384,  00010110 10100000 00000000 00000000
    0, // 1385,  10010110 10100000 00000000 00000000
    1, // 1386,  01010110 10100000 00000000 00000000
    0, // 1387,  11010110 10100000 00000000 00000000
    2, // 1388,  00110110 10100000 00000000 00000000
    0, // 1389,  10110110 10100000 00000000 00000000
    1, // 1390,  01110110 10100000 00000000 00000000
    0, // 1391,  11110110 10100000 00000000 00000000
    4, // 1392,  00001110 10100000 00000000 00000000
    0, // 1393,  10001110 10100000 00000000 00000000
    1, // 1394,  01001110 10100000 00000000 00000000
    0, // 1395,  11001110 10100000 00000000 00000000
    2, // 1396,  00101110 10100000 00000000 00000000
    0, // 1397,  10101110 10100000 00000000 00000000
    1, // 1398,  01101110 10100000 00000000 00000000
    0, // 1399,  11101110 10100000 00000000 00000000
    3, // 1400,  00011110 10100000 00000000 00000000
    0, // 1401,  10011110 10100000 00000000 00000000
    1, // 1402,  01011110 10100000 00000000 00000000
    0, // 1403,  11011110 10100000 00000000 00000000
    2, // 1404,  00111110 10100000 00000000 00000000
    0, // 1405,  10111110 10100000 00000000 00000000
    1, // 1406,  01111110 10100000 00000000 00000000
    0, // 1407,  11111110 10100000 00000000 00000000
    7, // 1408,  00000001 10100000 00000000 00000000
    0, // 1409,  10000001 10100000 00000000 00000000
    1, // 1410,  01000001 10100000 00000000 00000000
    0, // 1411,  11000001 10100000 00000000 00000000
    2, // 1412,  00100001 10100000 00000000 00000000
    0, // 1413,  10100001 10100000 00000000 00000000
    1, // 1414,  01100001 10100000 00000000 00000000
    0, // 1415,  11100001 10100000 00000000 00000000
    3, // 1416,  00010001 10100000 00000000 00000000
    0, // 1417,  10010001 10100000 00000000 00000000
    1, // 1418,  01010001 10100000 00000000 00000000
    0, // 1419,  11010001 10100000 00000000 00000000
    2, // 1420,  00110001 10100000 00000000 00000000
    0, // 1421,  10110001 10100000 00000000 00000000
    1, // 1422,  01110001 10100000 00000000 00000000
    0, // 1423,  11110001 10100000 00000000 00000000
    4, // 1424,  00001001 10100000 00000000 00000000
    0, // 1425,  10001001 10100000 00000000 00000000
    1, // 1426,  01001001 10100000 00000000 00000000
    0, // 1427,  11001001 10100000 00000000 00000000
    2, // 1428,  00101001 10100000 00000000 00000000
    0, // 1429,  10101001 10100000 00000000 00000000
    1, // 1430,  01101001 10100000 00000000 00000000
    0, // 1431,  11101001 10100000 00000000 00000000
    3, // 1432,  00011001 10100000 00000000 00000000
    0, // 1433,  10011001 10100000 00000000 00000000
    1, // 1434,  01011001 10100000 00000000 00000000
    0, // 1435,  11011001 10100000 00000000 00000000
    2, // 1436,  00111001 10100000 00000000 00000000
    0, // 1437,  10111001 10100000 00000000 00000000
    1, // 1438,  01111001 10100000 00000000 00000000
    0, // 1439,  11111001 10100000 00000000 00000000
    5, // 1440,  00000101 10100000 00000000 00000000
    0, // 1441,  10000101 10100000 00000000 00000000
    1, // 1442,  01000101 10100000 00000000 00000000
    0, // 1443,  11000101 10100000 00000000 00000000
    2, // 1444,  00100101 10100000 00000000 00000000
    0, // 1445,  10100101 10100000 00000000 00000000
    1, // 1446,  01100101 10100000 00000000 00000000
    0, // 1447,  11100101 10100000 00000000 00000000
    3, // 1448,  00010101 10100000 00000000 00000000
    0, // 1449,  10010101 10100000 00000000 00000000
    1, // 1450,  01010101 10100000 00000000 00000000
    0, // 1451,  11010101 10100000 00000000 00000000
    2, // 1452,  00110101 10100000 00000000 00000000
    0, // 1453,  10110101 10100000 00000000 00000000
    1, // 1454,  01110101 10100000 00000000 00000000
    0, // 1455,  11110101 10100000 00000000 00000000
    4, // 1456,  00001101 10100000 00000000 00000000
    0, // 1457,  10001101 10100000 00000000 00000000
    1, // 1458,  01001101 10100000 00000000 00000000
    0, // 1459,  11001101 10100000 00000000 00000000
    2, // 1460,  00101101 10100000 00000000 00000000
    0, // 1461,  10101101 10100000 00000000 00000000
    1, // 1462,  01101101 10100000 00000000 00000000
    0, // 1463,  11101101 10100000 00000000 00000000
    3, // 1464,  00011101 10100000 00000000 00000000
    0, // 1465,  10011101 10100000 00000000 00000000
    1, // 1466,  01011101 10100000 00000000 00000000
    0, // 1467,  11011101 10100000 00000000 00000000
    2, // 1468,  00111101 10100000 00000000 00000000
    0, // 1469,  10111101 10100000 00000000 00000000
    1, // 1470,  01111101 10100000 00000000 00000000
    0, // 1471,  11111101 10100000 00000000 00000000
    6, // 1472,  00000011 10100000 00000000 00000000
    0, // 1473,  10000011 10100000 00000000 00000000
    1, // 1474,  01000011 10100000 00000000 00000000
    0, // 1475,  11000011 10100000 00000000 00000000
    2, // 1476,  00100011 10100000 00000000 00000000
    0, // 1477,  10100011 10100000 00000000 00000000
    1, // 1478,  01100011 10100000 00000000 00000000
    0, // 1479,  11100011 10100000 00000000 00000000
    3, // 1480,  00010011 10100000 00000000 00000000
    0, // 1481,  10010011 10100000 00000000 00000000
    1, // 1482,  01010011 10100000 00000000 00000000
    0, // 1483,  11010011 10100000 00000000 00000000
    2, // 1484,  00110011 10100000 00000000 00000000
    0, // 1485,  10110011 10100000 00000000 00000000
    1, // 1486,  01110011 10100000 00000000 00000000
    0, // 1487,  11110011 10100000 00000000 00000000
    4, // 1488,  00001011 10100000 00000000 00000000
    0, // 1489,  10001011 10100000 00000000 00000000
    1, // 1490,  01001011 10100000 00000000 00000000
    0, // 1491,  11001011 10100000 00000000 00000000
    2, // 1492,  00101011 10100000 00000000 00000000
    0, // 1493,  10101011 10100000 00000000 00000000
    1, // 1494,  01101011 10100000 00000000 00000000
    0, // 1495,  11101011 10100000 00000000 00000000
    3, // 1496,  00011011 10100000 00000000 00000000
    0, // 1497,  10011011 10100000 00000000 00000000
    1, // 1498,  01011011 10100000 00000000 00000000
    0, // 1499,  11011011 10100000 00000000 00000000
    2, // 1500,  00111011 10100000 00000000 00000000
    0, // 1501,  10111011 10100000 00000000 00000000
    1, // 1502,  01111011 10100000 00000000 00000000
    0, // 1503,  11111011 10100000 00000000 00000000
    5, // 1504,  00000111 10100000 00000000 00000000
    0, // 1505,  10000111 10100000 00000000 00000000
    1, // 1506,  01000111 10100000 00000000 00000000
    0, // 1507,  11000111 10100000 00000000 00000000
    2, // 1508,  00100111 10100000 00000000 00000000
    0, // 1509,  10100111 10100000 00000000 00000000
    1, // 1510,  01100111 10100000 00000000 00000000
    0, // 1511,  11100111 10100000 00000000 00000000
    3, // 1512,  00010111 10100000 00000000 00000000
    0, // 1513,  10010111 10100000 00000000 00000000
    1, // 1514,  01010111 10100000 00000000 00000000
    0, // 1515,  11010111 10100000 00000000 00000000
    2, // 1516,  00110111 10100000 00000000 00000000
    0, // 1517,  10110111 10100000 00000000 00000000
    1, // 1518,  01110111 10100000 00000000 00000000
    0, // 1519,  11110111 10100000 00000000 00000000
    4, // 1520,  00001111 10100000 00000000 00000000
    0, // 1521,  10001111 10100000 00000000 00000000
    1, // 1522,  01001111 10100000 00000000 00000000
    0, // 1523,  11001111 10100000 00000000 00000000
    2, // 1524,  00101111 10100000 00000000 00000000
    0, // 1525,  10101111 10100000 00000000 00000000
    1, // 1526,  01101111 10100000 00000000 00000000
    0, // 1527,  11101111 10100000 00000000 00000000
    3, // 1528,  00011111 10100000 00000000 00000000
    0, // 1529,  10011111 10100000 00000000 00000000
    1, // 1530,  01011111 10100000 00000000 00000000
    0, // 1531,  11011111 10100000 00000000 00000000
    2, // 1532,  00111111 10100000 00000000 00000000
    0, // 1533,  10111111 10100000 00000000 00000000
    1, // 1534,  01111111 10100000 00000000 00000000
    0, // 1535,  11111111 10100000 00000000 00000000
    9, // 1536,  00000000 01100000 00000000 00000000
    0, // 1537,  10000000 01100000 00000000 00000000
    1, // 1538,  01000000 01100000 00000000 00000000
    0, // 1539,  11000000 01100000 00000000 00000000
    2, // 1540,  00100000 01100000 00000000 00000000
    0, // 1541,  10100000 01100000 00000000 00000000
    1, // 1542,  01100000 01100000 00000000 00000000
    0, // 1543,  11100000 01100000 00000000 00000000
    3, // 1544,  00010000 01100000 00000000 00000000
    0, // 1545,  10010000 01100000 00000000 00000000
    1, // 1546,  01010000 01100000 00000000 00000000
    0, // 1547,  11010000 01100000 00000000 00000000
    2, // 1548,  00110000 01100000 00000000 00000000
    0, // 1549,  10110000 01100000 00000000 00000000
    1, // 1550,  01110000 01100000 00000000 00000000
    0, // 1551,  11110000 01100000 00000000 00000000
    4, // 1552,  00001000 01100000 00000000 00000000
    0, // 1553,  10001000 01100000 00000000 00000000
    1, // 1554,  01001000 01100000 00000000 00000000
    0, // 1555,  11001000 01100000 00000000 00000000
    2, // 1556,  00101000 01100000 00000000 00000000
    0, // 1557,  10101000 01100000 00000000 00000000
    1, // 1558,  01101000 01100000 00000000 00000000
    0, // 1559,  11101000 01100000 00000000 00000000
    3, // 1560,  00011000 01100000 00000000 00000000
    0, // 1561,  10011000 01100000 00000000 00000000
    1, // 1562,  01011000 01100000 00000000 00000000
    0, // 1563,  11011000 01100000 00000000 00000000
    2, // 1564,  00111000 01100000 00000000 00000000
    0, // 1565,  10111000 01100000 00000000 00000000
    1, // 1566,  01111000 01100000 00000000 00000000
    0, // 1567,  11111000 01100000 00000000 00000000
    5, // 1568,  00000100 01100000 00000000 00000000
    0, // 1569,  10000100 01100000 00000000 00000000
    1, // 1570,  01000100 01100000 00000000 00000000
    0, // 1571,  11000100 01100000 00000000 00000000
    2, // 1572,  00100100 01100000 00000000 00000000
    0, // 1573,  10100100 01100000 00000000 00000000
    1, // 1574,  01100100 01100000 00000000 00000000
    0, // 1575,  11100100 01100000 00000000 00000000
    3, // 1576,  00010100 01100000 00000000 00000000
    0, // 1577,  10010100 01100000 00000000 00000000
    1, // 1578,  01010100 01100000 00000000 00000000
    0, // 1579,  11010100 01100000 00000000 00000000
    2, // 1580,  00110100 01100000 00000000 00000000
    0, // 1581,  10110100 01100000 00000000 00000000
    1, // 1582,  01110100 01100000 00000000 00000000
    0, // 1583,  11110100 01100000 00000000 00000000
    4, // 1584,  00001100 01100000 00000000 00000000
    0, // 1585,  10001100 01100000 00000000 00000000
    1, // 1586,  01001100 01100000 00000000 00000000
    0, // 1587,  11001100 01100000 00000000 00000000
    2, // 1588,  00101100 01100000 00000000 00000000
    0, // 1589,  10101100 01100000 00000000 00000000
    1, // 1590,  01101100 01100000 00000000 00000000
    0, // 1591,  11101100 01100000 00000000 00000000
    3, // 1592,  00011100 01100000 00000000 00000000
    0, // 1593,  10011100 01100000 00000000 00000000
    1, // 1594,  01011100 01100000 00000000 00000000
    0, // 1595,  11011100 01100000 00000000 00000000
    2, // 1596,  00111100 01100000 00000000 00000000
    0, // 1597,  10111100 01100000 00000000 00000000
    1, // 1598,  01111100 01100000 00000000 00000000
    0, // 1599,  11111100 01100000 00000000 00000000
    6, // 1600,  00000010 01100000 00000000 00000000
    0, // 1601,  10000010 01100000 00000000 00000000
    1, // 1602,  01000010 01100000 00000000 00000000
    0, // 1603,  11000010 01100000 00000000 00000000
    2, // 1604,  00100010 01100000 00000000 00000000
    0, // 1605,  10100010 01100000 00000000 00000000
    1, // 1606,  01100010 01100000 00000000 00000000
    0, // 1607,  11100010 01100000 00000000 00000000
    3, // 1608,  00010010 01100000 00000000 00000000
    0, // 1609,  10010010 01100000 00000000 00000000
    1, // 1610,  01010010 01100000 00000000 00000000
    0, // 1611,  11010010 01100000 00000000 00000000
    2, // 1612,  00110010 01100000 00000000 00000000
    0, // 1613,  10110010 01100000 00000000 00000000
    1, // 1614,  01110010 01100000 00000000 00000000
    0, // 1615,  11110010 01100000 00000000 00000000
    4, // 1616,  00001010 01100000 00000000 00000000
    0, // 1617,  10001010 01100000 00000000 00000000
    1, // 1618,  01001010 01100000 00000000 00000000
    0, // 1619,  11001010 01100000 00000000 00000000
    2, // 1620,  00101010 01100000 00000000 00000000
    0, // 1621,  10101010 01100000 00000000 00000000
    1, // 1622,  01101010 01100000 00000000 00000000
    0, // 1623,  11101010 01100000 00000000 00000000
    3, // 1624,  00011010 01100000 00000000 00000000
    0, // 1625,  10011010 01100000 00000000 00000000
    1, // 1626,  01011010 01100000 00000000 00000000
    0, // 1627,  11011010 01100000 00000000 00000000
    2, // 1628,  00111010 01100000 00000000 00000000
    0, // 1629,  10111010 01100000 00000000 00000000
    1, // 1630,  01111010 01100000 00000000 00000000
    0, // 1631,  11111010 01100000 00000000 00000000
    5, // 1632,  00000110 01100000 00000000 00000000
    0, // 1633,  10000110 01100000 00000000 00000000
    1, // 1634,  01000110 01100000 00000000 00000000
    0, // 1635,  11000110 01100000 00000000 00000000
    2, // 1636,  00100110 01100000 00000000 00000000
    0, // 1637,  10100110 01100000 00000000 00000000
    1, // 1638,  01100110 01100000 00000000 00000000
    0, // 1639,  11100110 01100000 00000000 00000000
    3, // 1640,  00010110 01100000 00000000 00000000
    0, // 1641,  10010110 01100000 00000000 00000000
    1, // 1642,  01010110 01100000 00000000 00000000
    0, // 1643,  11010110 01100000 00000000 00000000
    2, // 1644,  00110110 01100000 00000000 00000000
    0, // 1645,  10110110 01100000 00000000 00000000
    1, // 1646,  01110110 01100000 00000000 00000000
    0, // 1647,  11110110 01100000 00000000 00000000
    4, // 1648,  00001110 01100000 00000000 00000000
    0, // 1649,  10001110 01100000 00000000 00000000
    1, // 1650,  01001110 01100000 00000000 00000000
    0, // 1651,  11001110 01100000 00000000 00000000
    2, // 1652,  00101110 01100000 00000000 00000000
    0, // 1653,  10101110 01100000 00000000 00000000
    1, // 1654,  01101110 01100000 00000000 00000000
    0, // 1655,  11101110 01100000 00000000 00000000
    3, // 1656,  00011110 01100000 00000000 00000000
    0, // 1657,  10011110 01100000 00000000 00000000
    1, // 1658,  01011110 01100000 00000000 00000000
    0, // 1659,  11011110 01100000 00000000 00000000
    2, // 1660,  00111110 01100000 00000000 00000000
    0, // 1661,  10111110 01100000 00000000 00000000
    1, // 1662,  01111110 01100000 00000000 00000000
    0, // 1663,  11111110 01100000 00000000 00000000
    7, // 1664,  00000001 01100000 00000000 00000000
    0, // 1665,  10000001 01100000 00000000 00000000
    1, // 1666,  01000001 01100000 00000000 00000000
    0, // 1667,  11000001 01100000 00000000 00000000
    2, // 1668,  00100001 01100000 00000000 00000000
    0, // 1669,  10100001 01100000 00000000 00000000
    1, // 1670,  01100001 01100000 00000000 00000000
    0, // 1671,  11100001 01100000 00000000 00000000
    3, // 1672,  00010001 01100000 00000000 00000000
    0, // 1673,  10010001 01100000 00000000 00000000
    1, // 1674,  01010001 01100000 00000000 00000000
    0, // 1675,  11010001 01100000 00000000 00000000
    2, // 1676,  00110001 01100000 00000000 00000000
    0, // 1677,  10110001 01100000 00000000 00000000
    1, // 1678,  01110001 01100000 00000000 00000000
    0, // 1679,  11110001 01100000 00000000 00000000
    4, // 1680,  00001001 01100000 00000000 00000000
    0, // 1681,  10001001 01100000 00000000 00000000
    1, // 1682,  01001001 01100000 00000000 00000000
    0, // 1683,  11001001 01100000 00000000 00000000
    2, // 1684,  00101001 01100000 00000000 00000000
    0, // 1685,  10101001 01100000 00000000 00000000
    1, // 1686,  01101001 01100000 00000000 00000000
    0, // 1687,  11101001 01100000 00000000 00000000
    3, // 1688,  00011001 01100000 00000000 00000000
    0, // 1689,  10011001 01100000 00000000 00000000
    1, // 1690,  01011001 01100000 00000000 00000000
    0, // 1691,  11011001 01100000 00000000 00000000
    2, // 1692,  00111001 01100000 00000000 00000000
    0, // 1693,  10111001 01100000 00000000 00000000
    1, // 1694,  01111001 01100000 00000000 00000000
    0, // 1695,  11111001 01100000 00000000 00000000
    5, // 1696,  00000101 01100000 00000000 00000000
    0, // 1697,  10000101 01100000 00000000 00000000
    1, // 1698,  01000101 01100000 00000000 00000000
    0, // 1699,  11000101 01100000 00000000 00000000
    2, // 1700,  00100101 01100000 00000000 00000000
    0, // 1701,  10100101 01100000 00000000 00000000
    1, // 1702,  01100101 01100000 00000000 00000000
    0, // 1703,  11100101 01100000 00000000 00000000
    3, // 1704,  00010101 01100000 00000000 00000000
    0, // 1705,  10010101 01100000 00000000 00000000
    1, // 1706,  01010101 01100000 00000000 00000000
    0, // 1707,  11010101 01100000 00000000 00000000
    2, // 1708,  00110101 01100000 00000000 00000000
    0, // 1709,  10110101 01100000 00000000 00000000
    1, // 1710,  01110101 01100000 00000000 00000000
    0, // 1711,  11110101 01100000 00000000 00000000
    4, // 1712,  00001101 01100000 00000000 00000000
    0, // 1713,  10001101 01100000 00000000 00000000
    1, // 1714,  01001101 01100000 00000000 00000000
    0, // 1715,  11001101 01100000 00000000 00000000
    2, // 1716,  00101101 01100000 00000000 00000000
    0, // 1717,  10101101 01100000 00000000 00000000
    1, // 1718,  01101101 01100000 00000000 00000000
    0, // 1719,  11101101 01100000 00000000 00000000
    3, // 1720,  00011101 01100000 00000000 00000000
    0, // 1721,  10011101 01100000 00000000 00000000
    1, // 1722,  01011101 01100000 00000000 00000000
    0, // 1723,  11011101 01100000 00000000 00000000
    2, // 1724,  00111101 01100000 00000000 00000000
    0, // 1725,  10111101 01100000 00000000 00000000
    1, // 1726,  01111101 01100000 00000000 00000000
    0, // 1727,  11111101 01100000 00000000 00000000
    6, // 1728,  00000011 01100000 00000000 00000000
    0, // 1729,  10000011 01100000 00000000 00000000
    1, // 1730,  01000011 01100000 00000000 00000000
    0, // 1731,  11000011 01100000 00000000 00000000
    2, // 1732,  00100011 01100000 00000000 00000000
    0, // 1733,  10100011 01100000 00000000 00000000
    1, // 1734,  01100011 01100000 00000000 00000000
    0, // 1735,  11100011 01100000 00000000 00000000
    3, // 1736,  00010011 01100000 00000000 00000000
    0, // 1737,  10010011 01100000 00000000 00000000
    1, // 1738,  01010011 01100000 00000000 00000000
    0, // 1739,  11010011 01100000 00000000 00000000
    2, // 1740,  00110011 01100000 00000000 00000000
    0, // 1741,  10110011 01100000 00000000 00000000
    1, // 1742,  01110011 01100000 00000000 00000000
    0, // 1743,  11110011 01100000 00000000 00000000
    4, // 1744,  00001011 01100000 00000000 00000000
    0, // 1745,  10001011 01100000 00000000 00000000
    1, // 1746,  01001011 01100000 00000000 00000000
    0, // 1747,  11001011 01100000 00000000 00000000
    2, // 1748,  00101011 01100000 00000000 00000000
    0, // 1749,  10101011 01100000 00000000 00000000
    1, // 1750,  01101011 01100000 00000000 00000000
    0, // 1751,  11101011 01100000 00000000 00000000
    3, // 1752,  00011011 01100000 00000000 00000000
    0, // 1753,  10011011 01100000 00000000 00000000
    1, // 1754,  01011011 01100000 00000000 00000000
    0, // 1755,  11011011 01100000 00000000 00000000
    2, // 1756,  00111011 01100000 00000000 00000000
    0, // 1757,  10111011 01100000 00000000 00000000
    1, // 1758,  01111011 01100000 00000000 00000000
    0, // 1759,  11111011 01100000 00000000 00000000
    5, // 1760,  00000111 01100000 00000000 00000000
    0, // 1761,  10000111 01100000 00000000 00000000
    1, // 1762,  01000111 01100000 00000000 00000000
    0, // 1763,  11000111 01100000 00000000 00000000
    2, // 1764,  00100111 01100000 00000000 00000000
    0, // 1765,  10100111 01100000 00000000 00000000
    1, // 1766,  01100111 01100000 00000000 00000000
    0, // 1767,  11100111 01100000 00000000 00000000
    3, // 1768,  00010111 01100000 00000000 00000000
    0, // 1769,  10010111 01100000 00000000 00000000
    1, // 1770,  01010111 01100000 00000000 00000000
    0, // 1771,  11010111 01100000 00000000 00000000
    2, // 1772,  00110111 01100000 00000000 00000000
    0, // 1773,  10110111 01100000 00000000 00000000
    1, // 1774,  01110111 01100000 00000000 00000000
    0, // 1775,  11110111 01100000 00000000 00000000
    4, // 1776,  00001111 01100000 00000000 00000000
    0, // 1777,  10001111 01100000 00000000 00000000
    1, // 1778,  01001111 01100000 00000000 00000000
    0, // 1779,  11001111 01100000 00000000 00000000
    2, // 1780,  00101111 01100000 00000000 00000000
    0, // 1781,  10101111 01100000 00000000 00000000
    1, // 1782,  01101111 01100000 00000000 00000000
    0, // 1783,  11101111 01100000 00000000 00000000
    3, // 1784,  00011111 01100000 00000000 00000000
    0, // 1785,  10011111 01100000 00000000 00000000
    1, // 1786,  01011111 01100000 00000000 00000000
    0, // 1787,  11011111 01100000 00000000 00000000
    2, // 1788,  00111111 01100000 00000000 00000000
    0, // 1789,  10111111 01100000 00000000 00000000
    1, // 1790,  01111111 01100000 00000000 00000000
    0, // 1791,  11111111 01100000 00000000 00000000
    8, // 1792,  00000000 11100000 00000000 00000000
    0, // 1793,  10000000 11100000 00000000 00000000
    1, // 1794,  01000000 11100000 00000000 00000000
    0, // 1795,  11000000 11100000 00000000 00000000
    2, // 1796,  00100000 11100000 00000000 00000000
    0, // 1797,  10100000 11100000 00000000 00000000
    1, // 1798,  01100000 11100000 00000000 00000000
    0, // 1799,  11100000 11100000 00000000 00000000
    3, // 1800,  00010000 11100000 00000000 00000000
    0, // 1801,  10010000 11100000 00000000 00000000
    1, // 1802,  01010000 11100000 00000000 00000000
    0, // 1803,  11010000 11100000 00000000 00000000
    2, // 1804,  00110000 11100000 00000000 00000000
    0, // 1805,  10110000 11100000 00000000 00000000
    1, // 1806,  01110000 11100000 00000000 00000000
    0, // 1807,  11110000 11100000 00000000 00000000
    4, // 1808,  00001000 11100000 00000000 00000000
    0, // 1809,  10001000 11100000 00000000 00000000
    1, // 1810,  01001000 11100000 00000000 00000000
    0, // 1811,  11001000 11100000 00000000 00000000
    2, // 1812,  00101000 11100000 00000000 00000000
    0, // 1813,  10101000 11100000 00000000 00000000
    1, // 1814,  01101000 11100000 00000000 00000000
    0, // 1815,  11101000 11100000 00000000 00000000
    3, // 1816,  00011000 11100000 00000000 00000000
    0, // 1817,  10011000 11100000 00000000 00000000
    1, // 1818,  01011000 11100000 00000000 00000000
    0, // 1819,  11011000 11100000 00000000 00000000
    2, // 1820,  00111000 11100000 00000000 00000000
    0, // 1821,  10111000 11100000 00000000 00000000
    1, // 1822,  01111000 11100000 00000000 00000000
    0, // 1823,  11111000 11100000 00000000 00000000
    5, // 1824,  00000100 11100000 00000000 00000000
    0, // 1825,  10000100 11100000 00000000 00000000
    1, // 1826,  01000100 11100000 00000000 00000000
    0, // 1827,  11000100 11100000 00000000 00000000
    2, // 1828,  00100100 11100000 00000000 00000000
    0, // 1829,  10100100 11100000 00000000 00000000
    1, // 1830,  01100100 11100000 00000000 00000000
    0, // 1831,  11100100 11100000 00000000 00000000
    3, // 1832,  00010100 11100000 00000000 00000000
    0, // 1833,  10010100 11100000 00000000 00000000
    1, // 1834,  01010100 11100000 00000000 00000000
    0, // 1835,  11010100 11100000 00000000 00000000
    2, // 1836,  00110100 11100000 00000000 00000000
    0, // 1837,  10110100 11100000 00000000 00000000
    1, // 1838,  01110100 11100000 00000000 00000000
    0, // 1839,  11110100 11100000 00000000 00000000
    4, // 1840,  00001100 11100000 00000000 00000000
    0, // 1841,  10001100 11100000 00000000 00000000
    1, // 1842,  01001100 11100000 00000000 00000000
    0, // 1843,  11001100 11100000 00000000 00000000
    2, // 1844,  00101100 11100000 00000000 00000000
    0, // 1845,  10101100 11100000 00000000 00000000
    1, // 1846,  01101100 11100000 00000000 00000000
    0, // 1847,  11101100 11100000 00000000 00000000
    3, // 1848,  00011100 11100000 00000000 00000000
    0, // 1849,  10011100 11100000 00000000 00000000
    1, // 1850,  01011100 11100000 00000000 00000000
    0, // 1851,  11011100 11100000 00000000 00000000
    2, // 1852,  00111100 11100000 00000000 00000000
    0, // 1853,  10111100 11100000 00000000 00000000
    1, // 1854,  01111100 11100000 00000000 00000000
    0, // 1855,  11111100 11100000 00000000 00000000
    6, // 1856,  00000010 11100000 00000000 00000000
    0, // 1857,  10000010 11100000 00000000 00000000
    1, // 1858,  01000010 11100000 00000000 00000000
    0, // 1859,  11000010 11100000 00000000 00000000
    2, // 1860,  00100010 11100000 00000000 00000000
    0, // 1861,  10100010 11100000 00000000 00000000
    1, // 1862,  01100010 11100000 00000000 00000000
    0, // 1863,  11100010 11100000 00000000 00000000
    3, // 1864,  00010010 11100000 00000000 00000000
    0, // 1865,  10010010 11100000 00000000 00000000
    1, // 1866,  01010010 11100000 00000000 00000000
    0, // 1867,  11010010 11100000 00000000 00000000
    2, // 1868,  00110010 11100000 00000000 00000000
    0, // 1869,  10110010 11100000 00000000 00000000
    1, // 1870,  01110010 11100000 00000000 00000000
    0, // 1871,  11110010 11100000 00000000 00000000
    4, // 1872,  00001010 11100000 00000000 00000000
    0, // 1873,  10001010 11100000 00000000 00000000
    1, // 1874,  01001010 11100000 00000000 00000000
    0, // 1875,  11001010 11100000 00000000 00000000
    2, // 1876,  00101010 11100000 00000000 00000000
    0, // 1877,  10101010 11100000 00000000 00000000
    1, // 1878,  01101010 11100000 00000000 00000000
    0, // 1879,  11101010 11100000 00000000 00000000
    3, // 1880,  00011010 11100000 00000000 00000000
    0, // 1881,  10011010 11100000 00000000 00000000
    1, // 1882,  01011010 11100000 00000000 00000000
    0, // 1883,  11011010 11100000 00000000 00000000
    2, // 1884,  00111010 11100000 00000000 00000000
    0, // 1885,  10111010 11100000 00000000 00000000
    1, // 1886,  01111010 11100000 00000000 00000000
    0, // 1887,  11111010 11100000 00000000 00000000
    5, // 1888,  00000110 11100000 00000000 00000000
    0, // 1889,  10000110 11100000 00000000 00000000
    1, // 1890,  01000110 11100000 00000000 00000000
    0, // 1891,  11000110 11100000 00000000 00000000
    2, // 1892,  00100110 11100000 00000000 00000000
    0, // 1893,  10100110 11100000 00000000 00000000
    1, // 1894,  01100110 11100000 00000000 00000000
    0, // 1895,  11100110 11100000 00000000 00000000
    3, // 1896,  00010110 11100000 00000000 00000000
    0, // 1897,  10010110 11100000 00000000 00000000
    1, // 1898,  01010110 11100000 00000000 00000000
    0, // 1899,  11010110 11100000 00000000 00000000
    2, // 1900,  00110110 11100000 00000000 00000000
    0, // 1901,  10110110 11100000 00000000 00000000
    1, // 1902,  01110110 11100000 00000000 00000000
    0, // 1903,  11110110 11100000 00000000 00000000
    4, // 1904,  00001110 11100000 00000000 00000000
    0, // 1905,  10001110 11100000 00000000 00000000
    1, // 1906,  01001110 11100000 00000000 00000000
    0, // 1907,  11001110 11100000 00000000 00000000
    2, // 1908,  00101110 11100000 00000000 00000000
    0, // 1909,  10101110 11100000 00000000 00000000
    1, // 1910,  01101110 11100000 00000000 00000000
    0, // 1911,  11101110 11100000 00000000 00000000
    3, // 1912,  00011110 11100000 00000000 00000000
    0, // 1913,  10011110 11100000 00000000 00000000
    1, // 1914,  01011110 11100000 00000000 00000000
    0, // 1915,  11011110 11100000 00000000 00000000
    2, // 1916,  00111110 11100000 00000000 00000000
    0, // 1917,  10111110 11100000 00000000 00000000
    1, // 1918,  01111110 11100000 00000000 00000000
    0, // 1919,  11111110 11100000 00000000 00000000
    7, // 1920,  00000001 11100000 00000000 00000000
    0, // 1921,  10000001 11100000 00000000 00000000
    1, // 1922,  01000001 11100000 00000000 00000000
    0, // 1923,  11000001 11100000 00000000 00000000
    2, // 1924,  00100001 11100000 00000000 00000000
    0, // 1925,  10100001 11100000 00000000 00000000
    1, // 1926,  01100001 11100000 00000000 00000000
    0, // 1927,  11100001 11100000 00000000 00000000
    3, // 1928,  00010001 11100000 00000000 00000000
    0, // 1929,  10010001 11100000 00000000 00000000
    1, // 1930,  01010001 11100000 00000000 00000000
    0, // 1931,  11010001 11100000 00000000 00000000
    2, // 1932,  00110001 11100000 00000000 00000000
    0, // 1933,  10110001 11100000 00000000 00000000
    1, // 1934,  01110001 11100000 00000000 00000000
    0, // 1935,  11110001 11100000 00000000 00000000
    4, // 1936,  00001001 11100000 00000000 00000000
    0, // 1937,  10001001 11100000 00000000 00000000
    1, // 1938,  01001001 11100000 00000000 00000000
    0, // 1939,  11001001 11100000 00000000 00000000
    2, // 1940,  00101001 11100000 00000000 00000000
    0, // 1941,  10101001 11100000 00000000 00000000
    1, // 1942,  01101001 11100000 00000000 00000000
    0, // 1943,  11101001 11100000 00000000 00000000
    3, // 1944,  00011001 11100000 00000000 00000000
    0, // 1945,  10011001 11100000 00000000 00000000
    1, // 1946,  01011001 11100000 00000000 00000000
    0, // 1947,  11011001 11100000 00000000 00000000
    2, // 1948,  00111001 11100000 00000000 00000000
    0, // 1949,  10111001 11100000 00000000 00000000
    1, // 1950,  01111001 11100000 00000000 00000000
    0, // 1951,  11111001 11100000 00000000 00000000
    5, // 1952,  00000101 11100000 00000000 00000000
    0, // 1953,  10000101 11100000 00000000 00000000
    1, // 1954,  01000101 11100000 00000000 00000000
    0, // 1955,  11000101 11100000 00000000 00000000
    2, // 1956,  00100101 11100000 00000000 00000000
    0, // 1957,  10100101 11100000 00000000 00000000
    1, // 1958,  01100101 11100000 00000000 00000000
    0, // 1959,  11100101 11100000 00000000 00000000
    3, // 1960,  00010101 11100000 00000000 00000000
    0, // 1961,  10010101 11100000 00000000 00000000
    1, // 1962,  01010101 11100000 00000000 00000000
    0, // 1963,  11010101 11100000 00000000 00000000
    2, // 1964,  00110101 11100000 00000000 00000000
    0, // 1965,  10110101 11100000 00000000 00000000
    1, // 1966,  01110101 11100000 00000000 00000000
    0, // 1967,  11110101 11100000 00000000 00000000
    4, // 1968,  00001101 11100000 00000000 00000000
    0, // 1969,  10001101 11100000 00000000 00000000
    1, // 1970,  01001101 11100000 00000000 00000000
    0, // 1971,  11001101 11100000 00000000 00000000
    2, // 1972,  00101101 11100000 00000000 00000000
    0, // 1973,  10101101 11100000 00000000 00000000
    1, // 1974,  01101101 11100000 00000000 00000000
    0, // 1975,  11101101 11100000 00000000 00000000
    3, // 1976,  00011101 11100000 00000000 00000000
    0, // 1977,  10011101 11100000 00000000 00000000
    1, // 1978,  01011101 11100000 00000000 00000000
    0, // 1979,  11011101 11100000 00000000 00000000
    2, // 1980,  00111101 11100000 00000000 00000000
    0, // 1981,  10111101 11100000 00000000 00000000
    1, // 1982,  01111101 11100000 00000000 00000000
    0, // 1983,  11111101 11100000 00000000 00000000
    6, // 1984,  00000011 11100000 00000000 00000000
    0, // 1985,  10000011 11100000 00000000 00000000
    1, // 1986,  01000011 11100000 00000000 00000000
    0, // 1987,  11000011 11100000 00000000 00000000
    2, // 1988,  00100011 11100000 00000000 00000000
    0, // 1989,  10100011 11100000 00000000 00000000
    1, // 1990,  01100011 11100000 00000000 00000000
    0, // 1991,  11100011 11100000 00000000 00000000
    3, // 1992,  00010011 11100000 00000000 00000000
    0, // 1993,  10010011 11100000 00000000 00000000
    1, // 1994,  01010011 11100000 00000000 00000000
    0, // 1995,  11010011 11100000 00000000 00000000
    2, // 1996,  00110011 11100000 00000000 00000000
    0, // 1997,  10110011 11100000 00000000 00000000
    1, // 1998,  01110011 11100000 00000000 00000000
    0, // 1999,  11110011 11100000 00000000 00000000
    4, // 2000,  00001011 11100000 00000000 00000000
    0, // 2001,  10001011 11100000 00000000 00000000
    1, // 2002,  01001011 11100000 00000000 00000000
    0, // 2003,  11001011 11100000 00000000 00000000
    2, // 2004,  00101011 11100000 00000000 00000000
    0, // 2005,  10101011 11100000 00000000 00000000
    1, // 2006,  01101011 11100000 00000000 00000000
    0, // 2007,  11101011 11100000 00000000 00000000
    3, // 2008,  00011011 11100000 00000000 00000000
    0, // 2009,  10011011 11100000 00000000 00000000
    1, // 2010,  01011011 11100000 00000000 00000000
    0, // 2011,  11011011 11100000 00000000 00000000
    2, // 2012,  00111011 11100000 00000000 00000000
    0, // 2013,  10111011 11100000 00000000 00000000
    1, // 2014,  01111011 11100000 00000000 00000000
    0, // 2015,  11111011 11100000 00000000 00000000
    5, // 2016,  00000111 11100000 00000000 00000000
    0, // 2017,  10000111 11100000 00000000 00000000
    1, // 2018,  01000111 11100000 00000000 00000000
    0, // 2019,  11000111 11100000 00000000 00000000
    2, // 2020,  00100111 11100000 00000000 00000000
    0, // 2021,  10100111 11100000 00000000 00000000
    1, // 2022,  01100111 11100000 00000000 00000000
    0, // 2023,  11100111 11100000 00000000 00000000
    3, // 2024,  00010111 11100000 00000000 00000000
    0, // 2025,  10010111 11100000 00000000 00000000
    1, // 2026,  01010111 11100000 00000000 00000000
    0, // 2027,  11010111 11100000 00000000 00000000
    2, // 2028,  00110111 11100000 00000000 00000000
    0, // 2029,  10110111 11100000 00000000 00000000
    1, // 2030,  01110111 11100000 00000000 00000000
    0, // 2031,  11110111 11100000 00000000 00000000
    4, // 2032,  00001111 11100000 00000000 00000000
    0, // 2033,  10001111 11100000 00000000 00000000
    1, // 2034,  01001111 11100000 00000000 00000000
    0, // 2035,  11001111 11100000 00000000 00000000
    2, // 2036,  00101111 11100000 00000000 00000000
    0, // 2037,  10101111 11100000 00000000 00000000
    1, // 2038,  01101111 11100000 00000000 00000000
    0, // 2039,  11101111 11100000 00000000 00000000
    3, // 2040,  00011111 11100000 00000000 00000000
    0, // 2041,  10011111 11100000 00000000 00000000
    1, // 2042,  01011111 11100000 00000000 00000000
    0, // 2043,  11011111 11100000 00000000 00000000
    2, // 2044,  00111111 11100000 00000000 00000000
    0, // 2045,  10111111 11100000 00000000 00000000
    1, // 2046,  01111111 11100000 00000000 00000000
    0, // 2047,  11111111 11100000 00000000 00000000
    11, // 2048,  00000000 00010000 00000000 00000000
    0, // 2049,  10000000 00010000 00000000 00000000
    1, // 2050,  01000000 00010000 00000000 00000000
    0, // 2051,  11000000 00010000 00000000 00000000
    2, // 2052,  00100000 00010000 00000000 00000000
    0, // 2053,  10100000 00010000 00000000 00000000
    1, // 2054,  01100000 00010000 00000000 00000000
    0, // 2055,  11100000 00010000 00000000 00000000
    3, // 2056,  00010000 00010000 00000000 00000000
    0, // 2057,  10010000 00010000 00000000 00000000
    1, // 2058,  01010000 00010000 00000000 00000000
    0, // 2059,  11010000 00010000 00000000 00000000
    2, // 2060,  00110000 00010000 00000000 00000000
    0, // 2061,  10110000 00010000 00000000 00000000
    1, // 2062,  01110000 00010000 00000000 00000000
    0, // 2063,  11110000 00010000 00000000 00000000
    4, // 2064,  00001000 00010000 00000000 00000000
    0, // 2065,  10001000 00010000 00000000 00000000
    1, // 2066,  01001000 00010000 00000000 00000000
    0, // 2067,  11001000 00010000 00000000 00000000
    2, // 2068,  00101000 00010000 00000000 00000000
    0, // 2069,  10101000 00010000 00000000 00000000
    1, // 2070,  01101000 00010000 00000000 00000000
    0, // 2071,  11101000 00010000 00000000 00000000
    3, // 2072,  00011000 00010000 00000000 00000000
    0, // 2073,  10011000 00010000 00000000 00000000
    1, // 2074,  01011000 00010000 00000000 00000000
    0, // 2075,  11011000 00010000 00000000 00000000
    2, // 2076,  00111000 00010000 00000000 00000000
    0, // 2077,  10111000 00010000 00000000 00000000
    1, // 2078,  01111000 00010000 00000000 00000000
    0, // 2079,  11111000 00010000 00000000 00000000
    5, // 2080,  00000100 00010000 00000000 00000000
    0, // 2081,  10000100 00010000 00000000 00000000
    1, // 2082,  01000100 00010000 00000000 00000000
    0, // 2083,  11000100 00010000 00000000 00000000
    2, // 2084,  00100100 00010000 00000000 00000000
    0, // 2085,  10100100 00010000 00000000 00000000
    1, // 2086,  01100100 00010000 00000000 00000000
    0, // 2087,  11100100 00010000 00000000 00000000
    3, // 2088,  00010100 00010000 00000000 00000000
    0, // 2089,  10010100 00010000 00000000 00000000
    1, // 2090,  01010100 00010000 00000000 00000000
    0, // 2091,  11010100 00010000 00000000 00000000
    2, // 2092,  00110100 00010000 00000000 00000000
    0, // 2093,  10110100 00010000 00000000 00000000
    1, // 2094,  01110100 00010000 00000000 00000000
    0, // 2095,  11110100 00010000 00000000 00000000
    4, // 2096,  00001100 00010000 00000000 00000000
    0, // 2097,  10001100 00010000 00000000 00000000
    1, // 2098,  01001100 00010000 00000000 00000000
    0, // 2099,  11001100 00010000 00000000 00000000
    2, // 2100,  00101100 00010000 00000000 00000000
    0, // 2101,  10101100 00010000 00000000 00000000
    1, // 2102,  01101100 00010000 00000000 00000000
    0, // 2103,  11101100 00010000 00000000 00000000
    3, // 2104,  00011100 00010000 00000000 00000000
    0, // 2105,  10011100 00010000 00000000 00000000
    1, // 2106,  01011100 00010000 00000000 00000000
    0, // 2107,  11011100 00010000 00000000 00000000
    2, // 2108,  00111100 00010000 00000000 00000000
    0, // 2109,  10111100 00010000 00000000 00000000
    1, // 2110,  01111100 00010000 00000000 00000000
    0, // 2111,  11111100 00010000 00000000 00000000
    6, // 2112,  00000010 00010000 00000000 00000000
    0, // 2113,  10000010 00010000 00000000 00000000
    1, // 2114,  01000010 00010000 00000000 00000000
    0, // 2115,  11000010 00010000 00000000 00000000
    2, // 2116,  00100010 00010000 00000000 00000000
    0, // 2117,  10100010 00010000 00000000 00000000
    1, // 2118,  01100010 00010000 00000000 00000000
    0, // 2119,  11100010 00010000 00000000 00000000
    3, // 2120,  00010010 00010000 00000000 00000000
    0, // 2121,  10010010 00010000 00000000 00000000
    1, // 2122,  01010010 00010000 00000000 00000000
    0, // 2123,  11010010 00010000 00000000 00000000
    2, // 2124,  00110010 00010000 00000000 00000000
    0, // 2125,  10110010 00010000 00000000 00000000
    1, // 2126,  01110010 00010000 00000000 00000000
    0, // 2127,  11110010 00010000 00000000 00000000
    4, // 2128,  00001010 00010000 00000000 00000000
    0, // 2129,  10001010 00010000 00000000 00000000
    1, // 2130,  01001010 00010000 00000000 00000000
    0, // 2131,  11001010 00010000 00000000 00000000
    2, // 2132,  00101010 00010000 00000000 00000000
    0, // 2133,  10101010 00010000 00000000 00000000
    1, // 2134,  01101010 00010000 00000000 00000000
    0, // 2135,  11101010 00010000 00000000 00000000
    3, // 2136,  00011010 00010000 00000000 00000000
    0, // 2137,  10011010 00010000 00000000 00000000
    1, // 2138,  01011010 00010000 00000000 00000000
    0, // 2139,  11011010 00010000 00000000 00000000
    2, // 2140,  00111010 00010000 00000000 00000000
    0, // 2141,  10111010 00010000 00000000 00000000
    1, // 2142,  01111010 00010000 00000000 00000000
    0, // 2143,  11111010 00010000 00000000 00000000
    5, // 2144,  00000110 00010000 00000000 00000000
    0, // 2145,  10000110 00010000 00000000 00000000
    1, // 2146,  01000110 00010000 00000000 00000000
    0, // 2147,  11000110 00010000 00000000 00000000
    2, // 2148,  00100110 00010000 00000000 00000000
    0, // 2149,  10100110 00010000 00000000 00000000
    1, // 2150,  01100110 00010000 00000000 00000000
    0, // 2151,  11100110 00010000 00000000 00000000
    3, // 2152,  00010110 00010000 00000000 00000000
    0, // 2153,  10010110 00010000 00000000 00000000
    1, // 2154,  01010110 00010000 00000000 00000000
    0, // 2155,  11010110 00010000 00000000 00000000
    2, // 2156,  00110110 00010000 00000000 00000000
    0, // 2157,  10110110 00010000 00000000 00000000
    1, // 2158,  01110110 00010000 00000000 00000000
    0, // 2159,  11110110 00010000 00000000 00000000
    4, // 2160,  00001110 00010000 00000000 00000000
    0, // 2161,  10001110 00010000 00000000 00000000
    1, // 2162,  01001110 00010000 00000000 00000000
    0, // 2163,  11001110 00010000 00000000 00000000
    2, // 2164,  00101110 00010000 00000000 00000000
    0, // 2165,  10101110 00010000 00000000 00000000
    1, // 2166,  01101110 00010000 00000000 00000000
    0, // 2167,  11101110 00010000 00000000 00000000
    3, // 2168,  00011110 00010000 00000000 00000000
    0, // 2169,  10011110 00010000 00000000 00000000
    1, // 2170,  01011110 00010000 00000000 00000000
    0, // 2171,  11011110 00010000 00000000 00000000
    2, // 2172,  00111110 00010000 00000000 00000000
    0, // 2173,  10111110 00010000 00000000 00000000
    1, // 2174,  01111110 00010000 00000000 00000000
    0, // 2175,  11111110 00010000 00000000 00000000
    7, // 2176,  00000001 00010000 00000000 00000000
    0, // 2177,  10000001 00010000 00000000 00000000
    1, // 2178,  01000001 00010000 00000000 00000000
    0, // 2179,  11000001 00010000 00000000 00000000
    2, // 2180,  00100001 00010000 00000000 00000000
    0, // 2181,  10100001 00010000 00000000 00000000
    1, // 2182,  01100001 00010000 00000000 00000000
    0, // 2183,  11100001 00010000 00000000 00000000
    3, // 2184,  00010001 00010000 00000000 00000000
    0, // 2185,  10010001 00010000 00000000 00000000
    1, // 2186,  01010001 00010000 00000000 00000000
    0, // 2187,  11010001 00010000 00000000 00000000
    2, // 2188,  00110001 00010000 00000000 00000000
    0, // 2189,  10110001 00010000 00000000 00000000
    1, // 2190,  01110001 00010000 00000000 00000000
    0, // 2191,  11110001 00010000 00000000 00000000
    4, // 2192,  00001001 00010000 00000000 00000000
    0, // 2193,  10001001 00010000 00000000 00000000
    1, // 2194,  01001001 00010000 00000000 00000000
    0, // 2195,  11001001 00010000 00000000 00000000
    2, // 2196,  00101001 00010000 00000000 00000000
    0, // 2197,  10101001 00010000 00000000 00000000
    1, // 2198,  01101001 00010000 00000000 00000000
    0, // 2199,  11101001 00010000 00000000 00000000
    3, // 2200,  00011001 00010000 00000000 00000000
    0, // 2201,  10011001 00010000 00000000 00000000
    1, // 2202,  01011001 00010000 00000000 00000000
    0, // 2203,  11011001 00010000 00000000 00000000
    2, // 2204,  00111001 00010000 00000000 00000000
    0, // 2205,  10111001 00010000 00000000 00000000
    1, // 2206,  01111001 00010000 00000000 00000000
    0, // 2207,  11111001 00010000 00000000 00000000
    5, // 2208,  00000101 00010000 00000000 00000000
    0, // 2209,  10000101 00010000 00000000 00000000
    1, // 2210,  01000101 00010000 00000000 00000000
    0, // 2211,  11000101 00010000 00000000 00000000
    2, // 2212,  00100101 00010000 00000000 00000000
    0, // 2213,  10100101 00010000 00000000 00000000
    1, // 2214,  01100101 00010000 00000000 00000000
    0, // 2215,  11100101 00010000 00000000 00000000
    3, // 2216,  00010101 00010000 00000000 00000000
    0, // 2217,  10010101 00010000 00000000 00000000
    1, // 2218,  01010101 00010000 00000000 00000000
    0, // 2219,  11010101 00010000 00000000 00000000
    2, // 2220,  00110101 00010000 00000000 00000000
    0, // 2221,  10110101 00010000 00000000 00000000
    1, // 2222,  01110101 00010000 00000000 00000000
    0, // 2223,  11110101 00010000 00000000 00000000
    4, // 2224,  00001101 00010000 00000000 00000000
    0, // 2225,  10001101 00010000 00000000 00000000
    1, // 2226,  01001101 00010000 00000000 00000000
    0, // 2227,  11001101 00010000 00000000 00000000
    2, // 2228,  00101101 00010000 00000000 00000000
    0, // 2229,  10101101 00010000 00000000 00000000
    1, // 2230,  01101101 00010000 00000000 00000000
    0, // 2231,  11101101 00010000 00000000 00000000
    3, // 2232,  00011101 00010000 00000000 00000000
    0, // 2233,  10011101 00010000 00000000 00000000
    1, // 2234,  01011101 00010000 00000000 00000000
    0, // 2235,  11011101 00010000 00000000 00000000
    2, // 2236,  00111101 00010000 00000000 00000000
    0, // 2237,  10111101 00010000 00000000 00000000
    1, // 2238,  01111101 00010000 00000000 00000000
    0, // 2239,  11111101 00010000 00000000 00000000
    6, // 2240,  00000011 00010000 00000000 00000000
    0, // 2241,  10000011 00010000 00000000 00000000
    1, // 2242,  01000011 00010000 00000000 00000000
    0, // 2243,  11000011 00010000 00000000 00000000
    2, // 2244,  00100011 00010000 00000000 00000000
    0, // 2245,  10100011 00010000 00000000 00000000
    1, // 2246,  01100011 00010000 00000000 00000000
    0, // 2247,  11100011 00010000 00000000 00000000
    3, // 2248,  00010011 00010000 00000000 00000000
    0, // 2249,  10010011 00010000 00000000 00000000
    1, // 2250,  01010011 00010000 00000000 00000000
    0, // 2251,  11010011 00010000 00000000 00000000
    2, // 2252,  00110011 00010000 00000000 00000000
    0, // 2253,  10110011 00010000 00000000 00000000
    1, // 2254,  01110011 00010000 00000000 00000000
    0, // 2255,  11110011 00010000 00000000 00000000
    4, // 2256,  00001011 00010000 00000000 00000000
    0, // 2257,  10001011 00010000 00000000 00000000
    1, // 2258,  01001011 00010000 00000000 00000000
    0, // 2259,  11001011 00010000 00000000 00000000
    2, // 2260,  00101011 00010000 00000000 00000000
    0, // 2261,  10101011 00010000 00000000 00000000
    1, // 2262,  01101011 00010000 00000000 00000000
    0, // 2263,  11101011 00010000 00000000 00000000
    3, // 2264,  00011011 00010000 00000000 00000000
    0, // 2265,  10011011 00010000 00000000 00000000
    1, // 2266,  01011011 00010000 00000000 00000000
    0, // 2267,  11011011 00010000 00000000 00000000
    2, // 2268,  00111011 00010000 00000000 00000000
    0, // 2269,  10111011 00010000 00000000 00000000
    1, // 2270,  01111011 00010000 00000000 00000000
    0, // 2271,  11111011 00010000 00000000 00000000
    5, // 2272,  00000111 00010000 00000000 00000000
    0, // 2273,  10000111 00010000 00000000 00000000
    1, // 2274,  01000111 00010000 00000000 00000000
    0, // 2275,  11000111 00010000 00000000 00000000
    2, // 2276,  00100111 00010000 00000000 00000000
    0, // 2277,  10100111 00010000 00000000 00000000
    1, // 2278,  01100111 00010000 00000000 00000000
    0, // 2279,  11100111 00010000 00000000 00000000
    3, // 2280,  00010111 00010000 00000000 00000000
    0, // 2281,  10010111 00010000 00000000 00000000
    1, // 2282,  01010111 00010000 00000000 00000000
    0, // 2283,  11010111 00010000 00000000 00000000
    2, // 2284,  00110111 00010000 00000000 00000000
    0, // 2285,  10110111 00010000 00000000 00000000
    1, // 2286,  01110111 00010000 00000000 00000000
    0, // 2287,  11110111 00010000 00000000 00000000
    4, // 2288,  00001111 00010000 00000000 00000000
    0, // 2289,  10001111 00010000 00000000 00000000
    1, // 2290,  01001111 00010000 00000000 00000000
    0, // 2291,  11001111 00010000 00000000 00000000
    2, // 2292,  00101111 00010000 00000000 00000000
    0, // 2293,  10101111 00010000 00000000 00000000
    1, // 2294,  01101111 00010000 00000000 00000000
    0, // 2295,  11101111 00010000 00000000 00000000
    3, // 2296,  00011111 00010000 00000000 00000000
    0, // 2297,  10011111 00010000 00000000 00000000
    1, // 2298,  01011111 00010000 00000000 00000000
    0, // 2299,  11011111 00010000 00000000 00000000
    2, // 2300,  00111111 00010000 00000000 00000000
    0, // 2301,  10111111 00010000 00000000 00000000
    1, // 2302,  01111111 00010000 00000000 00000000
    0, // 2303,  11111111 00010000 00000000 00000000
    8, // 2304,  00000000 10010000 00000000 00000000
    0, // 2305,  10000000 10010000 00000000 00000000
    1, // 2306,  01000000 10010000 00000000 00000000
    0, // 2307,  11000000 10010000 00000000 00000000
    2, // 2308,  00100000 10010000 00000000 00000000
    0, // 2309,  10100000 10010000 00000000 00000000
    1, // 2310,  01100000 10010000 00000000 00000000
    0, // 2311,  11100000 10010000 00000000 00000000
    3, // 2312,  00010000 10010000 00000000 00000000
    0, // 2313,  10010000 10010000 00000000 00000000
    1, // 2314,  01010000 10010000 00000000 00000000
    0, // 2315,  11010000 10010000 00000000 00000000
    2, // 2316,  00110000 10010000 00000000 00000000
    0, // 2317,  10110000 10010000 00000000 00000000
    1, // 2318,  01110000 10010000 00000000 00000000
    0, // 2319,  11110000 10010000 00000000 00000000
    4, // 2320,  00001000 10010000 00000000 00000000
    0, // 2321,  10001000 10010000 00000000 00000000
    1, // 2322,  01001000 10010000 00000000 00000000
    0, // 2323,  11001000 10010000 00000000 00000000
    2, // 2324,  00101000 10010000 00000000 00000000
    0, // 2325,  10101000 10010000 00000000 00000000
    1, // 2326,  01101000 10010000 00000000 00000000
    0, // 2327,  11101000 10010000 00000000 00000000
    3, // 2328,  00011000 10010000 00000000 00000000
    0, // 2329,  10011000 10010000 00000000 00000000
    1, // 2330,  01011000 10010000 00000000 00000000
    0, // 2331,  11011000 10010000 00000000 00000000
    2, // 2332,  00111000 10010000 00000000 00000000
    0, // 2333,  10111000 10010000 00000000 00000000
    1, // 2334,  01111000 10010000 00000000 00000000
    0, // 2335,  11111000 10010000 00000000 00000000
    5, // 2336,  00000100 10010000 00000000 00000000
    0, // 2337,  10000100 10010000 00000000 00000000
    1, // 2338,  01000100 10010000 00000000 00000000
    0, // 2339,  11000100 10010000 00000000 00000000
    2, // 2340,  00100100 10010000 00000000 00000000
    0, // 2341,  10100100 10010000 00000000 00000000
    1, // 2342,  01100100 10010000 00000000 00000000
    0, // 2343,  11100100 10010000 00000000 00000000
    3, // 2344,  00010100 10010000 00000000 00000000
    0, // 2345,  10010100 10010000 00000000 00000000
    1, // 2346,  01010100 10010000 00000000 00000000
    0, // 2347,  11010100 10010000 00000000 00000000
    2, // 2348,  00110100 10010000 00000000 00000000
    0, // 2349,  10110100 10010000 00000000 00000000
    1, // 2350,  01110100 10010000 00000000 00000000
    0, // 2351,  11110100 10010000 00000000 00000000
    4, // 2352,  00001100 10010000 00000000 00000000
    0, // 2353,  10001100 10010000 00000000 00000000
    1, // 2354,  01001100 10010000 00000000 00000000
    0, // 2355,  11001100 10010000 00000000 00000000
    2, // 2356,  00101100 10010000 00000000 00000000
    0, // 2357,  10101100 10010000 00000000 00000000
    1, // 2358,  01101100 10010000 00000000 00000000
    0, // 2359,  11101100 10010000 00000000 00000000
    3, // 2360,  00011100 10010000 00000000 00000000
    0, // 2361,  10011100 10010000 00000000 00000000
    1, // 2362,  01011100 10010000 00000000 00000000
    0, // 2363,  11011100 10010000 00000000 00000000
    2, // 2364,  00111100 10010000 00000000 00000000
    0, // 2365,  10111100 10010000 00000000 00000000
    1, // 2366,  01111100 10010000 00000000 00000000
    0, // 2367,  11111100 10010000 00000000 00000000
    6, // 2368,  00000010 10010000 00000000 00000000
    0, // 2369,  10000010 10010000 00000000 00000000
    1, // 2370,  01000010 10010000 00000000 00000000
    0, // 2371,  11000010 10010000 00000000 00000000
    2, // 2372,  00100010 10010000 00000000 00000000
    0, // 2373,  10100010 10010000 00000000 00000000
    1, // 2374,  01100010 10010000 00000000 00000000
    0, // 2375,  11100010 10010000 00000000 00000000
    3, // 2376,  00010010 10010000 00000000 00000000
    0, // 2377,  10010010 10010000 00000000 00000000
    1, // 2378,  01010010 10010000 00000000 00000000
    0, // 2379,  11010010 10010000 00000000 00000000
    2, // 2380,  00110010 10010000 00000000 00000000
    0, // 2381,  10110010 10010000 00000000 00000000
    1, // 2382,  01110010 10010000 00000000 00000000
    0, // 2383,  11110010 10010000 00000000 00000000
    4, // 2384,  00001010 10010000 00000000 00000000
    0, // 2385,  10001010 10010000 00000000 00000000
    1, // 2386,  01001010 10010000 00000000 00000000
    0, // 2387,  11001010 10010000 00000000 00000000
    2, // 2388,  00101010 10010000 00000000 00000000
    0, // 2389,  10101010 10010000 00000000 00000000
    1, // 2390,  01101010 10010000 00000000 00000000
    0, // 2391,  11101010 10010000 00000000 00000000
    3, // 2392,  00011010 10010000 00000000 00000000
    0, // 2393,  10011010 10010000 00000000 00000000
    1, // 2394,  01011010 10010000 00000000 00000000
    0, // 2395,  11011010 10010000 00000000 00000000
    2, // 2396,  00111010 10010000 00000000 00000000
    0, // 2397,  10111010 10010000 00000000 00000000
    1, // 2398,  01111010 10010000 00000000 00000000
    0, // 2399,  11111010 10010000 00000000 00000000
    5, // 2400,  00000110 10010000 00000000 00000000
    0, // 2401,  10000110 10010000 00000000 00000000
    1, // 2402,  01000110 10010000 00000000 00000000
    0, // 2403,  11000110 10010000 00000000 00000000
    2, // 2404,  00100110 10010000 00000000 00000000
    0, // 2405,  10100110 10010000 00000000 00000000
    1, // 2406,  01100110 10010000 00000000 00000000
    0, // 2407,  11100110 10010000 00000000 00000000
    3, // 2408,  00010110 10010000 00000000 00000000
    0, // 2409,  10010110 10010000 00000000 00000000
    1, // 2410,  01010110 10010000 00000000 00000000
    0, // 2411,  11010110 10010000 00000000 00000000
    2, // 2412,  00110110 10010000 00000000 00000000
    0, // 2413,  10110110 10010000 00000000 00000000
    1, // 2414,  01110110 10010000 00000000 00000000
    0, // 2415,  11110110 10010000 00000000 00000000
    4, // 2416,  00001110 10010000 00000000 00000000
    0, // 2417,  10001110 10010000 00000000 00000000
    1, // 2418,  01001110 10010000 00000000 00000000
    0, // 2419,  11001110 10010000 00000000 00000000
    2, // 2420,  00101110 10010000 00000000 00000000
    0, // 2421,  10101110 10010000 00000000 00000000
    1, // 2422,  01101110 10010000 00000000 00000000
    0, // 2423,  11101110 10010000 00000000 00000000
    3, // 2424,  00011110 10010000 00000000 00000000
    0, // 2425,  10011110 10010000 00000000 00000000
    1, // 2426,  01011110 10010000 00000000 00000000
    0, // 2427,  11011110 10010000 00000000 00000000
    2, // 2428,  00111110 10010000 00000000 00000000
    0, // 2429,  10111110 10010000 00000000 00000000
    1, // 2430,  01111110 10010000 00000000 00000000
    0, // 2431,  11111110 10010000 00000000 00000000
    7, // 2432,  00000001 10010000 00000000 00000000
    0, // 2433,  10000001 10010000 00000000 00000000
    1, // 2434,  01000001 10010000 00000000 00000000
    0, // 2435,  11000001 10010000 00000000 00000000
    2, // 2436,  00100001 10010000 00000000 00000000
    0, // 2437,  10100001 10010000 00000000 00000000
    1, // 2438,  01100001 10010000 00000000 00000000
    0, // 2439,  11100001 10010000 00000000 00000000
    3, // 2440,  00010001 10010000 00000000 00000000
    0, // 2441,  10010001 10010000 00000000 00000000
    1, // 2442,  01010001 10010000 00000000 00000000
    0, // 2443,  11010001 10010000 00000000 00000000
    2, // 2444,  00110001 10010000 00000000 00000000
    0, // 2445,  10110001 10010000 00000000 00000000
    1, // 2446,  01110001 10010000 00000000 00000000
    0, // 2447,  11110001 10010000 00000000 00000000
    4, // 2448,  00001001 10010000 00000000 00000000
    0, // 2449,  10001001 10010000 00000000 00000000
    1, // 2450,  01001001 10010000 00000000 00000000
    0, // 2451,  11001001 10010000 00000000 00000000
    2, // 2452,  00101001 10010000 00000000 00000000
    0, // 2453,  10101001 10010000 00000000 00000000
    1, // 2454,  01101001 10010000 00000000 00000000
    0, // 2455,  11101001 10010000 00000000 00000000
    3, // 2456,  00011001 10010000 00000000 00000000
    0, // 2457,  10011001 10010000 00000000 00000000
    1, // 2458,  01011001 10010000 00000000 00000000
    0, // 2459,  11011001 10010000 00000000 00000000
    2, // 2460,  00111001 10010000 00000000 00000000
    0, // 2461,  10111001 10010000 00000000 00000000
    1, // 2462,  01111001 10010000 00000000 00000000
    0, // 2463,  11111001 10010000 00000000 00000000
    5, // 2464,  00000101 10010000 00000000 00000000
    0, // 2465,  10000101 10010000 00000000 00000000
    1, // 2466,  01000101 10010000 00000000 00000000
    0, // 2467,  11000101 10010000 00000000 00000000
    2, // 2468,  00100101 10010000 00000000 00000000
    0, // 2469,  10100101 10010000 00000000 00000000
    1, // 2470,  01100101 10010000 00000000 00000000
    0, // 2471,  11100101 10010000 00000000 00000000
    3, // 2472,  00010101 10010000 00000000 00000000
    0, // 2473,  10010101 10010000 00000000 00000000
    1, // 2474,  01010101 10010000 00000000 00000000
    0, // 2475,  11010101 10010000 00000000 00000000
    2, // 2476,  00110101 10010000 00000000 00000000
    0, // 2477,  10110101 10010000 00000000 00000000
    1, // 2478,  01110101 10010000 00000000 00000000
    0, // 2479,  11110101 10010000 00000000 00000000
    4, // 2480,  00001101 10010000 00000000 00000000
    0, // 2481,  10001101 10010000 00000000 00000000
    1, // 2482,  01001101 10010000 00000000 00000000
    0, // 2483,  11001101 10010000 00000000 00000000
    2, // 2484,  00101101 10010000 00000000 00000000
    0, // 2485,  10101101 10010000 00000000 00000000
    1, // 2486,  01101101 10010000 00000000 00000000
    0, // 2487,  11101101 10010000 00000000 00000000
    3, // 2488,  00011101 10010000 00000000 00000000
    0, // 2489,  10011101 10010000 00000000 00000000
    1, // 2490,  01011101 10010000 00000000 00000000
    0, // 2491,  11011101 10010000 00000000 00000000
    2, // 2492,  00111101 10010000 00000000 00000000
    0, // 2493,  10111101 10010000 00000000 00000000
    1, // 2494,  01111101 10010000 00000000 00000000
    0, // 2495,  11111101 10010000 00000000 00000000
    6, // 2496,  00000011 10010000 00000000 00000000
    0, // 2497,  10000011 10010000 00000000 00000000
    1, // 2498,  01000011 10010000 00000000 00000000
    0, // 2499,  11000011 10010000 00000000 00000000
    2, // 2500,  00100011 10010000 00000000 00000000
    0, // 2501,  10100011 10010000 00000000 00000000
    1, // 2502,  01100011 10010000 00000000 00000000
    0, // 2503,  11100011 10010000 00000000 00000000
    3, // 2504,  00010011 10010000 00000000 00000000
    0, // 2505,  10010011 10010000 00000000 00000000
    1, // 2506,  01010011 10010000 00000000 00000000
    0, // 2507,  11010011 10010000 00000000 00000000
    2, // 2508,  00110011 10010000 00000000 00000000
    0, // 2509,  10110011 10010000 00000000 00000000
    1, // 2510,  01110011 10010000 00000000 00000000
    0, // 2511,  11110011 10010000 00000000 00000000
    4, // 2512,  00001011 10010000 00000000 00000000
    0, // 2513,  10001011 10010000 00000000 00000000
    1, // 2514,  01001011 10010000 00000000 00000000
    0, // 2515,  11001011 10010000 00000000 00000000
    2, // 2516,  00101011 10010000 00000000 00000000
    0, // 2517,  10101011 10010000 00000000 00000000
    1, // 2518,  01101011 10010000 00000000 00000000
    0, // 2519,  11101011 10010000 00000000 00000000
    3, // 2520,  00011011 10010000 00000000 00000000
    0, // 2521,  10011011 10010000 00000000 00000000
    1, // 2522,  01011011 10010000 00000000 00000000
    0, // 2523,  11011011 10010000 00000000 00000000
    2, // 2524,  00111011 10010000 00000000 00000000
    0, // 2525,  10111011 10010000 00000000 00000000
    1, // 2526,  01111011 10010000 00000000 00000000
    0, // 2527,  11111011 10010000 00000000 00000000
    5, // 2528,  00000111 10010000 00000000 00000000
    0, // 2529,  10000111 10010000 00000000 00000000
    1, // 2530,  01000111 10010000 00000000 00000000
    0, // 2531,  11000111 10010000 00000000 00000000
    2, // 2532,  00100111 10010000 00000000 00000000
    0, // 2533,  10100111 10010000 00000000 00000000
    1, // 2534,  01100111 10010000 00000000 00000000
    0, // 2535,  11100111 10010000 00000000 00000000
    3, // 2536,  00010111 10010000 00000000 00000000
    0, // 2537,  10010111 10010000 00000000 00000000
    1, // 2538,  01010111 10010000 00000000 00000000
    0, // 2539,  11010111 10010000 00000000 00000000
    2, // 2540,  00110111 10010000 00000000 00000000
    0, // 2541,  10110111 10010000 00000000 00000000
    1, // 2542,  01110111 10010000 00000000 00000000
    0, // 2543,  11110111 10010000 00000000 00000000
    4, // 2544,  00001111 10010000 00000000 00000000
    0, // 2545,  10001111 10010000 00000000 00000000
    1, // 2546,  01001111 10010000 00000000 00000000
    0, // 2547,  11001111 10010000 00000000 00000000
    2, // 2548,  00101111 10010000 00000000 00000000
    0, // 2549,  10101111 10010000 00000000 00000000
    1, // 2550,  01101111 10010000 00000000 00000000
    0, // 2551,  11101111 10010000 00000000 00000000
    3, // 2552,  00011111 10010000 00000000 00000000
    0, // 2553,  10011111 10010000 00000000 00000000
    1, // 2554,  01011111 10010000 00000000 00000000
    0, // 2555,  11011111 10010000 00000000 00000000
    2, // 2556,  00111111 10010000 00000000 00000000
    0, // 2557,  10111111 10010000 00000000 00000000
    1, // 2558,  01111111 10010000 00000000 00000000
    0, // 2559,  11111111 10010000 00000000 00000000
    9, // 2560,  00000000 01010000 00000000 00000000
    0, // 2561,  10000000 01010000 00000000 00000000
    1, // 2562,  01000000 01010000 00000000 00000000
    0, // 2563,  11000000 01010000 00000000 00000000
    2, // 2564,  00100000 01010000 00000000 00000000
    0, // 2565,  10100000 01010000 00000000 00000000
    1, // 2566,  01100000 01010000 00000000 00000000
    0, // 2567,  11100000 01010000 00000000 00000000
    3, // 2568,  00010000 01010000 00000000 00000000
    0, // 2569,  10010000 01010000 00000000 00000000
    1, // 2570,  01010000 01010000 00000000 00000000
    0, // 2571,  11010000 01010000 00000000 00000000
    2, // 2572,  00110000 01010000 00000000 00000000
    0, // 2573,  10110000 01010000 00000000 00000000
    1, // 2574,  01110000 01010000 00000000 00000000
    0, // 2575,  11110000 01010000 00000000 00000000
    4, // 2576,  00001000 01010000 00000000 00000000
    0, // 2577,  10001000 01010000 00000000 00000000
    1, // 2578,  01001000 01010000 00000000 00000000
    0, // 2579,  11001000 01010000 00000000 00000000
    2, // 2580,  00101000 01010000 00000000 00000000
    0, // 2581,  10101000 01010000 00000000 00000000
    1, // 2582,  01101000 01010000 00000000 00000000
    0, // 2583,  11101000 01010000 00000000 00000000
    3, // 2584,  00011000 01010000 00000000 00000000
    0, // 2585,  10011000 01010000 00000000 00000000
    1, // 2586,  01011000 01010000 00000000 00000000
    0, // 2587,  11011000 01010000 00000000 00000000
    2, // 2588,  00111000 01010000 00000000 00000000
    0, // 2589,  10111000 01010000 00000000 00000000
    1, // 2590,  01111000 01010000 00000000 00000000
    0, // 2591,  11111000 01010000 00000000 00000000
    5, // 2592,  00000100 01010000 00000000 00000000
    0, // 2593,  10000100 01010000 00000000 00000000
    1, // 2594,  01000100 01010000 00000000 00000000
    0, // 2595,  11000100 01010000 00000000 00000000
    2, // 2596,  00100100 01010000 00000000 00000000
    0, // 2597,  10100100 01010000 00000000 00000000
    1, // 2598,  01100100 01010000 00000000 00000000
    0, // 2599,  11100100 01010000 00000000 00000000
    3, // 2600,  00010100 01010000 00000000 00000000
    0, // 2601,  10010100 01010000 00000000 00000000
    1, // 2602,  01010100 01010000 00000000 00000000
    0, // 2603,  11010100 01010000 00000000 00000000
    2, // 2604,  00110100 01010000 00000000 00000000
    0, // 2605,  10110100 01010000 00000000 00000000
    1, // 2606,  01110100 01010000 00000000 00000000
    0, // 2607,  11110100 01010000 00000000 00000000
    4, // 2608,  00001100 01010000 00000000 00000000
    0, // 2609,  10001100 01010000 00000000 00000000
    1, // 2610,  01001100 01010000 00000000 00000000
    0, // 2611,  11001100 01010000 00000000 00000000
    2, // 2612,  00101100 01010000 00000000 00000000
    0, // 2613,  10101100 01010000 00000000 00000000
    1, // 2614,  01101100 01010000 00000000 00000000
    0, // 2615,  11101100 01010000 00000000 00000000
    3, // 2616,  00011100 01010000 00000000 00000000
    0, // 2617,  10011100 01010000 00000000 00000000
    1, // 2618,  01011100 01010000 00000000 00000000
    0, // 2619,  11011100 01010000 00000000 00000000
    2, // 2620,  00111100 01010000 00000000 00000000
    0, // 2621,  10111100 01010000 00000000 00000000
    1, // 2622,  01111100 01010000 00000000 00000000
    0, // 2623,  11111100 01010000 00000000 00000000
    6, // 2624,  00000010 01010000 00000000 00000000
    0, // 2625,  10000010 01010000 00000000 00000000
    1, // 2626,  01000010 01010000 00000000 00000000
    0, // 2627,  11000010 01010000 00000000 00000000
    2, // 2628,  00100010 01010000 00000000 00000000
    0, // 2629,  10100010 01010000 00000000 00000000
    1, // 2630,  01100010 01010000 00000000 00000000
    0, // 2631,  11100010 01010000 00000000 00000000
    3, // 2632,  00010010 01010000 00000000 00000000
    0, // 2633,  10010010 01010000 00000000 00000000
    1, // 2634,  01010010 01010000 00000000 00000000
    0, // 2635,  11010010 01010000 00000000 00000000
    2, // 2636,  00110010 01010000 00000000 00000000
    0, // 2637,  10110010 01010000 00000000 00000000
    1, // 2638,  01110010 01010000 00000000 00000000
    0, // 2639,  11110010 01010000 00000000 00000000
    4, // 2640,  00001010 01010000 00000000 00000000
    0, // 2641,  10001010 01010000 00000000 00000000
    1, // 2642,  01001010 01010000 00000000 00000000
    0, // 2643,  11001010 01010000 00000000 00000000
    2, // 2644,  00101010 01010000 00000000 00000000
    0, // 2645,  10101010 01010000 00000000 00000000
    1, // 2646,  01101010 01010000 00000000 00000000
    0, // 2647,  11101010 01010000 00000000 00000000
    3, // 2648,  00011010 01010000 00000000 00000000
    0, // 2649,  10011010 01010000 00000000 00000000
    1, // 2650,  01011010 01010000 00000000 00000000
    0, // 2651,  11011010 01010000 00000000 00000000
    2, // 2652,  00111010 01010000 00000000 00000000
    0, // 2653,  10111010 01010000 00000000 00000000
    1, // 2654,  01111010 01010000 00000000 00000000
    0, // 2655,  11111010 01010000 00000000 00000000
    5, // 2656,  00000110 01010000 00000000 00000000
    0, // 2657,  10000110 01010000 00000000 00000000
    1, // 2658,  01000110 01010000 00000000 00000000
    0, // 2659,  11000110 01010000 00000000 00000000
    2, // 2660,  00100110 01010000 00000000 00000000
    0, // 2661,  10100110 01010000 00000000 00000000
    1, // 2662,  01100110 01010000 00000000 00000000
    0, // 2663,  11100110 01010000 00000000 00000000
    3, // 2664,  00010110 01010000 00000000 00000000
    0, // 2665,  10010110 01010000 00000000 00000000
    1, // 2666,  01010110 01010000 00000000 00000000
    0, // 2667,  11010110 01010000 00000000 00000000
    2, // 2668,  00110110 01010000 00000000 00000000
    0, // 2669,  10110110 01010000 00000000 00000000
    1, // 2670,  01110110 01010000 00000000 00000000
    0, // 2671,  11110110 01010000 00000000 00000000
    4, // 2672,  00001110 01010000 00000000 00000000
    0, // 2673,  10001110 01010000 00000000 00000000
    1, // 2674,  01001110 01010000 00000000 00000000
    0, // 2675,  11001110 01010000 00000000 00000000
    2, // 2676,  00101110 01010000 00000000 00000000
    0, // 2677,  10101110 01010000 00000000 00000000
    1, // 2678,  01101110 01010000 00000000 00000000
    0, // 2679,  11101110 01010000 00000000 00000000
    3, // 2680,  00011110 01010000 00000000 00000000
    0, // 2681,  10011110 01010000 00000000 00000000
    1, // 2682,  01011110 01010000 00000000 00000000
    0, // 2683,  11011110 01010000 00000000 00000000
    2, // 2684,  00111110 01010000 00000000 00000000
    0, // 2685,  10111110 01010000 00000000 00000000
    1, // 2686,  01111110 01010000 00000000 00000000
    0, // 2687,  11111110 01010000 00000000 00000000
    7, // 2688,  00000001 01010000 00000000 00000000
    0, // 2689,  10000001 01010000 00000000 00000000
    1, // 2690,  01000001 01010000 00000000 00000000
    0, // 2691,  11000001 01010000 00000000 00000000
    2, // 2692,  00100001 01010000 00000000 00000000
    0, // 2693,  10100001 01010000 00000000 00000000
    1, // 2694,  01100001 01010000 00000000 00000000
    0, // 2695,  11100001 01010000 00000000 00000000
    3, // 2696,  00010001 01010000 00000000 00000000
    0, // 2697,  10010001 01010000 00000000 00000000
    1, // 2698,  01010001 01010000 00000000 00000000
    0, // 2699,  11010001 01010000 00000000 00000000
    2, // 2700,  00110001 01010000 00000000 00000000
    0, // 2701,  10110001 01010000 00000000 00000000
    1, // 2702,  01110001 01010000 00000000 00000000
    0, // 2703,  11110001 01010000 00000000 00000000
    4, // 2704,  00001001 01010000 00000000 00000000
    0, // 2705,  10001001 01010000 00000000 00000000
    1, // 2706,  01001001 01010000 00000000 00000000
    0, // 2707,  11001001 01010000 00000000 00000000
    2, // 2708,  00101001 01010000 00000000 00000000
    0, // 2709,  10101001 01010000 00000000 00000000
    1, // 2710,  01101001 01010000 00000000 00000000
    0, // 2711,  11101001 01010000 00000000 00000000
    3, // 2712,  00011001 01010000 00000000 00000000
    0, // 2713,  10011001 01010000 00000000 00000000
    1, // 2714,  01011001 01010000 00000000 00000000
    0, // 2715,  11011001 01010000 00000000 00000000
    2, // 2716,  00111001 01010000 00000000 00000000
    0, // 2717,  10111001 01010000 00000000 00000000
    1, // 2718,  01111001 01010000 00000000 00000000
    0, // 2719,  11111001 01010000 00000000 00000000
    5, // 2720,  00000101 01010000 00000000 00000000
    0, // 2721,  10000101 01010000 00000000 00000000
    1, // 2722,  01000101 01010000 00000000 00000000
    0, // 2723,  11000101 01010000 00000000 00000000
    2, // 2724,  00100101 01010000 00000000 00000000
    0, // 2725,  10100101 01010000 00000000 00000000
    1, // 2726,  01100101 01010000 00000000 00000000
    0, // 2727,  11100101 01010000 00000000 00000000
    3, // 2728,  00010101 01010000 00000000 00000000
    0, // 2729,  10010101 01010000 00000000 00000000
    1, // 2730,  01010101 01010000 00000000 00000000
    0, // 2731,  11010101 01010000 00000000 00000000
    2, // 2732,  00110101 01010000 00000000 00000000
    0, // 2733,  10110101 01010000 00000000 00000000
    1, // 2734,  01110101 01010000 00000000 00000000
    0, // 2735,  11110101 01010000 00000000 00000000
    4, // 2736,  00001101 01010000 00000000 00000000
    0, // 2737,  10001101 01010000 00000000 00000000
    1, // 2738,  01001101 01010000 00000000 00000000
    0, // 2739,  11001101 01010000 00000000 00000000
    2, // 2740,  00101101 01010000 00000000 00000000
    0, // 2741,  10101101 01010000 00000000 00000000
    1, // 2742,  01101101 01010000 00000000 00000000
    0, // 2743,  11101101 01010000 00000000 00000000
    3, // 2744,  00011101 01010000 00000000 00000000
    0, // 2745,  10011101 01010000 00000000 00000000
    1, // 2746,  01011101 01010000 00000000 00000000
    0, // 2747,  11011101 01010000 00000000 00000000
    2, // 2748,  00111101 01010000 00000000 00000000
    0, // 2749,  10111101 01010000 00000000 00000000
    1, // 2750,  01111101 01010000 00000000 00000000
    0, // 2751,  11111101 01010000 00000000 00000000
    6, // 2752,  00000011 01010000 00000000 00000000
    0, // 2753,  10000011 01010000 00000000 00000000
    1, // 2754,  01000011 01010000 00000000 00000000
    0, // 2755,  11000011 01010000 00000000 00000000
    2, // 2756,  00100011 01010000 00000000 00000000
    0, // 2757,  10100011 01010000 00000000 00000000
    1, // 2758,  01100011 01010000 00000000 00000000
    0, // 2759,  11100011 01010000 00000000 00000000
    3, // 2760,  00010011 01010000 00000000 00000000
    0, // 2761,  10010011 01010000 00000000 00000000
    1, // 2762,  01010011 01010000 00000000 00000000
    0, // 2763,  11010011 01010000 00000000 00000000
    2, // 2764,  00110011 01010000 00000000 00000000
    0, // 2765,  10110011 01010000 00000000 00000000
    1, // 2766,  01110011 01010000 00000000 00000000
    0, // 2767,  11110011 01010000 00000000 00000000
    4, // 2768,  00001011 01010000 00000000 00000000
    0, // 2769,  10001011 01010000 00000000 00000000
    1, // 2770,  01001011 01010000 00000000 00000000
    0, // 2771,  11001011 01010000 00000000 00000000
    2, // 2772,  00101011 01010000 00000000 00000000
    0, // 2773,  10101011 01010000 00000000 00000000
    1, // 2774,  01101011 01010000 00000000 00000000
    0, // 2775,  11101011 01010000 00000000 00000000
    3, // 2776,  00011011 01010000 00000000 00000000
    0, // 2777,  10011011 01010000 00000000 00000000
    1, // 2778,  01011011 01010000 00000000 00000000
    0, // 2779,  11011011 01010000 00000000 00000000
    2, // 2780,  00111011 01010000 00000000 00000000
    0, // 2781,  10111011 01010000 00000000 00000000
    1, // 2782,  01111011 01010000 00000000 00000000
    0, // 2783,  11111011 01010000 00000000 00000000
    5, // 2784,  00000111 01010000 00000000 00000000
    0, // 2785,  10000111 01010000 00000000 00000000
    1, // 2786,  01000111 01010000 00000000 00000000
    0, // 2787,  11000111 01010000 00000000 00000000
    2, // 2788,  00100111 01010000 00000000 00000000
    0, // 2789,  10100111 01010000 00000000 00000000
    1, // 2790,  01100111 01010000 00000000 00000000
    0, // 2791,  11100111 01010000 00000000 00000000
    3, // 2792,  00010111 01010000 00000000 00000000
    0, // 2793,  10010111 01010000 00000000 00000000
    1, // 2794,  01010111 01010000 00000000 00000000
    0, // 2795,  11010111 01010000 00000000 00000000
    2, // 2796,  00110111 01010000 00000000 00000000
    0, // 2797,  10110111 01010000 00000000 00000000
    1, // 2798,  01110111 01010000 00000000 00000000
    0, // 2799,  11110111 01010000 00000000 00000000
    4, // 2800,  00001111 01010000 00000000 00000000
    0, // 2801,  10001111 01010000 00000000 00000000
    1, // 2802,  01001111 01010000 00000000 00000000
    0, // 2803,  11001111 01010000 00000000 00000000
    2, // 2804,  00101111 01010000 00000000 00000000
    0, // 2805,  10101111 01010000 00000000 00000000
    1, // 2806,  01101111 01010000 00000000 00000000
    0, // 2807,  11101111 01010000 00000000 00000000
    3, // 2808,  00011111 01010000 00000000 00000000
    0, // 2809,  10011111 01010000 00000000 00000000
    1, // 2810,  01011111 01010000 00000000 00000000
    0, // 2811,  11011111 01010000 00000000 00000000
    2, // 2812,  00111111 01010000 00000000 00000000
    0, // 2813,  10111111 01010000 00000000 00000000
    1, // 2814,  01111111 01010000 00000000 00000000
    0, // 2815,  11111111 01010000 00000000 00000000
    8, // 2816,  00000000 11010000 00000000 00000000
    0, // 2817,  10000000 11010000 00000000 00000000
    1, // 2818,  01000000 11010000 00000000 00000000
    0, // 2819,  11000000 11010000 00000000 00000000
    2, // 2820,  00100000 11010000 00000000 00000000
    0, // 2821,  10100000 11010000 00000000 00000000
    1, // 2822,  01100000 11010000 00000000 00000000
    0, // 2823,  11100000 11010000 00000000 00000000
    3, // 2824,  00010000 11010000 00000000 00000000
    0, // 2825,  10010000 11010000 00000000 00000000
    1, // 2826,  01010000 11010000 00000000 00000000
    0, // 2827,  11010000 11010000 00000000 00000000
    2, // 2828,  00110000 11010000 00000000 00000000
    0, // 2829,  10110000 11010000 00000000 00000000
    1, // 2830,  01110000 11010000 00000000 00000000
    0, // 2831,  11110000 11010000 00000000 00000000
    4, // 2832,  00001000 11010000 00000000 00000000
    0, // 2833,  10001000 11010000 00000000 00000000
    1, // 2834,  01001000 11010000 00000000 00000000
    0, // 2835,  11001000 11010000 00000000 00000000
    2, // 2836,  00101000 11010000 00000000 00000000
    0, // 2837,  10101000 11010000 00000000 00000000
    1, // 2838,  01101000 11010000 00000000 00000000
    0, // 2839,  11101000 11010000 00000000 00000000
    3, // 2840,  00011000 11010000 00000000 00000000
    0, // 2841,  10011000 11010000 00000000 00000000
    1, // 2842,  01011000 11010000 00000000 00000000
    0, // 2843,  11011000 11010000 00000000 00000000
    2, // 2844,  00111000 11010000 00000000 00000000
    0, // 2845,  10111000 11010000 00000000 00000000
    1, // 2846,  01111000 11010000 00000000 00000000
    0, // 2847,  11111000 11010000 00000000 00000000
    5, // 2848,  00000100 11010000 00000000 00000000
    0, // 2849,  10000100 11010000 00000000 00000000
    1, // 2850,  01000100 11010000 00000000 00000000
    0, // 2851,  11000100 11010000 00000000 00000000
    2, // 2852,  00100100 11010000 00000000 00000000
    0, // 2853,  10100100 11010000 00000000 00000000
    1, // 2854,  01100100 11010000 00000000 00000000
    0, // 2855,  11100100 11010000 00000000 00000000
    3, // 2856,  00010100 11010000 00000000 00000000
    0, // 2857,  10010100 11010000 00000000 00000000
    1, // 2858,  01010100 11010000 00000000 00000000
    0, // 2859,  11010100 11010000 00000000 00000000
    2, // 2860,  00110100 11010000 00000000 00000000
    0, // 2861,  10110100 11010000 00000000 00000000
    1, // 2862,  01110100 11010000 00000000 00000000
    0, // 2863,  11110100 11010000 00000000 00000000
    4, // 2864,  00001100 11010000 00000000 00000000
    0, // 2865,  10001100 11010000 00000000 00000000
    1, // 2866,  01001100 11010000 00000000 00000000
    0, // 2867,  11001100 11010000 00000000 00000000
    2, // 2868,  00101100 11010000 00000000 00000000
    0, // 2869,  10101100 11010000 00000000 00000000
    1, // 2870,  01101100 11010000 00000000 00000000
    0, // 2871,  11101100 11010000 00000000 00000000
    3, // 2872,  00011100 11010000 00000000 00000000
    0, // 2873,  10011100 11010000 00000000 00000000
    1, // 2874,  01011100 11010000 00000000 00000000
    0, // 2875,  11011100 11010000 00000000 00000000
    2, // 2876,  00111100 11010000 00000000 00000000
    0, // 2877,  10111100 11010000 00000000 00000000
    1, // 2878,  01111100 11010000 00000000 00000000
    0, // 2879,  11111100 11010000 00000000 00000000
    6, // 2880,  00000010 11010000 00000000 00000000
    0, // 2881,  10000010 11010000 00000000 00000000
    1, // 2882,  01000010 11010000 00000000 00000000
    0, // 2883,  11000010 11010000 00000000 00000000
    2, // 2884,  00100010 11010000 00000000 00000000
    0, // 2885,  10100010 11010000 00000000 00000000
    1, // 2886,  01100010 11010000 00000000 00000000
    0, // 2887,  11100010 11010000 00000000 00000000
    3, // 2888,  00010010 11010000 00000000 00000000
    0, // 2889,  10010010 11010000 00000000 00000000
    1, // 2890,  01010010 11010000 00000000 00000000
    0, // 2891,  11010010 11010000 00000000 00000000
    2, // 2892,  00110010 11010000 00000000 00000000
    0, // 2893,  10110010 11010000 00000000 00000000
    1, // 2894,  01110010 11010000 00000000 00000000
    0, // 2895,  11110010 11010000 00000000 00000000
    4, // 2896,  00001010 11010000 00000000 00000000
    0, // 2897,  10001010 11010000 00000000 00000000
    1, // 2898,  01001010 11010000 00000000 00000000
    0, // 2899,  11001010 11010000 00000000 00000000
    2, // 2900,  00101010 11010000 00000000 00000000
    0, // 2901,  10101010 11010000 00000000 00000000
    1, // 2902,  01101010 11010000 00000000 00000000
    0, // 2903,  11101010 11010000 00000000 00000000
    3, // 2904,  00011010 11010000 00000000 00000000
    0, // 2905,  10011010 11010000 00000000 00000000
    1, // 2906,  01011010 11010000 00000000 00000000
    0, // 2907,  11011010 11010000 00000000 00000000
    2, // 2908,  00111010 11010000 00000000 00000000
    0, // 2909,  10111010 11010000 00000000 00000000
    1, // 2910,  01111010 11010000 00000000 00000000
    0, // 2911,  11111010 11010000 00000000 00000000
    5, // 2912,  00000110 11010000 00000000 00000000
    0, // 2913,  10000110 11010000 00000000 00000000
    1, // 2914,  01000110 11010000 00000000 00000000
    0, // 2915,  11000110 11010000 00000000 00000000
    2, // 2916,  00100110 11010000 00000000 00000000
    0, // 2917,  10100110 11010000 00000000 00000000
    1, // 2918,  01100110 11010000 00000000 00000000
    0, // 2919,  11100110 11010000 00000000 00000000
    3, // 2920,  00010110 11010000 00000000 00000000
    0, // 2921,  10010110 11010000 00000000 00000000
    1, // 2922,  01010110 11010000 00000000 00000000
    0, // 2923,  11010110 11010000 00000000 00000000
    2, // 2924,  00110110 11010000 00000000 00000000
    0, // 2925,  10110110 11010000 00000000 00000000
    1, // 2926,  01110110 11010000 00000000 00000000
    0, // 2927,  11110110 11010000 00000000 00000000
    4, // 2928,  00001110 11010000 00000000 00000000
    0, // 2929,  10001110 11010000 00000000 00000000
    1, // 2930,  01001110 11010000 00000000 00000000
    0, // 2931,  11001110 11010000 00000000 00000000
    2, // 2932,  00101110 11010000 00000000 00000000
    0, // 2933,  10101110 11010000 00000000 00000000
    1, // 2934,  01101110 11010000 00000000 00000000
    0, // 2935,  11101110 11010000 00000000 00000000
    3, // 2936,  00011110 11010000 00000000 00000000
    0, // 2937,  10011110 11010000 00000000 00000000
    1, // 2938,  01011110 11010000 00000000 00000000
    0, // 2939,  11011110 11010000 00000000 00000000
    2, // 2940,  00111110 11010000 00000000 00000000
    0, // 2941,  10111110 11010000 00000000 00000000
    1, // 2942,  01111110 11010000 00000000 00000000
    0, // 2943,  11111110 11010000 00000000 00000000
    7, // 2944,  00000001 11010000 00000000 00000000
    0, // 2945,  10000001 11010000 00000000 00000000
    1, // 2946,  01000001 11010000 00000000 00000000
    0, // 2947,  11000001 11010000 00000000 00000000
    2, // 2948,  00100001 11010000 00000000 00000000
    0, // 2949,  10100001 11010000 00000000 00000000
    1, // 2950,  01100001 11010000 00000000 00000000
    0, // 2951,  11100001 11010000 00000000 00000000
    3, // 2952,  00010001 11010000 00000000 00000000
    0, // 2953,  10010001 11010000 00000000 00000000
    1, // 2954,  01010001 11010000 00000000 00000000
    0, // 2955,  11010001 11010000 00000000 00000000
    2, // 2956,  00110001 11010000 00000000 00000000
    0, // 2957,  10110001 11010000 00000000 00000000
    1, // 2958,  01110001 11010000 00000000 00000000
    0, // 2959,  11110001 11010000 00000000 00000000
    4, // 2960,  00001001 11010000 00000000 00000000
    0, // 2961,  10001001 11010000 00000000 00000000
    1, // 2962,  01001001 11010000 00000000 00000000
    0, // 2963,  11001001 11010000 00000000 00000000
    2, // 2964,  00101001 11010000 00000000 00000000
    0, // 2965,  10101001 11010000 00000000 00000000
    1, // 2966,  01101001 11010000 00000000 00000000
    0, // 2967,  11101001 11010000 00000000 00000000
    3, // 2968,  00011001 11010000 00000000 00000000
    0, // 2969,  10011001 11010000 00000000 00000000
    1, // 2970,  01011001 11010000 00000000 00000000
    0, // 2971,  11011001 11010000 00000000 00000000
    2, // 2972,  00111001 11010000 00000000 00000000
    0, // 2973,  10111001 11010000 00000000 00000000
    1, // 2974,  01111001 11010000 00000000 00000000
    0, // 2975,  11111001 11010000 00000000 00000000
    5, // 2976,  00000101 11010000 00000000 00000000
    0, // 2977,  10000101 11010000 00000000 00000000
    1, // 2978,  01000101 11010000 00000000 00000000
    0, // 2979,  11000101 11010000 00000000 00000000
    2, // 2980,  00100101 11010000 00000000 00000000
    0, // 2981,  10100101 11010000 00000000 00000000
    1, // 2982,  01100101 11010000 00000000 00000000
    0, // 2983,  11100101 11010000 00000000 00000000
    3, // 2984,  00010101 11010000 00000000 00000000
    0, // 2985,  10010101 11010000 00000000 00000000
    1, // 2986,  01010101 11010000 00000000 00000000
    0, // 2987,  11010101 11010000 00000000 00000000
    2, // 2988,  00110101 11010000 00000000 00000000
    0, // 2989,  10110101 11010000 00000000 00000000
    1, // 2990,  01110101 11010000 00000000 00000000
    0, // 2991,  11110101 11010000 00000000 00000000
    4, // 2992,  00001101 11010000 00000000 00000000
    0, // 2993,  10001101 11010000 00000000 00000000
    1, // 2994,  01001101 11010000 00000000 00000000
    0, // 2995,  11001101 11010000 00000000 00000000
    2, // 2996,  00101101 11010000 00000000 00000000
    0, // 2997,  10101101 11010000 00000000 00000000
    1, // 2998,  01101101 11010000 00000000 00000000
    0, // 2999,  11101101 11010000 00000000 00000000
    3, // 3000,  00011101 11010000 00000000 00000000
    0, // 3001,  10011101 11010000 00000000 00000000
    1, // 3002,  01011101 11010000 00000000 00000000
    0, // 3003,  11011101 11010000 00000000 00000000
    2, // 3004,  00111101 11010000 00000000 00000000
    0, // 3005,  10111101 11010000 00000000 00000000
    1, // 3006,  01111101 11010000 00000000 00000000
    0, // 3007,  11111101 11010000 00000000 00000000
    6, // 3008,  00000011 11010000 00000000 00000000
    0, // 3009,  10000011 11010000 00000000 00000000
    1, // 3010,  01000011 11010000 00000000 00000000
    0, // 3011,  11000011 11010000 00000000 00000000
    2, // 3012,  00100011 11010000 00000000 00000000
    0, // 3013,  10100011 11010000 00000000 00000000
    1, // 3014,  01100011 11010000 00000000 00000000
    0, // 3015,  11100011 11010000 00000000 00000000
    3, // 3016,  00010011 11010000 00000000 00000000
    0, // 3017,  10010011 11010000 00000000 00000000
    1, // 3018,  01010011 11010000 00000000 00000000
    0, // 3019,  11010011 11010000 00000000 00000000
    2, // 3020,  00110011 11010000 00000000 00000000
    0, // 3021,  10110011 11010000 00000000 00000000
    1, // 3022,  01110011 11010000 00000000 00000000
    0, // 3023,  11110011 11010000 00000000 00000000
    4, // 3024,  00001011 11010000 00000000 00000000
    0, // 3025,  10001011 11010000 00000000 00000000
    1, // 3026,  01001011 11010000 00000000 00000000
    0, // 3027,  11001011 11010000 00000000 00000000
    2, // 3028,  00101011 11010000 00000000 00000000
    0, // 3029,  10101011 11010000 00000000 00000000
    1, // 3030,  01101011 11010000 00000000 00000000
    0, // 3031,  11101011 11010000 00000000 00000000
    3, // 3032,  00011011 11010000 00000000 00000000
    0, // 3033,  10011011 11010000 00000000 00000000
    1, // 3034,  01011011 11010000 00000000 00000000
    0, // 3035,  11011011 11010000 00000000 00000000
    2, // 3036,  00111011 11010000 00000000 00000000
    0, // 3037,  10111011 11010000 00000000 00000000
    1, // 3038,  01111011 11010000 00000000 00000000
    0, // 3039,  11111011 11010000 00000000 00000000
    5, // 3040,  00000111 11010000 00000000 00000000
    0, // 3041,  10000111 11010000 00000000 00000000
    1, // 3042,  01000111 11010000 00000000 00000000
    0, // 3043,  11000111 11010000 00000000 00000000
    2, // 3044,  00100111 11010000 00000000 00000000
    0, // 3045,  10100111 11010000 00000000 00000000
    1, // 3046,  01100111 11010000 00000000 00000000
    0, // 3047,  11100111 11010000 00000000 00000000
    3, // 3048,  00010111 11010000 00000000 00000000
    0, // 3049,  10010111 11010000 00000000 00000000
    1, // 3050,  01010111 11010000 00000000 00000000
    0, // 3051,  11010111 11010000 00000000 00000000
    2, // 3052,  00110111 11010000 00000000 00000000
    0, // 3053,  10110111 11010000 00000000 00000000
    1, // 3054,  01110111 11010000 00000000 00000000
    0, // 3055,  11110111 11010000 00000000 00000000
    4, // 3056,  00001111 11010000 00000000 00000000
    0, // 3057,  10001111 11010000 00000000 00000000
    1, // 3058,  01001111 11010000 00000000 00000000
    0, // 3059,  11001111 11010000 00000000 00000000
    2, // 3060,  00101111 11010000 00000000 00000000
    0, // 3061,  10101111 11010000 00000000 00000000
    1, // 3062,  01101111 11010000 00000000 00000000
    0, // 3063,  11101111 11010000 00000000 00000000
    3, // 3064,  00011111 11010000 00000000 00000000
    0, // 3065,  10011111 11010000 00000000 00000000
    1, // 3066,  01011111 11010000 00000000 00000000
    0, // 3067,  11011111 11010000 00000000 00000000
    2, // 3068,  00111111 11010000 00000000 00000000
    0, // 3069,  10111111 11010000 00000000 00000000
    1, // 3070,  01111111 11010000 00000000 00000000
    0, // 3071,  11111111 11010000 00000000 00000000
    10, // 3072,  00000000 00110000 00000000 00000000
    0, // 3073,  10000000 00110000 00000000 00000000
    1, // 3074,  01000000 00110000 00000000 00000000
    0, // 3075,  11000000 00110000 00000000 00000000
    2, // 3076,  00100000 00110000 00000000 00000000
    0, // 3077,  10100000 00110000 00000000 00000000
    1, // 3078,  01100000 00110000 00000000 00000000
    0, // 3079,  11100000 00110000 00000000 00000000
    3, // 3080,  00010000 00110000 00000000 00000000
    0, // 3081,  10010000 00110000 00000000 00000000
    1, // 3082,  01010000 00110000 00000000 00000000
    0, // 3083,  11010000 00110000 00000000 00000000
    2, // 3084,  00110000 00110000 00000000 00000000
    0, // 3085,  10110000 00110000 00000000 00000000
    1, // 3086,  01110000 00110000 00000000 00000000
    0, // 3087,  11110000 00110000 00000000 00000000
    4, // 3088,  00001000 00110000 00000000 00000000
    0, // 3089,  10001000 00110000 00000000 00000000
    1, // 3090,  01001000 00110000 00000000 00000000
    0, // 3091,  11001000 00110000 00000000 00000000
    2, // 3092,  00101000 00110000 00000000 00000000
    0, // 3093,  10101000 00110000 00000000 00000000
    1, // 3094,  01101000 00110000 00000000 00000000
    0, // 3095,  11101000 00110000 00000000 00000000
    3, // 3096,  00011000 00110000 00000000 00000000
    0, // 3097,  10011000 00110000 00000000 00000000
    1, // 3098,  01011000 00110000 00000000 00000000
    0, // 3099,  11011000 00110000 00000000 00000000
    2, // 3100,  00111000 00110000 00000000 00000000
    0, // 3101,  10111000 00110000 00000000 00000000
    1, // 3102,  01111000 00110000 00000000 00000000
    0, // 3103,  11111000 00110000 00000000 00000000
    5, // 3104,  00000100 00110000 00000000 00000000
    0, // 3105,  10000100 00110000 00000000 00000000
    1, // 3106,  01000100 00110000 00000000 00000000
    0, // 3107,  11000100 00110000 00000000 00000000
    2, // 3108,  00100100 00110000 00000000 00000000
    0, // 3109,  10100100 00110000 00000000 00000000
    1, // 3110,  01100100 00110000 00000000 00000000
    0, // 3111,  11100100 00110000 00000000 00000000
    3, // 3112,  00010100 00110000 00000000 00000000
    0, // 3113,  10010100 00110000 00000000 00000000
    1, // 3114,  01010100 00110000 00000000 00000000
    0, // 3115,  11010100 00110000 00000000 00000000
    2, // 3116,  00110100 00110000 00000000 00000000
    0, // 3117,  10110100 00110000 00000000 00000000
    1, // 3118,  01110100 00110000 00000000 00000000
    0, // 3119,  11110100 00110000 00000000 00000000
    4, // 3120,  00001100 00110000 00000000 00000000
    0, // 3121,  10001100 00110000 00000000 00000000
    1, // 3122,  01001100 00110000 00000000 00000000
    0, // 3123,  11001100 00110000 00000000 00000000
    2, // 3124,  00101100 00110000 00000000 00000000
    0, // 3125,  10101100 00110000 00000000 00000000
    1, // 3126,  01101100 00110000 00000000 00000000
    0, // 3127,  11101100 00110000 00000000 00000000
    3, // 3128,  00011100 00110000 00000000 00000000
    0, // 3129,  10011100 00110000 00000000 00000000
    1, // 3130,  01011100 00110000 00000000 00000000
    0, // 3131,  11011100 00110000 00000000 00000000
    2, // 3132,  00111100 00110000 00000000 00000000
    0, // 3133,  10111100 00110000 00000000 00000000
    1, // 3134,  01111100 00110000 00000000 00000000
    0, // 3135,  11111100 00110000 00000000 00000000
    6, // 3136,  00000010 00110000 00000000 00000000
    0, // 3137,  10000010 00110000 00000000 00000000
    1, // 3138,  01000010 00110000 00000000 00000000
    0, // 3139,  11000010 00110000 00000000 00000000
    2, // 3140,  00100010 00110000 00000000 00000000
    0, // 3141,  10100010 00110000 00000000 00000000
    1, // 3142,  01100010 00110000 00000000 00000000
    0, // 3143,  11100010 00110000 00000000 00000000
    3, // 3144,  00010010 00110000 00000000 00000000
    0, // 3145,  10010010 00110000 00000000 00000000
    1, // 3146,  01010010 00110000 00000000 00000000
    0, // 3147,  11010010 00110000 00000000 00000000
    2, // 3148,  00110010 00110000 00000000 00000000
    0, // 3149,  10110010 00110000 00000000 00000000
    1, // 3150,  01110010 00110000 00000000 00000000
    0, // 3151,  11110010 00110000 00000000 00000000
    4, // 3152,  00001010 00110000 00000000 00000000
    0, // 3153,  10001010 00110000 00000000 00000000
    1, // 3154,  01001010 00110000 00000000 00000000
    0, // 3155,  11001010 00110000 00000000 00000000
    2, // 3156,  00101010 00110000 00000000 00000000
    0, // 3157,  10101010 00110000 00000000 00000000
    1, // 3158,  01101010 00110000 00000000 00000000
    0, // 3159,  11101010 00110000 00000000 00000000
    3, // 3160,  00011010 00110000 00000000 00000000
    0, // 3161,  10011010 00110000 00000000 00000000
    1, // 3162,  01011010 00110000 00000000 00000000
    0, // 3163,  11011010 00110000 00000000 00000000
    2, // 3164,  00111010 00110000 00000000 00000000
    0, // 3165,  10111010 00110000 00000000 00000000
    1, // 3166,  01111010 00110000 00000000 00000000
    0, // 3167,  11111010 00110000 00000000 00000000
    5, // 3168,  00000110 00110000 00000000 00000000
    0, // 3169,  10000110 00110000 00000000 00000000
    1, // 3170,  01000110 00110000 00000000 00000000
    0, // 3171,  11000110 00110000 00000000 00000000
    2, // 3172,  00100110 00110000 00000000 00000000
    0, // 3173,  10100110 00110000 00000000 00000000
    1, // 3174,  01100110 00110000 00000000 00000000
    0, // 3175,  11100110 00110000 00000000 00000000
    3, // 3176,  00010110 00110000 00000000 00000000
    0, // 3177,  10010110 00110000 00000000 00000000
    1, // 3178,  01010110 00110000 00000000 00000000
    0, // 3179,  11010110 00110000 00000000 00000000
    2, // 3180,  00110110 00110000 00000000 00000000
    0, // 3181,  10110110 00110000 00000000 00000000
    1, // 3182,  01110110 00110000 00000000 00000000
    0, // 3183,  11110110 00110000 00000000 00000000
    4, // 3184,  00001110 00110000 00000000 00000000
    0, // 3185,  10001110 00110000 00000000 00000000
    1, // 3186,  01001110 00110000 00000000 00000000
    0, // 3187,  11001110 00110000 00000000 00000000
    2, // 3188,  00101110 00110000 00000000 00000000
    0, // 3189,  10101110 00110000 00000000 00000000
    1, // 3190,  01101110 00110000 00000000 00000000
    0, // 3191,  11101110 00110000 00000000 00000000
    3, // 3192,  00011110 00110000 00000000 00000000
    0, // 3193,  10011110 00110000 00000000 00000000
    1, // 3194,  01011110 00110000 00000000 00000000
    0, // 3195,  11011110 00110000 00000000 00000000
    2, // 3196,  00111110 00110000 00000000 00000000
    0, // 3197,  10111110 00110000 00000000 00000000
    1, // 3198,  01111110 00110000 00000000 00000000
    0, // 3199,  11111110 00110000 00000000 00000000
    7, // 3200,  00000001 00110000 00000000 00000000
    0, // 3201,  10000001 00110000 00000000 00000000
    1, // 3202,  01000001 00110000 00000000 00000000
    0, // 3203,  11000001 00110000 00000000 00000000
    2, // 3204,  00100001 00110000 00000000 00000000
    0, // 3205,  10100001 00110000 00000000 00000000
    1, // 3206,  01100001 00110000 00000000 00000000
    0, // 3207,  11100001 00110000 00000000 00000000
    3, // 3208,  00010001 00110000 00000000 00000000
    0, // 3209,  10010001 00110000 00000000 00000000
    1, // 3210,  01010001 00110000 00000000 00000000
    0, // 3211,  11010001 00110000 00000000 00000000
    2, // 3212,  00110001 00110000 00000000 00000000
    0, // 3213,  10110001 00110000 00000000 00000000
    1, // 3214,  01110001 00110000 00000000 00000000
    0, // 3215,  11110001 00110000 00000000 00000000
    4, // 3216,  00001001 00110000 00000000 00000000
    0, // 3217,  10001001 00110000 00000000 00000000
    1, // 3218,  01001001 00110000 00000000 00000000
    0, // 3219,  11001001 00110000 00000000 00000000
    2, // 3220,  00101001 00110000 00000000 00000000
    0, // 3221,  10101001 00110000 00000000 00000000
    1, // 3222,  01101001 00110000 00000000 00000000
    0, // 3223,  11101001 00110000 00000000 00000000
    3, // 3224,  00011001 00110000 00000000 00000000
    0, // 3225,  10011001 00110000 00000000 00000000
    1, // 3226,  01011001 00110000 00000000 00000000
    0, // 3227,  11011001 00110000 00000000 00000000
    2, // 3228,  00111001 00110000 00000000 00000000
    0, // 3229,  10111001 00110000 00000000 00000000
    1, // 3230,  01111001 00110000 00000000 00000000
    0, // 3231,  11111001 00110000 00000000 00000000
    5, // 3232,  00000101 00110000 00000000 00000000
    0, // 3233,  10000101 00110000 00000000 00000000
    1, // 3234,  01000101 00110000 00000000 00000000
    0, // 3235,  11000101 00110000 00000000 00000000
    2, // 3236,  00100101 00110000 00000000 00000000
    0, // 3237,  10100101 00110000 00000000 00000000
    1, // 3238,  01100101 00110000 00000000 00000000
    0, // 3239,  11100101 00110000 00000000 00000000
    3, // 3240,  00010101 00110000 00000000 00000000
    0, // 3241,  10010101 00110000 00000000 00000000
    1, // 3242,  01010101 00110000 00000000 00000000
    0, // 3243,  11010101 00110000 00000000 00000000
    2, // 3244,  00110101 00110000 00000000 00000000
    0, // 3245,  10110101 00110000 00000000 00000000
    1, // 3246,  01110101 00110000 00000000 00000000
    0, // 3247,  11110101 00110000 00000000 00000000
    4, // 3248,  00001101 00110000 00000000 00000000
    0, // 3249,  10001101 00110000 00000000 00000000
    1, // 3250,  01001101 00110000 00000000 00000000
    0, // 3251,  11001101 00110000 00000000 00000000
    2, // 3252,  00101101 00110000 00000000 00000000
    0, // 3253,  10101101 00110000 00000000 00000000
    1, // 3254,  01101101 00110000 00000000 00000000
    0, // 3255,  11101101 00110000 00000000 00000000
    3, // 3256,  00011101 00110000 00000000 00000000
    0, // 3257,  10011101 00110000 00000000 00000000
    1, // 3258,  01011101 00110000 00000000 00000000
    0, // 3259,  11011101 00110000 00000000 00000000
    2, // 3260,  00111101 00110000 00000000 00000000
    0, // 3261,  10111101 00110000 00000000 00000000
    1, // 3262,  01111101 00110000 00000000 00000000
    0, // 3263,  11111101 00110000 00000000 00000000
    6, // 3264,  00000011 00110000 00000000 00000000
    0, // 3265,  10000011 00110000 00000000 00000000
    1, // 3266,  01000011 00110000 00000000 00000000
    0, // 3267,  11000011 00110000 00000000 00000000
    2, // 3268,  00100011 00110000 00000000 00000000
    0, // 3269,  10100011 00110000 00000000 00000000
    1, // 3270,  01100011 00110000 00000000 00000000
    0, // 3271,  11100011 00110000 00000000 00000000
    3, // 3272,  00010011 00110000 00000000 00000000
    0, // 3273,  10010011 00110000 00000000 00000000
    1, // 3274,  01010011 00110000 00000000 00000000
    0, // 3275,  11010011 00110000 00000000 00000000
    2, // 3276,  00110011 00110000 00000000 00000000
    0, // 3277,  10110011 00110000 00000000 00000000
    1, // 3278,  01110011 00110000 00000000 00000000
    0, // 3279,  11110011 00110000 00000000 00000000
    4, // 3280,  00001011 00110000 00000000 00000000
    0, // 3281,  10001011 00110000 00000000 00000000
    1, // 3282,  01001011 00110000 00000000 00000000
    0, // 3283,  11001011 00110000 00000000 00000000
    2, // 3284,  00101011 00110000 00000000 00000000
    0, // 3285,  10101011 00110000 00000000 00000000
    1, // 3286,  01101011 00110000 00000000 00000000
    0, // 3287,  11101011 00110000 00000000 00000000
    3, // 3288,  00011011 00110000 00000000 00000000
    0, // 3289,  10011011 00110000 00000000 00000000
    1, // 3290,  01011011 00110000 00000000 00000000
    0, // 3291,  11011011 00110000 00000000 00000000
    2, // 3292,  00111011 00110000 00000000 00000000
    0, // 3293,  10111011 00110000 00000000 00000000
    1, // 3294,  01111011 00110000 00000000 00000000
    0, // 3295,  11111011 00110000 00000000 00000000
    5, // 3296,  00000111 00110000 00000000 00000000
    0, // 3297,  10000111 00110000 00000000 00000000
    1, // 3298,  01000111 00110000 00000000 00000000
    0, // 3299,  11000111 00110000 00000000 00000000
    2, // 3300,  00100111 00110000 00000000 00000000
    0, // 3301,  10100111 00110000 00000000 00000000
    1, // 3302,  01100111 00110000 00000000 00000000
    0, // 3303,  11100111 00110000 00000000 00000000
    3, // 3304,  00010111 00110000 00000000 00000000
    0, // 3305,  10010111 00110000 00000000 00000000
    1, // 3306,  01010111 00110000 00000000 00000000
    0, // 3307,  11010111 00110000 00000000 00000000
    2, // 3308,  00110111 00110000 00000000 00000000
    0, // 3309,  10110111 00110000 00000000 00000000
    1, // 3310,  01110111 00110000 00000000 00000000
    0, // 3311,  11110111 00110000 00000000 00000000
    4, // 3312,  00001111 00110000 00000000 00000000
    0, // 3313,  10001111 00110000 00000000 00000000
    1, // 3314,  01001111 00110000 00000000 00000000
    0, // 3315,  11001111 00110000 00000000 00000000
    2, // 3316,  00101111 00110000 00000000 00000000
    0, // 3317,  10101111 00110000 00000000 00000000
    1, // 3318,  01101111 00110000 00000000 00000000
    0, // 3319,  11101111 00110000 00000000 00000000
    3, // 3320,  00011111 00110000 00000000 00000000
    0, // 3321,  10011111 00110000 00000000 00000000
    1, // 3322,  01011111 00110000 00000000 00000000
    0, // 3323,  11011111 00110000 00000000 00000000
    2, // 3324,  00111111 00110000 00000000 00000000
    0, // 3325,  10111111 00110000 00000000 00000000
    1, // 3326,  01111111 00110000 00000000 00000000
    0, // 3327,  11111111 00110000 00000000 00000000
    8, // 3328,  00000000 10110000 00000000 00000000
    0, // 3329,  10000000 10110000 00000000 00000000
    1, // 3330,  01000000 10110000 00000000 00000000
    0, // 3331,  11000000 10110000 00000000 00000000
    2, // 3332,  00100000 10110000 00000000 00000000
    0, // 3333,  10100000 10110000 00000000 00000000
    1, // 3334,  01100000 10110000 00000000 00000000
    0, // 3335,  11100000 10110000 00000000 00000000
    3, // 3336,  00010000 10110000 00000000 00000000
    0, // 3337,  10010000 10110000 00000000 00000000
    1, // 3338,  01010000 10110000 00000000 00000000
    0, // 3339,  11010000 10110000 00000000 00000000
    2, // 3340,  00110000 10110000 00000000 00000000
    0, // 3341,  10110000 10110000 00000000 00000000
    1, // 3342,  01110000 10110000 00000000 00000000
    0, // 3343,  11110000 10110000 00000000 00000000
    4, // 3344,  00001000 10110000 00000000 00000000
    0, // 3345,  10001000 10110000 00000000 00000000
    1, // 3346,  01001000 10110000 00000000 00000000
    0, // 3347,  11001000 10110000 00000000 00000000
    2, // 3348,  00101000 10110000 00000000 00000000
    0, // 3349,  10101000 10110000 00000000 00000000
    1, // 3350,  01101000 10110000 00000000 00000000
    0, // 3351,  11101000 10110000 00000000 00000000
    3, // 3352,  00011000 10110000 00000000 00000000
    0, // 3353,  10011000 10110000 00000000 00000000
    1, // 3354,  01011000 10110000 00000000 00000000
    0, // 3355,  11011000 10110000 00000000 00000000
    2, // 3356,  00111000 10110000 00000000 00000000
    0, // 3357,  10111000 10110000 00000000 00000000
    1, // 3358,  01111000 10110000 00000000 00000000
    0, // 3359,  11111000 10110000 00000000 00000000
    5, // 3360,  00000100 10110000 00000000 00000000
    0, // 3361,  10000100 10110000 00000000 00000000
    1, // 3362,  01000100 10110000 00000000 00000000
    0, // 3363,  11000100 10110000 00000000 00000000
    2, // 3364,  00100100 10110000 00000000 00000000
    0, // 3365,  10100100 10110000 00000000 00000000
    1, // 3366,  01100100 10110000 00000000 00000000
    0, // 3367,  11100100 10110000 00000000 00000000
    3, // 3368,  00010100 10110000 00000000 00000000
    0, // 3369,  10010100 10110000 00000000 00000000
    1, // 3370,  01010100 10110000 00000000 00000000
    0, // 3371,  11010100 10110000 00000000 00000000
    2, // 3372,  00110100 10110000 00000000 00000000
    0, // 3373,  10110100 10110000 00000000 00000000
    1, // 3374,  01110100 10110000 00000000 00000000
    0, // 3375,  11110100 10110000 00000000 00000000
    4, // 3376,  00001100 10110000 00000000 00000000
    0, // 3377,  10001100 10110000 00000000 00000000
    1, // 3378,  01001100 10110000 00000000 00000000
    0, // 3379,  11001100 10110000 00000000 00000000
    2, // 3380,  00101100 10110000 00000000 00000000
    0, // 3381,  10101100 10110000 00000000 00000000
    1, // 3382,  01101100 10110000 00000000 00000000
    0, // 3383,  11101100 10110000 00000000 00000000
    3, // 3384,  00011100 10110000 00000000 00000000
    0, // 3385,  10011100 10110000 00000000 00000000
    1, // 3386,  01011100 10110000 00000000 00000000
    0, // 3387,  11011100 10110000 00000000 00000000
    2, // 3388,  00111100 10110000 00000000 00000000
    0, // 3389,  10111100 10110000 00000000 00000000
    1, // 3390,  01111100 10110000 00000000 00000000
    0, // 3391,  11111100 10110000 00000000 00000000
    6, // 3392,  00000010 10110000 00000000 00000000
    0, // 3393,  10000010 10110000 00000000 00000000
    1, // 3394,  01000010 10110000 00000000 00000000
    0, // 3395,  11000010 10110000 00000000 00000000
    2, // 3396,  00100010 10110000 00000000 00000000
    0, // 3397,  10100010 10110000 00000000 00000000
    1, // 3398,  01100010 10110000 00000000 00000000
    0, // 3399,  11100010 10110000 00000000 00000000
    3, // 3400,  00010010 10110000 00000000 00000000
    0, // 3401,  10010010 10110000 00000000 00000000
    1, // 3402,  01010010 10110000 00000000 00000000
    0, // 3403,  11010010 10110000 00000000 00000000
    2, // 3404,  00110010 10110000 00000000 00000000
    0, // 3405,  10110010 10110000 00000000 00000000
    1, // 3406,  01110010 10110000 00000000 00000000
    0, // 3407,  11110010 10110000 00000000 00000000
    4, // 3408,  00001010 10110000 00000000 00000000
    0, // 3409,  10001010 10110000 00000000 00000000
    1, // 3410,  01001010 10110000 00000000 00000000
    0, // 3411,  11001010 10110000 00000000 00000000
    2, // 3412,  00101010 10110000 00000000 00000000
    0, // 3413,  10101010 10110000 00000000 00000000
    1, // 3414,  01101010 10110000 00000000 00000000
    0, // 3415,  11101010 10110000 00000000 00000000
    3, // 3416,  00011010 10110000 00000000 00000000
    0, // 3417,  10011010 10110000 00000000 00000000
    1, // 3418,  01011010 10110000 00000000 00000000
    0, // 3419,  11011010 10110000 00000000 00000000
    2, // 3420,  00111010 10110000 00000000 00000000
    0, // 3421,  10111010 10110000 00000000 00000000
    1, // 3422,  01111010 10110000 00000000 00000000
    0, // 3423,  11111010 10110000 00000000 00000000
    5, // 3424,  00000110 10110000 00000000 00000000
    0, // 3425,  10000110 10110000 00000000 00000000
    1, // 3426,  01000110 10110000 00000000 00000000
    0, // 3427,  11000110 10110000 00000000 00000000
    2, // 3428,  00100110 10110000 00000000 00000000
    0, // 3429,  10100110 10110000 00000000 00000000
    1, // 3430,  01100110 10110000 00000000 00000000
    0, // 3431,  11100110 10110000 00000000 00000000
    3, // 3432,  00010110 10110000 00000000 00000000
    0, // 3433,  10010110 10110000 00000000 00000000
    1, // 3434,  01010110 10110000 00000000 00000000
    0, // 3435,  11010110 10110000 00000000 00000000
    2, // 3436,  00110110 10110000 00000000 00000000
    0, // 3437,  10110110 10110000 00000000 00000000
    1, // 3438,  01110110 10110000 00000000 00000000
    0, // 3439,  11110110 10110000 00000000 00000000
    4, // 3440,  00001110 10110000 00000000 00000000
    0, // 3441,  10001110 10110000 00000000 00000000
    1, // 3442,  01001110 10110000 00000000 00000000
    0, // 3443,  11001110 10110000 00000000 00000000
    2, // 3444,  00101110 10110000 00000000 00000000
    0, // 3445,  10101110 10110000 00000000 00000000
    1, // 3446,  01101110 10110000 00000000 00000000
    0, // 3447,  11101110 10110000 00000000 00000000
    3, // 3448,  00011110 10110000 00000000 00000000
    0, // 3449,  10011110 10110000 00000000 00000000
    1, // 3450,  01011110 10110000 00000000 00000000
    0, // 3451,  11011110 10110000 00000000 00000000
    2, // 3452,  00111110 10110000 00000000 00000000
    0, // 3453,  10111110 10110000 00000000 00000000
    1, // 3454,  01111110 10110000 00000000 00000000
    0, // 3455,  11111110 10110000 00000000 00000000
    7, // 3456,  00000001 10110000 00000000 00000000
    0, // 3457,  10000001 10110000 00000000 00000000
    1, // 3458,  01000001 10110000 00000000 00000000
    0, // 3459,  11000001 10110000 00000000 00000000
    2, // 3460,  00100001 10110000 00000000 00000000
    0, // 3461,  10100001 10110000 00000000 00000000
    1, // 3462,  01100001 10110000 00000000 00000000
    0, // 3463,  11100001 10110000 00000000 00000000
    3, // 3464,  00010001 10110000 00000000 00000000
    0, // 3465,  10010001 10110000 00000000 00000000
    1, // 3466,  01010001 10110000 00000000 00000000
    0, // 3467,  11010001 10110000 00000000 00000000
    2, // 3468,  00110001 10110000 00000000 00000000
    0, // 3469,  10110001 10110000 00000000 00000000
    1, // 3470,  01110001 10110000 00000000 00000000
    0, // 3471,  11110001 10110000 00000000 00000000
    4, // 3472,  00001001 10110000 00000000 00000000
    0, // 3473,  10001001 10110000 00000000 00000000
    1, // 3474,  01001001 10110000 00000000 00000000
    0, // 3475,  11001001 10110000 00000000 00000000
    2, // 3476,  00101001 10110000 00000000 00000000
    0, // 3477,  10101001 10110000 00000000 00000000
    1, // 3478,  01101001 10110000 00000000 00000000
    0, // 3479,  11101001 10110000 00000000 00000000
    3, // 3480,  00011001 10110000 00000000 00000000
    0, // 3481,  10011001 10110000 00000000 00000000
    1, // 3482,  01011001 10110000 00000000 00000000
    0, // 3483,  11011001 10110000 00000000 00000000
    2, // 3484,  00111001 10110000 00000000 00000000
    0, // 3485,  10111001 10110000 00000000 00000000
    1, // 3486,  01111001 10110000 00000000 00000000
    0, // 3487,  11111001 10110000 00000000 00000000
    5, // 3488,  00000101 10110000 00000000 00000000
    0, // 3489,  10000101 10110000 00000000 00000000
    1, // 3490,  01000101 10110000 00000000 00000000
    0, // 3491,  11000101 10110000 00000000 00000000
    2, // 3492,  00100101 10110000 00000000 00000000
    0, // 3493,  10100101 10110000 00000000 00000000
    1, // 3494,  01100101 10110000 00000000 00000000
    0, // 3495,  11100101 10110000 00000000 00000000
    3, // 3496,  00010101 10110000 00000000 00000000
    0, // 3497,  10010101 10110000 00000000 00000000
    1, // 3498,  01010101 10110000 00000000 00000000
    0, // 3499,  11010101 10110000 00000000 00000000
    2, // 3500,  00110101 10110000 00000000 00000000
    0, // 3501,  10110101 10110000 00000000 00000000
    1, // 3502,  01110101 10110000 00000000 00000000
    0, // 3503,  11110101 10110000 00000000 00000000
    4, // 3504,  00001101 10110000 00000000 00000000
    0, // 3505,  10001101 10110000 00000000 00000000
    1, // 3506,  01001101 10110000 00000000 00000000
    0, // 3507,  11001101 10110000 00000000 00000000
    2, // 3508,  00101101 10110000 00000000 00000000
    0, // 3509,  10101101 10110000 00000000 00000000
    1, // 3510,  01101101 10110000 00000000 00000000
    0, // 3511,  11101101 10110000 00000000 00000000
    3, // 3512,  00011101 10110000 00000000 00000000
    0, // 3513,  10011101 10110000 00000000 00000000
    1, // 3514,  01011101 10110000 00000000 00000000
    0, // 3515,  11011101 10110000 00000000 00000000
    2, // 3516,  00111101 10110000 00000000 00000000
    0, // 3517,  10111101 10110000 00000000 00000000
    1, // 3518,  01111101 10110000 00000000 00000000
    0, // 3519,  11111101 10110000 00000000 00000000
    6, // 3520,  00000011 10110000 00000000 00000000
    0, // 3521,  10000011 10110000 00000000 00000000
    1, // 3522,  01000011 10110000 00000000 00000000
    0, // 3523,  11000011 10110000 00000000 00000000
    2, // 3524,  00100011 10110000 00000000 00000000
    0, // 3525,  10100011 10110000 00000000 00000000
    1, // 3526,  01100011 10110000 00000000 00000000
    0, // 3527,  11100011 10110000 00000000 00000000
    3, // 3528,  00010011 10110000 00000000 00000000
    0, // 3529,  10010011 10110000 00000000 00000000
    1, // 3530,  01010011 10110000 00000000 00000000
    0, // 3531,  11010011 10110000 00000000 00000000
    2, // 3532,  00110011 10110000 00000000 00000000
    0, // 3533,  10110011 10110000 00000000 00000000
    1, // 3534,  01110011 10110000 00000000 00000000
    0, // 3535,  11110011 10110000 00000000 00000000
    4, // 3536,  00001011 10110000 00000000 00000000
    0, // 3537,  10001011 10110000 00000000 00000000
    1, // 3538,  01001011 10110000 00000000 00000000
    0, // 3539,  11001011 10110000 00000000 00000000
    2, // 3540,  00101011 10110000 00000000 00000000
    0, // 3541,  10101011 10110000 00000000 00000000
    1, // 3542,  01101011 10110000 00000000 00000000
    0, // 3543,  11101011 10110000 00000000 00000000
    3, // 3544,  00011011 10110000 00000000 00000000
    0, // 3545,  10011011 10110000 00000000 00000000
    1, // 3546,  01011011 10110000 00000000 00000000
    0, // 3547,  11011011 10110000 00000000 00000000
    2, // 3548,  00111011 10110000 00000000 00000000
    0, // 3549,  10111011 10110000 00000000 00000000
    1, // 3550,  01111011 10110000 00000000 00000000
    0, // 3551,  11111011 10110000 00000000 00000000
    5, // 3552,  00000111 10110000 00000000 00000000
    0, // 3553,  10000111 10110000 00000000 00000000
    1, // 3554,  01000111 10110000 00000000 00000000
    0, // 3555,  11000111 10110000 00000000 00000000
    2, // 3556,  00100111 10110000 00000000 00000000
    0, // 3557,  10100111 10110000 00000000 00000000
    1, // 3558,  01100111 10110000 00000000 00000000
    0, // 3559,  11100111 10110000 00000000 00000000
    3, // 3560,  00010111 10110000 00000000 00000000
    0, // 3561,  10010111 10110000 00000000 00000000
    1, // 3562,  01010111 10110000 00000000 00000000
    0, // 3563,  11010111 10110000 00000000 00000000
    2, // 3564,  00110111 10110000 00000000 00000000
    0, // 3565,  10110111 10110000 00000000 00000000
    1, // 3566,  01110111 10110000 00000000 00000000
    0, // 3567,  11110111 10110000 00000000 00000000
    4, // 3568,  00001111 10110000 00000000 00000000
    0, // 3569,  10001111 10110000 00000000 00000000
    1, // 3570,  01001111 10110000 00000000 00000000
    0, // 3571,  11001111 10110000 00000000 00000000
    2, // 3572,  00101111 10110000 00000000 00000000
    0, // 3573,  10101111 10110000 00000000 00000000
    1, // 3574,  01101111 10110000 00000000 00000000
    0, // 3575,  11101111 10110000 00000000 00000000
    3, // 3576,  00011111 10110000 00000000 00000000
    0, // 3577,  10011111 10110000 00000000 00000000
    1, // 3578,  01011111 10110000 00000000 00000000
    0, // 3579,  11011111 10110000 00000000 00000000
    2, // 3580,  00111111 10110000 00000000 00000000
    0, // 3581,  10111111 10110000 00000000 00000000
    1, // 3582,  01111111 10110000 00000000 00000000
    0, // 3583,  11111111 10110000 00000000 00000000
    9, // 3584,  00000000 01110000 00000000 00000000
    0, // 3585,  10000000 01110000 00000000 00000000
    1, // 3586,  01000000 01110000 00000000 00000000
    0, // 3587,  11000000 01110000 00000000 00000000
    2, // 3588,  00100000 01110000 00000000 00000000
    0, // 3589,  10100000 01110000 00000000 00000000
    1, // 3590,  01100000 01110000 00000000 00000000
    0, // 3591,  11100000 01110000 00000000 00000000
    3, // 3592,  00010000 01110000 00000000 00000000
    0, // 3593,  10010000 01110000 00000000 00000000
    1, // 3594,  01010000 01110000 00000000 00000000
    0, // 3595,  11010000 01110000 00000000 00000000
    2, // 3596,  00110000 01110000 00000000 00000000
    0, // 3597,  10110000 01110000 00000000 00000000
    1, // 3598,  01110000 01110000 00000000 00000000
    0, // 3599,  11110000 01110000 00000000 00000000
    4, // 3600,  00001000 01110000 00000000 00000000
    0, // 3601,  10001000 01110000 00000000 00000000
    1, // 3602,  01001000 01110000 00000000 00000000
    0, // 3603,  11001000 01110000 00000000 00000000
    2, // 3604,  00101000 01110000 00000000 00000000
    0, // 3605,  10101000 01110000 00000000 00000000
    1, // 3606,  01101000 01110000 00000000 00000000
    0, // 3607,  11101000 01110000 00000000 00000000
    3, // 3608,  00011000 01110000 00000000 00000000
    0, // 3609,  10011000 01110000 00000000 00000000
    1, // 3610,  01011000 01110000 00000000 00000000
    0, // 3611,  11011000 01110000 00000000 00000000
    2, // 3612,  00111000 01110000 00000000 00000000
    0, // 3613,  10111000 01110000 00000000 00000000
    1, // 3614,  01111000 01110000 00000000 00000000
    0, // 3615,  11111000 01110000 00000000 00000000
    5, // 3616,  00000100 01110000 00000000 00000000
    0, // 3617,  10000100 01110000 00000000 00000000
    1, // 3618,  01000100 01110000 00000000 00000000
    0, // 3619,  11000100 01110000 00000000 00000000
    2, // 3620,  00100100 01110000 00000000 00000000
    0, // 3621,  10100100 01110000 00000000 00000000
    1, // 3622,  01100100 01110000 00000000 00000000
    0, // 3623,  11100100 01110000 00000000 00000000
    3, // 3624,  00010100 01110000 00000000 00000000
    0, // 3625,  10010100 01110000 00000000 00000000
    1, // 3626,  01010100 01110000 00000000 00000000
    0, // 3627,  11010100 01110000 00000000 00000000
    2, // 3628,  00110100 01110000 00000000 00000000
    0, // 3629,  10110100 01110000 00000000 00000000
    1, // 3630,  01110100 01110000 00000000 00000000
    0, // 3631,  11110100 01110000 00000000 00000000
    4, // 3632,  00001100 01110000 00000000 00000000
    0, // 3633,  10001100 01110000 00000000 00000000
    1, // 3634,  01001100 01110000 00000000 00000000
    0, // 3635,  11001100 01110000 00000000 00000000
    2, // 3636,  00101100 01110000 00000000 00000000
    0, // 3637,  10101100 01110000 00000000 00000000
    1, // 3638,  01101100 01110000 00000000 00000000
    0, // 3639,  11101100 01110000 00000000 00000000
    3, // 3640,  00011100 01110000 00000000 00000000
    0, // 3641,  10011100 01110000 00000000 00000000
    1, // 3642,  01011100 01110000 00000000 00000000
    0, // 3643,  11011100 01110000 00000000 00000000
    2, // 3644,  00111100 01110000 00000000 00000000
    0, // 3645,  10111100 01110000 00000000 00000000
    1, // 3646,  01111100 01110000 00000000 00000000
    0, // 3647,  11111100 01110000 00000000 00000000
    6, // 3648,  00000010 01110000 00000000 00000000
    0, // 3649,  10000010 01110000 00000000 00000000
    1, // 3650,  01000010 01110000 00000000 00000000
    0, // 3651,  11000010 01110000 00000000 00000000
    2, // 3652,  00100010 01110000 00000000 00000000
    0, // 3653,  10100010 01110000 00000000 00000000
    1, // 3654,  01100010 01110000 00000000 00000000
    0, // 3655,  11100010 01110000 00000000 00000000
    3, // 3656,  00010010 01110000 00000000 00000000
    0, // 3657,  10010010 01110000 00000000 00000000
    1, // 3658,  01010010 01110000 00000000 00000000
    0, // 3659,  11010010 01110000 00000000 00000000
    2, // 3660,  00110010 01110000 00000000 00000000
    0, // 3661,  10110010 01110000 00000000 00000000
    1, // 3662,  01110010 01110000 00000000 00000000
    0, // 3663,  11110010 01110000 00000000 00000000
    4, // 3664,  00001010 01110000 00000000 00000000
    0, // 3665,  10001010 01110000 00000000 00000000
    1, // 3666,  01001010 01110000 00000000 00000000
    0, // 3667,  11001010 01110000 00000000 00000000
    2, // 3668,  00101010 01110000 00000000 00000000
    0, // 3669,  10101010 01110000 00000000 00000000
    1, // 3670,  01101010 01110000 00000000 00000000
    0, // 3671,  11101010 01110000 00000000 00000000
    3, // 3672,  00011010 01110000 00000000 00000000
    0, // 3673,  10011010 01110000 00000000 00000000
    1, // 3674,  01011010 01110000 00000000 00000000
    0, // 3675,  11011010 01110000 00000000 00000000
    2, // 3676,  00111010 01110000 00000000 00000000
    0, // 3677,  10111010 01110000 00000000 00000000
    1, // 3678,  01111010 01110000 00000000 00000000
    0, // 3679,  11111010 01110000 00000000 00000000
    5, // 3680,  00000110 01110000 00000000 00000000
    0, // 3681,  10000110 01110000 00000000 00000000
    1, // 3682,  01000110 01110000 00000000 00000000
    0, // 3683,  11000110 01110000 00000000 00000000
    2, // 3684,  00100110 01110000 00000000 00000000
    0, // 3685,  10100110 01110000 00000000 00000000
    1, // 3686,  01100110 01110000 00000000 00000000
    0, // 3687,  11100110 01110000 00000000 00000000
    3, // 3688,  00010110 01110000 00000000 00000000
    0, // 3689,  10010110 01110000 00000000 00000000
    1, // 3690,  01010110 01110000 00000000 00000000
    0, // 3691,  11010110 01110000 00000000 00000000
    2, // 3692,  00110110 01110000 00000000 00000000
    0, // 3693,  10110110 01110000 00000000 00000000
    1, // 3694,  01110110 01110000 00000000 00000000
    0, // 3695,  11110110 01110000 00000000 00000000
    4, // 3696,  00001110 01110000 00000000 00000000
    0, // 3697,  10001110 01110000 00000000 00000000
    1, // 3698,  01001110 01110000 00000000 00000000
    0, // 3699,  11001110 01110000 00000000 00000000
    2, // 3700,  00101110 01110000 00000000 00000000
    0, // 3701,  10101110 01110000 00000000 00000000
    1, // 3702,  01101110 01110000 00000000 00000000
    0, // 3703,  11101110 01110000 00000000 00000000
    3, // 3704,  00011110 01110000 00000000 00000000
    0, // 3705,  10011110 01110000 00000000 00000000
    1, // 3706,  01011110 01110000 00000000 00000000
    0, // 3707,  11011110 01110000 00000000 00000000
    2, // 3708,  00111110 01110000 00000000 00000000
    0, // 3709,  10111110 01110000 00000000 00000000
    1, // 3710,  01111110 01110000 00000000 00000000
    0, // 3711,  11111110 01110000 00000000 00000000
    7, // 3712,  00000001 01110000 00000000 00000000
    0, // 3713,  10000001 01110000 00000000 00000000
    1, // 3714,  01000001 01110000 00000000 00000000
    0, // 3715,  11000001 01110000 00000000 00000000
    2, // 3716,  00100001 01110000 00000000 00000000
    0, // 3717,  10100001 01110000 00000000 00000000
    1, // 3718,  01100001 01110000 00000000 00000000
    0, // 3719,  11100001 01110000 00000000 00000000
    3, // 3720,  00010001 01110000 00000000 00000000
    0, // 3721,  10010001 01110000 00000000 00000000
    1, // 3722,  01010001 01110000 00000000 00000000
    0, // 3723,  11010001 01110000 00000000 00000000
    2, // 3724,  00110001 01110000 00000000 00000000
    0, // 3725,  10110001 01110000 00000000 00000000
    1, // 3726,  01110001 01110000 00000000 00000000
    0, // 3727,  11110001 01110000 00000000 00000000
    4, // 3728,  00001001 01110000 00000000 00000000
    0, // 3729,  10001001 01110000 00000000 00000000
    1, // 3730,  01001001 01110000 00000000 00000000
    0, // 3731,  11001001 01110000 00000000 00000000
    2, // 3732,  00101001 01110000 00000000 00000000
    0, // 3733,  10101001 01110000 00000000 00000000
    1, // 3734,  01101001 01110000 00000000 00000000
    0, // 3735,  11101001 01110000 00000000 00000000
    3, // 3736,  00011001 01110000 00000000 00000000
    0, // 3737,  10011001 01110000 00000000 00000000
    1, // 3738,  01011001 01110000 00000000 00000000
    0, // 3739,  11011001 01110000 00000000 00000000
    2, // 3740,  00111001 01110000 00000000 00000000
    0, // 3741,  10111001 01110000 00000000 00000000
    1, // 3742,  01111001 01110000 00000000 00000000
    0, // 3743,  11111001 01110000 00000000 00000000
    5, // 3744,  00000101 01110000 00000000 00000000
    0, // 3745,  10000101 01110000 00000000 00000000
    1, // 3746,  01000101 01110000 00000000 00000000
    0, // 3747,  11000101 01110000 00000000 00000000
    2, // 3748,  00100101 01110000 00000000 00000000
    0, // 3749,  10100101 01110000 00000000 00000000
    1, // 3750,  01100101 01110000 00000000 00000000
    0, // 3751,  11100101 01110000 00000000 00000000
    3, // 3752,  00010101 01110000 00000000 00000000
    0, // 3753,  10010101 01110000 00000000 00000000
    1, // 3754,  01010101 01110000 00000000 00000000
    0, // 3755,  11010101 01110000 00000000 00000000
    2, // 3756,  00110101 01110000 00000000 00000000
    0, // 3757,  10110101 01110000 00000000 00000000
    1, // 3758,  01110101 01110000 00000000 00000000
    0, // 3759,  11110101 01110000 00000000 00000000
    4, // 3760,  00001101 01110000 00000000 00000000
    0, // 3761,  10001101 01110000 00000000 00000000
    1, // 3762,  01001101 01110000 00000000 00000000
    0, // 3763,  11001101 01110000 00000000 00000000
    2, // 3764,  00101101 01110000 00000000 00000000
    0, // 3765,  10101101 01110000 00000000 00000000
    1, // 3766,  01101101 01110000 00000000 00000000
    0, // 3767,  11101101 01110000 00000000 00000000
    3, // 3768,  00011101 01110000 00000000 00000000
    0, // 3769,  10011101 01110000 00000000 00000000
    1, // 3770,  01011101 01110000 00000000 00000000
    0, // 3771,  11011101 01110000 00000000 00000000
    2, // 3772,  00111101 01110000 00000000 00000000
    0, // 3773,  10111101 01110000 00000000 00000000
    1, // 3774,  01111101 01110000 00000000 00000000
    0, // 3775,  11111101 01110000 00000000 00000000
    6, // 3776,  00000011 01110000 00000000 00000000
    0, // 3777,  10000011 01110000 00000000 00000000
    1, // 3778,  01000011 01110000 00000000 00000000
    0, // 3779,  11000011 01110000 00000000 00000000
    2, // 3780,  00100011 01110000 00000000 00000000
    0, // 3781,  10100011 01110000 00000000 00000000
    1, // 3782,  01100011 01110000 00000000 00000000
    0, // 3783,  11100011 01110000 00000000 00000000
    3, // 3784,  00010011 01110000 00000000 00000000
    0, // 3785,  10010011 01110000 00000000 00000000
    1, // 3786,  01010011 01110000 00000000 00000000
    0, // 3787,  11010011 01110000 00000000 00000000
    2, // 3788,  00110011 01110000 00000000 00000000
    0, // 3789,  10110011 01110000 00000000 00000000
    1, // 3790,  01110011 01110000 00000000 00000000
    0, // 3791,  11110011 01110000 00000000 00000000
    4, // 3792,  00001011 01110000 00000000 00000000
    0, // 3793,  10001011 01110000 00000000 00000000
    1, // 3794,  01001011 01110000 00000000 00000000
    0, // 3795,  11001011 01110000 00000000 00000000
    2, // 3796,  00101011 01110000 00000000 00000000
    0, // 3797,  10101011 01110000 00000000 00000000
    1, // 3798,  01101011 01110000 00000000 00000000
    0, // 3799,  11101011 01110000 00000000 00000000
    3, // 3800,  00011011 01110000 00000000 00000000
    0, // 3801,  10011011 01110000 00000000 00000000
    1, // 3802,  01011011 01110000 00000000 00000000
    0, // 3803,  11011011 01110000 00000000 00000000
    2, // 3804,  00111011 01110000 00000000 00000000
    0, // 3805,  10111011 01110000 00000000 00000000
    1, // 3806,  01111011 01110000 00000000 00000000
    0, // 3807,  11111011 01110000 00000000 00000000
    5, // 3808,  00000111 01110000 00000000 00000000
    0, // 3809,  10000111 01110000 00000000 00000000
    1, // 3810,  01000111 01110000 00000000 00000000
    0, // 3811,  11000111 01110000 00000000 00000000
    2, // 3812,  00100111 01110000 00000000 00000000
    0, // 3813,  10100111 01110000 00000000 00000000
    1, // 3814,  01100111 01110000 00000000 00000000
    0, // 3815,  11100111 01110000 00000000 00000000
    3, // 3816,  00010111 01110000 00000000 00000000
    0, // 3817,  10010111 01110000 00000000 00000000
    1, // 3818,  01010111 01110000 00000000 00000000
    0, // 3819,  11010111 01110000 00000000 00000000
    2, // 3820,  00110111 01110000 00000000 00000000
    0, // 3821,  10110111 01110000 00000000 00000000
    1, // 3822,  01110111 01110000 00000000 00000000
    0, // 3823,  11110111 01110000 00000000 00000000
    4, // 3824,  00001111 01110000 00000000 00000000
    0, // 3825,  10001111 01110000 00000000 00000000
    1, // 3826,  01001111 01110000 00000000 00000000
    0, // 3827,  11001111 01110000 00000000 00000000
    2, // 3828,  00101111 01110000 00000000 00000000
    0, // 3829,  10101111 01110000 00000000 00000000
    1, // 3830,  01101111 01110000 00000000 00000000
    0, // 3831,  11101111 01110000 00000000 00000000
    3, // 3832,  00011111 01110000 00000000 00000000
    0, // 3833,  10011111 01110000 00000000 00000000
    1, // 3834,  01011111 01110000 00000000 00000000
    0, // 3835,  11011111 01110000 00000000 00000000
    2, // 3836,  00111111 01110000 00000000 00000000
    0, // 3837,  10111111 01110000 00000000 00000000
    1, // 3838,  01111111 01110000 00000000 00000000
    0, // 3839,  11111111 01110000 00000000 00000000
    8, // 3840,  00000000 11110000 00000000 00000000
    0, // 3841,  10000000 11110000 00000000 00000000
    1, // 3842,  01000000 11110000 00000000 00000000
    0, // 3843,  11000000 11110000 00000000 00000000
    2, // 3844,  00100000 11110000 00000000 00000000
    0, // 3845,  10100000 11110000 00000000 00000000
    1, // 3846,  01100000 11110000 00000000 00000000
    0, // 3847,  11100000 11110000 00000000 00000000
    3, // 3848,  00010000 11110000 00000000 00000000
    0, // 3849,  10010000 11110000 00000000 00000000
    1, // 3850,  01010000 11110000 00000000 00000000
    0, // 3851,  11010000 11110000 00000000 00000000
    2, // 3852,  00110000 11110000 00000000 00000000
    0, // 3853,  10110000 11110000 00000000 00000000
    1, // 3854,  01110000 11110000 00000000 00000000
    0, // 3855,  11110000 11110000 00000000 00000000
    4, // 3856,  00001000 11110000 00000000 00000000
    0, // 3857,  10001000 11110000 00000000 00000000
    1, // 3858,  01001000 11110000 00000000 00000000
    0, // 3859,  11001000 11110000 00000000 00000000
    2, // 3860,  00101000 11110000 00000000 00000000
    0, // 3861,  10101000 11110000 00000000 00000000
    1, // 3862,  01101000 11110000 00000000 00000000
    0, // 3863,  11101000 11110000 00000000 00000000
    3, // 3864,  00011000 11110000 00000000 00000000
    0, // 3865,  10011000 11110000 00000000 00000000
    1, // 3866,  01011000 11110000 00000000 00000000
    0, // 3867,  11011000 11110000 00000000 00000000
    2, // 3868,  00111000 11110000 00000000 00000000
    0, // 3869,  10111000 11110000 00000000 00000000
    1, // 3870,  01111000 11110000 00000000 00000000
    0, // 3871,  11111000 11110000 00000000 00000000
    5, // 3872,  00000100 11110000 00000000 00000000
    0, // 3873,  10000100 11110000 00000000 00000000
    1, // 3874,  01000100 11110000 00000000 00000000
    0, // 3875,  11000100 11110000 00000000 00000000
    2, // 3876,  00100100 11110000 00000000 00000000
    0, // 3877,  10100100 11110000 00000000 00000000
    1, // 3878,  01100100 11110000 00000000 00000000
    0, // 3879,  11100100 11110000 00000000 00000000
    3, // 3880,  00010100 11110000 00000000 00000000
    0, // 3881,  10010100 11110000 00000000 00000000
    1, // 3882,  01010100 11110000 00000000 00000000
    0, // 3883,  11010100 11110000 00000000 00000000
    2, // 3884,  00110100 11110000 00000000 00000000
    0, // 3885,  10110100 11110000 00000000 00000000
    1, // 3886,  01110100 11110000 00000000 00000000
    0, // 3887,  11110100 11110000 00000000 00000000
    4, // 3888,  00001100 11110000 00000000 00000000
    0, // 3889,  10001100 11110000 00000000 00000000
    1, // 3890,  01001100 11110000 00000000 00000000
    0, // 3891,  11001100 11110000 00000000 00000000
    2, // 3892,  00101100 11110000 00000000 00000000
    0, // 3893,  10101100 11110000 00000000 00000000
    1, // 3894,  01101100 11110000 00000000 00000000
    0, // 3895,  11101100 11110000 00000000 00000000
    3, // 3896,  00011100 11110000 00000000 00000000
    0, // 3897,  10011100 11110000 00000000 00000000
    1, // 3898,  01011100 11110000 00000000 00000000
    0, // 3899,  11011100 11110000 00000000 00000000
    2, // 3900,  00111100 11110000 00000000 00000000
    0, // 3901,  10111100 11110000 00000000 00000000
    1, // 3902,  01111100 11110000 00000000 00000000
    0, // 3903,  11111100 11110000 00000000 00000000
    6, // 3904,  00000010 11110000 00000000 00000000
    0, // 3905,  10000010 11110000 00000000 00000000
    1, // 3906,  01000010 11110000 00000000 00000000
    0, // 3907,  11000010 11110000 00000000 00000000
    2, // 3908,  00100010 11110000 00000000 00000000
    0, // 3909,  10100010 11110000 00000000 00000000
    1, // 3910,  01100010 11110000 00000000 00000000
    0, // 3911,  11100010 11110000 00000000 00000000
    3, // 3912,  00010010 11110000 00000000 00000000
    0, // 3913,  10010010 11110000 00000000 00000000
    1, // 3914,  01010010 11110000 00000000 00000000
    0, // 3915,  11010010 11110000 00000000 00000000
    2, // 3916,  00110010 11110000 00000000 00000000
    0, // 3917,  10110010 11110000 00000000 00000000
    1, // 3918,  01110010 11110000 00000000 00000000
    0, // 3919,  11110010 11110000 00000000 00000000
    4, // 3920,  00001010 11110000 00000000 00000000
    0, // 3921,  10001010 11110000 00000000 00000000
    1, // 3922,  01001010 11110000 00000000 00000000
    0, // 3923,  11001010 11110000 00000000 00000000
    2, // 3924,  00101010 11110000 00000000 00000000
    0, // 3925,  10101010 11110000 00000000 00000000
    1, // 3926,  01101010 11110000 00000000 00000000
    0, // 3927,  11101010 11110000 00000000 00000000
    3, // 3928,  00011010 11110000 00000000 00000000
    0, // 3929,  10011010 11110000 00000000 00000000
    1, // 3930,  01011010 11110000 00000000 00000000
    0, // 3931,  11011010 11110000 00000000 00000000
    2, // 3932,  00111010 11110000 00000000 00000000
    0, // 3933,  10111010 11110000 00000000 00000000
    1, // 3934,  01111010 11110000 00000000 00000000
    0, // 3935,  11111010 11110000 00000000 00000000
    5, // 3936,  00000110 11110000 00000000 00000000
    0, // 3937,  10000110 11110000 00000000 00000000
    1, // 3938,  01000110 11110000 00000000 00000000
    0, // 3939,  11000110 11110000 00000000 00000000
    2, // 3940,  00100110 11110000 00000000 00000000
    0, // 3941,  10100110 11110000 00000000 00000000
    1, // 3942,  01100110 11110000 00000000 00000000
    0, // 3943,  11100110 11110000 00000000 00000000
    3, // 3944,  00010110 11110000 00000000 00000000
    0, // 3945,  10010110 11110000 00000000 00000000
    1, // 3946,  01010110 11110000 00000000 00000000
    0, // 3947,  11010110 11110000 00000000 00000000
    2, // 3948,  00110110 11110000 00000000 00000000
    0, // 3949,  10110110 11110000 00000000 00000000
    1, // 3950,  01110110 11110000 00000000 00000000
    0, // 3951,  11110110 11110000 00000000 00000000
    4, // 3952,  00001110 11110000 00000000 00000000
    0, // 3953,  10001110 11110000 00000000 00000000
    1, // 3954,  01001110 11110000 00000000 00000000
    0, // 3955,  11001110 11110000 00000000 00000000
    2, // 3956,  00101110 11110000 00000000 00000000
    0, // 3957,  10101110 11110000 00000000 00000000
    1, // 3958,  01101110 11110000 00000000 00000000
    0, // 3959,  11101110 11110000 00000000 00000000
    3, // 3960,  00011110 11110000 00000000 00000000
    0, // 3961,  10011110 11110000 00000000 00000000
    1, // 3962,  01011110 11110000 00000000 00000000
    0, // 3963,  11011110 11110000 00000000 00000000
    2, // 3964,  00111110 11110000 00000000 00000000
    0, // 3965,  10111110 11110000 00000000 00000000
    1, // 3966,  01111110 11110000 00000000 00000000
    0, // 3967,  11111110 11110000 00000000 00000000
    7, // 3968,  00000001 11110000 00000000 00000000
    0, // 3969,  10000001 11110000 00000000 00000000
    1, // 3970,  01000001 11110000 00000000 00000000
    0, // 3971,  11000001 11110000 00000000 00000000
    2, // 3972,  00100001 11110000 00000000 00000000
    0, // 3973,  10100001 11110000 00000000 00000000
    1, // 3974,  01100001 11110000 00000000 00000000
    0, // 3975,  11100001 11110000 00000000 00000000
    3, // 3976,  00010001 11110000 00000000 00000000
    0, // 3977,  10010001 11110000 00000000 00000000
    1, // 3978,  01010001 11110000 00000000 00000000
    0, // 3979,  11010001 11110000 00000000 00000000
    2, // 3980,  00110001 11110000 00000000 00000000
    0, // 3981,  10110001 11110000 00000000 00000000
    1, // 3982,  01110001 11110000 00000000 00000000
    0, // 3983,  11110001 11110000 00000000 00000000
    4, // 3984,  00001001 11110000 00000000 00000000
    0, // 3985,  10001001 11110000 00000000 00000000
    1, // 3986,  01001001 11110000 00000000 00000000
    0, // 3987,  11001001 11110000 00000000 00000000
    2, // 3988,  00101001 11110000 00000000 00000000
    0, // 3989,  10101001 11110000 00000000 00000000
    1, // 3990,  01101001 11110000 00000000 00000000
    0, // 3991,  11101001 11110000 00000000 00000000
    3, // 3992,  00011001 11110000 00000000 00000000
    0, // 3993,  10011001 11110000 00000000 00000000
    1, // 3994,  01011001 11110000 00000000 00000000
    0, // 3995,  11011001 11110000 00000000 00000000
    2, // 3996,  00111001 11110000 00000000 00000000
    0, // 3997,  10111001 11110000 00000000 00000000
    1, // 3998,  01111001 11110000 00000000 00000000
    0, // 3999,  11111001 11110000 00000000 00000000
    5, // 4000,  00000101 11110000 00000000 00000000
    0, // 4001,  10000101 11110000 00000000 00000000
    1, // 4002,  01000101 11110000 00000000 00000000
    0, // 4003,  11000101 11110000 00000000 00000000
    2, // 4004,  00100101 11110000 00000000 00000000
    0, // 4005,  10100101 11110000 00000000 00000000
    1, // 4006,  01100101 11110000 00000000 00000000
    0, // 4007,  11100101 11110000 00000000 00000000
    3, // 4008,  00010101 11110000 00000000 00000000
    0, // 4009,  10010101 11110000 00000000 00000000
    1, // 4010,  01010101 11110000 00000000 00000000
    0, // 4011,  11010101 11110000 00000000 00000000
    2, // 4012,  00110101 11110000 00000000 00000000
    0, // 4013,  10110101 11110000 00000000 00000000
    1, // 4014,  01110101 11110000 00000000 00000000
    0, // 4015,  11110101 11110000 00000000 00000000
    4, // 4016,  00001101 11110000 00000000 00000000
    0, // 4017,  10001101 11110000 00000000 00000000
    1, // 4018,  01001101 11110000 00000000 00000000
    0, // 4019,  11001101 11110000 00000000 00000000
    2, // 4020,  00101101 11110000 00000000 00000000
    0, // 4021,  10101101 11110000 00000000 00000000
    1, // 4022,  01101101 11110000 00000000 00000000
    0, // 4023,  11101101 11110000 00000000 00000000
    3, // 4024,  00011101 11110000 00000000 00000000
    0, // 4025,  10011101 11110000 00000000 00000000
    1, // 4026,  01011101 11110000 00000000 00000000
    0, // 4027,  11011101 11110000 00000000 00000000
    2, // 4028,  00111101 11110000 00000000 00000000
    0, // 4029,  10111101 11110000 00000000 00000000
    1, // 4030,  01111101 11110000 00000000 00000000
    0, // 4031,  11111101 11110000 00000000 00000000
    6, // 4032,  00000011 11110000 00000000 00000000
    0, // 4033,  10000011 11110000 00000000 00000000
    1, // 4034,  01000011 11110000 00000000 00000000
    0, // 4035,  11000011 11110000 00000000 00000000
    2, // 4036,  00100011 11110000 00000000 00000000
    0, // 4037,  10100011 11110000 00000000 00000000
    1, // 4038,  01100011 11110000 00000000 00000000
    0, // 4039,  11100011 11110000 00000000 00000000
    3, // 4040,  00010011 11110000 00000000 00000000
    0, // 4041,  10010011 11110000 00000000 00000000
    1, // 4042,  01010011 11110000 00000000 00000000
    0, // 4043,  11010011 11110000 00000000 00000000
    2, // 4044,  00110011 11110000 00000000 00000000
    0, // 4045,  10110011 11110000 00000000 00000000
    1, // 4046,  01110011 11110000 00000000 00000000
    0, // 4047,  11110011 11110000 00000000 00000000
    4, // 4048,  00001011 11110000 00000000 00000000
    0, // 4049,  10001011 11110000 00000000 00000000
    1, // 4050,  01001011 11110000 00000000 00000000
    0, // 4051,  11001011 11110000 00000000 00000000
    2, // 4052,  00101011 11110000 00000000 00000000
    0, // 4053,  10101011 11110000 00000000 00000000
    1, // 4054,  01101011 11110000 00000000 00000000
    0, // 4055,  11101011 11110000 00000000 00000000
    3, // 4056,  00011011 11110000 00000000 00000000
    0, // 4057,  10011011 11110000 00000000 00000000
    1, // 4058,  01011011 11110000 00000000 00000000
    0, // 4059,  11011011 11110000 00000000 00000000
    2, // 4060,  00111011 11110000 00000000 00000000
    0, // 4061,  10111011 11110000 00000000 00000000
    1, // 4062,  01111011 11110000 00000000 00000000
    0, // 4063,  11111011 11110000 00000000 00000000
    5, // 4064,  00000111 11110000 00000000 00000000
    0, // 4065,  10000111 11110000 00000000 00000000
    1, // 4066,  01000111 11110000 00000000 00000000
    0, // 4067,  11000111 11110000 00000000 00000000
    2, // 4068,  00100111 11110000 00000000 00000000
    0, // 4069,  10100111 11110000 00000000 00000000
    1, // 4070,  01100111 11110000 00000000 00000000
    0, // 4071,  11100111 11110000 00000000 00000000
    3, // 4072,  00010111 11110000 00000000 00000000
    0, // 4073,  10010111 11110000 00000000 00000000
    1, // 4074,  01010111 11110000 00000000 00000000
    0, // 4075,  11010111 11110000 00000000 00000000
    2, // 4076,  00110111 11110000 00000000 00000000
    0, // 4077,  10110111 11110000 00000000 00000000
    1, // 4078,  01110111 11110000 00000000 00000000
    0, // 4079,  11110111 11110000 00000000 00000000
    4, // 4080,  00001111 11110000 00000000 00000000
    0, // 4081,  10001111 11110000 00000000 00000000
    1, // 4082,  01001111 11110000 00000000 00000000
    0, // 4083,  11001111 11110000 00000000 00000000
    2, // 4084,  00101111 11110000 00000000 00000000
    0, // 4085,  10101111 11110000 00000000 00000000
    1, // 4086,  01101111 11110000 00000000 00000000
    0, // 4087,  11101111 11110000 00000000 00000000
    3, // 4088,  00011111 11110000 00000000 00000000
    0, // 4089,  10011111 11110000 00000000 00000000
    1, // 4090,  01011111 11110000 00000000 00000000
    0, // 4091,  11011111 11110000 00000000 00000000
    2, // 4092,  00111111 11110000 00000000 00000000
    0, // 4093,  10111111 11110000 00000000 00000000
    1, // 4094,  01111111 11110000 00000000 00000000
    0, // 4095,  11111111 11110000 00000000 00000000
    12, // 4096,  00000000 00001000 00000000 00000000
    0, // 4097,  10000000 00001000 00000000 00000000
    1, // 4098,  01000000 00001000 00000000 00000000
    0, // 4099,  11000000 00001000 00000000 00000000
    2, // 4100,  00100000 00001000 00000000 00000000
    0, // 4101,  10100000 00001000 00000000 00000000
    1, // 4102,  01100000 00001000 00000000 00000000
    0, // 4103,  11100000 00001000 00000000 00000000
    3, // 4104,  00010000 00001000 00000000 00000000
    0, // 4105,  10010000 00001000 00000000 00000000
    1, // 4106,  01010000 00001000 00000000 00000000
    0, // 4107,  11010000 00001000 00000000 00000000
    2, // 4108,  00110000 00001000 00000000 00000000
    0, // 4109,  10110000 00001000 00000000 00000000
    1, // 4110,  01110000 00001000 00000000 00000000
    0, // 4111,  11110000 00001000 00000000 00000000
    4, // 4112,  00001000 00001000 00000000 00000000
    0, // 4113,  10001000 00001000 00000000 00000000
    1, // 4114,  01001000 00001000 00000000 00000000
    0, // 4115,  11001000 00001000 00000000 00000000
    2, // 4116,  00101000 00001000 00000000 00000000
    0, // 4117,  10101000 00001000 00000000 00000000
    1, // 4118,  01101000 00001000 00000000 00000000
    0, // 4119,  11101000 00001000 00000000 00000000
    3, // 4120,  00011000 00001000 00000000 00000000
    0, // 4121,  10011000 00001000 00000000 00000000
    1, // 4122,  01011000 00001000 00000000 00000000
    0, // 4123,  11011000 00001000 00000000 00000000
    2, // 4124,  00111000 00001000 00000000 00000000
    0, // 4125,  10111000 00001000 00000000 00000000
    1, // 4126,  01111000 00001000 00000000 00000000
    0, // 4127,  11111000 00001000 00000000 00000000
    5, // 4128,  00000100 00001000 00000000 00000000
    0, // 4129,  10000100 00001000 00000000 00000000
    1, // 4130,  01000100 00001000 00000000 00000000
    0, // 4131,  11000100 00001000 00000000 00000000
    2, // 4132,  00100100 00001000 00000000 00000000
    0, // 4133,  10100100 00001000 00000000 00000000
    1, // 4134,  01100100 00001000 00000000 00000000
    0, // 4135,  11100100 00001000 00000000 00000000
    3, // 4136,  00010100 00001000 00000000 00000000
    0, // 4137,  10010100 00001000 00000000 00000000
    1, // 4138,  01010100 00001000 00000000 00000000
    0, // 4139,  11010100 00001000 00000000 00000000
    2, // 4140,  00110100 00001000 00000000 00000000
    0, // 4141,  10110100 00001000 00000000 00000000
    1, // 4142,  01110100 00001000 00000000 00000000
    0, // 4143,  11110100 00001000 00000000 00000000
    4, // 4144,  00001100 00001000 00000000 00000000
    0, // 4145,  10001100 00001000 00000000 00000000
    1, // 4146,  01001100 00001000 00000000 00000000
    0, // 4147,  11001100 00001000 00000000 00000000
    2, // 4148,  00101100 00001000 00000000 00000000
    0, // 4149,  10101100 00001000 00000000 00000000
    1, // 4150,  01101100 00001000 00000000 00000000
    0, // 4151,  11101100 00001000 00000000 00000000
    3, // 4152,  00011100 00001000 00000000 00000000
    0, // 4153,  10011100 00001000 00000000 00000000
    1, // 4154,  01011100 00001000 00000000 00000000
    0, // 4155,  11011100 00001000 00000000 00000000
    2, // 4156,  00111100 00001000 00000000 00000000
    0, // 4157,  10111100 00001000 00000000 00000000
    1, // 4158,  01111100 00001000 00000000 00000000
    0, // 4159,  11111100 00001000 00000000 00000000
    6, // 4160,  00000010 00001000 00000000 00000000
    0, // 4161,  10000010 00001000 00000000 00000000
    1, // 4162,  01000010 00001000 00000000 00000000
    0, // 4163,  11000010 00001000 00000000 00000000
    2, // 4164,  00100010 00001000 00000000 00000000
    0, // 4165,  10100010 00001000 00000000 00000000
    1, // 4166,  01100010 00001000 00000000 00000000
    0, // 4167,  11100010 00001000 00000000 00000000
    3, // 4168,  00010010 00001000 00000000 00000000
    0, // 4169,  10010010 00001000 00000000 00000000
    1, // 4170,  01010010 00001000 00000000 00000000
    0, // 4171,  11010010 00001000 00000000 00000000
    2, // 4172,  00110010 00001000 00000000 00000000
    0, // 4173,  10110010 00001000 00000000 00000000
    1, // 4174,  01110010 00001000 00000000 00000000
    0, // 4175,  11110010 00001000 00000000 00000000
    4, // 4176,  00001010 00001000 00000000 00000000
    0, // 4177,  10001010 00001000 00000000 00000000
    1, // 4178,  01001010 00001000 00000000 00000000
    0, // 4179,  11001010 00001000 00000000 00000000
    2, // 4180,  00101010 00001000 00000000 00000000
    0, // 4181,  10101010 00001000 00000000 00000000
    1, // 4182,  01101010 00001000 00000000 00000000
    0, // 4183,  11101010 00001000 00000000 00000000
    3, // 4184,  00011010 00001000 00000000 00000000
    0, // 4185,  10011010 00001000 00000000 00000000
    1, // 4186,  01011010 00001000 00000000 00000000
    0, // 4187,  11011010 00001000 00000000 00000000
    2, // 4188,  00111010 00001000 00000000 00000000
    0, // 4189,  10111010 00001000 00000000 00000000
    1, // 4190,  01111010 00001000 00000000 00000000
    0, // 4191,  11111010 00001000 00000000 00000000
    5, // 4192,  00000110 00001000 00000000 00000000
    0, // 4193,  10000110 00001000 00000000 00000000
    1, // 4194,  01000110 00001000 00000000 00000000
    0, // 4195,  11000110 00001000 00000000 00000000
    2, // 4196,  00100110 00001000 00000000 00000000
    0, // 4197,  10100110 00001000 00000000 00000000
    1, // 4198,  01100110 00001000 00000000 00000000
    0, // 4199,  11100110 00001000 00000000 00000000
    3, // 4200,  00010110 00001000 00000000 00000000
    0, // 4201,  10010110 00001000 00000000 00000000
    1, // 4202,  01010110 00001000 00000000 00000000
    0, // 4203,  11010110 00001000 00000000 00000000
    2, // 4204,  00110110 00001000 00000000 00000000
    0, // 4205,  10110110 00001000 00000000 00000000
    1, // 4206,  01110110 00001000 00000000 00000000
    0, // 4207,  11110110 00001000 00000000 00000000
    4, // 4208,  00001110 00001000 00000000 00000000
    0, // 4209,  10001110 00001000 00000000 00000000
    1, // 4210,  01001110 00001000 00000000 00000000
    0, // 4211,  11001110 00001000 00000000 00000000
    2, // 4212,  00101110 00001000 00000000 00000000
    0, // 4213,  10101110 00001000 00000000 00000000
    1, // 4214,  01101110 00001000 00000000 00000000
    0, // 4215,  11101110 00001000 00000000 00000000
    3, // 4216,  00011110 00001000 00000000 00000000
    0, // 4217,  10011110 00001000 00000000 00000000
    1, // 4218,  01011110 00001000 00000000 00000000
    0, // 4219,  11011110 00001000 00000000 00000000
    2, // 4220,  00111110 00001000 00000000 00000000
    0, // 4221,  10111110 00001000 00000000 00000000
    1, // 4222,  01111110 00001000 00000000 00000000
    0, // 4223,  11111110 00001000 00000000 00000000
    7, // 4224,  00000001 00001000 00000000 00000000
    0, // 4225,  10000001 00001000 00000000 00000000
    1, // 4226,  01000001 00001000 00000000 00000000
    0, // 4227,  11000001 00001000 00000000 00000000
    2, // 4228,  00100001 00001000 00000000 00000000
    0, // 4229,  10100001 00001000 00000000 00000000
    1, // 4230,  01100001 00001000 00000000 00000000
    0, // 4231,  11100001 00001000 00000000 00000000
    3, // 4232,  00010001 00001000 00000000 00000000
    0, // 4233,  10010001 00001000 00000000 00000000
    1, // 4234,  01010001 00001000 00000000 00000000
    0, // 4235,  11010001 00001000 00000000 00000000
    2, // 4236,  00110001 00001000 00000000 00000000
    0, // 4237,  10110001 00001000 00000000 00000000
    1, // 4238,  01110001 00001000 00000000 00000000
    0, // 4239,  11110001 00001000 00000000 00000000
    4, // 4240,  00001001 00001000 00000000 00000000
    0, // 4241,  10001001 00001000 00000000 00000000
    1, // 4242,  01001001 00001000 00000000 00000000
    0, // 4243,  11001001 00001000 00000000 00000000
    2, // 4244,  00101001 00001000 00000000 00000000
    0, // 4245,  10101001 00001000 00000000 00000000
    1, // 4246,  01101001 00001000 00000000 00000000
    0, // 4247,  11101001 00001000 00000000 00000000
    3, // 4248,  00011001 00001000 00000000 00000000
    0, // 4249,  10011001 00001000 00000000 00000000
    1, // 4250,  01011001 00001000 00000000 00000000
    0, // 4251,  11011001 00001000 00000000 00000000
    2, // 4252,  00111001 00001000 00000000 00000000
    0, // 4253,  10111001 00001000 00000000 00000000
    1, // 4254,  01111001 00001000 00000000 00000000
    0, // 4255,  11111001 00001000 00000000 00000000
    5, // 4256,  00000101 00001000 00000000 00000000
    0, // 4257,  10000101 00001000 00000000 00000000
    1, // 4258,  01000101 00001000 00000000 00000000
    0, // 4259,  11000101 00001000 00000000 00000000
    2, // 4260,  00100101 00001000 00000000 00000000
    0, // 4261,  10100101 00001000 00000000 00000000
    1, // 4262,  01100101 00001000 00000000 00000000
    0, // 4263,  11100101 00001000 00000000 00000000
    3, // 4264,  00010101 00001000 00000000 00000000
    0, // 4265,  10010101 00001000 00000000 00000000
    1, // 4266,  01010101 00001000 00000000 00000000
    0, // 4267,  11010101 00001000 00000000 00000000
    2, // 4268,  00110101 00001000 00000000 00000000
    0, // 4269,  10110101 00001000 00000000 00000000
    1, // 4270,  01110101 00001000 00000000 00000000
    0, // 4271,  11110101 00001000 00000000 00000000
    4, // 4272,  00001101 00001000 00000000 00000000
    0, // 4273,  10001101 00001000 00000000 00000000
    1, // 4274,  01001101 00001000 00000000 00000000
    0, // 4275,  11001101 00001000 00000000 00000000
    2, // 4276,  00101101 00001000 00000000 00000000
    0, // 4277,  10101101 00001000 00000000 00000000
    1, // 4278,  01101101 00001000 00000000 00000000
    0, // 4279,  11101101 00001000 00000000 00000000
    3, // 4280,  00011101 00001000 00000000 00000000
    0, // 4281,  10011101 00001000 00000000 00000000
    1, // 4282,  01011101 00001000 00000000 00000000
    0, // 4283,  11011101 00001000 00000000 00000000
    2, // 4284,  00111101 00001000 00000000 00000000
    0, // 4285,  10111101 00001000 00000000 00000000
    1, // 4286,  01111101 00001000 00000000 00000000
    0, // 4287,  11111101 00001000 00000000 00000000
    6, // 4288,  00000011 00001000 00000000 00000000
    0, // 4289,  10000011 00001000 00000000 00000000
    1, // 4290,  01000011 00001000 00000000 00000000
    0, // 4291,  11000011 00001000 00000000 00000000
    2, // 4292,  00100011 00001000 00000000 00000000
    0, // 4293,  10100011 00001000 00000000 00000000
    1, // 4294,  01100011 00001000 00000000 00000000
    0, // 4295,  11100011 00001000 00000000 00000000
    3, // 4296,  00010011 00001000 00000000 00000000
    0, // 4297,  10010011 00001000 00000000 00000000
    1, // 4298,  01010011 00001000 00000000 00000000
    0, // 4299,  11010011 00001000 00000000 00000000
    2, // 4300,  00110011 00001000 00000000 00000000
    0, // 4301,  10110011 00001000 00000000 00000000
    1, // 4302,  01110011 00001000 00000000 00000000
    0, // 4303,  11110011 00001000 00000000 00000000
    4, // 4304,  00001011 00001000 00000000 00000000
    0, // 4305,  10001011 00001000 00000000 00000000
    1, // 4306,  01001011 00001000 00000000 00000000
    0, // 4307,  11001011 00001000 00000000 00000000
    2, // 4308,  00101011 00001000 00000000 00000000
    0, // 4309,  10101011 00001000 00000000 00000000
    1, // 4310,  01101011 00001000 00000000 00000000
    0, // 4311,  11101011 00001000 00000000 00000000
    3, // 4312,  00011011 00001000 00000000 00000000
    0, // 4313,  10011011 00001000 00000000 00000000
    1, // 4314,  01011011 00001000 00000000 00000000
    0, // 4315,  11011011 00001000 00000000 00000000
    2, // 4316,  00111011 00001000 00000000 00000000
    0, // 4317,  10111011 00001000 00000000 00000000
    1, // 4318,  01111011 00001000 00000000 00000000
    0, // 4319,  11111011 00001000 00000000 00000000
    5, // 4320,  00000111 00001000 00000000 00000000
    0, // 4321,  10000111 00001000 00000000 00000000
    1, // 4322,  01000111 00001000 00000000 00000000
    0, // 4323,  11000111 00001000 00000000 00000000
    2, // 4324,  00100111 00001000 00000000 00000000
    0, // 4325,  10100111 00001000 00000000 00000000
    1, // 4326,  01100111 00001000 00000000 00000000
    0, // 4327,  11100111 00001000 00000000 00000000
    3, // 4328,  00010111 00001000 00000000 00000000
    0, // 4329,  10010111 00001000 00000000 00000000
    1, // 4330,  01010111 00001000 00000000 00000000
    0, // 4331,  11010111 00001000 00000000 00000000
    2, // 4332,  00110111 00001000 00000000 00000000
    0, // 4333,  10110111 00001000 00000000 00000000
    1, // 4334,  01110111 00001000 00000000 00000000
    0, // 4335,  11110111 00001000 00000000 00000000
    4, // 4336,  00001111 00001000 00000000 00000000
    0, // 4337,  10001111 00001000 00000000 00000000
    1, // 4338,  01001111 00001000 00000000 00000000
    0, // 4339,  11001111 00001000 00000000 00000000
    2, // 4340,  00101111 00001000 00000000 00000000
    0, // 4341,  10101111 00001000 00000000 00000000
    1, // 4342,  01101111 00001000 00000000 00000000
    0, // 4343,  11101111 00001000 00000000 00000000
    3, // 4344,  00011111 00001000 00000000 00000000
    0, // 4345,  10011111 00001000 00000000 00000000
    1, // 4346,  01011111 00001000 00000000 00000000
    0, // 4347,  11011111 00001000 00000000 00000000
    2, // 4348,  00111111 00001000 00000000 00000000
    0, // 4349,  10111111 00001000 00000000 00000000
    1, // 4350,  01111111 00001000 00000000 00000000
    0, // 4351,  11111111 00001000 00000000 00000000
    8, // 4352,  00000000 10001000 00000000 00000000
    0, // 4353,  10000000 10001000 00000000 00000000
    1, // 4354,  01000000 10001000 00000000 00000000
    0, // 4355,  11000000 10001000 00000000 00000000
    2, // 4356,  00100000 10001000 00000000 00000000
    0, // 4357,  10100000 10001000 00000000 00000000
    1, // 4358,  01100000 10001000 00000000 00000000
    0, // 4359,  11100000 10001000 00000000 00000000
    3, // 4360,  00010000 10001000 00000000 00000000
    0, // 4361,  10010000 10001000 00000000 00000000
    1, // 4362,  01010000 10001000 00000000 00000000
    0, // 4363,  11010000 10001000 00000000 00000000
    2, // 4364,  00110000 10001000 00000000 00000000
    0, // 4365,  10110000 10001000 00000000 00000000
    1, // 4366,  01110000 10001000 00000000 00000000
    0, // 4367,  11110000 10001000 00000000 00000000
    4, // 4368,  00001000 10001000 00000000 00000000
    0, // 4369,  10001000 10001000 00000000 00000000
    1, // 4370,  01001000 10001000 00000000 00000000
    0, // 4371,  11001000 10001000 00000000 00000000
    2, // 4372,  00101000 10001000 00000000 00000000
    0, // 4373,  10101000 10001000 00000000 00000000
    1, // 4374,  01101000 10001000 00000000 00000000
    0, // 4375,  11101000 10001000 00000000 00000000
    3, // 4376,  00011000 10001000 00000000 00000000
    0, // 4377,  10011000 10001000 00000000 00000000
    1, // 4378,  01011000 10001000 00000000 00000000
    0, // 4379,  11011000 10001000 00000000 00000000
    2, // 4380,  00111000 10001000 00000000 00000000
    0, // 4381,  10111000 10001000 00000000 00000000
    1, // 4382,  01111000 10001000 00000000 00000000
    0, // 4383,  11111000 10001000 00000000 00000000
    5, // 4384,  00000100 10001000 00000000 00000000
    0, // 4385,  10000100 10001000 00000000 00000000
    1, // 4386,  01000100 10001000 00000000 00000000
    0, // 4387,  11000100 10001000 00000000 00000000
    2, // 4388,  00100100 10001000 00000000 00000000
    0, // 4389,  10100100 10001000 00000000 00000000
    1, // 4390,  01100100 10001000 00000000 00000000
    0, // 4391,  11100100 10001000 00000000 00000000
    3, // 4392,  00010100 10001000 00000000 00000000
    0, // 4393,  10010100 10001000 00000000 00000000
    1, // 4394,  01010100 10001000 00000000 00000000
    0, // 4395,  11010100 10001000 00000000 00000000
    2, // 4396,  00110100 10001000 00000000 00000000
    0, // 4397,  10110100 10001000 00000000 00000000
    1, // 4398,  01110100 10001000 00000000 00000000
    0, // 4399,  11110100 10001000 00000000 00000000
    4, // 4400,  00001100 10001000 00000000 00000000
    0, // 4401,  10001100 10001000 00000000 00000000
    1, // 4402,  01001100 10001000 00000000 00000000
    0, // 4403,  11001100 10001000 00000000 00000000
    2, // 4404,  00101100 10001000 00000000 00000000
    0, // 4405,  10101100 10001000 00000000 00000000
    1, // 4406,  01101100 10001000 00000000 00000000
    0, // 4407,  11101100 10001000 00000000 00000000
    3, // 4408,  00011100 10001000 00000000 00000000
    0, // 4409,  10011100 10001000 00000000 00000000
    1, // 4410,  01011100 10001000 00000000 00000000
    0, // 4411,  11011100 10001000 00000000 00000000
    2, // 4412,  00111100 10001000 00000000 00000000
    0, // 4413,  10111100 10001000 00000000 00000000
    1, // 4414,  01111100 10001000 00000000 00000000
    0, // 4415,  11111100 10001000 00000000 00000000
    6, // 4416,  00000010 10001000 00000000 00000000
    0, // 4417,  10000010 10001000 00000000 00000000
    1, // 4418,  01000010 10001000 00000000 00000000
    0, // 4419,  11000010 10001000 00000000 00000000
    2, // 4420,  00100010 10001000 00000000 00000000
    0, // 4421,  10100010 10001000 00000000 00000000
    1, // 4422,  01100010 10001000 00000000 00000000
    0, // 4423,  11100010 10001000 00000000 00000000
    3, // 4424,  00010010 10001000 00000000 00000000
    0, // 4425,  10010010 10001000 00000000 00000000
    1, // 4426,  01010010 10001000 00000000 00000000
    0, // 4427,  11010010 10001000 00000000 00000000
    2, // 4428,  00110010 10001000 00000000 00000000
    0, // 4429,  10110010 10001000 00000000 00000000
    1, // 4430,  01110010 10001000 00000000 00000000
    0, // 4431,  11110010 10001000 00000000 00000000
    4, // 4432,  00001010 10001000 00000000 00000000
    0, // 4433,  10001010 10001000 00000000 00000000
    1, // 4434,  01001010 10001000 00000000 00000000
    0, // 4435,  11001010 10001000 00000000 00000000
    2, // 4436,  00101010 10001000 00000000 00000000
    0, // 4437,  10101010 10001000 00000000 00000000
    1, // 4438,  01101010 10001000 00000000 00000000
    0, // 4439,  11101010 10001000 00000000 00000000
    3, // 4440,  00011010 10001000 00000000 00000000
    0, // 4441,  10011010 10001000 00000000 00000000
    1, // 4442,  01011010 10001000 00000000 00000000
    0, // 4443,  11011010 10001000 00000000 00000000
    2, // 4444,  00111010 10001000 00000000 00000000
    0, // 4445,  10111010 10001000 00000000 00000000
    1, // 4446,  01111010 10001000 00000000 00000000
    0, // 4447,  11111010 10001000 00000000 00000000
    5, // 4448,  00000110 10001000 00000000 00000000
    0, // 4449,  10000110 10001000 00000000 00000000
    1, // 4450,  01000110 10001000 00000000 00000000
    0, // 4451,  11000110 10001000 00000000 00000000
    2, // 4452,  00100110 10001000 00000000 00000000
    0, // 4453,  10100110 10001000 00000000 00000000
    1, // 4454,  01100110 10001000 00000000 00000000
    0, // 4455,  11100110 10001000 00000000 00000000
    3, // 4456,  00010110 10001000 00000000 00000000
    0, // 4457,  10010110 10001000 00000000 00000000
    1, // 4458,  01010110 10001000 00000000 00000000
    0, // 4459,  11010110 10001000 00000000 00000000
    2, // 4460,  00110110 10001000 00000000 00000000
    0, // 4461,  10110110 10001000 00000000 00000000
    1, // 4462,  01110110 10001000 00000000 00000000
    0, // 4463,  11110110 10001000 00000000 00000000
    4, // 4464,  00001110 10001000 00000000 00000000
    0, // 4465,  10001110 10001000 00000000 00000000
    1, // 4466,  01001110 10001000 00000000 00000000
    0, // 4467,  11001110 10001000 00000000 00000000
    2, // 4468,  00101110 10001000 00000000 00000000
    0, // 4469,  10101110 10001000 00000000 00000000
    1, // 4470,  01101110 10001000 00000000 00000000
    0, // 4471,  11101110 10001000 00000000 00000000
    3, // 4472,  00011110 10001000 00000000 00000000
    0, // 4473,  10011110 10001000 00000000 00000000
    1, // 4474,  01011110 10001000 00000000 00000000
    0, // 4475,  11011110 10001000 00000000 00000000
    2, // 4476,  00111110 10001000 00000000 00000000
    0, // 4477,  10111110 10001000 00000000 00000000
    1, // 4478,  01111110 10001000 00000000 00000000
    0, // 4479,  11111110 10001000 00000000 00000000
    7, // 4480,  00000001 10001000 00000000 00000000
    0, // 4481,  10000001 10001000 00000000 00000000
    1, // 4482,  01000001 10001000 00000000 00000000
    0, // 4483,  11000001 10001000 00000000 00000000
    2, // 4484,  00100001 10001000 00000000 00000000
    0, // 4485,  10100001 10001000 00000000 00000000
    1, // 4486,  01100001 10001000 00000000 00000000
    0, // 4487,  11100001 10001000 00000000 00000000
    3, // 4488,  00010001 10001000 00000000 00000000
    0, // 4489,  10010001 10001000 00000000 00000000
    1, // 4490,  01010001 10001000 00000000 00000000
    0, // 4491,  11010001 10001000 00000000 00000000
    2, // 4492,  00110001 10001000 00000000 00000000
    0, // 4493,  10110001 10001000 00000000 00000000
    1, // 4494,  01110001 10001000 00000000 00000000
    0, // 4495,  11110001 10001000 00000000 00000000
    4, // 4496,  00001001 10001000 00000000 00000000
    0, // 4497,  10001001 10001000 00000000 00000000
    1, // 4498,  01001001 10001000 00000000 00000000
    0, // 4499,  11001001 10001000 00000000 00000000
    2, // 4500,  00101001 10001000 00000000 00000000
    0, // 4501,  10101001 10001000 00000000 00000000
    1, // 4502,  01101001 10001000 00000000 00000000
    0, // 4503,  11101001 10001000 00000000 00000000
    3, // 4504,  00011001 10001000 00000000 00000000
    0, // 4505,  10011001 10001000 00000000 00000000
    1, // 4506,  01011001 10001000 00000000 00000000
    0, // 4507,  11011001 10001000 00000000 00000000
    2, // 4508,  00111001 10001000 00000000 00000000
    0, // 4509,  10111001 10001000 00000000 00000000
    1, // 4510,  01111001 10001000 00000000 00000000
    0, // 4511,  11111001 10001000 00000000 00000000
    5, // 4512,  00000101 10001000 00000000 00000000
    0, // 4513,  10000101 10001000 00000000 00000000
    1, // 4514,  01000101 10001000 00000000 00000000
    0, // 4515,  11000101 10001000 00000000 00000000
    2, // 4516,  00100101 10001000 00000000 00000000
    0, // 4517,  10100101 10001000 00000000 00000000
    1, // 4518,  01100101 10001000 00000000 00000000
    0, // 4519,  11100101 10001000 00000000 00000000
    3, // 4520,  00010101 10001000 00000000 00000000
    0, // 4521,  10010101 10001000 00000000 00000000
    1, // 4522,  01010101 10001000 00000000 00000000
    0, // 4523,  11010101 10001000 00000000 00000000
    2, // 4524,  00110101 10001000 00000000 00000000
    0, // 4525,  10110101 10001000 00000000 00000000
    1, // 4526,  01110101 10001000 00000000 00000000
    0, // 4527,  11110101 10001000 00000000 00000000
    4, // 4528,  00001101 10001000 00000000 00000000
    0, // 4529,  10001101 10001000 00000000 00000000
    1, // 4530,  01001101 10001000 00000000 00000000
    0, // 4531,  11001101 10001000 00000000 00000000
    2, // 4532,  00101101 10001000 00000000 00000000
    0, // 4533,  10101101 10001000 00000000 00000000
    1, // 4534,  01101101 10001000 00000000 00000000
    0, // 4535,  11101101 10001000 00000000 00000000
    3, // 4536,  00011101 10001000 00000000 00000000
    0, // 4537,  10011101 10001000 00000000 00000000
    1, // 4538,  01011101 10001000 00000000 00000000
    0, // 4539,  11011101 10001000 00000000 00000000
    2, // 4540,  00111101 10001000 00000000 00000000
    0, // 4541,  10111101 10001000 00000000 00000000
    1, // 4542,  01111101 10001000 00000000 00000000
    0, // 4543,  11111101 10001000 00000000 00000000
    6, // 4544,  00000011 10001000 00000000 00000000
    0, // 4545,  10000011 10001000 00000000 00000000
    1, // 4546,  01000011 10001000 00000000 00000000
    0, // 4547,  11000011 10001000 00000000 00000000
    2, // 4548,  00100011 10001000 00000000 00000000
    0, // 4549,  10100011 10001000 00000000 00000000
    1, // 4550,  01100011 10001000 00000000 00000000
    0, // 4551,  11100011 10001000 00000000 00000000
    3, // 4552,  00010011 10001000 00000000 00000000
    0, // 4553,  10010011 10001000 00000000 00000000
    1, // 4554,  01010011 10001000 00000000 00000000
    0, // 4555,  11010011 10001000 00000000 00000000
    2, // 4556,  00110011 10001000 00000000 00000000
    0, // 4557,  10110011 10001000 00000000 00000000
    1, // 4558,  01110011 10001000 00000000 00000000
    0, // 4559,  11110011 10001000 00000000 00000000
    4, // 4560,  00001011 10001000 00000000 00000000
    0, // 4561,  10001011 10001000 00000000 00000000
    1, // 4562,  01001011 10001000 00000000 00000000
    0, // 4563,  11001011 10001000 00000000 00000000
    2, // 4564,  00101011 10001000 00000000 00000000
    0, // 4565,  10101011 10001000 00000000 00000000
    1, // 4566,  01101011 10001000 00000000 00000000
    0, // 4567,  11101011 10001000 00000000 00000000
    3, // 4568,  00011011 10001000 00000000 00000000
    0, // 4569,  10011011 10001000 00000000 00000000
    1, // 4570,  01011011 10001000 00000000 00000000
    0, // 4571,  11011011 10001000 00000000 00000000
    2, // 4572,  00111011 10001000 00000000 00000000
    0, // 4573,  10111011 10001000 00000000 00000000
    1, // 4574,  01111011 10001000 00000000 00000000
    0, // 4575,  11111011 10001000 00000000 00000000
    5, // 4576,  00000111 10001000 00000000 00000000
    0, // 4577,  10000111 10001000 00000000 00000000
    1, // 4578,  01000111 10001000 00000000 00000000
    0, // 4579,  11000111 10001000 00000000 00000000
    2, // 4580,  00100111 10001000 00000000 00000000
    0, // 4581,  10100111 10001000 00000000 00000000
    1, // 4582,  01100111 10001000 00000000 00000000
    0, // 4583,  11100111 10001000 00000000 00000000
    3, // 4584,  00010111 10001000 00000000 00000000
    0, // 4585,  10010111 10001000 00000000 00000000
    1, // 4586,  01010111 10001000 00000000 00000000
    0, // 4587,  11010111 10001000 00000000 00000000
    2, // 4588,  00110111 10001000 00000000 00000000
    0, // 4589,  10110111 10001000 00000000 00000000
    1, // 4590,  01110111 10001000 00000000 00000000
    0, // 4591,  11110111 10001000 00000000 00000000
    4, // 4592,  00001111 10001000 00000000 00000000
    0, // 4593,  10001111 10001000 00000000 00000000
    1, // 4594,  01001111 10001000 00000000 00000000
    0, // 4595,  11001111 10001000 00000000 00000000
    2, // 4596,  00101111 10001000 00000000 00000000
    0, // 4597,  10101111 10001000 00000000 00000000
    1, // 4598,  01101111 10001000 00000000 00000000
    0, // 4599,  11101111 10001000 00000000 00000000
    3, // 4600,  00011111 10001000 00000000 00000000
    0, // 4601,  10011111 10001000 00000000 00000000
    1, // 4602,  01011111 10001000 00000000 00000000
    0, // 4603,  11011111 10001000 00000000 00000000
    2, // 4604,  00111111 10001000 00000000 00000000
    0, // 4605,  10111111 10001000 00000000 00000000
    1, // 4606,  01111111 10001000 00000000 00000000
    0, // 4607,  11111111 10001000 00000000 00000000
    9, // 4608,  00000000 01001000 00000000 00000000
    0, // 4609,  10000000 01001000 00000000 00000000
    1, // 4610,  01000000 01001000 00000000 00000000
    0, // 4611,  11000000 01001000 00000000 00000000
    2, // 4612,  00100000 01001000 00000000 00000000
    0, // 4613,  10100000 01001000 00000000 00000000
    1, // 4614,  01100000 01001000 00000000 00000000
    0, // 4615,  11100000 01001000 00000000 00000000
    3, // 4616,  00010000 01001000 00000000 00000000
    0, // 4617,  10010000 01001000 00000000 00000000
    1, // 4618,  01010000 01001000 00000000 00000000
    0, // 4619,  11010000 01001000 00000000 00000000
    2, // 4620,  00110000 01001000 00000000 00000000
    0, // 4621,  10110000 01001000 00000000 00000000
    1, // 4622,  01110000 01001000 00000000 00000000
    0, // 4623,  11110000 01001000 00000000 00000000
    4, // 4624,  00001000 01001000 00000000 00000000
    0, // 4625,  10001000 01001000 00000000 00000000
    1, // 4626,  01001000 01001000 00000000 00000000
    0, // 4627,  11001000 01001000 00000000 00000000
    2, // 4628,  00101000 01001000 00000000 00000000
    0, // 4629,  10101000 01001000 00000000 00000000
    1, // 4630,  01101000 01001000 00000000 00000000
    0, // 4631,  11101000 01001000 00000000 00000000
    3, // 4632,  00011000 01001000 00000000 00000000
    0, // 4633,  10011000 01001000 00000000 00000000
    1, // 4634,  01011000 01001000 00000000 00000000
    0, // 4635,  11011000 01001000 00000000 00000000
    2, // 4636,  00111000 01001000 00000000 00000000
    0, // 4637,  10111000 01001000 00000000 00000000
    1, // 4638,  01111000 01001000 00000000 00000000
    0, // 4639,  11111000 01001000 00000000 00000000
    5, // 4640,  00000100 01001000 00000000 00000000
    0, // 4641,  10000100 01001000 00000000 00000000
    1, // 4642,  01000100 01001000 00000000 00000000
    0, // 4643,  11000100 01001000 00000000 00000000
    2, // 4644,  00100100 01001000 00000000 00000000
    0, // 4645,  10100100 01001000 00000000 00000000
    1, // 4646,  01100100 01001000 00000000 00000000
    0, // 4647,  11100100 01001000 00000000 00000000
    3, // 4648,  00010100 01001000 00000000 00000000
    0, // 4649,  10010100 01001000 00000000 00000000
    1, // 4650,  01010100 01001000 00000000 00000000
    0, // 4651,  11010100 01001000 00000000 00000000
    2, // 4652,  00110100 01001000 00000000 00000000
    0, // 4653,  10110100 01001000 00000000 00000000
    1, // 4654,  01110100 01001000 00000000 00000000
    0, // 4655,  11110100 01001000 00000000 00000000
    4, // 4656,  00001100 01001000 00000000 00000000
    0, // 4657,  10001100 01001000 00000000 00000000
    1, // 4658,  01001100 01001000 00000000 00000000
    0, // 4659,  11001100 01001000 00000000 00000000
    2, // 4660,  00101100 01001000 00000000 00000000
    0, // 4661,  10101100 01001000 00000000 00000000
    1, // 4662,  01101100 01001000 00000000 00000000
    0, // 4663,  11101100 01001000 00000000 00000000
    3, // 4664,  00011100 01001000 00000000 00000000
    0, // 4665,  10011100 01001000 00000000 00000000
    1, // 4666,  01011100 01001000 00000000 00000000
    0, // 4667,  11011100 01001000 00000000 00000000
    2, // 4668,  00111100 01001000 00000000 00000000
    0, // 4669,  10111100 01001000 00000000 00000000
    1, // 4670,  01111100 01001000 00000000 00000000
    0, // 4671,  11111100 01001000 00000000 00000000
    6, // 4672,  00000010 01001000 00000000 00000000
    0, // 4673,  10000010 01001000 00000000 00000000
    1, // 4674,  01000010 01001000 00000000 00000000
    0, // 4675,  11000010 01001000 00000000 00000000
    2, // 4676,  00100010 01001000 00000000 00000000
    0, // 4677,  10100010 01001000 00000000 00000000
    1, // 4678,  01100010 01001000 00000000 00000000
    0, // 4679,  11100010 01001000 00000000 00000000
    3, // 4680,  00010010 01001000 00000000 00000000
    0, // 4681,  10010010 01001000 00000000 00000000
    1, // 4682,  01010010 01001000 00000000 00000000
    0, // 4683,  11010010 01001000 00000000 00000000
    2, // 4684,  00110010 01001000 00000000 00000000
    0, // 4685,  10110010 01001000 00000000 00000000
    1, // 4686,  01110010 01001000 00000000 00000000
    0, // 4687,  11110010 01001000 00000000 00000000
    4, // 4688,  00001010 01001000 00000000 00000000
    0, // 4689,  10001010 01001000 00000000 00000000
    1, // 4690,  01001010 01001000 00000000 00000000
    0, // 4691,  11001010 01001000 00000000 00000000
    2, // 4692,  00101010 01001000 00000000 00000000
    0, // 4693,  10101010 01001000 00000000 00000000
    1, // 4694,  01101010 01001000 00000000 00000000
    0, // 4695,  11101010 01001000 00000000 00000000
    3, // 4696,  00011010 01001000 00000000 00000000
    0, // 4697,  10011010 01001000 00000000 00000000
    1, // 4698,  01011010 01001000 00000000 00000000
    0, // 4699,  11011010 01001000 00000000 00000000
    2, // 4700,  00111010 01001000 00000000 00000000
    0, // 4701,  10111010 01001000 00000000 00000000
    1, // 4702,  01111010 01001000 00000000 00000000
    0, // 4703,  11111010 01001000 00000000 00000000
    5, // 4704,  00000110 01001000 00000000 00000000
    0, // 4705,  10000110 01001000 00000000 00000000
    1, // 4706,  01000110 01001000 00000000 00000000
    0, // 4707,  11000110 01001000 00000000 00000000
    2, // 4708,  00100110 01001000 00000000 00000000
    0, // 4709,  10100110 01001000 00000000 00000000
    1, // 4710,  01100110 01001000 00000000 00000000
    0, // 4711,  11100110 01001000 00000000 00000000
    3, // 4712,  00010110 01001000 00000000 00000000
    0, // 4713,  10010110 01001000 00000000 00000000
    1, // 4714,  01010110 01001000 00000000 00000000
    0, // 4715,  11010110 01001000 00000000 00000000
    2, // 4716,  00110110 01001000 00000000 00000000
    0, // 4717,  10110110 01001000 00000000 00000000
    1, // 4718,  01110110 01001000 00000000 00000000
    0, // 4719,  11110110 01001000 00000000 00000000
    4, // 4720,  00001110 01001000 00000000 00000000
    0, // 4721,  10001110 01001000 00000000 00000000
    1, // 4722,  01001110 01001000 00000000 00000000
    0, // 4723,  11001110 01001000 00000000 00000000
    2, // 4724,  00101110 01001000 00000000 00000000
    0, // 4725,  10101110 01001000 00000000 00000000
    1, // 4726,  01101110 01001000 00000000 00000000
    0, // 4727,  11101110 01001000 00000000 00000000
    3, // 4728,  00011110 01001000 00000000 00000000
    0, // 4729,  10011110 01001000 00000000 00000000
    1, // 4730,  01011110 01001000 00000000 00000000
    0, // 4731,  11011110 01001000 00000000 00000000
    2, // 4732,  00111110 01001000 00000000 00000000
    0, // 4733,  10111110 01001000 00000000 00000000
    1, // 4734,  01111110 01001000 00000000 00000000
    0, // 4735,  11111110 01001000 00000000 00000000
    7, // 4736,  00000001 01001000 00000000 00000000
    0, // 4737,  10000001 01001000 00000000 00000000
    1, // 4738,  01000001 01001000 00000000 00000000
    0, // 4739,  11000001 01001000 00000000 00000000
    2, // 4740,  00100001 01001000 00000000 00000000
    0, // 4741,  10100001 01001000 00000000 00000000
    1, // 4742,  01100001 01001000 00000000 00000000
    0, // 4743,  11100001 01001000 00000000 00000000
    3, // 4744,  00010001 01001000 00000000 00000000
    0, // 4745,  10010001 01001000 00000000 00000000
    1, // 4746,  01010001 01001000 00000000 00000000
    0, // 4747,  11010001 01001000 00000000 00000000
    2, // 4748,  00110001 01001000 00000000 00000000
    0, // 4749,  10110001 01001000 00000000 00000000
    1, // 4750,  01110001 01001000 00000000 00000000
    0, // 4751,  11110001 01001000 00000000 00000000
    4, // 4752,  00001001 01001000 00000000 00000000
    0, // 4753,  10001001 01001000 00000000 00000000
    1, // 4754,  01001001 01001000 00000000 00000000
    0, // 4755,  11001001 01001000 00000000 00000000
    2, // 4756,  00101001 01001000 00000000 00000000
    0, // 4757,  10101001 01001000 00000000 00000000
    1, // 4758,  01101001 01001000 00000000 00000000
    0, // 4759,  11101001 01001000 00000000 00000000
    3, // 4760,  00011001 01001000 00000000 00000000
    0, // 4761,  10011001 01001000 00000000 00000000
    1, // 4762,  01011001 01001000 00000000 00000000
    0, // 4763,  11011001 01001000 00000000 00000000
    2, // 4764,  00111001 01001000 00000000 00000000
    0, // 4765,  10111001 01001000 00000000 00000000
    1, // 4766,  01111001 01001000 00000000 00000000
    0, // 4767,  11111001 01001000 00000000 00000000
    5, // 4768,  00000101 01001000 00000000 00000000
    0, // 4769,  10000101 01001000 00000000 00000000
    1, // 4770,  01000101 01001000 00000000 00000000
    0, // 4771,  11000101 01001000 00000000 00000000
    2, // 4772,  00100101 01001000 00000000 00000000
    0, // 4773,  10100101 01001000 00000000 00000000
    1, // 4774,  01100101 01001000 00000000 00000000
    0, // 4775,  11100101 01001000 00000000 00000000
    3, // 4776,  00010101 01001000 00000000 00000000
    0, // 4777,  10010101 01001000 00000000 00000000
    1, // 4778,  01010101 01001000 00000000 00000000
    0, // 4779,  11010101 01001000 00000000 00000000
    2, // 4780,  00110101 01001000 00000000 00000000
    0, // 4781,  10110101 01001000 00000000 00000000
    1, // 4782,  01110101 01001000 00000000 00000000
    0, // 4783,  11110101 01001000 00000000 00000000
    4, // 4784,  00001101 01001000 00000000 00000000
    0, // 4785,  10001101 01001000 00000000 00000000
    1, // 4786,  01001101 01001000 00000000 00000000
    0, // 4787,  11001101 01001000 00000000 00000000
    2, // 4788,  00101101 01001000 00000000 00000000
    0, // 4789,  10101101 01001000 00000000 00000000
    1, // 4790,  01101101 01001000 00000000 00000000
    0, // 4791,  11101101 01001000 00000000 00000000
    3, // 4792,  00011101 01001000 00000000 00000000
    0, // 4793,  10011101 01001000 00000000 00000000
    1, // 4794,  01011101 01001000 00000000 00000000
    0, // 4795,  11011101 01001000 00000000 00000000
    2, // 4796,  00111101 01001000 00000000 00000000
    0, // 4797,  10111101 01001000 00000000 00000000
    1, // 4798,  01111101 01001000 00000000 00000000
    0, // 4799,  11111101 01001000 00000000 00000000
    6, // 4800,  00000011 01001000 00000000 00000000
    0, // 4801,  10000011 01001000 00000000 00000000
    1, // 4802,  01000011 01001000 00000000 00000000
    0, // 4803,  11000011 01001000 00000000 00000000
    2, // 4804,  00100011 01001000 00000000 00000000
    0, // 4805,  10100011 01001000 00000000 00000000
    1, // 4806,  01100011 01001000 00000000 00000000
    0, // 4807,  11100011 01001000 00000000 00000000
    3, // 4808,  00010011 01001000 00000000 00000000
    0, // 4809,  10010011 01001000 00000000 00000000
    1, // 4810,  01010011 01001000 00000000 00000000
    0, // 4811,  11010011 01001000 00000000 00000000
    2, // 4812,  00110011 01001000 00000000 00000000
    0, // 4813,  10110011 01001000 00000000 00000000
    1, // 4814,  01110011 01001000 00000000 00000000
    0, // 4815,  11110011 01001000 00000000 00000000
    4, // 4816,  00001011 01001000 00000000 00000000
    0, // 4817,  10001011 01001000 00000000 00000000
    1, // 4818,  01001011 01001000 00000000 00000000
    0, // 4819,  11001011 01001000 00000000 00000000
    2, // 4820,  00101011 01001000 00000000 00000000
    0, // 4821,  10101011 01001000 00000000 00000000
    1, // 4822,  01101011 01001000 00000000 00000000
    0, // 4823,  11101011 01001000 00000000 00000000
    3, // 4824,  00011011 01001000 00000000 00000000
    0, // 4825,  10011011 01001000 00000000 00000000
    1, // 4826,  01011011 01001000 00000000 00000000
    0, // 4827,  11011011 01001000 00000000 00000000
    2, // 4828,  00111011 01001000 00000000 00000000
    0, // 4829,  10111011 01001000 00000000 00000000
    1, // 4830,  01111011 01001000 00000000 00000000
    0, // 4831,  11111011 01001000 00000000 00000000
    5, // 4832,  00000111 01001000 00000000 00000000
    0, // 4833,  10000111 01001000 00000000 00000000
    1, // 4834,  01000111 01001000 00000000 00000000
    0, // 4835,  11000111 01001000 00000000 00000000
    2, // 4836,  00100111 01001000 00000000 00000000
    0, // 4837,  10100111 01001000 00000000 00000000
    1, // 4838,  01100111 01001000 00000000 00000000
    0, // 4839,  11100111 01001000 00000000 00000000
    3, // 4840,  00010111 01001000 00000000 00000000
    0, // 4841,  10010111 01001000 00000000 00000000
    1, // 4842,  01010111 01001000 00000000 00000000
    0, // 4843,  11010111 01001000 00000000 00000000
    2, // 4844,  00110111 01001000 00000000 00000000
    0, // 4845,  10110111 01001000 00000000 00000000
    1, // 4846,  01110111 01001000 00000000 00000000
    0, // 4847,  11110111 01001000 00000000 00000000
    4, // 4848,  00001111 01001000 00000000 00000000
    0, // 4849,  10001111 01001000 00000000 00000000
    1, // 4850,  01001111 01001000 00000000 00000000
    0, // 4851,  11001111 01001000 00000000 00000000
    2, // 4852,  00101111 01001000 00000000 00000000
    0, // 4853,  10101111 01001000 00000000 00000000
    1, // 4854,  01101111 01001000 00000000 00000000
    0, // 4855,  11101111 01001000 00000000 00000000
    3, // 4856,  00011111 01001000 00000000 00000000
    0, // 4857,  10011111 01001000 00000000 00000000
    1, // 4858,  01011111 01001000 00000000 00000000
    0, // 4859,  11011111 01001000 00000000 00000000
    2, // 4860,  00111111 01001000 00000000 00000000
    0, // 4861,  10111111 01001000 00000000 00000000
    1, // 4862,  01111111 01001000 00000000 00000000
    0, // 4863,  11111111 01001000 00000000 00000000
    8, // 4864,  00000000 11001000 00000000 00000000
    0, // 4865,  10000000 11001000 00000000 00000000
    1, // 4866,  01000000 11001000 00000000 00000000
    0, // 4867,  11000000 11001000 00000000 00000000
    2, // 4868,  00100000 11001000 00000000 00000000
    0, // 4869,  10100000 11001000 00000000 00000000
    1, // 4870,  01100000 11001000 00000000 00000000
    0, // 4871,  11100000 11001000 00000000 00000000
    3, // 4872,  00010000 11001000 00000000 00000000
    0, // 4873,  10010000 11001000 00000000 00000000
    1, // 4874,  01010000 11001000 00000000 00000000
    0, // 4875,  11010000 11001000 00000000 00000000
    2, // 4876,  00110000 11001000 00000000 00000000
    0, // 4877,  10110000 11001000 00000000 00000000
    1, // 4878,  01110000 11001000 00000000 00000000
    0, // 4879,  11110000 11001000 00000000 00000000
    4, // 4880,  00001000 11001000 00000000 00000000
    0, // 4881,  10001000 11001000 00000000 00000000
    1, // 4882,  01001000 11001000 00000000 00000000
    0, // 4883,  11001000 11001000 00000000 00000000
    2, // 4884,  00101000 11001000 00000000 00000000
    0, // 4885,  10101000 11001000 00000000 00000000
    1, // 4886,  01101000 11001000 00000000 00000000
    0, // 4887,  11101000 11001000 00000000 00000000
    3, // 4888,  00011000 11001000 00000000 00000000
    0, // 4889,  10011000 11001000 00000000 00000000
    1, // 4890,  01011000 11001000 00000000 00000000
    0, // 4891,  11011000 11001000 00000000 00000000
    2, // 4892,  00111000 11001000 00000000 00000000
    0, // 4893,  10111000 11001000 00000000 00000000
    1, // 4894,  01111000 11001000 00000000 00000000
    0, // 4895,  11111000 11001000 00000000 00000000
    5, // 4896,  00000100 11001000 00000000 00000000
    0, // 4897,  10000100 11001000 00000000 00000000
    1, // 4898,  01000100 11001000 00000000 00000000
    0, // 4899,  11000100 11001000 00000000 00000000
    2, // 4900,  00100100 11001000 00000000 00000000
    0, // 4901,  10100100 11001000 00000000 00000000
    1, // 4902,  01100100 11001000 00000000 00000000
    0, // 4903,  11100100 11001000 00000000 00000000
    3, // 4904,  00010100 11001000 00000000 00000000
    0, // 4905,  10010100 11001000 00000000 00000000
    1, // 4906,  01010100 11001000 00000000 00000000
    0, // 4907,  11010100 11001000 00000000 00000000
    2, // 4908,  00110100 11001000 00000000 00000000
    0, // 4909,  10110100 11001000 00000000 00000000
    1, // 4910,  01110100 11001000 00000000 00000000
    0, // 4911,  11110100 11001000 00000000 00000000
    4, // 4912,  00001100 11001000 00000000 00000000
    0, // 4913,  10001100 11001000 00000000 00000000
    1, // 4914,  01001100 11001000 00000000 00000000
    0, // 4915,  11001100 11001000 00000000 00000000
    2, // 4916,  00101100 11001000 00000000 00000000
    0, // 4917,  10101100 11001000 00000000 00000000
    1, // 4918,  01101100 11001000 00000000 00000000
    0, // 4919,  11101100 11001000 00000000 00000000
    3, // 4920,  00011100 11001000 00000000 00000000
    0, // 4921,  10011100 11001000 00000000 00000000
    1, // 4922,  01011100 11001000 00000000 00000000
    0, // 4923,  11011100 11001000 00000000 00000000
    2, // 4924,  00111100 11001000 00000000 00000000
    0, // 4925,  10111100 11001000 00000000 00000000
    1, // 4926,  01111100 11001000 00000000 00000000
    0, // 4927,  11111100 11001000 00000000 00000000
    6, // 4928,  00000010 11001000 00000000 00000000
    0, // 4929,  10000010 11001000 00000000 00000000
    1, // 4930,  01000010 11001000 00000000 00000000
    0, // 4931,  11000010 11001000 00000000 00000000
    2, // 4932,  00100010 11001000 00000000 00000000
    0, // 4933,  10100010 11001000 00000000 00000000
    1, // 4934,  01100010 11001000 00000000 00000000
    0, // 4935,  11100010 11001000 00000000 00000000
    3, // 4936,  00010010 11001000 00000000 00000000
    0, // 4937,  10010010 11001000 00000000 00000000
    1, // 4938,  01010010 11001000 00000000 00000000
    0, // 4939,  11010010 11001000 00000000 00000000
    2, // 4940,  00110010 11001000 00000000 00000000
    0, // 4941,  10110010 11001000 00000000 00000000
    1, // 4942,  01110010 11001000 00000000 00000000
    0, // 4943,  11110010 11001000 00000000 00000000
    4, // 4944,  00001010 11001000 00000000 00000000
    0, // 4945,  10001010 11001000 00000000 00000000
    1, // 4946,  01001010 11001000 00000000 00000000
    0, // 4947,  11001010 11001000 00000000 00000000
    2, // 4948,  00101010 11001000 00000000 00000000
    0, // 4949,  10101010 11001000 00000000 00000000
    1, // 4950,  01101010 11001000 00000000 00000000
    0, // 4951,  11101010 11001000 00000000 00000000
    3, // 4952,  00011010 11001000 00000000 00000000
    0, // 4953,  10011010 11001000 00000000 00000000
    1, // 4954,  01011010 11001000 00000000 00000000
    0, // 4955,  11011010 11001000 00000000 00000000
    2, // 4956,  00111010 11001000 00000000 00000000
    0, // 4957,  10111010 11001000 00000000 00000000
    1, // 4958,  01111010 11001000 00000000 00000000
    0, // 4959,  11111010 11001000 00000000 00000000
    5, // 4960,  00000110 11001000 00000000 00000000
    0, // 4961,  10000110 11001000 00000000 00000000
    1, // 4962,  01000110 11001000 00000000 00000000
    0, // 4963,  11000110 11001000 00000000 00000000
    2, // 4964,  00100110 11001000 00000000 00000000
    0, // 4965,  10100110 11001000 00000000 00000000
    1, // 4966,  01100110 11001000 00000000 00000000
    0, // 4967,  11100110 11001000 00000000 00000000
    3, // 4968,  00010110 11001000 00000000 00000000
    0, // 4969,  10010110 11001000 00000000 00000000
    1, // 4970,  01010110 11001000 00000000 00000000
    0, // 4971,  11010110 11001000 00000000 00000000
    2, // 4972,  00110110 11001000 00000000 00000000
    0, // 4973,  10110110 11001000 00000000 00000000
    1, // 4974,  01110110 11001000 00000000 00000000
    0, // 4975,  11110110 11001000 00000000 00000000
    4, // 4976,  00001110 11001000 00000000 00000000
    0, // 4977,  10001110 11001000 00000000 00000000
    1, // 4978,  01001110 11001000 00000000 00000000
    0, // 4979,  11001110 11001000 00000000 00000000
    2, // 4980,  00101110 11001000 00000000 00000000
    0, // 4981,  10101110 11001000 00000000 00000000
    1, // 4982,  01101110 11001000 00000000 00000000
    0, // 4983,  11101110 11001000 00000000 00000000
    3, // 4984,  00011110 11001000 00000000 00000000
    0, // 4985,  10011110 11001000 00000000 00000000
    1, // 4986,  01011110 11001000 00000000 00000000
    0, // 4987,  11011110 11001000 00000000 00000000
    2, // 4988,  00111110 11001000 00000000 00000000
    0, // 4989,  10111110 11001000 00000000 00000000
    1, // 4990,  01111110 11001000 00000000 00000000
    0, // 4991,  11111110 11001000 00000000 00000000
    7, // 4992,  00000001 11001000 00000000 00000000
    0, // 4993,  10000001 11001000 00000000 00000000
    1, // 4994,  01000001 11001000 00000000 00000000
    0, // 4995,  11000001 11001000 00000000 00000000
    2, // 4996,  00100001 11001000 00000000 00000000
    0, // 4997,  10100001 11001000 00000000 00000000
    1, // 4998,  01100001 11001000 00000000 00000000
    0, // 4999,  11100001 11001000 00000000 00000000
    3, // 5000,  00010001 11001000 00000000 00000000
    0, // 5001,  10010001 11001000 00000000 00000000
    1, // 5002,  01010001 11001000 00000000 00000000
    0, // 5003,  11010001 11001000 00000000 00000000
    2, // 5004,  00110001 11001000 00000000 00000000
    0, // 5005,  10110001 11001000 00000000 00000000
    1, // 5006,  01110001 11001000 00000000 00000000
    0, // 5007,  11110001 11001000 00000000 00000000
    4, // 5008,  00001001 11001000 00000000 00000000
    0, // 5009,  10001001 11001000 00000000 00000000
    1, // 5010,  01001001 11001000 00000000 00000000
    0, // 5011,  11001001 11001000 00000000 00000000
    2, // 5012,  00101001 11001000 00000000 00000000
    0, // 5013,  10101001 11001000 00000000 00000000
    1, // 5014,  01101001 11001000 00000000 00000000
    0, // 5015,  11101001 11001000 00000000 00000000
    3, // 5016,  00011001 11001000 00000000 00000000
    0, // 5017,  10011001 11001000 00000000 00000000
    1, // 5018,  01011001 11001000 00000000 00000000
    0, // 5019,  11011001 11001000 00000000 00000000
    2, // 5020,  00111001 11001000 00000000 00000000
    0, // 5021,  10111001 11001000 00000000 00000000
    1, // 5022,  01111001 11001000 00000000 00000000
    0, // 5023,  11111001 11001000 00000000 00000000
    5, // 5024,  00000101 11001000 00000000 00000000
    0, // 5025,  10000101 11001000 00000000 00000000
    1, // 5026,  01000101 11001000 00000000 00000000
    0, // 5027,  11000101 11001000 00000000 00000000
    2, // 5028,  00100101 11001000 00000000 00000000
    0, // 5029,  10100101 11001000 00000000 00000000
    1, // 5030,  01100101 11001000 00000000 00000000
    0, // 5031,  11100101 11001000 00000000 00000000
    3, // 5032,  00010101 11001000 00000000 00000000
    0, // 5033,  10010101 11001000 00000000 00000000
    1, // 5034,  01010101 11001000 00000000 00000000
    0, // 5035,  11010101 11001000 00000000 00000000
    2, // 5036,  00110101 11001000 00000000 00000000
    0, // 5037,  10110101 11001000 00000000 00000000
    1, // 5038,  01110101 11001000 00000000 00000000
    0, // 5039,  11110101 11001000 00000000 00000000
    4, // 5040,  00001101 11001000 00000000 00000000
    0, // 5041,  10001101 11001000 00000000 00000000
    1, // 5042,  01001101 11001000 00000000 00000000
    0, // 5043,  11001101 11001000 00000000 00000000
    2, // 5044,  00101101 11001000 00000000 00000000
    0, // 5045,  10101101 11001000 00000000 00000000
    1, // 5046,  01101101 11001000 00000000 00000000
    0, // 5047,  11101101 11001000 00000000 00000000
    3, // 5048,  00011101 11001000 00000000 00000000
    0, // 5049,  10011101 11001000 00000000 00000000
    1, // 5050,  01011101 11001000 00000000 00000000
    0, // 5051,  11011101 11001000 00000000 00000000
    2, // 5052,  00111101 11001000 00000000 00000000
    0, // 5053,  10111101 11001000 00000000 00000000
    1, // 5054,  01111101 11001000 00000000 00000000
    0, // 5055,  11111101 11001000 00000000 00000000
    6, // 5056,  00000011 11001000 00000000 00000000
    0, // 5057,  10000011 11001000 00000000 00000000
    1, // 5058,  01000011 11001000 00000000 00000000
    0, // 5059,  11000011 11001000 00000000 00000000
    2, // 5060,  00100011 11001000 00000000 00000000
    0, // 5061,  10100011 11001000 00000000 00000000
    1, // 5062,  01100011 11001000 00000000 00000000
    0, // 5063,  11100011 11001000 00000000 00000000
    3, // 5064,  00010011 11001000 00000000 00000000
    0, // 5065,  10010011 11001000 00000000 00000000
    1, // 5066,  01010011 11001000 00000000 00000000
    0, // 5067,  11010011 11001000 00000000 00000000
    2, // 5068,  00110011 11001000 00000000 00000000
    0, // 5069,  10110011 11001000 00000000 00000000
    1, // 5070,  01110011 11001000 00000000 00000000
    0, // 5071,  11110011 11001000 00000000 00000000
    4, // 5072,  00001011 11001000 00000000 00000000
    0, // 5073,  10001011 11001000 00000000 00000000
    1, // 5074,  01001011 11001000 00000000 00000000
    0, // 5075,  11001011 11001000 00000000 00000000
    2, // 5076,  00101011 11001000 00000000 00000000
    0, // 5077,  10101011 11001000 00000000 00000000
    1, // 5078,  01101011 11001000 00000000 00000000
    0, // 5079,  11101011 11001000 00000000 00000000
    3, // 5080,  00011011 11001000 00000000 00000000
    0, // 5081,  10011011 11001000 00000000 00000000
    1, // 5082,  01011011 11001000 00000000 00000000
    0, // 5083,  11011011 11001000 00000000 00000000
    2, // 5084,  00111011 11001000 00000000 00000000
    0, // 5085,  10111011 11001000 00000000 00000000
    1, // 5086,  01111011 11001000 00000000 00000000
    0, // 5087,  11111011 11001000 00000000 00000000
    5, // 5088,  00000111 11001000 00000000 00000000
    0, // 5089,  10000111 11001000 00000000 00000000
    1, // 5090,  01000111 11001000 00000000 00000000
    0, // 5091,  11000111 11001000 00000000 00000000
    2, // 5092,  00100111 11001000 00000000 00000000
    0, // 5093,  10100111 11001000 00000000 00000000
    1, // 5094,  01100111 11001000 00000000 00000000
    0, // 5095,  11100111 11001000 00000000 00000000
    3, // 5096,  00010111 11001000 00000000 00000000
    0, // 5097,  10010111 11001000 00000000 00000000
    1, // 5098,  01010111 11001000 00000000 00000000
    0, // 5099,  11010111 11001000 00000000 00000000
    2, // 5100,  00110111 11001000 00000000 00000000
    0, // 5101,  10110111 11001000 00000000 00000000
    1, // 5102,  01110111 11001000 00000000 00000000
    0, // 5103,  11110111 11001000 00000000 00000000
    4, // 5104,  00001111 11001000 00000000 00000000
    0, // 5105,  10001111 11001000 00000000 00000000
    1, // 5106,  01001111 11001000 00000000 00000000
    0, // 5107,  11001111 11001000 00000000 00000000
    2, // 5108,  00101111 11001000 00000000 00000000
    0, // 5109,  10101111 11001000 00000000 00000000
    1, // 5110,  01101111 11001000 00000000 00000000
    0, // 5111,  11101111 11001000 00000000 00000000
    3, // 5112,  00011111 11001000 00000000 00000000
    0, // 5113,  10011111 11001000 00000000 00000000
    1, // 5114,  01011111 11001000 00000000 00000000
    0, // 5115,  11011111 11001000 00000000 00000000
    2, // 5116,  00111111 11001000 00000000 00000000
    0, // 5117,  10111111 11001000 00000000 00000000
    1, // 5118,  01111111 11001000 00000000 00000000
    0, // 5119,  11111111 11001000 00000000 00000000
    10, // 5120,  00000000 00101000 00000000 00000000
    0, // 5121,  10000000 00101000 00000000 00000000
    1, // 5122,  01000000 00101000 00000000 00000000
    0, // 5123,  11000000 00101000 00000000 00000000
    2, // 5124,  00100000 00101000 00000000 00000000
    0, // 5125,  10100000 00101000 00000000 00000000
    1, // 5126,  01100000 00101000 00000000 00000000
    0, // 5127,  11100000 00101000 00000000 00000000
    3, // 5128,  00010000 00101000 00000000 00000000
    0, // 5129,  10010000 00101000 00000000 00000000
    1, // 5130,  01010000 00101000 00000000 00000000
    0, // 5131,  11010000 00101000 00000000 00000000
    2, // 5132,  00110000 00101000 00000000 00000000
    0, // 5133,  10110000 00101000 00000000 00000000
    1, // 5134,  01110000 00101000 00000000 00000000
    0, // 5135,  11110000 00101000 00000000 00000000
    4, // 5136,  00001000 00101000 00000000 00000000
    0, // 5137,  10001000 00101000 00000000 00000000
    1, // 5138,  01001000 00101000 00000000 00000000
    0, // 5139,  11001000 00101000 00000000 00000000
    2, // 5140,  00101000 00101000 00000000 00000000
    0, // 5141,  10101000 00101000 00000000 00000000
    1, // 5142,  01101000 00101000 00000000 00000000
    0, // 5143,  11101000 00101000 00000000 00000000
    3, // 5144,  00011000 00101000 00000000 00000000
    0, // 5145,  10011000 00101000 00000000 00000000
    1, // 5146,  01011000 00101000 00000000 00000000
    0, // 5147,  11011000 00101000 00000000 00000000
    2, // 5148,  00111000 00101000 00000000 00000000
    0, // 5149,  10111000 00101000 00000000 00000000
    1, // 5150,  01111000 00101000 00000000 00000000
    0, // 5151,  11111000 00101000 00000000 00000000
    5, // 5152,  00000100 00101000 00000000 00000000
    0, // 5153,  10000100 00101000 00000000 00000000
    1, // 5154,  01000100 00101000 00000000 00000000
    0, // 5155,  11000100 00101000 00000000 00000000
    2, // 5156,  00100100 00101000 00000000 00000000
    0, // 5157,  10100100 00101000 00000000 00000000
    1, // 5158,  01100100 00101000 00000000 00000000
    0, // 5159,  11100100 00101000 00000000 00000000
    3, // 5160,  00010100 00101000 00000000 00000000
    0, // 5161,  10010100 00101000 00000000 00000000
    1, // 5162,  01010100 00101000 00000000 00000000
    0, // 5163,  11010100 00101000 00000000 00000000
    2, // 5164,  00110100 00101000 00000000 00000000
    0, // 5165,  10110100 00101000 00000000 00000000
    1, // 5166,  01110100 00101000 00000000 00000000
    0, // 5167,  11110100 00101000 00000000 00000000
    4, // 5168,  00001100 00101000 00000000 00000000
    0, // 5169,  10001100 00101000 00000000 00000000
    1, // 5170,  01001100 00101000 00000000 00000000
    0, // 5171,  11001100 00101000 00000000 00000000
    2, // 5172,  00101100 00101000 00000000 00000000
    0, // 5173,  10101100 00101000 00000000 00000000
    1, // 5174,  01101100 00101000 00000000 00000000
    0, // 5175,  11101100 00101000 00000000 00000000
    3, // 5176,  00011100 00101000 00000000 00000000
    0, // 5177,  10011100 00101000 00000000 00000000
    1, // 5178,  01011100 00101000 00000000 00000000
    0, // 5179,  11011100 00101000 00000000 00000000
    2, // 5180,  00111100 00101000 00000000 00000000
    0, // 5181,  10111100 00101000 00000000 00000000
    1, // 5182,  01111100 00101000 00000000 00000000
    0, // 5183,  11111100 00101000 00000000 00000000
    6, // 5184,  00000010 00101000 00000000 00000000
    0, // 5185,  10000010 00101000 00000000 00000000
    1, // 5186,  01000010 00101000 00000000 00000000
    0, // 5187,  11000010 00101000 00000000 00000000
    2, // 5188,  00100010 00101000 00000000 00000000
    0, // 5189,  10100010 00101000 00000000 00000000
    1, // 5190,  01100010 00101000 00000000 00000000
    0, // 5191,  11100010 00101000 00000000 00000000
    3, // 5192,  00010010 00101000 00000000 00000000
    0, // 5193,  10010010 00101000 00000000 00000000
    1, // 5194,  01010010 00101000 00000000 00000000
    0, // 5195,  11010010 00101000 00000000 00000000
    2, // 5196,  00110010 00101000 00000000 00000000
    0, // 5197,  10110010 00101000 00000000 00000000
    1, // 5198,  01110010 00101000 00000000 00000000
    0, // 5199,  11110010 00101000 00000000 00000000
    4, // 5200,  00001010 00101000 00000000 00000000
    0, // 5201,  10001010 00101000 00000000 00000000
    1, // 5202,  01001010 00101000 00000000 00000000
    0, // 5203,  11001010 00101000 00000000 00000000
    2, // 5204,  00101010 00101000 00000000 00000000
    0, // 5205,  10101010 00101000 00000000 00000000
    1, // 5206,  01101010 00101000 00000000 00000000
    0, // 5207,  11101010 00101000 00000000 00000000
    3, // 5208,  00011010 00101000 00000000 00000000
    0, // 5209,  10011010 00101000 00000000 00000000
    1, // 5210,  01011010 00101000 00000000 00000000
    0, // 5211,  11011010 00101000 00000000 00000000
    2, // 5212,  00111010 00101000 00000000 00000000
    0, // 5213,  10111010 00101000 00000000 00000000
    1, // 5214,  01111010 00101000 00000000 00000000
    0, // 5215,  11111010 00101000 00000000 00000000
    5, // 5216,  00000110 00101000 00000000 00000000
    0, // 5217,  10000110 00101000 00000000 00000000
    1, // 5218,  01000110 00101000 00000000 00000000
    0, // 5219,  11000110 00101000 00000000 00000000
    2, // 5220,  00100110 00101000 00000000 00000000
    0, // 5221,  10100110 00101000 00000000 00000000
    1, // 5222,  01100110 00101000 00000000 00000000
    0, // 5223,  11100110 00101000 00000000 00000000
    3, // 5224,  00010110 00101000 00000000 00000000
    0, // 5225,  10010110 00101000 00000000 00000000
    1, // 5226,  01010110 00101000 00000000 00000000
    0, // 5227,  11010110 00101000 00000000 00000000
    2, // 5228,  00110110 00101000 00000000 00000000
    0, // 5229,  10110110 00101000 00000000 00000000
    1, // 5230,  01110110 00101000 00000000 00000000
    0, // 5231,  11110110 00101000 00000000 00000000
    4, // 5232,  00001110 00101000 00000000 00000000
    0, // 5233,  10001110 00101000 00000000 00000000
    1, // 5234,  01001110 00101000 00000000 00000000
    0, // 5235,  11001110 00101000 00000000 00000000
    2, // 5236,  00101110 00101000 00000000 00000000
    0, // 5237,  10101110 00101000 00000000 00000000
    1, // 5238,  01101110 00101000 00000000 00000000
    0, // 5239,  11101110 00101000 00000000 00000000
    3, // 5240,  00011110 00101000 00000000 00000000
    0, // 5241,  10011110 00101000 00000000 00000000
    1, // 5242,  01011110 00101000 00000000 00000000
    0, // 5243,  11011110 00101000 00000000 00000000
    2, // 5244,  00111110 00101000 00000000 00000000
    0, // 5245,  10111110 00101000 00000000 00000000
    1, // 5246,  01111110 00101000 00000000 00000000
    0, // 5247,  11111110 00101000 00000000 00000000
    7, // 5248,  00000001 00101000 00000000 00000000
    0, // 5249,  10000001 00101000 00000000 00000000
    1, // 5250,  01000001 00101000 00000000 00000000
    0, // 5251,  11000001 00101000 00000000 00000000
    2, // 5252,  00100001 00101000 00000000 00000000
    0, // 5253,  10100001 00101000 00000000 00000000
    1, // 5254,  01100001 00101000 00000000 00000000
    0, // 5255,  11100001 00101000 00000000 00000000
    3, // 5256,  00010001 00101000 00000000 00000000
    0, // 5257,  10010001 00101000 00000000 00000000
    1, // 5258,  01010001 00101000 00000000 00000000
    0, // 5259,  11010001 00101000 00000000 00000000
    2, // 5260,  00110001 00101000 00000000 00000000
    0, // 5261,  10110001 00101000 00000000 00000000
    1, // 5262,  01110001 00101000 00000000 00000000
    0, // 5263,  11110001 00101000 00000000 00000000
    4, // 5264,  00001001 00101000 00000000 00000000
    0, // 5265,  10001001 00101000 00000000 00000000
    1, // 5266,  01001001 00101000 00000000 00000000
    0, // 5267,  11001001 00101000 00000000 00000000
    2, // 5268,  00101001 00101000 00000000 00000000
    0, // 5269,  10101001 00101000 00000000 00000000
    1, // 5270,  01101001 00101000 00000000 00000000
    0, // 5271,  11101001 00101000 00000000 00000000
    3, // 5272,  00011001 00101000 00000000 00000000
    0, // 5273,  10011001 00101000 00000000 00000000
    1, // 5274,  01011001 00101000 00000000 00000000
    0, // 5275,  11011001 00101000 00000000 00000000
    2, // 5276,  00111001 00101000 00000000 00000000
    0, // 5277,  10111001 00101000 00000000 00000000
    1, // 5278,  01111001 00101000 00000000 00000000
    0, // 5279,  11111001 00101000 00000000 00000000
    5, // 5280,  00000101 00101000 00000000 00000000
    0, // 5281,  10000101 00101000 00000000 00000000
    1, // 5282,  01000101 00101000 00000000 00000000
    0, // 5283,  11000101 00101000 00000000 00000000
    2, // 5284,  00100101 00101000 00000000 00000000
    0, // 5285,  10100101 00101000 00000000 00000000
    1, // 5286,  01100101 00101000 00000000 00000000
    0, // 5287,  11100101 00101000 00000000 00000000
    3, // 5288,  00010101 00101000 00000000 00000000
    0, // 5289,  10010101 00101000 00000000 00000000
    1, // 5290,  01010101 00101000 00000000 00000000
    0, // 5291,  11010101 00101000 00000000 00000000
    2, // 5292,  00110101 00101000 00000000 00000000
    0, // 5293,  10110101 00101000 00000000 00000000
    1, // 5294,  01110101 00101000 00000000 00000000
    0, // 5295,  11110101 00101000 00000000 00000000
    4, // 5296,  00001101 00101000 00000000 00000000
    0, // 5297,  10001101 00101000 00000000 00000000
    1, // 5298,  01001101 00101000 00000000 00000000
    0, // 5299,  11001101 00101000 00000000 00000000
    2, // 5300,  00101101 00101000 00000000 00000000
    0, // 5301,  10101101 00101000 00000000 00000000
    1, // 5302,  01101101 00101000 00000000 00000000
    0, // 5303,  11101101 00101000 00000000 00000000
    3, // 5304,  00011101 00101000 00000000 00000000
    0, // 5305,  10011101 00101000 00000000 00000000
    1, // 5306,  01011101 00101000 00000000 00000000
    0, // 5307,  11011101 00101000 00000000 00000000
    2, // 5308,  00111101 00101000 00000000 00000000
    0, // 5309,  10111101 00101000 00000000 00000000
    1, // 5310,  01111101 00101000 00000000 00000000
    0, // 5311,  11111101 00101000 00000000 00000000
    6, // 5312,  00000011 00101000 00000000 00000000
    0, // 5313,  10000011 00101000 00000000 00000000
    1, // 5314,  01000011 00101000 00000000 00000000
    0, // 5315,  11000011 00101000 00000000 00000000
    2, // 5316,  00100011 00101000 00000000 00000000
    0, // 5317,  10100011 00101000 00000000 00000000
    1, // 5318,  01100011 00101000 00000000 00000000
    0, // 5319,  11100011 00101000 00000000 00000000
    3, // 5320,  00010011 00101000 00000000 00000000
    0, // 5321,  10010011 00101000 00000000 00000000
    1, // 5322,  01010011 00101000 00000000 00000000
    0, // 5323,  11010011 00101000 00000000 00000000
    2, // 5324,  00110011 00101000 00000000 00000000
    0, // 5325,  10110011 00101000 00000000 00000000
    1, // 5326,  01110011 00101000 00000000 00000000
    0, // 5327,  11110011 00101000 00000000 00000000
    4, // 5328,  00001011 00101000 00000000 00000000
    0, // 5329,  10001011 00101000 00000000 00000000
    1, // 5330,  01001011 00101000 00000000 00000000
    0, // 5331,  11001011 00101000 00000000 00000000
    2, // 5332,  00101011 00101000 00000000 00000000
    0, // 5333,  10101011 00101000 00000000 00000000
    1, // 5334,  01101011 00101000 00000000 00000000
    0, // 5335,  11101011 00101000 00000000 00000000
    3, // 5336,  00011011 00101000 00000000 00000000
    0, // 5337,  10011011 00101000 00000000 00000000
    1, // 5338,  01011011 00101000 00000000 00000000
    0, // 5339,  11011011 00101000 00000000 00000000
    2, // 5340,  00111011 00101000 00000000 00000000
    0, // 5341,  10111011 00101000 00000000 00000000
    1, // 5342,  01111011 00101000 00000000 00000000
    0, // 5343,  11111011 00101000 00000000 00000000
    5, // 5344,  00000111 00101000 00000000 00000000
    0, // 5345,  10000111 00101000 00000000 00000000
    1, // 5346,  01000111 00101000 00000000 00000000
    0, // 5347,  11000111 00101000 00000000 00000000
    2, // 5348,  00100111 00101000 00000000 00000000
    0, // 5349,  10100111 00101000 00000000 00000000
    1, // 5350,  01100111 00101000 00000000 00000000
    0, // 5351,  11100111 00101000 00000000 00000000
    3, // 5352,  00010111 00101000 00000000 00000000
    0, // 5353,  10010111 00101000 00000000 00000000
    1, // 5354,  01010111 00101000 00000000 00000000
    0, // 5355,  11010111 00101000 00000000 00000000
    2, // 5356,  00110111 00101000 00000000 00000000
    0, // 5357,  10110111 00101000 00000000 00000000
    1, // 5358,  01110111 00101000 00000000 00000000
    0, // 5359,  11110111 00101000 00000000 00000000
    4, // 5360,  00001111 00101000 00000000 00000000
    0, // 5361,  10001111 00101000 00000000 00000000
    1, // 5362,  01001111 00101000 00000000 00000000
    0, // 5363,  11001111 00101000 00000000 00000000
    2, // 5364,  00101111 00101000 00000000 00000000
    0, // 5365,  10101111 00101000 00000000 00000000
    1, // 5366,  01101111 00101000 00000000 00000000
    0, // 5367,  11101111 00101000 00000000 00000000
    3, // 5368,  00011111 00101000 00000000 00000000
    0, // 5369,  10011111 00101000 00000000 00000000
    1, // 5370,  01011111 00101000 00000000 00000000
    0, // 5371,  11011111 00101000 00000000 00000000
    2, // 5372,  00111111 00101000 00000000 00000000
    0, // 5373,  10111111 00101000 00000000 00000000
    1, // 5374,  01111111 00101000 00000000 00000000
    0, // 5375,  11111111 00101000 00000000 00000000
    8, // 5376,  00000000 10101000 00000000 00000000
    0, // 5377,  10000000 10101000 00000000 00000000
    1, // 5378,  01000000 10101000 00000000 00000000
    0, // 5379,  11000000 10101000 00000000 00000000
    2, // 5380,  00100000 10101000 00000000 00000000
    0, // 5381,  10100000 10101000 00000000 00000000
    1, // 5382,  01100000 10101000 00000000 00000000
    0, // 5383,  11100000 10101000 00000000 00000000
    3, // 5384,  00010000 10101000 00000000 00000000
    0, // 5385,  10010000 10101000 00000000 00000000
    1, // 5386,  01010000 10101000 00000000 00000000
    0, // 5387,  11010000 10101000 00000000 00000000
    2, // 5388,  00110000 10101000 00000000 00000000
    0, // 5389,  10110000 10101000 00000000 00000000
    1, // 5390,  01110000 10101000 00000000 00000000
    0, // 5391,  11110000 10101000 00000000 00000000
    4, // 5392,  00001000 10101000 00000000 00000000
    0, // 5393,  10001000 10101000 00000000 00000000
    1, // 5394,  01001000 10101000 00000000 00000000
    0, // 5395,  11001000 10101000 00000000 00000000
    2, // 5396,  00101000 10101000 00000000 00000000
    0, // 5397,  10101000 10101000 00000000 00000000
    1, // 5398,  01101000 10101000 00000000 00000000
    0, // 5399,  11101000 10101000 00000000 00000000
    3, // 5400,  00011000 10101000 00000000 00000000
    0, // 5401,  10011000 10101000 00000000 00000000
    1, // 5402,  01011000 10101000 00000000 00000000
    0, // 5403,  11011000 10101000 00000000 00000000
    2, // 5404,  00111000 10101000 00000000 00000000
    0, // 5405,  10111000 10101000 00000000 00000000
    1, // 5406,  01111000 10101000 00000000 00000000
    0, // 5407,  11111000 10101000 00000000 00000000
    5, // 5408,  00000100 10101000 00000000 00000000
    0, // 5409,  10000100 10101000 00000000 00000000
    1, // 5410,  01000100 10101000 00000000 00000000
    0, // 5411,  11000100 10101000 00000000 00000000
    2, // 5412,  00100100 10101000 00000000 00000000
    0, // 5413,  10100100 10101000 00000000 00000000
    1, // 5414,  01100100 10101000 00000000 00000000
    0, // 5415,  11100100 10101000 00000000 00000000
    3, // 5416,  00010100 10101000 00000000 00000000
    0, // 5417,  10010100 10101000 00000000 00000000
    1, // 5418,  01010100 10101000 00000000 00000000
    0, // 5419,  11010100 10101000 00000000 00000000
    2, // 5420,  00110100 10101000 00000000 00000000
    0, // 5421,  10110100 10101000 00000000 00000000
    1, // 5422,  01110100 10101000 00000000 00000000
    0, // 5423,  11110100 10101000 00000000 00000000
    4, // 5424,  00001100 10101000 00000000 00000000
    0, // 5425,  10001100 10101000 00000000 00000000
    1, // 5426,  01001100 10101000 00000000 00000000
    0, // 5427,  11001100 10101000 00000000 00000000
    2, // 5428,  00101100 10101000 00000000 00000000
    0, // 5429,  10101100 10101000 00000000 00000000
    1, // 5430,  01101100 10101000 00000000 00000000
    0, // 5431,  11101100 10101000 00000000 00000000
    3, // 5432,  00011100 10101000 00000000 00000000
    0, // 5433,  10011100 10101000 00000000 00000000
    1, // 5434,  01011100 10101000 00000000 00000000
    0, // 5435,  11011100 10101000 00000000 00000000
    2, // 5436,  00111100 10101000 00000000 00000000
    0, // 5437,  10111100 10101000 00000000 00000000
    1, // 5438,  01111100 10101000 00000000 00000000
    0, // 5439,  11111100 10101000 00000000 00000000
    6, // 5440,  00000010 10101000 00000000 00000000
    0, // 5441,  10000010 10101000 00000000 00000000
    1, // 5442,  01000010 10101000 00000000 00000000
    0, // 5443,  11000010 10101000 00000000 00000000
    2, // 5444,  00100010 10101000 00000000 00000000
    0, // 5445,  10100010 10101000 00000000 00000000
    1, // 5446,  01100010 10101000 00000000 00000000
    0, // 5447,  11100010 10101000 00000000 00000000
    3, // 5448,  00010010 10101000 00000000 00000000
    0, // 5449,  10010010 10101000 00000000 00000000
    1, // 5450,  01010010 10101000 00000000 00000000
    0, // 5451,  11010010 10101000 00000000 00000000
    2, // 5452,  00110010 10101000 00000000 00000000
    0, // 5453,  10110010 10101000 00000000 00000000
    1, // 5454,  01110010 10101000 00000000 00000000
    0, // 5455,  11110010 10101000 00000000 00000000
    4, // 5456,  00001010 10101000 00000000 00000000
    0, // 5457,  10001010 10101000 00000000 00000000
    1, // 5458,  01001010 10101000 00000000 00000000
    0, // 5459,  11001010 10101000 00000000 00000000
    2, // 5460,  00101010 10101000 00000000 00000000
    0, // 5461,  10101010 10101000 00000000 00000000
    1, // 5462,  01101010 10101000 00000000 00000000
    0, // 5463,  11101010 10101000 00000000 00000000
    3, // 5464,  00011010 10101000 00000000 00000000
    0, // 5465,  10011010 10101000 00000000 00000000
    1, // 5466,  01011010 10101000 00000000 00000000
    0, // 5467,  11011010 10101000 00000000 00000000
    2, // 5468,  00111010 10101000 00000000 00000000
    0, // 5469,  10111010 10101000 00000000 00000000
    1, // 5470,  01111010 10101000 00000000 00000000
    0, // 5471,  11111010 10101000 00000000 00000000
    5, // 5472,  00000110 10101000 00000000 00000000
    0, // 5473,  10000110 10101000 00000000 00000000
    1, // 5474,  01000110 10101000 00000000 00000000
    0, // 5475,  11000110 10101000 00000000 00000000
    2, // 5476,  00100110 10101000 00000000 00000000
    0, // 5477,  10100110 10101000 00000000 00000000
    1, // 5478,  01100110 10101000 00000000 00000000
    0, // 5479,  11100110 10101000 00000000 00000000
    3, // 5480,  00010110 10101000 00000000 00000000
    0, // 5481,  10010110 10101000 00000000 00000000
    1, // 5482,  01010110 10101000 00000000 00000000
    0, // 5483,  11010110 10101000 00000000 00000000
    2, // 5484,  00110110 10101000 00000000 00000000
    0, // 5485,  10110110 10101000 00000000 00000000
    1, // 5486,  01110110 10101000 00000000 00000000
    0, // 5487,  11110110 10101000 00000000 00000000
    4, // 5488,  00001110 10101000 00000000 00000000
    0, // 5489,  10001110 10101000 00000000 00000000
    1, // 5490,  01001110 10101000 00000000 00000000
    0, // 5491,  11001110 10101000 00000000 00000000
    2, // 5492,  00101110 10101000 00000000 00000000
    0, // 5493,  10101110 10101000 00000000 00000000
    1, // 5494,  01101110 10101000 00000000 00000000
    0, // 5495,  11101110 10101000 00000000 00000000
    3, // 5496,  00011110 10101000 00000000 00000000
    0, // 5497,  10011110 10101000 00000000 00000000
    1, // 5498,  01011110 10101000 00000000 00000000
    0, // 5499,  11011110 10101000 00000000 00000000
    2, // 5500,  00111110 10101000 00000000 00000000
    0, // 5501,  10111110 10101000 00000000 00000000
    1, // 5502,  01111110 10101000 00000000 00000000
    0, // 5503,  11111110 10101000 00000000 00000000
    7, // 5504,  00000001 10101000 00000000 00000000
    0, // 5505,  10000001 10101000 00000000 00000000
    1, // 5506,  01000001 10101000 00000000 00000000
    0, // 5507,  11000001 10101000 00000000 00000000
    2, // 5508,  00100001 10101000 00000000 00000000
    0, // 5509,  10100001 10101000 00000000 00000000
    1, // 5510,  01100001 10101000 00000000 00000000
    0, // 5511,  11100001 10101000 00000000 00000000
    3, // 5512,  00010001 10101000 00000000 00000000
    0, // 5513,  10010001 10101000 00000000 00000000
    1, // 5514,  01010001 10101000 00000000 00000000
    0, // 5515,  11010001 10101000 00000000 00000000
    2, // 5516,  00110001 10101000 00000000 00000000
    0, // 5517,  10110001 10101000 00000000 00000000
    1, // 5518,  01110001 10101000 00000000 00000000
    0, // 5519,  11110001 10101000 00000000 00000000
    4, // 5520,  00001001 10101000 00000000 00000000
    0, // 5521,  10001001 10101000 00000000 00000000
    1, // 5522,  01001001 10101000 00000000 00000000
    0, // 5523,  11001001 10101000 00000000 00000000
    2, // 5524,  00101001 10101000 00000000 00000000
    0, // 5525,  10101001 10101000 00000000 00000000
    1, // 5526,  01101001 10101000 00000000 00000000
    0, // 5527,  11101001 10101000 00000000 00000000
    3, // 5528,  00011001 10101000 00000000 00000000
    0, // 5529,  10011001 10101000 00000000 00000000
    1, // 5530,  01011001 10101000 00000000 00000000
    0, // 5531,  11011001 10101000 00000000 00000000
    2, // 5532,  00111001 10101000 00000000 00000000
    0, // 5533,  10111001 10101000 00000000 00000000
    1, // 5534,  01111001 10101000 00000000 00000000
    0, // 5535,  11111001 10101000 00000000 00000000
    5, // 5536,  00000101 10101000 00000000 00000000
    0, // 5537,  10000101 10101000 00000000 00000000
    1, // 5538,  01000101 10101000 00000000 00000000
    0, // 5539,  11000101 10101000 00000000 00000000
    2, // 5540,  00100101 10101000 00000000 00000000
    0, // 5541,  10100101 10101000 00000000 00000000
    1, // 5542,  01100101 10101000 00000000 00000000
    0, // 5543,  11100101 10101000 00000000 00000000
    3, // 5544,  00010101 10101000 00000000 00000000
    0, // 5545,  10010101 10101000 00000000 00000000
    1, // 5546,  01010101 10101000 00000000 00000000
    0, // 5547,  11010101 10101000 00000000 00000000
    2, // 5548,  00110101 10101000 00000000 00000000
    0, // 5549,  10110101 10101000 00000000 00000000
    1, // 5550,  01110101 10101000 00000000 00000000
    0, // 5551,  11110101 10101000 00000000 00000000
    4, // 5552,  00001101 10101000 00000000 00000000
    0, // 5553,  10001101 10101000 00000000 00000000
    1, // 5554,  01001101 10101000 00000000 00000000
    0, // 5555,  11001101 10101000 00000000 00000000
    2, // 5556,  00101101 10101000 00000000 00000000
    0, // 5557,  10101101 10101000 00000000 00000000
    1, // 5558,  01101101 10101000 00000000 00000000
    0, // 5559,  11101101 10101000 00000000 00000000
    3, // 5560,  00011101 10101000 00000000 00000000
    0, // 5561,  10011101 10101000 00000000 00000000
    1, // 5562,  01011101 10101000 00000000 00000000
    0, // 5563,  11011101 10101000 00000000 00000000
    2, // 5564,  00111101 10101000 00000000 00000000
    0, // 5565,  10111101 10101000 00000000 00000000
    1, // 5566,  01111101 10101000 00000000 00000000
    0, // 5567,  11111101 10101000 00000000 00000000
    6, // 5568,  00000011 10101000 00000000 00000000
    0, // 5569,  10000011 10101000 00000000 00000000
    1, // 5570,  01000011 10101000 00000000 00000000
    0, // 5571,  11000011 10101000 00000000 00000000
    2, // 5572,  00100011 10101000 00000000 00000000
    0, // 5573,  10100011 10101000 00000000 00000000
    1, // 5574,  01100011 10101000 00000000 00000000
    0, // 5575,  11100011 10101000 00000000 00000000
    3, // 5576,  00010011 10101000 00000000 00000000
    0, // 5577,  10010011 10101000 00000000 00000000
    1, // 5578,  01010011 10101000 00000000 00000000
    0, // 5579,  11010011 10101000 00000000 00000000
    2, // 5580,  00110011 10101000 00000000 00000000
    0, // 5581,  10110011 10101000 00000000 00000000
    1, // 5582,  01110011 10101000 00000000 00000000
    0, // 5583,  11110011 10101000 00000000 00000000
    4, // 5584,  00001011 10101000 00000000 00000000
    0, // 5585,  10001011 10101000 00000000 00000000
    1, // 5586,  01001011 10101000 00000000 00000000
    0, // 5587,  11001011 10101000 00000000 00000000
    2, // 5588,  00101011 10101000 00000000 00000000
    0, // 5589,  10101011 10101000 00000000 00000000
    1, // 5590,  01101011 10101000 00000000 00000000
    0, // 5591,  11101011 10101000 00000000 00000000
    3, // 5592,  00011011 10101000 00000000 00000000
    0, // 5593,  10011011 10101000 00000000 00000000
    1, // 5594,  01011011 10101000 00000000 00000000
    0, // 5595,  11011011 10101000 00000000 00000000
    2, // 5596,  00111011 10101000 00000000 00000000
    0, // 5597,  10111011 10101000 00000000 00000000
    1, // 5598,  01111011 10101000 00000000 00000000
    0, // 5599,  11111011 10101000 00000000 00000000
    5, // 5600,  00000111 10101000 00000000 00000000
    0, // 5601,  10000111 10101000 00000000 00000000
    1, // 5602,  01000111 10101000 00000000 00000000
    0, // 5603,  11000111 10101000 00000000 00000000
    2, // 5604,  00100111 10101000 00000000 00000000
    0, // 5605,  10100111 10101000 00000000 00000000
    1, // 5606,  01100111 10101000 00000000 00000000
    0, // 5607,  11100111 10101000 00000000 00000000
    3, // 5608,  00010111 10101000 00000000 00000000
    0, // 5609,  10010111 10101000 00000000 00000000
    1, // 5610,  01010111 10101000 00000000 00000000
    0, // 5611,  11010111 10101000 00000000 00000000
    2, // 5612,  00110111 10101000 00000000 00000000
    0, // 5613,  10110111 10101000 00000000 00000000
    1, // 5614,  01110111 10101000 00000000 00000000
    0, // 5615,  11110111 10101000 00000000 00000000
    4, // 5616,  00001111 10101000 00000000 00000000
    0, // 5617,  10001111 10101000 00000000 00000000
    1, // 5618,  01001111 10101000 00000000 00000000
    0, // 5619,  11001111 10101000 00000000 00000000
    2, // 5620,  00101111 10101000 00000000 00000000
    0, // 5621,  10101111 10101000 00000000 00000000
    1, // 5622,  01101111 10101000 00000000 00000000
    0, // 5623,  11101111 10101000 00000000 00000000
    3, // 5624,  00011111 10101000 00000000 00000000
    0, // 5625,  10011111 10101000 00000000 00000000
    1, // 5626,  01011111 10101000 00000000 00000000
    0, // 5627,  11011111 10101000 00000000 00000000
    2, // 5628,  00111111 10101000 00000000 00000000
    0, // 5629,  10111111 10101000 00000000 00000000
    1, // 5630,  01111111 10101000 00000000 00000000
    0, // 5631,  11111111 10101000 00000000 00000000
    9, // 5632,  00000000 01101000 00000000 00000000
    0, // 5633,  10000000 01101000 00000000 00000000
    1, // 5634,  01000000 01101000 00000000 00000000
    0, // 5635,  11000000 01101000 00000000 00000000
    2, // 5636,  00100000 01101000 00000000 00000000
    0, // 5637,  10100000 01101000 00000000 00000000
    1, // 5638,  01100000 01101000 00000000 00000000
    0, // 5639,  11100000 01101000 00000000 00000000
    3, // 5640,  00010000 01101000 00000000 00000000
    0, // 5641,  10010000 01101000 00000000 00000000
    1, // 5642,  01010000 01101000 00000000 00000000
    0, // 5643,  11010000 01101000 00000000 00000000
    2, // 5644,  00110000 01101000 00000000 00000000
    0, // 5645,  10110000 01101000 00000000 00000000
    1, // 5646,  01110000 01101000 00000000 00000000
    0, // 5647,  11110000 01101000 00000000 00000000
    4, // 5648,  00001000 01101000 00000000 00000000
    0, // 5649,  10001000 01101000 00000000 00000000
    1, // 5650,  01001000 01101000 00000000 00000000
    0, // 5651,  11001000 01101000 00000000 00000000
    2, // 5652,  00101000 01101000 00000000 00000000
    0, // 5653,  10101000 01101000 00000000 00000000
    1, // 5654,  01101000 01101000 00000000 00000000
    0, // 5655,  11101000 01101000 00000000 00000000
    3, // 5656,  00011000 01101000 00000000 00000000
    0, // 5657,  10011000 01101000 00000000 00000000
    1, // 5658,  01011000 01101000 00000000 00000000
    0, // 5659,  11011000 01101000 00000000 00000000
    2, // 5660,  00111000 01101000 00000000 00000000
    0, // 5661,  10111000 01101000 00000000 00000000
    1, // 5662,  01111000 01101000 00000000 00000000
    0, // 5663,  11111000 01101000 00000000 00000000
    5, // 5664,  00000100 01101000 00000000 00000000
    0, // 5665,  10000100 01101000 00000000 00000000
    1, // 5666,  01000100 01101000 00000000 00000000
    0, // 5667,  11000100 01101000 00000000 00000000
    2, // 5668,  00100100 01101000 00000000 00000000
    0, // 5669,  10100100 01101000 00000000 00000000
    1, // 5670,  01100100 01101000 00000000 00000000
    0, // 5671,  11100100 01101000 00000000 00000000
    3, // 5672,  00010100 01101000 00000000 00000000
    0, // 5673,  10010100 01101000 00000000 00000000
    1, // 5674,  01010100 01101000 00000000 00000000
    0, // 5675,  11010100 01101000 00000000 00000000
    2, // 5676,  00110100 01101000 00000000 00000000
    0, // 5677,  10110100 01101000 00000000 00000000
    1, // 5678,  01110100 01101000 00000000 00000000
    0, // 5679,  11110100 01101000 00000000 00000000
    4, // 5680,  00001100 01101000 00000000 00000000
    0, // 5681,  10001100 01101000 00000000 00000000
    1, // 5682,  01001100 01101000 00000000 00000000
    0, // 5683,  11001100 01101000 00000000 00000000
    2, // 5684,  00101100 01101000 00000000 00000000
    0, // 5685,  10101100 01101000 00000000 00000000
    1, // 5686,  01101100 01101000 00000000 00000000
    0, // 5687,  11101100 01101000 00000000 00000000
    3, // 5688,  00011100 01101000 00000000 00000000
    0, // 5689,  10011100 01101000 00000000 00000000
    1, // 5690,  01011100 01101000 00000000 00000000
    0, // 5691,  11011100 01101000 00000000 00000000
    2, // 5692,  00111100 01101000 00000000 00000000
    0, // 5693,  10111100 01101000 00000000 00000000
    1, // 5694,  01111100 01101000 00000000 00000000
    0, // 5695,  11111100 01101000 00000000 00000000
    6, // 5696,  00000010 01101000 00000000 00000000
    0, // 5697,  10000010 01101000 00000000 00000000
    1, // 5698,  01000010 01101000 00000000 00000000
    0, // 5699,  11000010 01101000 00000000 00000000
    2, // 5700,  00100010 01101000 00000000 00000000
    0, // 5701,  10100010 01101000 00000000 00000000
    1, // 5702,  01100010 01101000 00000000 00000000
    0, // 5703,  11100010 01101000 00000000 00000000
    3, // 5704,  00010010 01101000 00000000 00000000
    0, // 5705,  10010010 01101000 00000000 00000000
    1, // 5706,  01010010 01101000 00000000 00000000
    0, // 5707,  11010010 01101000 00000000 00000000
    2, // 5708,  00110010 01101000 00000000 00000000
    0, // 5709,  10110010 01101000 00000000 00000000
    1, // 5710,  01110010 01101000 00000000 00000000
    0, // 5711,  11110010 01101000 00000000 00000000
    4, // 5712,  00001010 01101000 00000000 00000000
    0, // 5713,  10001010 01101000 00000000 00000000
    1, // 5714,  01001010 01101000 00000000 00000000
    0, // 5715,  11001010 01101000 00000000 00000000
    2, // 5716,  00101010 01101000 00000000 00000000
    0, // 5717,  10101010 01101000 00000000 00000000
    1, // 5718,  01101010 01101000 00000000 00000000
    0, // 5719,  11101010 01101000 00000000 00000000
    3, // 5720,  00011010 01101000 00000000 00000000
    0, // 5721,  10011010 01101000 00000000 00000000
    1, // 5722,  01011010 01101000 00000000 00000000
    0, // 5723,  11011010 01101000 00000000 00000000
    2, // 5724,  00111010 01101000 00000000 00000000
    0, // 5725,  10111010 01101000 00000000 00000000
    1, // 5726,  01111010 01101000 00000000 00000000
    0, // 5727,  11111010 01101000 00000000 00000000
    5, // 5728,  00000110 01101000 00000000 00000000
    0, // 5729,  10000110 01101000 00000000 00000000
    1, // 5730,  01000110 01101000 00000000 00000000
    0, // 5731,  11000110 01101000 00000000 00000000
    2, // 5732,  00100110 01101000 00000000 00000000
    0, // 5733,  10100110 01101000 00000000 00000000
    1, // 5734,  01100110 01101000 00000000 00000000
    0, // 5735,  11100110 01101000 00000000 00000000
    3, // 5736,  00010110 01101000 00000000 00000000
    0, // 5737,  10010110 01101000 00000000 00000000
    1, // 5738,  01010110 01101000 00000000 00000000
    0, // 5739,  11010110 01101000 00000000 00000000
    2, // 5740,  00110110 01101000 00000000 00000000
    0, // 5741,  10110110 01101000 00000000 00000000
    1, // 5742,  01110110 01101000 00000000 00000000
    0, // 5743,  11110110 01101000 00000000 00000000
    4, // 5744,  00001110 01101000 00000000 00000000
    0, // 5745,  10001110 01101000 00000000 00000000
    1, // 5746,  01001110 01101000 00000000 00000000
    0, // 5747,  11001110 01101000 00000000 00000000
    2, // 5748,  00101110 01101000 00000000 00000000
    0, // 5749,  10101110 01101000 00000000 00000000
    1, // 5750,  01101110 01101000 00000000 00000000
    0, // 5751,  11101110 01101000 00000000 00000000
    3, // 5752,  00011110 01101000 00000000 00000000
    0, // 5753,  10011110 01101000 00000000 00000000
    1, // 5754,  01011110 01101000 00000000 00000000
    0, // 5755,  11011110 01101000 00000000 00000000
    2, // 5756,  00111110 01101000 00000000 00000000
    0, // 5757,  10111110 01101000 00000000 00000000
    1, // 5758,  01111110 01101000 00000000 00000000
    0, // 5759,  11111110 01101000 00000000 00000000
    7, // 5760,  00000001 01101000 00000000 00000000
    0, // 5761,  10000001 01101000 00000000 00000000
    1, // 5762,  01000001 01101000 00000000 00000000
    0, // 5763,  11000001 01101000 00000000 00000000
    2, // 5764,  00100001 01101000 00000000 00000000
    0, // 5765,  10100001 01101000 00000000 00000000
    1, // 5766,  01100001 01101000 00000000 00000000
    0, // 5767,  11100001 01101000 00000000 00000000
    3, // 5768,  00010001 01101000 00000000 00000000
    0, // 5769,  10010001 01101000 00000000 00000000
    1, // 5770,  01010001 01101000 00000000 00000000
    0, // 5771,  11010001 01101000 00000000 00000000
    2, // 5772,  00110001 01101000 00000000 00000000
    0, // 5773,  10110001 01101000 00000000 00000000
    1, // 5774,  01110001 01101000 00000000 00000000
    0, // 5775,  11110001 01101000 00000000 00000000
    4, // 5776,  00001001 01101000 00000000 00000000
    0, // 5777,  10001001 01101000 00000000 00000000
    1, // 5778,  01001001 01101000 00000000 00000000
    0, // 5779,  11001001 01101000 00000000 00000000
    2, // 5780,  00101001 01101000 00000000 00000000
    0, // 5781,  10101001 01101000 00000000 00000000
    1, // 5782,  01101001 01101000 00000000 00000000
    0, // 5783,  11101001 01101000 00000000 00000000
    3, // 5784,  00011001 01101000 00000000 00000000
    0, // 5785,  10011001 01101000 00000000 00000000
    1, // 5786,  01011001 01101000 00000000 00000000
    0, // 5787,  11011001 01101000 00000000 00000000
    2, // 5788,  00111001 01101000 00000000 00000000
    0, // 5789,  10111001 01101000 00000000 00000000
    1, // 5790,  01111001 01101000 00000000 00000000
    0, // 5791,  11111001 01101000 00000000 00000000
    5, // 5792,  00000101 01101000 00000000 00000000
    0, // 5793,  10000101 01101000 00000000 00000000
    1, // 5794,  01000101 01101000 00000000 00000000
    0, // 5795,  11000101 01101000 00000000 00000000
    2, // 5796,  00100101 01101000 00000000 00000000
    0, // 5797,  10100101 01101000 00000000 00000000
    1, // 5798,  01100101 01101000 00000000 00000000
    0, // 5799,  11100101 01101000 00000000 00000000
    3, // 5800,  00010101 01101000 00000000 00000000
    0, // 5801,  10010101 01101000 00000000 00000000
    1, // 5802,  01010101 01101000 00000000 00000000
    0, // 5803,  11010101 01101000 00000000 00000000
    2, // 5804,  00110101 01101000 00000000 00000000
    0, // 5805,  10110101 01101000 00000000 00000000
    1, // 5806,  01110101 01101000 00000000 00000000
    0, // 5807,  11110101 01101000 00000000 00000000
    4, // 5808,  00001101 01101000 00000000 00000000
    0, // 5809,  10001101 01101000 00000000 00000000
    1, // 5810,  01001101 01101000 00000000 00000000
    0, // 5811,  11001101 01101000 00000000 00000000
    2, // 5812,  00101101 01101000 00000000 00000000
    0, // 5813,  10101101 01101000 00000000 00000000
    1, // 5814,  01101101 01101000 00000000 00000000
    0, // 5815,  11101101 01101000 00000000 00000000
    3, // 5816,  00011101 01101000 00000000 00000000
    0, // 5817,  10011101 01101000 00000000 00000000
    1, // 5818,  01011101 01101000 00000000 00000000
    0, // 5819,  11011101 01101000 00000000 00000000
    2, // 5820,  00111101 01101000 00000000 00000000
    0, // 5821,  10111101 01101000 00000000 00000000
    1, // 5822,  01111101 01101000 00000000 00000000
    0, // 5823,  11111101 01101000 00000000 00000000
    6, // 5824,  00000011 01101000 00000000 00000000
    0, // 5825,  10000011 01101000 00000000 00000000
    1, // 5826,  01000011 01101000 00000000 00000000
    0, // 5827,  11000011 01101000 00000000 00000000
    2, // 5828,  00100011 01101000 00000000 00000000
    0, // 5829,  10100011 01101000 00000000 00000000
    1, // 5830,  01100011 01101000 00000000 00000000
    0, // 5831,  11100011 01101000 00000000 00000000
    3, // 5832,  00010011 01101000 00000000 00000000
    0, // 5833,  10010011 01101000 00000000 00000000
    1, // 5834,  01010011 01101000 00000000 00000000
    0, // 5835,  11010011 01101000 00000000 00000000
    2, // 5836,  00110011 01101000 00000000 00000000
    0, // 5837,  10110011 01101000 00000000 00000000
    1, // 5838,  01110011 01101000 00000000 00000000
    0, // 5839,  11110011 01101000 00000000 00000000
    4, // 5840,  00001011 01101000 00000000 00000000
    0, // 5841,  10001011 01101000 00000000 00000000
    1, // 5842,  01001011 01101000 00000000 00000000
    0, // 5843,  11001011 01101000 00000000 00000000
    2, // 5844,  00101011 01101000 00000000 00000000
    0, // 5845,  10101011 01101000 00000000 00000000
    1, // 5846,  01101011 01101000 00000000 00000000
    0, // 5847,  11101011 01101000 00000000 00000000
    3, // 5848,  00011011 01101000 00000000 00000000
    0, // 5849,  10011011 01101000 00000000 00000000
    1, // 5850,  01011011 01101000 00000000 00000000
    0, // 5851,  11011011 01101000 00000000 00000000
    2, // 5852,  00111011 01101000 00000000 00000000
    0, // 5853,  10111011 01101000 00000000 00000000
    1, // 5854,  01111011 01101000 00000000 00000000
    0, // 5855,  11111011 01101000 00000000 00000000
    5, // 5856,  00000111 01101000 00000000 00000000
    0, // 5857,  10000111 01101000 00000000 00000000
    1, // 5858,  01000111 01101000 00000000 00000000
    0, // 5859,  11000111 01101000 00000000 00000000
    2, // 5860,  00100111 01101000 00000000 00000000
    0, // 5861,  10100111 01101000 00000000 00000000
    1, // 5862,  01100111 01101000 00000000 00000000
    0, // 5863,  11100111 01101000 00000000 00000000
    3, // 5864,  00010111 01101000 00000000 00000000
    0, // 5865,  10010111 01101000 00000000 00000000
    1, // 5866,  01010111 01101000 00000000 00000000
    0, // 5867,  11010111 01101000 00000000 00000000
    2, // 5868,  00110111 01101000 00000000 00000000
    0, // 5869,  10110111 01101000 00000000 00000000
    1, // 5870,  01110111 01101000 00000000 00000000
    0, // 5871,  11110111 01101000 00000000 00000000
    4, // 5872,  00001111 01101000 00000000 00000000
    0, // 5873,  10001111 01101000 00000000 00000000
    1, // 5874,  01001111 01101000 00000000 00000000
    0, // 5875,  11001111 01101000 00000000 00000000
    2, // 5876,  00101111 01101000 00000000 00000000
    0, // 5877,  10101111 01101000 00000000 00000000
    1, // 5878,  01101111 01101000 00000000 00000000
    0, // 5879,  11101111 01101000 00000000 00000000
    3, // 5880,  00011111 01101000 00000000 00000000
    0, // 5881,  10011111 01101000 00000000 00000000
    1, // 5882,  01011111 01101000 00000000 00000000
    0, // 5883,  11011111 01101000 00000000 00000000
    2, // 5884,  00111111 01101000 00000000 00000000
    0, // 5885,  10111111 01101000 00000000 00000000
    1, // 5886,  01111111 01101000 00000000 00000000
    0, // 5887,  11111111 01101000 00000000 00000000
    8, // 5888,  00000000 11101000 00000000 00000000
    0, // 5889,  10000000 11101000 00000000 00000000
    1, // 5890,  01000000 11101000 00000000 00000000
    0, // 5891,  11000000 11101000 00000000 00000000
    2, // 5892,  00100000 11101000 00000000 00000000
    0, // 5893,  10100000 11101000 00000000 00000000
    1, // 5894,  01100000 11101000 00000000 00000000
    0, // 5895,  11100000 11101000 00000000 00000000
    3, // 5896,  00010000 11101000 00000000 00000000
    0, // 5897,  10010000 11101000 00000000 00000000
    1, // 5898,  01010000 11101000 00000000 00000000
    0, // 5899,  11010000 11101000 00000000 00000000
    2, // 5900,  00110000 11101000 00000000 00000000
    0, // 5901,  10110000 11101000 00000000 00000000
    1, // 5902,  01110000 11101000 00000000 00000000
    0, // 5903,  11110000 11101000 00000000 00000000
    4, // 5904,  00001000 11101000 00000000 00000000
    0, // 5905,  10001000 11101000 00000000 00000000
    1, // 5906,  01001000 11101000 00000000 00000000
    0, // 5907,  11001000 11101000 00000000 00000000
    2, // 5908,  00101000 11101000 00000000 00000000
    0, // 5909,  10101000 11101000 00000000 00000000
    1, // 5910,  01101000 11101000 00000000 00000000
    0, // 5911,  11101000 11101000 00000000 00000000
    3, // 5912,  00011000 11101000 00000000 00000000
    0, // 5913,  10011000 11101000 00000000 00000000
    1, // 5914,  01011000 11101000 00000000 00000000
    0, // 5915,  11011000 11101000 00000000 00000000
    2, // 5916,  00111000 11101000 00000000 00000000
    0, // 5917,  10111000 11101000 00000000 00000000
    1, // 5918,  01111000 11101000 00000000 00000000
    0, // 5919,  11111000 11101000 00000000 00000000
    5, // 5920,  00000100 11101000 00000000 00000000
    0, // 5921,  10000100 11101000 00000000 00000000
    1, // 5922,  01000100 11101000 00000000 00000000
    0, // 5923,  11000100 11101000 00000000 00000000
    2, // 5924,  00100100 11101000 00000000 00000000
    0, // 5925,  10100100 11101000 00000000 00000000
    1, // 5926,  01100100 11101000 00000000 00000000
    0, // 5927,  11100100 11101000 00000000 00000000
    3, // 5928,  00010100 11101000 00000000 00000000
    0, // 5929,  10010100 11101000 00000000 00000000
    1, // 5930,  01010100 11101000 00000000 00000000
    0, // 5931,  11010100 11101000 00000000 00000000
    2, // 5932,  00110100 11101000 00000000 00000000
    0, // 5933,  10110100 11101000 00000000 00000000
    1, // 5934,  01110100 11101000 00000000 00000000
    0, // 5935,  11110100 11101000 00000000 00000000
    4, // 5936,  00001100 11101000 00000000 00000000
    0, // 5937,  10001100 11101000 00000000 00000000
    1, // 5938,  01001100 11101000 00000000 00000000
    0, // 5939,  11001100 11101000 00000000 00000000
    2, // 5940,  00101100 11101000 00000000 00000000
    0, // 5941,  10101100 11101000 00000000 00000000
    1, // 5942,  01101100 11101000 00000000 00000000
    0, // 5943,  11101100 11101000 00000000 00000000
    3, // 5944,  00011100 11101000 00000000 00000000
    0, // 5945,  10011100 11101000 00000000 00000000
    1, // 5946,  01011100 11101000 00000000 00000000
    0, // 5947,  11011100 11101000 00000000 00000000
    2, // 5948,  00111100 11101000 00000000 00000000
    0, // 5949,  10111100 11101000 00000000 00000000
    1, // 5950,  01111100 11101000 00000000 00000000
    0, // 5951,  11111100 11101000 00000000 00000000
    6, // 5952,  00000010 11101000 00000000 00000000
    0, // 5953,  10000010 11101000 00000000 00000000
    1, // 5954,  01000010 11101000 00000000 00000000
    0, // 5955,  11000010 11101000 00000000 00000000
    2, // 5956,  00100010 11101000 00000000 00000000
    0, // 5957,  10100010 11101000 00000000 00000000
    1, // 5958,  01100010 11101000 00000000 00000000
    0, // 5959,  11100010 11101000 00000000 00000000
    3, // 5960,  00010010 11101000 00000000 00000000
    0, // 5961,  10010010 11101000 00000000 00000000
    1, // 5962,  01010010 11101000 00000000 00000000
    0, // 5963,  11010010 11101000 00000000 00000000
    2, // 5964,  00110010 11101000 00000000 00000000
    0, // 5965,  10110010 11101000 00000000 00000000
    1, // 5966,  01110010 11101000 00000000 00000000
    0, // 5967,  11110010 11101000 00000000 00000000
    4, // 5968,  00001010 11101000 00000000 00000000
    0, // 5969,  10001010 11101000 00000000 00000000
    1, // 5970,  01001010 11101000 00000000 00000000
    0, // 5971,  11001010 11101000 00000000 00000000
    2, // 5972,  00101010 11101000 00000000 00000000
    0, // 5973,  10101010 11101000 00000000 00000000
    1, // 5974,  01101010 11101000 00000000 00000000
    0, // 5975,  11101010 11101000 00000000 00000000
    3, // 5976,  00011010 11101000 00000000 00000000
    0, // 5977,  10011010 11101000 00000000 00000000
    1, // 5978,  01011010 11101000 00000000 00000000
    0, // 5979,  11011010 11101000 00000000 00000000
    2, // 5980,  00111010 11101000 00000000 00000000
    0, // 5981,  10111010 11101000 00000000 00000000
    1, // 5982,  01111010 11101000 00000000 00000000
    0, // 5983,  11111010 11101000 00000000 00000000
    5, // 5984,  00000110 11101000 00000000 00000000
    0, // 5985,  10000110 11101000 00000000 00000000
    1, // 5986,  01000110 11101000 00000000 00000000
    0, // 5987,  11000110 11101000 00000000 00000000
    2, // 5988,  00100110 11101000 00000000 00000000
    0, // 5989,  10100110 11101000 00000000 00000000
    1, // 5990,  01100110 11101000 00000000 00000000
    0, // 5991,  11100110 11101000 00000000 00000000
    3, // 5992,  00010110 11101000 00000000 00000000
    0, // 5993,  10010110 11101000 00000000 00000000
    1, // 5994,  01010110 11101000 00000000 00000000
    0, // 5995,  11010110 11101000 00000000 00000000
    2, // 5996,  00110110 11101000 00000000 00000000
    0, // 5997,  10110110 11101000 00000000 00000000
    1, // 5998,  01110110 11101000 00000000 00000000
    0, // 5999,  11110110 11101000 00000000 00000000
    4, // 6000,  00001110 11101000 00000000 00000000
    0, // 6001,  10001110 11101000 00000000 00000000
    1, // 6002,  01001110 11101000 00000000 00000000
    0, // 6003,  11001110 11101000 00000000 00000000
    2, // 6004,  00101110 11101000 00000000 00000000
    0, // 6005,  10101110 11101000 00000000 00000000
    1, // 6006,  01101110 11101000 00000000 00000000
    0, // 6007,  11101110 11101000 00000000 00000000
    3, // 6008,  00011110 11101000 00000000 00000000
    0, // 6009,  10011110 11101000 00000000 00000000
    1, // 6010,  01011110 11101000 00000000 00000000
    0, // 6011,  11011110 11101000 00000000 00000000
    2, // 6012,  00111110 11101000 00000000 00000000
    0, // 6013,  10111110 11101000 00000000 00000000
    1, // 6014,  01111110 11101000 00000000 00000000
    0, // 6015,  11111110 11101000 00000000 00000000
    7, // 6016,  00000001 11101000 00000000 00000000
    0, // 6017,  10000001 11101000 00000000 00000000
    1, // 6018,  01000001 11101000 00000000 00000000
    0, // 6019,  11000001 11101000 00000000 00000000
    2, // 6020,  00100001 11101000 00000000 00000000
    0, // 6021,  10100001 11101000 00000000 00000000
    1, // 6022,  01100001 11101000 00000000 00000000
    0, // 6023,  11100001 11101000 00000000 00000000
    3, // 6024,  00010001 11101000 00000000 00000000
    0, // 6025,  10010001 11101000 00000000 00000000
    1, // 6026,  01010001 11101000 00000000 00000000
    0, // 6027,  11010001 11101000 00000000 00000000
    2, // 6028,  00110001 11101000 00000000 00000000
    0, // 6029,  10110001 11101000 00000000 00000000
    1, // 6030,  01110001 11101000 00000000 00000000
    0, // 6031,  11110001 11101000 00000000 00000000
    4, // 6032,  00001001 11101000 00000000 00000000
    0, // 6033,  10001001 11101000 00000000 00000000
    1, // 6034,  01001001 11101000 00000000 00000000
    0, // 6035,  11001001 11101000 00000000 00000000
    2, // 6036,  00101001 11101000 00000000 00000000
    0, // 6037,  10101001 11101000 00000000 00000000
    1, // 6038,  01101001 11101000 00000000 00000000
    0, // 6039,  11101001 11101000 00000000 00000000
    3, // 6040,  00011001 11101000 00000000 00000000
    0, // 6041,  10011001 11101000 00000000 00000000
    1, // 6042,  01011001 11101000 00000000 00000000
    0, // 6043,  11011001 11101000 00000000 00000000
    2, // 6044,  00111001 11101000 00000000 00000000
    0, // 6045,  10111001 11101000 00000000 00000000
    1, // 6046,  01111001 11101000 00000000 00000000
    0, // 6047,  11111001 11101000 00000000 00000000
    5, // 6048,  00000101 11101000 00000000 00000000
    0, // 6049,  10000101 11101000 00000000 00000000
    1, // 6050,  01000101 11101000 00000000 00000000
    0, // 6051,  11000101 11101000 00000000 00000000
    2, // 6052,  00100101 11101000 00000000 00000000
    0, // 6053,  10100101 11101000 00000000 00000000
    1, // 6054,  01100101 11101000 00000000 00000000
    0, // 6055,  11100101 11101000 00000000 00000000
    3, // 6056,  00010101 11101000 00000000 00000000
    0, // 6057,  10010101 11101000 00000000 00000000
    1, // 6058,  01010101 11101000 00000000 00000000
    0, // 6059,  11010101 11101000 00000000 00000000
    2, // 6060,  00110101 11101000 00000000 00000000
    0, // 6061,  10110101 11101000 00000000 00000000
    1, // 6062,  01110101 11101000 00000000 00000000
    0, // 6063,  11110101 11101000 00000000 00000000
    4, // 6064,  00001101 11101000 00000000 00000000
    0, // 6065,  10001101 11101000 00000000 00000000
    1, // 6066,  01001101 11101000 00000000 00000000
    0, // 6067,  11001101 11101000 00000000 00000000
    2, // 6068,  00101101 11101000 00000000 00000000
    0, // 6069,  10101101 11101000 00000000 00000000
    1, // 6070,  01101101 11101000 00000000 00000000
    0, // 6071,  11101101 11101000 00000000 00000000
    3, // 6072,  00011101 11101000 00000000 00000000
    0, // 6073,  10011101 11101000 00000000 00000000
    1, // 6074,  01011101 11101000 00000000 00000000
    0, // 6075,  11011101 11101000 00000000 00000000
    2, // 6076,  00111101 11101000 00000000 00000000
    0, // 6077,  10111101 11101000 00000000 00000000
    1, // 6078,  01111101 11101000 00000000 00000000
    0, // 6079,  11111101 11101000 00000000 00000000
    6, // 6080,  00000011 11101000 00000000 00000000
    0, // 6081,  10000011 11101000 00000000 00000000
    1, // 6082,  01000011 11101000 00000000 00000000
    0, // 6083,  11000011 11101000 00000000 00000000
    2, // 6084,  00100011 11101000 00000000 00000000
    0, // 6085,  10100011 11101000 00000000 00000000
    1, // 6086,  01100011 11101000 00000000 00000000
    0, // 6087,  11100011 11101000 00000000 00000000
    3, // 6088,  00010011 11101000 00000000 00000000
    0, // 6089,  10010011 11101000 00000000 00000000
    1, // 6090,  01010011 11101000 00000000 00000000
    0, // 6091,  11010011 11101000 00000000 00000000
    2, // 6092,  00110011 11101000 00000000 00000000
    0, // 6093,  10110011 11101000 00000000 00000000
    1, // 6094,  01110011 11101000 00000000 00000000
    0, // 6095,  11110011 11101000 00000000 00000000
    4, // 6096,  00001011 11101000 00000000 00000000
    0, // 6097,  10001011 11101000 00000000 00000000
    1, // 6098,  01001011 11101000 00000000 00000000
    0, // 6099,  11001011 11101000 00000000 00000000
    2, // 6100,  00101011 11101000 00000000 00000000
    0, // 6101,  10101011 11101000 00000000 00000000
    1, // 6102,  01101011 11101000 00000000 00000000
    0, // 6103,  11101011 11101000 00000000 00000000
    3, // 6104,  00011011 11101000 00000000 00000000
    0, // 6105,  10011011 11101000 00000000 00000000
    1, // 6106,  01011011 11101000 00000000 00000000
    0, // 6107,  11011011 11101000 00000000 00000000
    2, // 6108,  00111011 11101000 00000000 00000000
    0, // 6109,  10111011 11101000 00000000 00000000
    1, // 6110,  01111011 11101000 00000000 00000000
    0, // 6111,  11111011 11101000 00000000 00000000
    5, // 6112,  00000111 11101000 00000000 00000000
    0, // 6113,  10000111 11101000 00000000 00000000
    1, // 6114,  01000111 11101000 00000000 00000000
    0, // 6115,  11000111 11101000 00000000 00000000
    2, // 6116,  00100111 11101000 00000000 00000000
    0, // 6117,  10100111 11101000 00000000 00000000
    1, // 6118,  01100111 11101000 00000000 00000000
    0, // 6119,  11100111 11101000 00000000 00000000
    3, // 6120,  00010111 11101000 00000000 00000000
    0, // 6121,  10010111 11101000 00000000 00000000
    1, // 6122,  01010111 11101000 00000000 00000000
    0, // 6123,  11010111 11101000 00000000 00000000
    2, // 6124,  00110111 11101000 00000000 00000000
    0, // 6125,  10110111 11101000 00000000 00000000
    1, // 6126,  01110111 11101000 00000000 00000000
    0, // 6127,  11110111 11101000 00000000 00000000
    4, // 6128,  00001111 11101000 00000000 00000000
    0, // 6129,  10001111 11101000 00000000 00000000
    1, // 6130,  01001111 11101000 00000000 00000000
    0, // 6131,  11001111 11101000 00000000 00000000
    2, // 6132,  00101111 11101000 00000000 00000000
    0, // 6133,  10101111 11101000 00000000 00000000
    1, // 6134,  01101111 11101000 00000000 00000000
    0, // 6135,  11101111 11101000 00000000 00000000
    3, // 6136,  00011111 11101000 00000000 00000000
    0, // 6137,  10011111 11101000 00000000 00000000
    1, // 6138,  01011111 11101000 00000000 00000000
    0, // 6139,  11011111 11101000 00000000 00000000
    2, // 6140,  00111111 11101000 00000000 00000000
    0, // 6141,  10111111 11101000 00000000 00000000
    1, // 6142,  01111111 11101000 00000000 00000000
    0, // 6143,  11111111 11101000 00000000 00000000
    11, // 6144,  00000000 00011000 00000000 00000000
    0, // 6145,  10000000 00011000 00000000 00000000
    1, // 6146,  01000000 00011000 00000000 00000000
    0, // 6147,  11000000 00011000 00000000 00000000
    2, // 6148,  00100000 00011000 00000000 00000000
    0, // 6149,  10100000 00011000 00000000 00000000
    1, // 6150,  01100000 00011000 00000000 00000000
    0, // 6151,  11100000 00011000 00000000 00000000
    3, // 6152,  00010000 00011000 00000000 00000000
    0, // 6153,  10010000 00011000 00000000 00000000
    1, // 6154,  01010000 00011000 00000000 00000000
    0, // 6155,  11010000 00011000 00000000 00000000
    2, // 6156,  00110000 00011000 00000000 00000000
    0, // 6157,  10110000 00011000 00000000 00000000
    1, // 6158,  01110000 00011000 00000000 00000000
    0, // 6159,  11110000 00011000 00000000 00000000
    4, // 6160,  00001000 00011000 00000000 00000000
    0, // 6161,  10001000 00011000 00000000 00000000
    1, // 6162,  01001000 00011000 00000000 00000000
    0, // 6163,  11001000 00011000 00000000 00000000
    2, // 6164,  00101000 00011000 00000000 00000000
    0, // 6165,  10101000 00011000 00000000 00000000
    1, // 6166,  01101000 00011000 00000000 00000000
    0, // 6167,  11101000 00011000 00000000 00000000
    3, // 6168,  00011000 00011000 00000000 00000000
    0, // 6169,  10011000 00011000 00000000 00000000
    1, // 6170,  01011000 00011000 00000000 00000000
    0, // 6171,  11011000 00011000 00000000 00000000
    2, // 6172,  00111000 00011000 00000000 00000000
    0, // 6173,  10111000 00011000 00000000 00000000
    1, // 6174,  01111000 00011000 00000000 00000000
    0, // 6175,  11111000 00011000 00000000 00000000
    5, // 6176,  00000100 00011000 00000000 00000000
    0, // 6177,  10000100 00011000 00000000 00000000
    1, // 6178,  01000100 00011000 00000000 00000000
    0, // 6179,  11000100 00011000 00000000 00000000
    2, // 6180,  00100100 00011000 00000000 00000000
    0, // 6181,  10100100 00011000 00000000 00000000
    1, // 6182,  01100100 00011000 00000000 00000000
    0, // 6183,  11100100 00011000 00000000 00000000
    3, // 6184,  00010100 00011000 00000000 00000000
    0, // 6185,  10010100 00011000 00000000 00000000
    1, // 6186,  01010100 00011000 00000000 00000000
    0, // 6187,  11010100 00011000 00000000 00000000
    2, // 6188,  00110100 00011000 00000000 00000000
    0, // 6189,  10110100 00011000 00000000 00000000
    1, // 6190,  01110100 00011000 00000000 00000000
    0, // 6191,  11110100 00011000 00000000 00000000
    4, // 6192,  00001100 00011000 00000000 00000000
    0, // 6193,  10001100 00011000 00000000 00000000
    1, // 6194,  01001100 00011000 00000000 00000000
    0, // 6195,  11001100 00011000 00000000 00000000
    2, // 6196,  00101100 00011000 00000000 00000000
    0, // 6197,  10101100 00011000 00000000 00000000
    1, // 6198,  01101100 00011000 00000000 00000000
    0, // 6199,  11101100 00011000 00000000 00000000
    3, // 6200,  00011100 00011000 00000000 00000000
    0, // 6201,  10011100 00011000 00000000 00000000
    1, // 6202,  01011100 00011000 00000000 00000000
    0, // 6203,  11011100 00011000 00000000 00000000
    2, // 6204,  00111100 00011000 00000000 00000000
    0, // 6205,  10111100 00011000 00000000 00000000
    1, // 6206,  01111100 00011000 00000000 00000000
    0, // 6207,  11111100 00011000 00000000 00000000
    6, // 6208,  00000010 00011000 00000000 00000000
    0, // 6209,  10000010 00011000 00000000 00000000
    1, // 6210,  01000010 00011000 00000000 00000000
    0, // 6211,  11000010 00011000 00000000 00000000
    2, // 6212,  00100010 00011000 00000000 00000000
    0, // 6213,  10100010 00011000 00000000 00000000
    1, // 6214,  01100010 00011000 00000000 00000000
    0, // 6215,  11100010 00011000 00000000 00000000
    3, // 6216,  00010010 00011000 00000000 00000000
    0, // 6217,  10010010 00011000 00000000 00000000
    1, // 6218,  01010010 00011000 00000000 00000000
    0, // 6219,  11010010 00011000 00000000 00000000
    2, // 6220,  00110010 00011000 00000000 00000000
    0, // 6221,  10110010 00011000 00000000 00000000
    1, // 6222,  01110010 00011000 00000000 00000000
    0, // 6223,  11110010 00011000 00000000 00000000
    4, // 6224,  00001010 00011000 00000000 00000000
    0, // 6225,  10001010 00011000 00000000 00000000
    1, // 6226,  01001010 00011000 00000000 00000000
    0, // 6227,  11001010 00011000 00000000 00000000
    2, // 6228,  00101010 00011000 00000000 00000000
    0, // 6229,  10101010 00011000 00000000 00000000
    1, // 6230,  01101010 00011000 00000000 00000000
    0, // 6231,  11101010 00011000 00000000 00000000
    3, // 6232,  00011010 00011000 00000000 00000000
    0, // 6233,  10011010 00011000 00000000 00000000
    1, // 6234,  01011010 00011000 00000000 00000000
    0, // 6235,  11011010 00011000 00000000 00000000
    2, // 6236,  00111010 00011000 00000000 00000000
    0, // 6237,  10111010 00011000 00000000 00000000
    1, // 6238,  01111010 00011000 00000000 00000000
    0, // 6239,  11111010 00011000 00000000 00000000
    5, // 6240,  00000110 00011000 00000000 00000000
    0, // 6241,  10000110 00011000 00000000 00000000
    1, // 6242,  01000110 00011000 00000000 00000000
    0, // 6243,  11000110 00011000 00000000 00000000
    2, // 6244,  00100110 00011000 00000000 00000000
    0, // 6245,  10100110 00011000 00000000 00000000
    1, // 6246,  01100110 00011000 00000000 00000000
    0, // 6247,  11100110 00011000 00000000 00000000
    3, // 6248,  00010110 00011000 00000000 00000000
    0, // 6249,  10010110 00011000 00000000 00000000
    1, // 6250,  01010110 00011000 00000000 00000000
    0, // 6251,  11010110 00011000 00000000 00000000
    2, // 6252,  00110110 00011000 00000000 00000000
    0, // 6253,  10110110 00011000 00000000 00000000
    1, // 6254,  01110110 00011000 00000000 00000000
    0, // 6255,  11110110 00011000 00000000 00000000
    4, // 6256,  00001110 00011000 00000000 00000000
    0, // 6257,  10001110 00011000 00000000 00000000
    1, // 6258,  01001110 00011000 00000000 00000000
    0, // 6259,  11001110 00011000 00000000 00000000
    2, // 6260,  00101110 00011000 00000000 00000000
    0, // 6261,  10101110 00011000 00000000 00000000
    1, // 6262,  01101110 00011000 00000000 00000000
    0, // 6263,  11101110 00011000 00000000 00000000
    3, // 6264,  00011110 00011000 00000000 00000000
    0, // 6265,  10011110 00011000 00000000 00000000
    1, // 6266,  01011110 00011000 00000000 00000000
    0, // 6267,  11011110 00011000 00000000 00000000
    2, // 6268,  00111110 00011000 00000000 00000000
    0, // 6269,  10111110 00011000 00000000 00000000
    1, // 6270,  01111110 00011000 00000000 00000000
    0, // 6271,  11111110 00011000 00000000 00000000
    7, // 6272,  00000001 00011000 00000000 00000000
    0, // 6273,  10000001 00011000 00000000 00000000
    1, // 6274,  01000001 00011000 00000000 00000000
    0, // 6275,  11000001 00011000 00000000 00000000
    2, // 6276,  00100001 00011000 00000000 00000000
    0, // 6277,  10100001 00011000 00000000 00000000
    1, // 6278,  01100001 00011000 00000000 00000000
    0, // 6279,  11100001 00011000 00000000 00000000
    3, // 6280,  00010001 00011000 00000000 00000000
    0, // 6281,  10010001 00011000 00000000 00000000
    1, // 6282,  01010001 00011000 00000000 00000000
    0, // 6283,  11010001 00011000 00000000 00000000
    2, // 6284,  00110001 00011000 00000000 00000000
    0, // 6285,  10110001 00011000 00000000 00000000
    1, // 6286,  01110001 00011000 00000000 00000000
    0, // 6287,  11110001 00011000 00000000 00000000
    4, // 6288,  00001001 00011000 00000000 00000000
    0, // 6289,  10001001 00011000 00000000 00000000
    1, // 6290,  01001001 00011000 00000000 00000000
    0, // 6291,  11001001 00011000 00000000 00000000
    2, // 6292,  00101001 00011000 00000000 00000000
    0, // 6293,  10101001 00011000 00000000 00000000
    1, // 6294,  01101001 00011000 00000000 00000000
    0, // 6295,  11101001 00011000 00000000 00000000
    3, // 6296,  00011001 00011000 00000000 00000000
    0, // 6297,  10011001 00011000 00000000 00000000
    1, // 6298,  01011001 00011000 00000000 00000000
    0, // 6299,  11011001 00011000 00000000 00000000
    2, // 6300,  00111001 00011000 00000000 00000000
    0, // 6301,  10111001 00011000 00000000 00000000
    1, // 6302,  01111001 00011000 00000000 00000000
    0, // 6303,  11111001 00011000 00000000 00000000
    5, // 6304,  00000101 00011000 00000000 00000000
    0, // 6305,  10000101 00011000 00000000 00000000
    1, // 6306,  01000101 00011000 00000000 00000000
    0, // 6307,  11000101 00011000 00000000 00000000
    2, // 6308,  00100101 00011000 00000000 00000000
    0, // 6309,  10100101 00011000 00000000 00000000
    1, // 6310,  01100101 00011000 00000000 00000000
    0, // 6311,  11100101 00011000 00000000 00000000
    3, // 6312,  00010101 00011000 00000000 00000000
    0, // 6313,  10010101 00011000 00000000 00000000
    1, // 6314,  01010101 00011000 00000000 00000000
    0, // 6315,  11010101 00011000 00000000 00000000
    2, // 6316,  00110101 00011000 00000000 00000000
    0, // 6317,  10110101 00011000 00000000 00000000
    1, // 6318,  01110101 00011000 00000000 00000000
    0, // 6319,  11110101 00011000 00000000 00000000
    4, // 6320,  00001101 00011000 00000000 00000000
    0, // 6321,  10001101 00011000 00000000 00000000
    1, // 6322,  01001101 00011000 00000000 00000000
    0, // 6323,  11001101 00011000 00000000 00000000
    2, // 6324,  00101101 00011000 00000000 00000000
    0, // 6325,  10101101 00011000 00000000 00000000
    1, // 6326,  01101101 00011000 00000000 00000000
    0, // 6327,  11101101 00011000 00000000 00000000
    3, // 6328,  00011101 00011000 00000000 00000000
    0, // 6329,  10011101 00011000 00000000 00000000
    1, // 6330,  01011101 00011000 00000000 00000000
    0, // 6331,  11011101 00011000 00000000 00000000
    2, // 6332,  00111101 00011000 00000000 00000000
    0, // 6333,  10111101 00011000 00000000 00000000
    1, // 6334,  01111101 00011000 00000000 00000000
    0, // 6335,  11111101 00011000 00000000 00000000
    6, // 6336,  00000011 00011000 00000000 00000000
    0, // 6337,  10000011 00011000 00000000 00000000
    1, // 6338,  01000011 00011000 00000000 00000000
    0, // 6339,  11000011 00011000 00000000 00000000
    2, // 6340,  00100011 00011000 00000000 00000000
    0, // 6341,  10100011 00011000 00000000 00000000
    1, // 6342,  01100011 00011000 00000000 00000000
    0, // 6343,  11100011 00011000 00000000 00000000
    3, // 6344,  00010011 00011000 00000000 00000000
    0, // 6345,  10010011 00011000 00000000 00000000
    1, // 6346,  01010011 00011000 00000000 00000000
    0, // 6347,  11010011 00011000 00000000 00000000
    2, // 6348,  00110011 00011000 00000000 00000000
    0, // 6349,  10110011 00011000 00000000 00000000
    1, // 6350,  01110011 00011000 00000000 00000000
    0, // 6351,  11110011 00011000 00000000 00000000
    4, // 6352,  00001011 00011000 00000000 00000000
    0, // 6353,  10001011 00011000 00000000 00000000
    1, // 6354,  01001011 00011000 00000000 00000000
    0, // 6355,  11001011 00011000 00000000 00000000
    2, // 6356,  00101011 00011000 00000000 00000000
    0, // 6357,  10101011 00011000 00000000 00000000
    1, // 6358,  01101011 00011000 00000000 00000000
    0, // 6359,  11101011 00011000 00000000 00000000
    3, // 6360,  00011011 00011000 00000000 00000000
    0, // 6361,  10011011 00011000 00000000 00000000
    1, // 6362,  01011011 00011000 00000000 00000000
    0, // 6363,  11011011 00011000 00000000 00000000
    2, // 6364,  00111011 00011000 00000000 00000000
    0, // 6365,  10111011 00011000 00000000 00000000
    1, // 6366,  01111011 00011000 00000000 00000000
    0, // 6367,  11111011 00011000 00000000 00000000
    5, // 6368,  00000111 00011000 00000000 00000000
    0, // 6369,  10000111 00011000 00000000 00000000
    1, // 6370,  01000111 00011000 00000000 00000000
    0, // 6371,  11000111 00011000 00000000 00000000
    2, // 6372,  00100111 00011000 00000000 00000000
    0, // 6373,  10100111 00011000 00000000 00000000
    1, // 6374,  01100111 00011000 00000000 00000000
    0, // 6375,  11100111 00011000 00000000 00000000
    3, // 6376,  00010111 00011000 00000000 00000000
    0, // 6377,  10010111 00011000 00000000 00000000
    1, // 6378,  01010111 00011000 00000000 00000000
    0, // 6379,  11010111 00011000 00000000 00000000
    2, // 6380,  00110111 00011000 00000000 00000000
    0, // 6381,  10110111 00011000 00000000 00000000
    1, // 6382,  01110111 00011000 00000000 00000000
    0, // 6383,  11110111 00011000 00000000 00000000
    4, // 6384,  00001111 00011000 00000000 00000000
    0, // 6385,  10001111 00011000 00000000 00000000
    1, // 6386,  01001111 00011000 00000000 00000000
    0, // 6387,  11001111 00011000 00000000 00000000
    2, // 6388,  00101111 00011000 00000000 00000000
    0, // 6389,  10101111 00011000 00000000 00000000
    1, // 6390,  01101111 00011000 00000000 00000000
    0, // 6391,  11101111 00011000 00000000 00000000
    3, // 6392,  00011111 00011000 00000000 00000000
    0, // 6393,  10011111 00011000 00000000 00000000
    1, // 6394,  01011111 00011000 00000000 00000000
    0, // 6395,  11011111 00011000 00000000 00000000
    2, // 6396,  00111111 00011000 00000000 00000000
    0, // 6397,  10111111 00011000 00000000 00000000
    1, // 6398,  01111111 00011000 00000000 00000000
    0, // 6399,  11111111 00011000 00000000 00000000
    8, // 6400,  00000000 10011000 00000000 00000000
    0, // 6401,  10000000 10011000 00000000 00000000
    1, // 6402,  01000000 10011000 00000000 00000000
    0, // 6403,  11000000 10011000 00000000 00000000
    2, // 6404,  00100000 10011000 00000000 00000000
    0, // 6405,  10100000 10011000 00000000 00000000
    1, // 6406,  01100000 10011000 00000000 00000000
    0, // 6407,  11100000 10011000 00000000 00000000
    3, // 6408,  00010000 10011000 00000000 00000000
    0, // 6409,  10010000 10011000 00000000 00000000
    1, // 6410,  01010000 10011000 00000000 00000000
    0, // 6411,  11010000 10011000 00000000 00000000
    2, // 6412,  00110000 10011000 00000000 00000000
    0, // 6413,  10110000 10011000 00000000 00000000
    1, // 6414,  01110000 10011000 00000000 00000000
    0, // 6415,  11110000 10011000 00000000 00000000
    4, // 6416,  00001000 10011000 00000000 00000000
    0, // 6417,  10001000 10011000 00000000 00000000
    1, // 6418,  01001000 10011000 00000000 00000000
    0, // 6419,  11001000 10011000 00000000 00000000
    2, // 6420,  00101000 10011000 00000000 00000000
    0, // 6421,  10101000 10011000 00000000 00000000
    1, // 6422,  01101000 10011000 00000000 00000000
    0, // 6423,  11101000 10011000 00000000 00000000
    3, // 6424,  00011000 10011000 00000000 00000000
    0, // 6425,  10011000 10011000 00000000 00000000
    1, // 6426,  01011000 10011000 00000000 00000000
    0, // 6427,  11011000 10011000 00000000 00000000
    2, // 6428,  00111000 10011000 00000000 00000000
    0, // 6429,  10111000 10011000 00000000 00000000
    1, // 6430,  01111000 10011000 00000000 00000000
    0, // 6431,  11111000 10011000 00000000 00000000
    5, // 6432,  00000100 10011000 00000000 00000000
    0, // 6433,  10000100 10011000 00000000 00000000
    1, // 6434,  01000100 10011000 00000000 00000000
    0, // 6435,  11000100 10011000 00000000 00000000
    2, // 6436,  00100100 10011000 00000000 00000000
    0, // 6437,  10100100 10011000 00000000 00000000
    1, // 6438,  01100100 10011000 00000000 00000000
    0, // 6439,  11100100 10011000 00000000 00000000
    3, // 6440,  00010100 10011000 00000000 00000000
    0, // 6441,  10010100 10011000 00000000 00000000
    1, // 6442,  01010100 10011000 00000000 00000000
    0, // 6443,  11010100 10011000 00000000 00000000
    2, // 6444,  00110100 10011000 00000000 00000000
    0, // 6445,  10110100 10011000 00000000 00000000
    1, // 6446,  01110100 10011000 00000000 00000000
    0, // 6447,  11110100 10011000 00000000 00000000
    4, // 6448,  00001100 10011000 00000000 00000000
    0, // 6449,  10001100 10011000 00000000 00000000
    1, // 6450,  01001100 10011000 00000000 00000000
    0, // 6451,  11001100 10011000 00000000 00000000
    2, // 6452,  00101100 10011000 00000000 00000000
    0, // 6453,  10101100 10011000 00000000 00000000
    1, // 6454,  01101100 10011000 00000000 00000000
    0, // 6455,  11101100 10011000 00000000 00000000
    3, // 6456,  00011100 10011000 00000000 00000000
    0, // 6457,  10011100 10011000 00000000 00000000
    1, // 6458,  01011100 10011000 00000000 00000000
    0, // 6459,  11011100 10011000 00000000 00000000
    2, // 6460,  00111100 10011000 00000000 00000000
    0, // 6461,  10111100 10011000 00000000 00000000
    1, // 6462,  01111100 10011000 00000000 00000000
    0, // 6463,  11111100 10011000 00000000 00000000
    6, // 6464,  00000010 10011000 00000000 00000000
    0, // 6465,  10000010 10011000 00000000 00000000
    1, // 6466,  01000010 10011000 00000000 00000000
    0, // 6467,  11000010 10011000 00000000 00000000
    2, // 6468,  00100010 10011000 00000000 00000000
    0, // 6469,  10100010 10011000 00000000 00000000
    1, // 6470,  01100010 10011000 00000000 00000000
    0, // 6471,  11100010 10011000 00000000 00000000
    3, // 6472,  00010010 10011000 00000000 00000000
    0, // 6473,  10010010 10011000 00000000 00000000
    1, // 6474,  01010010 10011000 00000000 00000000
    0, // 6475,  11010010 10011000 00000000 00000000
    2, // 6476,  00110010 10011000 00000000 00000000
    0, // 6477,  10110010 10011000 00000000 00000000
    1, // 6478,  01110010 10011000 00000000 00000000
    0, // 6479,  11110010 10011000 00000000 00000000
    4, // 6480,  00001010 10011000 00000000 00000000
    0, // 6481,  10001010 10011000 00000000 00000000
    1, // 6482,  01001010 10011000 00000000 00000000
    0, // 6483,  11001010 10011000 00000000 00000000
    2, // 6484,  00101010 10011000 00000000 00000000
    0, // 6485,  10101010 10011000 00000000 00000000
    1, // 6486,  01101010 10011000 00000000 00000000
    0, // 6487,  11101010 10011000 00000000 00000000
    3, // 6488,  00011010 10011000 00000000 00000000
    0, // 6489,  10011010 10011000 00000000 00000000
    1, // 6490,  01011010 10011000 00000000 00000000
    0, // 6491,  11011010 10011000 00000000 00000000
    2, // 6492,  00111010 10011000 00000000 00000000
    0, // 6493,  10111010 10011000 00000000 00000000
    1, // 6494,  01111010 10011000 00000000 00000000
    0, // 6495,  11111010 10011000 00000000 00000000
    5, // 6496,  00000110 10011000 00000000 00000000
    0, // 6497,  10000110 10011000 00000000 00000000
    1, // 6498,  01000110 10011000 00000000 00000000
    0, // 6499,  11000110 10011000 00000000 00000000
    2, // 6500,  00100110 10011000 00000000 00000000
    0, // 6501,  10100110 10011000 00000000 00000000
    1, // 6502,  01100110 10011000 00000000 00000000
    0, // 6503,  11100110 10011000 00000000 00000000
    3, // 6504,  00010110 10011000 00000000 00000000
    0, // 6505,  10010110 10011000 00000000 00000000
    1, // 6506,  01010110 10011000 00000000 00000000
    0, // 6507,  11010110 10011000 00000000 00000000
    2, // 6508,  00110110 10011000 00000000 00000000
    0, // 6509,  10110110 10011000 00000000 00000000
    1, // 6510,  01110110 10011000 00000000 00000000
    0, // 6511,  11110110 10011000 00000000 00000000
    4, // 6512,  00001110 10011000 00000000 00000000
    0, // 6513,  10001110 10011000 00000000 00000000
    1, // 6514,  01001110 10011000 00000000 00000000
    0, // 6515,  11001110 10011000 00000000 00000000
    2, // 6516,  00101110 10011000 00000000 00000000
    0, // 6517,  10101110 10011000 00000000 00000000
    1, // 6518,  01101110 10011000 00000000 00000000
    0, // 6519,  11101110 10011000 00000000 00000000
    3, // 6520,  00011110 10011000 00000000 00000000
    0, // 6521,  10011110 10011000 00000000 00000000
    1, // 6522,  01011110 10011000 00000000 00000000
    0, // 6523,  11011110 10011000 00000000 00000000
    2, // 6524,  00111110 10011000 00000000 00000000
    0, // 6525,  10111110 10011000 00000000 00000000
    1, // 6526,  01111110 10011000 00000000 00000000
    0, // 6527,  11111110 10011000 00000000 00000000
    7, // 6528,  00000001 10011000 00000000 00000000
    0, // 6529,  10000001 10011000 00000000 00000000
    1, // 6530,  01000001 10011000 00000000 00000000
    0, // 6531,  11000001 10011000 00000000 00000000
    2, // 6532,  00100001 10011000 00000000 00000000
    0, // 6533,  10100001 10011000 00000000 00000000
    1, // 6534,  01100001 10011000 00000000 00000000
    0, // 6535,  11100001 10011000 00000000 00000000
    3, // 6536,  00010001 10011000 00000000 00000000
    0, // 6537,  10010001 10011000 00000000 00000000
    1, // 6538,  01010001 10011000 00000000 00000000
    0, // 6539,  11010001 10011000 00000000 00000000
    2, // 6540,  00110001 10011000 00000000 00000000
    0, // 6541,  10110001 10011000 00000000 00000000
    1, // 6542,  01110001 10011000 00000000 00000000
    0, // 6543,  11110001 10011000 00000000 00000000
    4, // 6544,  00001001 10011000 00000000 00000000
    0, // 6545,  10001001 10011000 00000000 00000000
    1, // 6546,  01001001 10011000 00000000 00000000
    0, // 6547,  11001001 10011000 00000000 00000000
    2, // 6548,  00101001 10011000 00000000 00000000
    0, // 6549,  10101001 10011000 00000000 00000000
    1, // 6550,  01101001 10011000 00000000 00000000
    0, // 6551,  11101001 10011000 00000000 00000000
    3, // 6552,  00011001 10011000 00000000 00000000
    0, // 6553,  10011001 10011000 00000000 00000000
    1, // 6554,  01011001 10011000 00000000 00000000
    0, // 6555,  11011001 10011000 00000000 00000000
    2, // 6556,  00111001 10011000 00000000 00000000
    0, // 6557,  10111001 10011000 00000000 00000000
    1, // 6558,  01111001 10011000 00000000 00000000
    0, // 6559,  11111001 10011000 00000000 00000000
    5, // 6560,  00000101 10011000 00000000 00000000
    0, // 6561,  10000101 10011000 00000000 00000000
    1, // 6562,  01000101 10011000 00000000 00000000
    0, // 6563,  11000101 10011000 00000000 00000000
    2, // 6564,  00100101 10011000 00000000 00000000
    0, // 6565,  10100101 10011000 00000000 00000000
    1, // 6566,  01100101 10011000 00000000 00000000
    0, // 6567,  11100101 10011000 00000000 00000000
    3, // 6568,  00010101 10011000 00000000 00000000
    0, // 6569,  10010101 10011000 00000000 00000000
    1, // 6570,  01010101 10011000 00000000 00000000
    0, // 6571,  11010101 10011000 00000000 00000000
    2, // 6572,  00110101 10011000 00000000 00000000
    0, // 6573,  10110101 10011000 00000000 00000000
    1, // 6574,  01110101 10011000 00000000 00000000
    0, // 6575,  11110101 10011000 00000000 00000000
    4, // 6576,  00001101 10011000 00000000 00000000
    0, // 6577,  10001101 10011000 00000000 00000000
    1, // 6578,  01001101 10011000 00000000 00000000
    0, // 6579,  11001101 10011000 00000000 00000000
    2, // 6580,  00101101 10011000 00000000 00000000
    0, // 6581,  10101101 10011000 00000000 00000000
    1, // 6582,  01101101 10011000 00000000 00000000
    0, // 6583,  11101101 10011000 00000000 00000000
    3, // 6584,  00011101 10011000 00000000 00000000
    0, // 6585,  10011101 10011000 00000000 00000000
    1, // 6586,  01011101 10011000 00000000 00000000
    0, // 6587,  11011101 10011000 00000000 00000000
    2, // 6588,  00111101 10011000 00000000 00000000
    0, // 6589,  10111101 10011000 00000000 00000000
    1, // 6590,  01111101 10011000 00000000 00000000
    0, // 6591,  11111101 10011000 00000000 00000000
    6, // 6592,  00000011 10011000 00000000 00000000
    0, // 6593,  10000011 10011000 00000000 00000000
    1, // 6594,  01000011 10011000 00000000 00000000
    0, // 6595,  11000011 10011000 00000000 00000000
    2, // 6596,  00100011 10011000 00000000 00000000
    0, // 6597,  10100011 10011000 00000000 00000000
    1, // 6598,  01100011 10011000 00000000 00000000
    0, // 6599,  11100011 10011000 00000000 00000000
    3, // 6600,  00010011 10011000 00000000 00000000
    0, // 6601,  10010011 10011000 00000000 00000000
    1, // 6602,  01010011 10011000 00000000 00000000
    0, // 6603,  11010011 10011000 00000000 00000000
    2, // 6604,  00110011 10011000 00000000 00000000
    0, // 6605,  10110011 10011000 00000000 00000000
    1, // 6606,  01110011 10011000 00000000 00000000
    0, // 6607,  11110011 10011000 00000000 00000000
    4, // 6608,  00001011 10011000 00000000 00000000
    0, // 6609,  10001011 10011000 00000000 00000000
    1, // 6610,  01001011 10011000 00000000 00000000
    0, // 6611,  11001011 10011000 00000000 00000000
    2, // 6612,  00101011 10011000 00000000 00000000
    0, // 6613,  10101011 10011000 00000000 00000000
    1, // 6614,  01101011 10011000 00000000 00000000
    0, // 6615,  11101011 10011000 00000000 00000000
    3, // 6616,  00011011 10011000 00000000 00000000
    0, // 6617,  10011011 10011000 00000000 00000000
    1, // 6618,  01011011 10011000 00000000 00000000
    0, // 6619,  11011011 10011000 00000000 00000000
    2, // 6620,  00111011 10011000 00000000 00000000
    0, // 6621,  10111011 10011000 00000000 00000000
    1, // 6622,  01111011 10011000 00000000 00000000
    0, // 6623,  11111011 10011000 00000000 00000000
    5, // 6624,  00000111 10011000 00000000 00000000
    0, // 6625,  10000111 10011000 00000000 00000000
    1, // 6626,  01000111 10011000 00000000 00000000
    0, // 6627,  11000111 10011000 00000000 00000000
    2, // 6628,  00100111 10011000 00000000 00000000
    0, // 6629,  10100111 10011000 00000000 00000000
    1, // 6630,  01100111 10011000 00000000 00000000
    0, // 6631,  11100111 10011000 00000000 00000000
    3, // 6632,  00010111 10011000 00000000 00000000
    0, // 6633,  10010111 10011000 00000000 00000000
    1, // 6634,  01010111 10011000 00000000 00000000
    0, // 6635,  11010111 10011000 00000000 00000000
    2, // 6636,  00110111 10011000 00000000 00000000
    0, // 6637,  10110111 10011000 00000000 00000000
    1, // 6638,  01110111 10011000 00000000 00000000
    0, // 6639,  11110111 10011000 00000000 00000000
    4, // 6640,  00001111 10011000 00000000 00000000
    0, // 6641,  10001111 10011000 00000000 00000000
    1, // 6642,  01001111 10011000 00000000 00000000
    0, // 6643,  11001111 10011000 00000000 00000000
    2, // 6644,  00101111 10011000 00000000 00000000
    0, // 6645,  10101111 10011000 00000000 00000000
    1, // 6646,  01101111 10011000 00000000 00000000
    0, // 6647,  11101111 10011000 00000000 00000000
    3, // 6648,  00011111 10011000 00000000 00000000
    0, // 6649,  10011111 10011000 00000000 00000000
    1, // 6650,  01011111 10011000 00000000 00000000
    0, // 6651,  11011111 10011000 00000000 00000000
    2, // 6652,  00111111 10011000 00000000 00000000
    0, // 6653,  10111111 10011000 00000000 00000000
    1, // 6654,  01111111 10011000 00000000 00000000
    0, // 6655,  11111111 10011000 00000000 00000000
    9, // 6656,  00000000 01011000 00000000 00000000
    0, // 6657,  10000000 01011000 00000000 00000000
    1, // 6658,  01000000 01011000 00000000 00000000
    0, // 6659,  11000000 01011000 00000000 00000000
    2, // 6660,  00100000 01011000 00000000 00000000
    0, // 6661,  10100000 01011000 00000000 00000000
    1, // 6662,  01100000 01011000 00000000 00000000
    0, // 6663,  11100000 01011000 00000000 00000000
    3, // 6664,  00010000 01011000 00000000 00000000
    0, // 6665,  10010000 01011000 00000000 00000000
    1, // 6666,  01010000 01011000 00000000 00000000
    0, // 6667,  11010000 01011000 00000000 00000000
    2, // 6668,  00110000 01011000 00000000 00000000
    0, // 6669,  10110000 01011000 00000000 00000000
    1, // 6670,  01110000 01011000 00000000 00000000
    0, // 6671,  11110000 01011000 00000000 00000000
    4, // 6672,  00001000 01011000 00000000 00000000
    0, // 6673,  10001000 01011000 00000000 00000000
    1, // 6674,  01001000 01011000 00000000 00000000
    0, // 6675,  11001000 01011000 00000000 00000000
    2, // 6676,  00101000 01011000 00000000 00000000
    0, // 6677,  10101000 01011000 00000000 00000000
    1, // 6678,  01101000 01011000 00000000 00000000
    0, // 6679,  11101000 01011000 00000000 00000000
    3, // 6680,  00011000 01011000 00000000 00000000
    0, // 6681,  10011000 01011000 00000000 00000000
    1, // 6682,  01011000 01011000 00000000 00000000
    0, // 6683,  11011000 01011000 00000000 00000000
    2, // 6684,  00111000 01011000 00000000 00000000
    0, // 6685,  10111000 01011000 00000000 00000000
    1, // 6686,  01111000 01011000 00000000 00000000
    0, // 6687,  11111000 01011000 00000000 00000000
    5, // 6688,  00000100 01011000 00000000 00000000
    0, // 6689,  10000100 01011000 00000000 00000000
    1, // 6690,  01000100 01011000 00000000 00000000
    0, // 6691,  11000100 01011000 00000000 00000000
    2, // 6692,  00100100 01011000 00000000 00000000
    0, // 6693,  10100100 01011000 00000000 00000000
    1, // 6694,  01100100 01011000 00000000 00000000
    0, // 6695,  11100100 01011000 00000000 00000000
    3, // 6696,  00010100 01011000 00000000 00000000
    0, // 6697,  10010100 01011000 00000000 00000000
    1, // 6698,  01010100 01011000 00000000 00000000
    0, // 6699,  11010100 01011000 00000000 00000000
    2, // 6700,  00110100 01011000 00000000 00000000
    0, // 6701,  10110100 01011000 00000000 00000000
    1, // 6702,  01110100 01011000 00000000 00000000
    0, // 6703,  11110100 01011000 00000000 00000000
    4, // 6704,  00001100 01011000 00000000 00000000
    0, // 6705,  10001100 01011000 00000000 00000000
    1, // 6706,  01001100 01011000 00000000 00000000
    0, // 6707,  11001100 01011000 00000000 00000000
    2, // 6708,  00101100 01011000 00000000 00000000
    0, // 6709,  10101100 01011000 00000000 00000000
    1, // 6710,  01101100 01011000 00000000 00000000
    0, // 6711,  11101100 01011000 00000000 00000000
    3, // 6712,  00011100 01011000 00000000 00000000
    0, // 6713,  10011100 01011000 00000000 00000000
    1, // 6714,  01011100 01011000 00000000 00000000
    0, // 6715,  11011100 01011000 00000000 00000000
    2, // 6716,  00111100 01011000 00000000 00000000
    0, // 6717,  10111100 01011000 00000000 00000000
    1, // 6718,  01111100 01011000 00000000 00000000
    0, // 6719,  11111100 01011000 00000000 00000000
    6, // 6720,  00000010 01011000 00000000 00000000
    0, // 6721,  10000010 01011000 00000000 00000000
    1, // 6722,  01000010 01011000 00000000 00000000
    0, // 6723,  11000010 01011000 00000000 00000000
    2, // 6724,  00100010 01011000 00000000 00000000
    0, // 6725,  10100010 01011000 00000000 00000000
    1, // 6726,  01100010 01011000 00000000 00000000
    0, // 6727,  11100010 01011000 00000000 00000000
    3, // 6728,  00010010 01011000 00000000 00000000
    0, // 6729,  10010010 01011000 00000000 00000000
    1, // 6730,  01010010 01011000 00000000 00000000
    0, // 6731,  11010010 01011000 00000000 00000000
    2, // 6732,  00110010 01011000 00000000 00000000
    0, // 6733,  10110010 01011000 00000000 00000000
    1, // 6734,  01110010 01011000 00000000 00000000
    0, // 6735,  11110010 01011000 00000000 00000000
    4, // 6736,  00001010 01011000 00000000 00000000
    0, // 6737,  10001010 01011000 00000000 00000000
    1, // 6738,  01001010 01011000 00000000 00000000
    0, // 6739,  11001010 01011000 00000000 00000000
    2, // 6740,  00101010 01011000 00000000 00000000
    0, // 6741,  10101010 01011000 00000000 00000000
    1, // 6742,  01101010 01011000 00000000 00000000
    0, // 6743,  11101010 01011000 00000000 00000000
    3, // 6744,  00011010 01011000 00000000 00000000
    0, // 6745,  10011010 01011000 00000000 00000000
    1, // 6746,  01011010 01011000 00000000 00000000
    0, // 6747,  11011010 01011000 00000000 00000000
    2, // 6748,  00111010 01011000 00000000 00000000
    0, // 6749,  10111010 01011000 00000000 00000000
    1, // 6750,  01111010 01011000 00000000 00000000
    0, // 6751,  11111010 01011000 00000000 00000000
    5, // 6752,  00000110 01011000 00000000 00000000
    0, // 6753,  10000110 01011000 00000000 00000000
    1, // 6754,  01000110 01011000 00000000 00000000
    0, // 6755,  11000110 01011000 00000000 00000000
    2, // 6756,  00100110 01011000 00000000 00000000
    0, // 6757,  10100110 01011000 00000000 00000000
    1, // 6758,  01100110 01011000 00000000 00000000
    0, // 6759,  11100110 01011000 00000000 00000000
    3, // 6760,  00010110 01011000 00000000 00000000
    0, // 6761,  10010110 01011000 00000000 00000000
    1, // 6762,  01010110 01011000 00000000 00000000
    0, // 6763,  11010110 01011000 00000000 00000000
    2, // 6764,  00110110 01011000 00000000 00000000
    0, // 6765,  10110110 01011000 00000000 00000000
    1, // 6766,  01110110 01011000 00000000 00000000
    0, // 6767,  11110110 01011000 00000000 00000000
    4, // 6768,  00001110 01011000 00000000 00000000
    0, // 6769,  10001110 01011000 00000000 00000000
    1, // 6770,  01001110 01011000 00000000 00000000
    0, // 6771,  11001110 01011000 00000000 00000000
    2, // 6772,  00101110 01011000 00000000 00000000
    0, // 6773,  10101110 01011000 00000000 00000000
    1, // 6774,  01101110 01011000 00000000 00000000
    0, // 6775,  11101110 01011000 00000000 00000000
    3, // 6776,  00011110 01011000 00000000 00000000
    0, // 6777,  10011110 01011000 00000000 00000000
    1, // 6778,  01011110 01011000 00000000 00000000
    0, // 6779,  11011110 01011000 00000000 00000000
    2, // 6780,  00111110 01011000 00000000 00000000
    0, // 6781,  10111110 01011000 00000000 00000000
    1, // 6782,  01111110 01011000 00000000 00000000
    0, // 6783,  11111110 01011000 00000000 00000000
    7, // 6784,  00000001 01011000 00000000 00000000
    0, // 6785,  10000001 01011000 00000000 00000000
    1, // 6786,  01000001 01011000 00000000 00000000
    0, // 6787,  11000001 01011000 00000000 00000000
    2, // 6788,  00100001 01011000 00000000 00000000
    0, // 6789,  10100001 01011000 00000000 00000000
    1, // 6790,  01100001 01011000 00000000 00000000
    0, // 6791,  11100001 01011000 00000000 00000000
    3, // 6792,  00010001 01011000 00000000 00000000
    0, // 6793,  10010001 01011000 00000000 00000000
    1, // 6794,  01010001 01011000 00000000 00000000
    0, // 6795,  11010001 01011000 00000000 00000000
    2, // 6796,  00110001 01011000 00000000 00000000
    0, // 6797,  10110001 01011000 00000000 00000000
    1, // 6798,  01110001 01011000 00000000 00000000
    0, // 6799,  11110001 01011000 00000000 00000000
    4, // 6800,  00001001 01011000 00000000 00000000
    0, // 6801,  10001001 01011000 00000000 00000000
    1, // 6802,  01001001 01011000 00000000 00000000
    0, // 6803,  11001001 01011000 00000000 00000000
    2, // 6804,  00101001 01011000 00000000 00000000
    0, // 6805,  10101001 01011000 00000000 00000000
    1, // 6806,  01101001 01011000 00000000 00000000
    0, // 6807,  11101001 01011000 00000000 00000000
    3, // 6808,  00011001 01011000 00000000 00000000
    0, // 6809,  10011001 01011000 00000000 00000000
    1, // 6810,  01011001 01011000 00000000 00000000
    0, // 6811,  11011001 01011000 00000000 00000000
    2, // 6812,  00111001 01011000 00000000 00000000
    0, // 6813,  10111001 01011000 00000000 00000000
    1, // 6814,  01111001 01011000 00000000 00000000
    0, // 6815,  11111001 01011000 00000000 00000000
    5, // 6816,  00000101 01011000 00000000 00000000
    0, // 6817,  10000101 01011000 00000000 00000000
    1, // 6818,  01000101 01011000 00000000 00000000
    0, // 6819,  11000101 01011000 00000000 00000000
    2, // 6820,  00100101 01011000 00000000 00000000
    0, // 6821,  10100101 01011000 00000000 00000000
    1, // 6822,  01100101 01011000 00000000 00000000
    0, // 6823,  11100101 01011000 00000000 00000000
    3, // 6824,  00010101 01011000 00000000 00000000
    0, // 6825,  10010101 01011000 00000000 00000000
    1, // 6826,  01010101 01011000 00000000 00000000
    0, // 6827,  11010101 01011000 00000000 00000000
    2, // 6828,  00110101 01011000 00000000 00000000
    0, // 6829,  10110101 01011000 00000000 00000000
    1, // 6830,  01110101 01011000 00000000 00000000
    0, // 6831,  11110101 01011000 00000000 00000000
    4, // 6832,  00001101 01011000 00000000 00000000
    0, // 6833,  10001101 01011000 00000000 00000000
    1, // 6834,  01001101 01011000 00000000 00000000
    0, // 6835,  11001101 01011000 00000000 00000000
    2, // 6836,  00101101 01011000 00000000 00000000
    0, // 6837,  10101101 01011000 00000000 00000000
    1, // 6838,  01101101 01011000 00000000 00000000
    0, // 6839,  11101101 01011000 00000000 00000000
    3, // 6840,  00011101 01011000 00000000 00000000
    0, // 6841,  10011101 01011000 00000000 00000000
    1, // 6842,  01011101 01011000 00000000 00000000
    0, // 6843,  11011101 01011000 00000000 00000000
    2, // 6844,  00111101 01011000 00000000 00000000
    0, // 6845,  10111101 01011000 00000000 00000000
    1, // 6846,  01111101 01011000 00000000 00000000
    0, // 6847,  11111101 01011000 00000000 00000000
    6, // 6848,  00000011 01011000 00000000 00000000
    0, // 6849,  10000011 01011000 00000000 00000000
    1, // 6850,  01000011 01011000 00000000 00000000
    0, // 6851,  11000011 01011000 00000000 00000000
    2, // 6852,  00100011 01011000 00000000 00000000
    0, // 6853,  10100011 01011000 00000000 00000000
    1, // 6854,  01100011 01011000 00000000 00000000
    0, // 6855,  11100011 01011000 00000000 00000000
    3, // 6856,  00010011 01011000 00000000 00000000
    0, // 6857,  10010011 01011000 00000000 00000000
    1, // 6858,  01010011 01011000 00000000 00000000
    0, // 6859,  11010011 01011000 00000000 00000000
    2, // 6860,  00110011 01011000 00000000 00000000
    0, // 6861,  10110011 01011000 00000000 00000000
    1, // 6862,  01110011 01011000 00000000 00000000
    0, // 6863,  11110011 01011000 00000000 00000000
    4, // 6864,  00001011 01011000 00000000 00000000
    0, // 6865,  10001011 01011000 00000000 00000000
    1, // 6866,  01001011 01011000 00000000 00000000
    0, // 6867,  11001011 01011000 00000000 00000000
    2, // 6868,  00101011 01011000 00000000 00000000
    0, // 6869,  10101011 01011000 00000000 00000000
    1, // 6870,  01101011 01011000 00000000 00000000
    0, // 6871,  11101011 01011000 00000000 00000000
    3, // 6872,  00011011 01011000 00000000 00000000
    0, // 6873,  10011011 01011000 00000000 00000000
    1, // 6874,  01011011 01011000 00000000 00000000
    0, // 6875,  11011011 01011000 00000000 00000000
    2, // 6876,  00111011 01011000 00000000 00000000
    0, // 6877,  10111011 01011000 00000000 00000000
    1, // 6878,  01111011 01011000 00000000 00000000
    0, // 6879,  11111011 01011000 00000000 00000000
    5, // 6880,  00000111 01011000 00000000 00000000
    0, // 6881,  10000111 01011000 00000000 00000000
    1, // 6882,  01000111 01011000 00000000 00000000
    0, // 6883,  11000111 01011000 00000000 00000000
    2, // 6884,  00100111 01011000 00000000 00000000
    0, // 6885,  10100111 01011000 00000000 00000000
    1, // 6886,  01100111 01011000 00000000 00000000
    0, // 6887,  11100111 01011000 00000000 00000000
    3, // 6888,  00010111 01011000 00000000 00000000
    0, // 6889,  10010111 01011000 00000000 00000000
    1, // 6890,  01010111 01011000 00000000 00000000
    0, // 6891,  11010111 01011000 00000000 00000000
    2, // 6892,  00110111 01011000 00000000 00000000
    0, // 6893,  10110111 01011000 00000000 00000000
    1, // 6894,  01110111 01011000 00000000 00000000
    0, // 6895,  11110111 01011000 00000000 00000000
    4, // 6896,  00001111 01011000 00000000 00000000
    0, // 6897,  10001111 01011000 00000000 00000000
    1, // 6898,  01001111 01011000 00000000 00000000
    0, // 6899,  11001111 01011000 00000000 00000000
    2, // 6900,  00101111 01011000 00000000 00000000
    0, // 6901,  10101111 01011000 00000000 00000000
    1, // 6902,  01101111 01011000 00000000 00000000
    0, // 6903,  11101111 01011000 00000000 00000000
    3, // 6904,  00011111 01011000 00000000 00000000
    0, // 6905,  10011111 01011000 00000000 00000000
    1, // 6906,  01011111 01011000 00000000 00000000
    0, // 6907,  11011111 01011000 00000000 00000000
    2, // 6908,  00111111 01011000 00000000 00000000
    0, // 6909,  10111111 01011000 00000000 00000000
    1, // 6910,  01111111 01011000 00000000 00000000
    0, // 6911,  11111111 01011000 00000000 00000000
    8, // 6912,  00000000 11011000 00000000 00000000
    0, // 6913,  10000000 11011000 00000000 00000000
    1, // 6914,  01000000 11011000 00000000 00000000
    0, // 6915,  11000000 11011000 00000000 00000000
    2, // 6916,  00100000 11011000 00000000 00000000
    0, // 6917,  10100000 11011000 00000000 00000000
    1, // 6918,  01100000 11011000 00000000 00000000
    0, // 6919,  11100000 11011000 00000000 00000000
    3, // 6920,  00010000 11011000 00000000 00000000
    0, // 6921,  10010000 11011000 00000000 00000000
    1, // 6922,  01010000 11011000 00000000 00000000
    0, // 6923,  11010000 11011000 00000000 00000000
    2, // 6924,  00110000 11011000 00000000 00000000
    0, // 6925,  10110000 11011000 00000000 00000000
    1, // 6926,  01110000 11011000 00000000 00000000
    0, // 6927,  11110000 11011000 00000000 00000000
    4, // 6928,  00001000 11011000 00000000 00000000
    0, // 6929,  10001000 11011000 00000000 00000000
    1, // 6930,  01001000 11011000 00000000 00000000
    0, // 6931,  11001000 11011000 00000000 00000000
    2, // 6932,  00101000 11011000 00000000 00000000
    0, // 6933,  10101000 11011000 00000000 00000000
    1, // 6934,  01101000 11011000 00000000 00000000
    0, // 6935,  11101000 11011000 00000000 00000000
    3, // 6936,  00011000 11011000 00000000 00000000
    0, // 6937,  10011000 11011000 00000000 00000000
    1, // 6938,  01011000 11011000 00000000 00000000
    0, // 6939,  11011000 11011000 00000000 00000000
    2, // 6940,  00111000 11011000 00000000 00000000
    0, // 6941,  10111000 11011000 00000000 00000000
    1, // 6942,  01111000 11011000 00000000 00000000
    0, // 6943,  11111000 11011000 00000000 00000000
    5, // 6944,  00000100 11011000 00000000 00000000
    0, // 6945,  10000100 11011000 00000000 00000000
    1, // 6946,  01000100 11011000 00000000 00000000
    0, // 6947,  11000100 11011000 00000000 00000000
    2, // 6948,  00100100 11011000 00000000 00000000
    0, // 6949,  10100100 11011000 00000000 00000000
    1, // 6950,  01100100 11011000 00000000 00000000
    0, // 6951,  11100100 11011000 00000000 00000000
    3, // 6952,  00010100 11011000 00000000 00000000
    0, // 6953,  10010100 11011000 00000000 00000000
    1, // 6954,  01010100 11011000 00000000 00000000
    0, // 6955,  11010100 11011000 00000000 00000000
    2, // 6956,  00110100 11011000 00000000 00000000
    0, // 6957,  10110100 11011000 00000000 00000000
    1, // 6958,  01110100 11011000 00000000 00000000
    0, // 6959,  11110100 11011000 00000000 00000000
    4, // 6960,  00001100 11011000 00000000 00000000
    0, // 6961,  10001100 11011000 00000000 00000000
    1, // 6962,  01001100 11011000 00000000 00000000
    0, // 6963,  11001100 11011000 00000000 00000000
    2, // 6964,  00101100 11011000 00000000 00000000
    0, // 6965,  10101100 11011000 00000000 00000000
    1, // 6966,  01101100 11011000 00000000 00000000
    0, // 6967,  11101100 11011000 00000000 00000000
    3, // 6968,  00011100 11011000 00000000 00000000
    0, // 6969,  10011100 11011000 00000000 00000000
    1, // 6970,  01011100 11011000 00000000 00000000
    0, // 6971,  11011100 11011000 00000000 00000000
    2, // 6972,  00111100 11011000 00000000 00000000
    0, // 6973,  10111100 11011000 00000000 00000000
    1, // 6974,  01111100 11011000 00000000 00000000
    0, // 6975,  11111100 11011000 00000000 00000000
    6, // 6976,  00000010 11011000 00000000 00000000
    0, // 6977,  10000010 11011000 00000000 00000000
    1, // 6978,  01000010 11011000 00000000 00000000
    0, // 6979,  11000010 11011000 00000000 00000000
    2, // 6980,  00100010 11011000 00000000 00000000
    0, // 6981,  10100010 11011000 00000000 00000000
    1, // 6982,  01100010 11011000 00000000 00000000
    0, // 6983,  11100010 11011000 00000000 00000000
    3, // 6984,  00010010 11011000 00000000 00000000
    0, // 6985,  10010010 11011000 00000000 00000000
    1, // 6986,  01010010 11011000 00000000 00000000
    0, // 6987,  11010010 11011000 00000000 00000000
    2, // 6988,  00110010 11011000 00000000 00000000
    0, // 6989,  10110010 11011000 00000000 00000000
    1, // 6990,  01110010 11011000 00000000 00000000
    0, // 6991,  11110010 11011000 00000000 00000000
    4, // 6992,  00001010 11011000 00000000 00000000
    0, // 6993,  10001010 11011000 00000000 00000000
    1, // 6994,  01001010 11011000 00000000 00000000
    0, // 6995,  11001010 11011000 00000000 00000000
    2, // 6996,  00101010 11011000 00000000 00000000
    0, // 6997,  10101010 11011000 00000000 00000000
    1, // 6998,  01101010 11011000 00000000 00000000
    0, // 6999,  11101010 11011000 00000000 00000000
    3, // 7000,  00011010 11011000 00000000 00000000
    0, // 7001,  10011010 11011000 00000000 00000000
    1, // 7002,  01011010 11011000 00000000 00000000
    0, // 7003,  11011010 11011000 00000000 00000000
    2, // 7004,  00111010 11011000 00000000 00000000
    0, // 7005,  10111010 11011000 00000000 00000000
    1, // 7006,  01111010 11011000 00000000 00000000
    0, // 7007,  11111010 11011000 00000000 00000000
    5, // 7008,  00000110 11011000 00000000 00000000
    0, // 7009,  10000110 11011000 00000000 00000000
    1, // 7010,  01000110 11011000 00000000 00000000
    0, // 7011,  11000110 11011000 00000000 00000000
    2, // 7012,  00100110 11011000 00000000 00000000
    0, // 7013,  10100110 11011000 00000000 00000000
    1, // 7014,  01100110 11011000 00000000 00000000
    0, // 7015,  11100110 11011000 00000000 00000000
    3, // 7016,  00010110 11011000 00000000 00000000
    0, // 7017,  10010110 11011000 00000000 00000000
    1, // 7018,  01010110 11011000 00000000 00000000
    0, // 7019,  11010110 11011000 00000000 00000000
    2, // 7020,  00110110 11011000 00000000 00000000
    0, // 7021,  10110110 11011000 00000000 00000000
    1, // 7022,  01110110 11011000 00000000 00000000
    0, // 7023,  11110110 11011000 00000000 00000000
    4, // 7024,  00001110 11011000 00000000 00000000
    0, // 7025,  10001110 11011000 00000000 00000000
    1, // 7026,  01001110 11011000 00000000 00000000
    0, // 7027,  11001110 11011000 00000000 00000000
    2, // 7028,  00101110 11011000 00000000 00000000
    0, // 7029,  10101110 11011000 00000000 00000000
    1, // 7030,  01101110 11011000 00000000 00000000
    0, // 7031,  11101110 11011000 00000000 00000000
    3, // 7032,  00011110 11011000 00000000 00000000
    0, // 7033,  10011110 11011000 00000000 00000000
    1, // 7034,  01011110 11011000 00000000 00000000
    0, // 7035,  11011110 11011000 00000000 00000000
    2, // 7036,  00111110 11011000 00000000 00000000
    0, // 7037,  10111110 11011000 00000000 00000000
    1, // 7038,  01111110 11011000 00000000 00000000
    0, // 7039,  11111110 11011000 00000000 00000000
    7, // 7040,  00000001 11011000 00000000 00000000
    0, // 7041,  10000001 11011000 00000000 00000000
    1, // 7042,  01000001 11011000 00000000 00000000
    0, // 7043,  11000001 11011000 00000000 00000000
    2, // 7044,  00100001 11011000 00000000 00000000
    0, // 7045,  10100001 11011000 00000000 00000000
    1, // 7046,  01100001 11011000 00000000 00000000
    0, // 7047,  11100001 11011000 00000000 00000000
    3, // 7048,  00010001 11011000 00000000 00000000
    0, // 7049,  10010001 11011000 00000000 00000000
    1, // 7050,  01010001 11011000 00000000 00000000
    0, // 7051,  11010001 11011000 00000000 00000000
    2, // 7052,  00110001 11011000 00000000 00000000
    0, // 7053,  10110001 11011000 00000000 00000000
    1, // 7054,  01110001 11011000 00000000 00000000
    0, // 7055,  11110001 11011000 00000000 00000000
    4, // 7056,  00001001 11011000 00000000 00000000
    0, // 7057,  10001001 11011000 00000000 00000000
    1, // 7058,  01001001 11011000 00000000 00000000
    0, // 7059,  11001001 11011000 00000000 00000000
    2, // 7060,  00101001 11011000 00000000 00000000
    0, // 7061,  10101001 11011000 00000000 00000000
    1, // 7062,  01101001 11011000 00000000 00000000
    0, // 7063,  11101001 11011000 00000000 00000000
    3, // 7064,  00011001 11011000 00000000 00000000
    0, // 7065,  10011001 11011000 00000000 00000000
    1, // 7066,  01011001 11011000 00000000 00000000
    0, // 7067,  11011001 11011000 00000000 00000000
    2, // 7068,  00111001 11011000 00000000 00000000
    0, // 7069,  10111001 11011000 00000000 00000000
    1, // 7070,  01111001 11011000 00000000 00000000
    0, // 7071,  11111001 11011000 00000000 00000000
    5, // 7072,  00000101 11011000 00000000 00000000
    0, // 7073,  10000101 11011000 00000000 00000000
    1, // 7074,  01000101 11011000 00000000 00000000
    0, // 7075,  11000101 11011000 00000000 00000000
    2, // 7076,  00100101 11011000 00000000 00000000
    0, // 7077,  10100101 11011000 00000000 00000000
    1, // 7078,  01100101 11011000 00000000 00000000
    0, // 7079,  11100101 11011000 00000000 00000000
    3, // 7080,  00010101 11011000 00000000 00000000
    0, // 7081,  10010101 11011000 00000000 00000000
    1, // 7082,  01010101 11011000 00000000 00000000
    0, // 7083,  11010101 11011000 00000000 00000000
    2, // 7084,  00110101 11011000 00000000 00000000
    0, // 7085,  10110101 11011000 00000000 00000000
    1, // 7086,  01110101 11011000 00000000 00000000
    0, // 7087,  11110101 11011000 00000000 00000000
    4, // 7088,  00001101 11011000 00000000 00000000
    0, // 7089,  10001101 11011000 00000000 00000000
    1, // 7090,  01001101 11011000 00000000 00000000
    0, // 7091,  11001101 11011000 00000000 00000000
    2, // 7092,  00101101 11011000 00000000 00000000
    0, // 7093,  10101101 11011000 00000000 00000000
    1, // 7094,  01101101 11011000 00000000 00000000
    0, // 7095,  11101101 11011000 00000000 00000000
    3, // 7096,  00011101 11011000 00000000 00000000
    0, // 7097,  10011101 11011000 00000000 00000000
    1, // 7098,  01011101 11011000 00000000 00000000
    0, // 7099,  11011101 11011000 00000000 00000000
    2, // 7100,  00111101 11011000 00000000 00000000
    0, // 7101,  10111101 11011000 00000000 00000000
    1, // 7102,  01111101 11011000 00000000 00000000
    0, // 7103,  11111101 11011000 00000000 00000000
    6, // 7104,  00000011 11011000 00000000 00000000
    0, // 7105,  10000011 11011000 00000000 00000000
    1, // 7106,  01000011 11011000 00000000 00000000
    0, // 7107,  11000011 11011000 00000000 00000000
    2, // 7108,  00100011 11011000 00000000 00000000
    0, // 7109,  10100011 11011000 00000000 00000000
    1, // 7110,  01100011 11011000 00000000 00000000
    0, // 7111,  11100011 11011000 00000000 00000000
    3, // 7112,  00010011 11011000 00000000 00000000
    0, // 7113,  10010011 11011000 00000000 00000000
    1, // 7114,  01010011 11011000 00000000 00000000
    0, // 7115,  11010011 11011000 00000000 00000000
    2, // 7116,  00110011 11011000 00000000 00000000
    0, // 7117,  10110011 11011000 00000000 00000000
    1, // 7118,  01110011 11011000 00000000 00000000
    0, // 7119,  11110011 11011000 00000000 00000000
    4, // 7120,  00001011 11011000 00000000 00000000
    0, // 7121,  10001011 11011000 00000000 00000000
    1, // 7122,  01001011 11011000 00000000 00000000
    0, // 7123,  11001011 11011000 00000000 00000000
    2, // 7124,  00101011 11011000 00000000 00000000
    0, // 7125,  10101011 11011000 00000000 00000000
    1, // 7126,  01101011 11011000 00000000 00000000
    0, // 7127,  11101011 11011000 00000000 00000000
    3, // 7128,  00011011 11011000 00000000 00000000
    0, // 7129,  10011011 11011000 00000000 00000000
    1, // 7130,  01011011 11011000 00000000 00000000
    0, // 7131,  11011011 11011000 00000000 00000000
    2, // 7132,  00111011 11011000 00000000 00000000
    0, // 7133,  10111011 11011000 00000000 00000000
    1, // 7134,  01111011 11011000 00000000 00000000
    0, // 7135,  11111011 11011000 00000000 00000000
    5, // 7136,  00000111 11011000 00000000 00000000
    0, // 7137,  10000111 11011000 00000000 00000000
    1, // 7138,  01000111 11011000 00000000 00000000
    0, // 7139,  11000111 11011000 00000000 00000000
    2, // 7140,  00100111 11011000 00000000 00000000
    0, // 7141,  10100111 11011000 00000000 00000000
    1, // 7142,  01100111 11011000 00000000 00000000
    0, // 7143,  11100111 11011000 00000000 00000000
    3, // 7144,  00010111 11011000 00000000 00000000
    0, // 7145,  10010111 11011000 00000000 00000000
    1, // 7146,  01010111 11011000 00000000 00000000
    0, // 7147,  11010111 11011000 00000000 00000000
    2, // 7148,  00110111 11011000 00000000 00000000
    0, // 7149,  10110111 11011000 00000000 00000000
    1, // 7150,  01110111 11011000 00000000 00000000
    0, // 7151,  11110111 11011000 00000000 00000000
    4, // 7152,  00001111 11011000 00000000 00000000
    0, // 7153,  10001111 11011000 00000000 00000000
    1, // 7154,  01001111 11011000 00000000 00000000
    0, // 7155,  11001111 11011000 00000000 00000000
    2, // 7156,  00101111 11011000 00000000 00000000
    0, // 7157,  10101111 11011000 00000000 00000000
    1, // 7158,  01101111 11011000 00000000 00000000
    0, // 7159,  11101111 11011000 00000000 00000000
    3, // 7160,  00011111 11011000 00000000 00000000
    0, // 7161,  10011111 11011000 00000000 00000000
    1, // 7162,  01011111 11011000 00000000 00000000
    0, // 7163,  11011111 11011000 00000000 00000000
    2, // 7164,  00111111 11011000 00000000 00000000
    0, // 7165,  10111111 11011000 00000000 00000000
    1, // 7166,  01111111 11011000 00000000 00000000
    0, // 7167,  11111111 11011000 00000000 00000000
    10, // 7168,  00000000 00111000 00000000 00000000
    0, // 7169,  10000000 00111000 00000000 00000000
    1, // 7170,  01000000 00111000 00000000 00000000
    0, // 7171,  11000000 00111000 00000000 00000000
    2, // 7172,  00100000 00111000 00000000 00000000
    0, // 7173,  10100000 00111000 00000000 00000000
    1, // 7174,  01100000 00111000 00000000 00000000
    0, // 7175,  11100000 00111000 00000000 00000000
    3, // 7176,  00010000 00111000 00000000 00000000
    0, // 7177,  10010000 00111000 00000000 00000000
    1, // 7178,  01010000 00111000 00000000 00000000
    0, // 7179,  11010000 00111000 00000000 00000000
    2, // 7180,  00110000 00111000 00000000 00000000
    0, // 7181,  10110000 00111000 00000000 00000000
    1, // 7182,  01110000 00111000 00000000 00000000
    0, // 7183,  11110000 00111000 00000000 00000000
    4, // 7184,  00001000 00111000 00000000 00000000
    0, // 7185,  10001000 00111000 00000000 00000000
    1, // 7186,  01001000 00111000 00000000 00000000
    0, // 7187,  11001000 00111000 00000000 00000000
    2, // 7188,  00101000 00111000 00000000 00000000
    0, // 7189,  10101000 00111000 00000000 00000000
    1, // 7190,  01101000 00111000 00000000 00000000
    0, // 7191,  11101000 00111000 00000000 00000000
    3, // 7192,  00011000 00111000 00000000 00000000
    0, // 7193,  10011000 00111000 00000000 00000000
    1, // 7194,  01011000 00111000 00000000 00000000
    0, // 7195,  11011000 00111000 00000000 00000000
    2, // 7196,  00111000 00111000 00000000 00000000
    0, // 7197,  10111000 00111000 00000000 00000000
    1, // 7198,  01111000 00111000 00000000 00000000
    0, // 7199,  11111000 00111000 00000000 00000000
    5, // 7200,  00000100 00111000 00000000 00000000
    0, // 7201,  10000100 00111000 00000000 00000000
    1, // 7202,  01000100 00111000 00000000 00000000
    0, // 7203,  11000100 00111000 00000000 00000000
    2, // 7204,  00100100 00111000 00000000 00000000
    0, // 7205,  10100100 00111000 00000000 00000000
    1, // 7206,  01100100 00111000 00000000 00000000
    0, // 7207,  11100100 00111000 00000000 00000000
    3, // 7208,  00010100 00111000 00000000 00000000
    0, // 7209,  10010100 00111000 00000000 00000000
    1, // 7210,  01010100 00111000 00000000 00000000
    0, // 7211,  11010100 00111000 00000000 00000000
    2, // 7212,  00110100 00111000 00000000 00000000
    0, // 7213,  10110100 00111000 00000000 00000000
    1, // 7214,  01110100 00111000 00000000 00000000
    0, // 7215,  11110100 00111000 00000000 00000000
    4, // 7216,  00001100 00111000 00000000 00000000
    0, // 7217,  10001100 00111000 00000000 00000000
    1, // 7218,  01001100 00111000 00000000 00000000
    0, // 7219,  11001100 00111000 00000000 00000000
    2, // 7220,  00101100 00111000 00000000 00000000
    0, // 7221,  10101100 00111000 00000000 00000000
    1, // 7222,  01101100 00111000 00000000 00000000
    0, // 7223,  11101100 00111000 00000000 00000000
    3, // 7224,  00011100 00111000 00000000 00000000
    0, // 7225,  10011100 00111000 00000000 00000000
    1, // 7226,  01011100 00111000 00000000 00000000
    0, // 7227,  11011100 00111000 00000000 00000000
    2, // 7228,  00111100 00111000 00000000 00000000
    0, // 7229,  10111100 00111000 00000000 00000000
    1, // 7230,  01111100 00111000 00000000 00000000
    0, // 7231,  11111100 00111000 00000000 00000000
    6, // 7232,  00000010 00111000 00000000 00000000
    0, // 7233,  10000010 00111000 00000000 00000000
    1, // 7234,  01000010 00111000 00000000 00000000
    0, // 7235,  11000010 00111000 00000000 00000000
    2, // 7236,  00100010 00111000 00000000 00000000
    0, // 7237,  10100010 00111000 00000000 00000000
    1, // 7238,  01100010 00111000 00000000 00000000
    0, // 7239,  11100010 00111000 00000000 00000000
    3, // 7240,  00010010 00111000 00000000 00000000
    0, // 7241,  10010010 00111000 00000000 00000000
    1, // 7242,  01010010 00111000 00000000 00000000
    0, // 7243,  11010010 00111000 00000000 00000000
    2, // 7244,  00110010 00111000 00000000 00000000
    0, // 7245,  10110010 00111000 00000000 00000000
    1, // 7246,  01110010 00111000 00000000 00000000
    0, // 7247,  11110010 00111000 00000000 00000000
    4, // 7248,  00001010 00111000 00000000 00000000
    0, // 7249,  10001010 00111000 00000000 00000000
    1, // 7250,  01001010 00111000 00000000 00000000
    0, // 7251,  11001010 00111000 00000000 00000000
    2, // 7252,  00101010 00111000 00000000 00000000
    0, // 7253,  10101010 00111000 00000000 00000000
    1, // 7254,  01101010 00111000 00000000 00000000
    0, // 7255,  11101010 00111000 00000000 00000000
    3, // 7256,  00011010 00111000 00000000 00000000
    0, // 7257,  10011010 00111000 00000000 00000000
    1, // 7258,  01011010 00111000 00000000 00000000
    0, // 7259,  11011010 00111000 00000000 00000000
    2, // 7260,  00111010 00111000 00000000 00000000
    0, // 7261,  10111010 00111000 00000000 00000000
    1, // 7262,  01111010 00111000 00000000 00000000
    0, // 7263,  11111010 00111000 00000000 00000000
    5, // 7264,  00000110 00111000 00000000 00000000
    0, // 7265,  10000110 00111000 00000000 00000000
    1, // 7266,  01000110 00111000 00000000 00000000
    0, // 7267,  11000110 00111000 00000000 00000000
    2, // 7268,  00100110 00111000 00000000 00000000
    0, // 7269,  10100110 00111000 00000000 00000000
    1, // 7270,  01100110 00111000 00000000 00000000
    0, // 7271,  11100110 00111000 00000000 00000000
    3, // 7272,  00010110 00111000 00000000 00000000
    0, // 7273,  10010110 00111000 00000000 00000000
    1, // 7274,  01010110 00111000 00000000 00000000
    0, // 7275,  11010110 00111000 00000000 00000000
    2, // 7276,  00110110 00111000 00000000 00000000
    0, // 7277,  10110110 00111000 00000000 00000000
    1, // 7278,  01110110 00111000 00000000 00000000
    0, // 7279,  11110110 00111000 00000000 00000000
    4, // 7280,  00001110 00111000 00000000 00000000
    0, // 7281,  10001110 00111000 00000000 00000000
    1, // 7282,  01001110 00111000 00000000 00000000
    0, // 7283,  11001110 00111000 00000000 00000000
    2, // 7284,  00101110 00111000 00000000 00000000
    0, // 7285,  10101110 00111000 00000000 00000000
    1, // 7286,  01101110 00111000 00000000 00000000
    0, // 7287,  11101110 00111000 00000000 00000000
    3, // 7288,  00011110 00111000 00000000 00000000
    0, // 7289,  10011110 00111000 00000000 00000000
    1, // 7290,  01011110 00111000 00000000 00000000
    0, // 7291,  11011110 00111000 00000000 00000000
    2, // 7292,  00111110 00111000 00000000 00000000
    0, // 7293,  10111110 00111000 00000000 00000000
    1, // 7294,  01111110 00111000 00000000 00000000
    0, // 7295,  11111110 00111000 00000000 00000000
    7, // 7296,  00000001 00111000 00000000 00000000
    0, // 7297,  10000001 00111000 00000000 00000000
    1, // 7298,  01000001 00111000 00000000 00000000
    0, // 7299,  11000001 00111000 00000000 00000000
    2, // 7300,  00100001 00111000 00000000 00000000
    0, // 7301,  10100001 00111000 00000000 00000000
    1, // 7302,  01100001 00111000 00000000 00000000
    0, // 7303,  11100001 00111000 00000000 00000000
    3, // 7304,  00010001 00111000 00000000 00000000
    0, // 7305,  10010001 00111000 00000000 00000000
    1, // 7306,  01010001 00111000 00000000 00000000
    0, // 7307,  11010001 00111000 00000000 00000000
    2, // 7308,  00110001 00111000 00000000 00000000
    0, // 7309,  10110001 00111000 00000000 00000000
    1, // 7310,  01110001 00111000 00000000 00000000
    0, // 7311,  11110001 00111000 00000000 00000000
    4, // 7312,  00001001 00111000 00000000 00000000
    0, // 7313,  10001001 00111000 00000000 00000000
    1, // 7314,  01001001 00111000 00000000 00000000
    0, // 7315,  11001001 00111000 00000000 00000000
    2, // 7316,  00101001 00111000 00000000 00000000
    0, // 7317,  10101001 00111000 00000000 00000000
    1, // 7318,  01101001 00111000 00000000 00000000
    0, // 7319,  11101001 00111000 00000000 00000000
    3, // 7320,  00011001 00111000 00000000 00000000
    0, // 7321,  10011001 00111000 00000000 00000000
    1, // 7322,  01011001 00111000 00000000 00000000
    0, // 7323,  11011001 00111000 00000000 00000000
    2, // 7324,  00111001 00111000 00000000 00000000
    0, // 7325,  10111001 00111000 00000000 00000000
    1, // 7326,  01111001 00111000 00000000 00000000
    0, // 7327,  11111001 00111000 00000000 00000000
    5, // 7328,  00000101 00111000 00000000 00000000
    0, // 7329,  10000101 00111000 00000000 00000000
    1, // 7330,  01000101 00111000 00000000 00000000
    0, // 7331,  11000101 00111000 00000000 00000000
    2, // 7332,  00100101 00111000 00000000 00000000
    0, // 7333,  10100101 00111000 00000000 00000000
    1, // 7334,  01100101 00111000 00000000 00000000
    0, // 7335,  11100101 00111000 00000000 00000000
    3, // 7336,  00010101 00111000 00000000 00000000
    0, // 7337,  10010101 00111000 00000000 00000000
    1, // 7338,  01010101 00111000 00000000 00000000
    0, // 7339,  11010101 00111000 00000000 00000000
    2, // 7340,  00110101 00111000 00000000 00000000
    0, // 7341,  10110101 00111000 00000000 00000000
    1, // 7342,  01110101 00111000 00000000 00000000
    0, // 7343,  11110101 00111000 00000000 00000000
    4, // 7344,  00001101 00111000 00000000 00000000
    0, // 7345,  10001101 00111000 00000000 00000000
    1, // 7346,  01001101 00111000 00000000 00000000
    0, // 7347,  11001101 00111000 00000000 00000000
    2, // 7348,  00101101 00111000 00000000 00000000
    0, // 7349,  10101101 00111000 00000000 00000000
    1, // 7350,  01101101 00111000 00000000 00000000
    0, // 7351,  11101101 00111000 00000000 00000000
    3, // 7352,  00011101 00111000 00000000 00000000
    0, // 7353,  10011101 00111000 00000000 00000000
    1, // 7354,  01011101 00111000 00000000 00000000
    0, // 7355,  11011101 00111000 00000000 00000000
    2, // 7356,  00111101 00111000 00000000 00000000
    0, // 7357,  10111101 00111000 00000000 00000000
    1, // 7358,  01111101 00111000 00000000 00000000
    0, // 7359,  11111101 00111000 00000000 00000000
    6, // 7360,  00000011 00111000 00000000 00000000
    0, // 7361,  10000011 00111000 00000000 00000000
    1, // 7362,  01000011 00111000 00000000 00000000
    0, // 7363,  11000011 00111000 00000000 00000000
    2, // 7364,  00100011 00111000 00000000 00000000
    0, // 7365,  10100011 00111000 00000000 00000000
    1, // 7366,  01100011 00111000 00000000 00000000
    0, // 7367,  11100011 00111000 00000000 00000000
    3, // 7368,  00010011 00111000 00000000 00000000
    0, // 7369,  10010011 00111000 00000000 00000000
    1, // 7370,  01010011 00111000 00000000 00000000
    0, // 7371,  11010011 00111000 00000000 00000000
    2, // 7372,  00110011 00111000 00000000 00000000
    0, // 7373,  10110011 00111000 00000000 00000000
    1, // 7374,  01110011 00111000 00000000 00000000
    0, // 7375,  11110011 00111000 00000000 00000000
    4, // 7376,  00001011 00111000 00000000 00000000
    0, // 7377,  10001011 00111000 00000000 00000000
    1, // 7378,  01001011 00111000 00000000 00000000
    0, // 7379,  11001011 00111000 00000000 00000000
    2, // 7380,  00101011 00111000 00000000 00000000
    0, // 7381,  10101011 00111000 00000000 00000000
    1, // 7382,  01101011 00111000 00000000 00000000
    0, // 7383,  11101011 00111000 00000000 00000000
    3, // 7384,  00011011 00111000 00000000 00000000
    0, // 7385,  10011011 00111000 00000000 00000000
    1, // 7386,  01011011 00111000 00000000 00000000
    0, // 7387,  11011011 00111000 00000000 00000000
    2, // 7388,  00111011 00111000 00000000 00000000
    0, // 7389,  10111011 00111000 00000000 00000000
    1, // 7390,  01111011 00111000 00000000 00000000
    0, // 7391,  11111011 00111000 00000000 00000000
    5, // 7392,  00000111 00111000 00000000 00000000
    0, // 7393,  10000111 00111000 00000000 00000000
    1, // 7394,  01000111 00111000 00000000 00000000
    0, // 7395,  11000111 00111000 00000000 00000000
    2, // 7396,  00100111 00111000 00000000 00000000
    0, // 7397,  10100111 00111000 00000000 00000000
    1, // 7398,  01100111 00111000 00000000 00000000
    0, // 7399,  11100111 00111000 00000000 00000000
    3, // 7400,  00010111 00111000 00000000 00000000
    0, // 7401,  10010111 00111000 00000000 00000000
    1, // 7402,  01010111 00111000 00000000 00000000
    0, // 7403,  11010111 00111000 00000000 00000000
    2, // 7404,  00110111 00111000 00000000 00000000
    0, // 7405,  10110111 00111000 00000000 00000000
    1, // 7406,  01110111 00111000 00000000 00000000
    0, // 7407,  11110111 00111000 00000000 00000000
    4, // 7408,  00001111 00111000 00000000 00000000
    0, // 7409,  10001111 00111000 00000000 00000000
    1, // 7410,  01001111 00111000 00000000 00000000
    0, // 7411,  11001111 00111000 00000000 00000000
    2, // 7412,  00101111 00111000 00000000 00000000
    0, // 7413,  10101111 00111000 00000000 00000000
    1, // 7414,  01101111 00111000 00000000 00000000
    0, // 7415,  11101111 00111000 00000000 00000000
    3, // 7416,  00011111 00111000 00000000 00000000
    0, // 7417,  10011111 00111000 00000000 00000000
    1, // 7418,  01011111 00111000 00000000 00000000
    0, // 7419,  11011111 00111000 00000000 00000000
    2, // 7420,  00111111 00111000 00000000 00000000
    0, // 7421,  10111111 00111000 00000000 00000000
    1, // 7422,  01111111 00111000 00000000 00000000
    0, // 7423,  11111111 00111000 00000000 00000000
    8, // 7424,  00000000 10111000 00000000 00000000
    0, // 7425,  10000000 10111000 00000000 00000000
    1, // 7426,  01000000 10111000 00000000 00000000
    0, // 7427,  11000000 10111000 00000000 00000000
    2, // 7428,  00100000 10111000 00000000 00000000
    0, // 7429,  10100000 10111000 00000000 00000000
    1, // 7430,  01100000 10111000 00000000 00000000
    0, // 7431,  11100000 10111000 00000000 00000000
    3, // 7432,  00010000 10111000 00000000 00000000
    0, // 7433,  10010000 10111000 00000000 00000000
    1, // 7434,  01010000 10111000 00000000 00000000
    0, // 7435,  11010000 10111000 00000000 00000000
    2, // 7436,  00110000 10111000 00000000 00000000
    0, // 7437,  10110000 10111000 00000000 00000000
    1, // 7438,  01110000 10111000 00000000 00000000
    0, // 7439,  11110000 10111000 00000000 00000000
    4, // 7440,  00001000 10111000 00000000 00000000
    0, // 7441,  10001000 10111000 00000000 00000000
    1, // 7442,  01001000 10111000 00000000 00000000
    0, // 7443,  11001000 10111000 00000000 00000000
    2, // 7444,  00101000 10111000 00000000 00000000
    0, // 7445,  10101000 10111000 00000000 00000000
    1, // 7446,  01101000 10111000 00000000 00000000
    0, // 7447,  11101000 10111000 00000000 00000000
    3, // 7448,  00011000 10111000 00000000 00000000
    0, // 7449,  10011000 10111000 00000000 00000000
    1, // 7450,  01011000 10111000 00000000 00000000
    0, // 7451,  11011000 10111000 00000000 00000000
    2, // 7452,  00111000 10111000 00000000 00000000
    0, // 7453,  10111000 10111000 00000000 00000000
    1, // 7454,  01111000 10111000 00000000 00000000
    0, // 7455,  11111000 10111000 00000000 00000000
    5, // 7456,  00000100 10111000 00000000 00000000
    0, // 7457,  10000100 10111000 00000000 00000000
    1, // 7458,  01000100 10111000 00000000 00000000
    0, // 7459,  11000100 10111000 00000000 00000000
    2, // 7460,  00100100 10111000 00000000 00000000
    0, // 7461,  10100100 10111000 00000000 00000000
    1, // 7462,  01100100 10111000 00000000 00000000
    0, // 7463,  11100100 10111000 00000000 00000000
    3, // 7464,  00010100 10111000 00000000 00000000
    0, // 7465,  10010100 10111000 00000000 00000000
    1, // 7466,  01010100 10111000 00000000 00000000
    0, // 7467,  11010100 10111000 00000000 00000000
    2, // 7468,  00110100 10111000 00000000 00000000
    0, // 7469,  10110100 10111000 00000000 00000000
    1, // 7470,  01110100 10111000 00000000 00000000
    0, // 7471,  11110100 10111000 00000000 00000000
    4, // 7472,  00001100 10111000 00000000 00000000
    0, // 7473,  10001100 10111000 00000000 00000000
    1, // 7474,  01001100 10111000 00000000 00000000
    0, // 7475,  11001100 10111000 00000000 00000000
    2, // 7476,  00101100 10111000 00000000 00000000
    0, // 7477,  10101100 10111000 00000000 00000000
    1, // 7478,  01101100 10111000 00000000 00000000
    0, // 7479,  11101100 10111000 00000000 00000000
    3, // 7480,  00011100 10111000 00000000 00000000
    0, // 7481,  10011100 10111000 00000000 00000000
    1, // 7482,  01011100 10111000 00000000 00000000
    0, // 7483,  11011100 10111000 00000000 00000000
    2, // 7484,  00111100 10111000 00000000 00000000
    0, // 7485,  10111100 10111000 00000000 00000000
    1, // 7486,  01111100 10111000 00000000 00000000
    0, // 7487,  11111100 10111000 00000000 00000000
    6, // 7488,  00000010 10111000 00000000 00000000
    0, // 7489,  10000010 10111000 00000000 00000000
    1, // 7490,  01000010 10111000 00000000 00000000
    0, // 7491,  11000010 10111000 00000000 00000000
    2, // 7492,  00100010 10111000 00000000 00000000
    0, // 7493,  10100010 10111000 00000000 00000000
    1, // 7494,  01100010 10111000 00000000 00000000
    0, // 7495,  11100010 10111000 00000000 00000000
    3, // 7496,  00010010 10111000 00000000 00000000
    0, // 7497,  10010010 10111000 00000000 00000000
    1, // 7498,  01010010 10111000 00000000 00000000
    0, // 7499,  11010010 10111000 00000000 00000000
    2, // 7500,  00110010 10111000 00000000 00000000
    0, // 7501,  10110010 10111000 00000000 00000000
    1, // 7502,  01110010 10111000 00000000 00000000
    0, // 7503,  11110010 10111000 00000000 00000000
    4, // 7504,  00001010 10111000 00000000 00000000
    0, // 7505,  10001010 10111000 00000000 00000000
    1, // 7506,  01001010 10111000 00000000 00000000
    0, // 7507,  11001010 10111000 00000000 00000000
    2, // 7508,  00101010 10111000 00000000 00000000
    0, // 7509,  10101010 10111000 00000000 00000000
    1, // 7510,  01101010 10111000 00000000 00000000
    0, // 7511,  11101010 10111000 00000000 00000000
    3, // 7512,  00011010 10111000 00000000 00000000
    0, // 7513,  10011010 10111000 00000000 00000000
    1, // 7514,  01011010 10111000 00000000 00000000
    0, // 7515,  11011010 10111000 00000000 00000000
    2, // 7516,  00111010 10111000 00000000 00000000
    0, // 7517,  10111010 10111000 00000000 00000000
    1, // 7518,  01111010 10111000 00000000 00000000
    0, // 7519,  11111010 10111000 00000000 00000000
    5, // 7520,  00000110 10111000 00000000 00000000
    0, // 7521,  10000110 10111000 00000000 00000000
    1, // 7522,  01000110 10111000 00000000 00000000
    0, // 7523,  11000110 10111000 00000000 00000000
    2, // 7524,  00100110 10111000 00000000 00000000
    0, // 7525,  10100110 10111000 00000000 00000000
    1, // 7526,  01100110 10111000 00000000 00000000
    0, // 7527,  11100110 10111000 00000000 00000000
    3, // 7528,  00010110 10111000 00000000 00000000
    0, // 7529,  10010110 10111000 00000000 00000000
    1, // 7530,  01010110 10111000 00000000 00000000
    0, // 7531,  11010110 10111000 00000000 00000000
    2, // 7532,  00110110 10111000 00000000 00000000
    0, // 7533,  10110110 10111000 00000000 00000000
    1, // 7534,  01110110 10111000 00000000 00000000
    0, // 7535,  11110110 10111000 00000000 00000000
    4, // 7536,  00001110 10111000 00000000 00000000
    0, // 7537,  10001110 10111000 00000000 00000000
    1, // 7538,  01001110 10111000 00000000 00000000
    0, // 7539,  11001110 10111000 00000000 00000000
    2, // 7540,  00101110 10111000 00000000 00000000
    0, // 7541,  10101110 10111000 00000000 00000000
    1, // 7542,  01101110 10111000 00000000 00000000
    0, // 7543,  11101110 10111000 00000000 00000000
    3, // 7544,  00011110 10111000 00000000 00000000
    0, // 7545,  10011110 10111000 00000000 00000000
    1, // 7546,  01011110 10111000 00000000 00000000
    0, // 7547,  11011110 10111000 00000000 00000000
    2, // 7548,  00111110 10111000 00000000 00000000
    0, // 7549,  10111110 10111000 00000000 00000000
    1, // 7550,  01111110 10111000 00000000 00000000
    0, // 7551,  11111110 10111000 00000000 00000000
    7, // 7552,  00000001 10111000 00000000 00000000
    0, // 7553,  10000001 10111000 00000000 00000000
    1, // 7554,  01000001 10111000 00000000 00000000
    0, // 7555,  11000001 10111000 00000000 00000000
    2, // 7556,  00100001 10111000 00000000 00000000
    0, // 7557,  10100001 10111000 00000000 00000000
    1, // 7558,  01100001 10111000 00000000 00000000
    0, // 7559,  11100001 10111000 00000000 00000000
    3, // 7560,  00010001 10111000 00000000 00000000
    0, // 7561,  10010001 10111000 00000000 00000000
    1, // 7562,  01010001 10111000 00000000 00000000
    0, // 7563,  11010001 10111000 00000000 00000000
    2, // 7564,  00110001 10111000 00000000 00000000
    0, // 7565,  10110001 10111000 00000000 00000000
    1, // 7566,  01110001 10111000 00000000 00000000
    0, // 7567,  11110001 10111000 00000000 00000000
    4, // 7568,  00001001 10111000 00000000 00000000
    0, // 7569,  10001001 10111000 00000000 00000000
    1, // 7570,  01001001 10111000 00000000 00000000
    0, // 7571,  11001001 10111000 00000000 00000000
    2, // 7572,  00101001 10111000 00000000 00000000
    0, // 7573,  10101001 10111000 00000000 00000000
    1, // 7574,  01101001 10111000 00000000 00000000
    0, // 7575,  11101001 10111000 00000000 00000000
    3, // 7576,  00011001 10111000 00000000 00000000
    0, // 7577,  10011001 10111000 00000000 00000000
    1, // 7578,  01011001 10111000 00000000 00000000
    0, // 7579,  11011001 10111000 00000000 00000000
    2, // 7580,  00111001 10111000 00000000 00000000
    0, // 7581,  10111001 10111000 00000000 00000000
    1, // 7582,  01111001 10111000 00000000 00000000
    0, // 7583,  11111001 10111000 00000000 00000000
    5, // 7584,  00000101 10111000 00000000 00000000
    0, // 7585,  10000101 10111000 00000000 00000000
    1, // 7586,  01000101 10111000 00000000 00000000
    0, // 7587,  11000101 10111000 00000000 00000000
    2, // 7588,  00100101 10111000 00000000 00000000
    0, // 7589,  10100101 10111000 00000000 00000000
    1, // 7590,  01100101 10111000 00000000 00000000
    0, // 7591,  11100101 10111000 00000000 00000000
    3, // 7592,  00010101 10111000 00000000 00000000
    0, // 7593,  10010101 10111000 00000000 00000000
    1, // 7594,  01010101 10111000 00000000 00000000
    0, // 7595,  11010101 10111000 00000000 00000000
    2, // 7596,  00110101 10111000 00000000 00000000
    0, // 7597,  10110101 10111000 00000000 00000000
    1, // 7598,  01110101 10111000 00000000 00000000
    0, // 7599,  11110101 10111000 00000000 00000000
    4, // 7600,  00001101 10111000 00000000 00000000
    0, // 7601,  10001101 10111000 00000000 00000000
    1, // 7602,  01001101 10111000 00000000 00000000
    0, // 7603,  11001101 10111000 00000000 00000000
    2, // 7604,  00101101 10111000 00000000 00000000
    0, // 7605,  10101101 10111000 00000000 00000000
    1, // 7606,  01101101 10111000 00000000 00000000
    0, // 7607,  11101101 10111000 00000000 00000000
    3, // 7608,  00011101 10111000 00000000 00000000
    0, // 7609,  10011101 10111000 00000000 00000000
    1, // 7610,  01011101 10111000 00000000 00000000
    0, // 7611,  11011101 10111000 00000000 00000000
    2, // 7612,  00111101 10111000 00000000 00000000
    0, // 7613,  10111101 10111000 00000000 00000000
    1, // 7614,  01111101 10111000 00000000 00000000
    0, // 7615,  11111101 10111000 00000000 00000000
    6, // 7616,  00000011 10111000 00000000 00000000
    0, // 7617,  10000011 10111000 00000000 00000000
    1, // 7618,  01000011 10111000 00000000 00000000
    0, // 7619,  11000011 10111000 00000000 00000000
    2, // 7620,  00100011 10111000 00000000 00000000
    0, // 7621,  10100011 10111000 00000000 00000000
    1, // 7622,  01100011 10111000 00000000 00000000
    0, // 7623,  11100011 10111000 00000000 00000000
    3, // 7624,  00010011 10111000 00000000 00000000
    0, // 7625,  10010011 10111000 00000000 00000000
    1, // 7626,  01010011 10111000 00000000 00000000
    0, // 7627,  11010011 10111000 00000000 00000000
    2, // 7628,  00110011 10111000 00000000 00000000
    0, // 7629,  10110011 10111000 00000000 00000000
    1, // 7630,  01110011 10111000 00000000 00000000
    0, // 7631,  11110011 10111000 00000000 00000000
    4, // 7632,  00001011 10111000 00000000 00000000
    0, // 7633,  10001011 10111000 00000000 00000000
    1, // 7634,  01001011 10111000 00000000 00000000
    0, // 7635,  11001011 10111000 00000000 00000000
    2, // 7636,  00101011 10111000 00000000 00000000
    0, // 7637,  10101011 10111000 00000000 00000000
    1, // 7638,  01101011 10111000 00000000 00000000
    0, // 7639,  11101011 10111000 00000000 00000000
    3, // 7640,  00011011 10111000 00000000 00000000
    0, // 7641,  10011011 10111000 00000000 00000000
    1, // 7642,  01011011 10111000 00000000 00000000
    0, // 7643,  11011011 10111000 00000000 00000000
    2, // 7644,  00111011 10111000 00000000 00000000
    0, // 7645,  10111011 10111000 00000000 00000000
    1, // 7646,  01111011 10111000 00000000 00000000
    0, // 7647,  11111011 10111000 00000000 00000000
    5, // 7648,  00000111 10111000 00000000 00000000
    0, // 7649,  10000111 10111000 00000000 00000000
    1, // 7650,  01000111 10111000 00000000 00000000
    0, // 7651,  11000111 10111000 00000000 00000000
    2, // 7652,  00100111 10111000 00000000 00000000
    0, // 7653,  10100111 10111000 00000000 00000000
    1, // 7654,  01100111 10111000 00000000 00000000
    0, // 7655,  11100111 10111000 00000000 00000000
    3, // 7656,  00010111 10111000 00000000 00000000
    0, // 7657,  10010111 10111000 00000000 00000000
    1, // 7658,  01010111 10111000 00000000 00000000
    0, // 7659,  11010111 10111000 00000000 00000000
    2, // 7660,  00110111 10111000 00000000 00000000
    0, // 7661,  10110111 10111000 00000000 00000000
    1, // 7662,  01110111 10111000 00000000 00000000
    0, // 7663,  11110111 10111000 00000000 00000000
    4, // 7664,  00001111 10111000 00000000 00000000
    0, // 7665,  10001111 10111000 00000000 00000000
    1, // 7666,  01001111 10111000 00000000 00000000
    0, // 7667,  11001111 10111000 00000000 00000000
    2, // 7668,  00101111 10111000 00000000 00000000
    0, // 7669,  10101111 10111000 00000000 00000000
    1, // 7670,  01101111 10111000 00000000 00000000
    0, // 7671,  11101111 10111000 00000000 00000000
    3, // 7672,  00011111 10111000 00000000 00000000
    0, // 7673,  10011111 10111000 00000000 00000000
    1, // 7674,  01011111 10111000 00000000 00000000
    0, // 7675,  11011111 10111000 00000000 00000000
    2, // 7676,  00111111 10111000 00000000 00000000
    0, // 7677,  10111111 10111000 00000000 00000000
    1, // 7678,  01111111 10111000 00000000 00000000
    0, // 7679,  11111111 10111000 00000000 00000000
    9, // 7680,  00000000 01111000 00000000 00000000
    0, // 7681,  10000000 01111000 00000000 00000000
    1, // 7682,  01000000 01111000 00000000 00000000
    0, // 7683,  11000000 01111000 00000000 00000000
    2, // 7684,  00100000 01111000 00000000 00000000
    0, // 7685,  10100000 01111000 00000000 00000000
    1, // 7686,  01100000 01111000 00000000 00000000
    0, // 7687,  11100000 01111000 00000000 00000000
    3, // 7688,  00010000 01111000 00000000 00000000
    0, // 7689,  10010000 01111000 00000000 00000000
    1, // 7690,  01010000 01111000 00000000 00000000
    0, // 7691,  11010000 01111000 00000000 00000000
    2, // 7692,  00110000 01111000 00000000 00000000
    0, // 7693,  10110000 01111000 00000000 00000000
    1, // 7694,  01110000 01111000 00000000 00000000
    0, // 7695,  11110000 01111000 00000000 00000000
    4, // 7696,  00001000 01111000 00000000 00000000
    0, // 7697,  10001000 01111000 00000000 00000000
    1, // 7698,  01001000 01111000 00000000 00000000
    0, // 7699,  11001000 01111000 00000000 00000000
    2, // 7700,  00101000 01111000 00000000 00000000
    0, // 7701,  10101000 01111000 00000000 00000000
    1, // 7702,  01101000 01111000 00000000 00000000
    0, // 7703,  11101000 01111000 00000000 00000000
    3, // 7704,  00011000 01111000 00000000 00000000
    0, // 7705,  10011000 01111000 00000000 00000000
    1, // 7706,  01011000 01111000 00000000 00000000
    0, // 7707,  11011000 01111000 00000000 00000000
    2, // 7708,  00111000 01111000 00000000 00000000
    0, // 7709,  10111000 01111000 00000000 00000000
    1, // 7710,  01111000 01111000 00000000 00000000
    0, // 7711,  11111000 01111000 00000000 00000000
    5, // 7712,  00000100 01111000 00000000 00000000
    0, // 7713,  10000100 01111000 00000000 00000000
    1, // 7714,  01000100 01111000 00000000 00000000
    0, // 7715,  11000100 01111000 00000000 00000000
    2, // 7716,  00100100 01111000 00000000 00000000
    0, // 7717,  10100100 01111000 00000000 00000000
    1, // 7718,  01100100 01111000 00000000 00000000
    0, // 7719,  11100100 01111000 00000000 00000000
    3, // 7720,  00010100 01111000 00000000 00000000
    0, // 7721,  10010100 01111000 00000000 00000000
    1, // 7722,  01010100 01111000 00000000 00000000
    0, // 7723,  11010100 01111000 00000000 00000000
    2, // 7724,  00110100 01111000 00000000 00000000
    0, // 7725,  10110100 01111000 00000000 00000000
    1, // 7726,  01110100 01111000 00000000 00000000
    0, // 7727,  11110100 01111000 00000000 00000000
    4, // 7728,  00001100 01111000 00000000 00000000
    0, // 7729,  10001100 01111000 00000000 00000000
    1, // 7730,  01001100 01111000 00000000 00000000
    0, // 7731,  11001100 01111000 00000000 00000000
    2, // 7732,  00101100 01111000 00000000 00000000
    0, // 7733,  10101100 01111000 00000000 00000000
    1, // 7734,  01101100 01111000 00000000 00000000
    0, // 7735,  11101100 01111000 00000000 00000000
    3, // 7736,  00011100 01111000 00000000 00000000
    0, // 7737,  10011100 01111000 00000000 00000000
    1, // 7738,  01011100 01111000 00000000 00000000
    0, // 7739,  11011100 01111000 00000000 00000000
    2, // 7740,  00111100 01111000 00000000 00000000
    0, // 7741,  10111100 01111000 00000000 00000000
    1, // 7742,  01111100 01111000 00000000 00000000
    0, // 7743,  11111100 01111000 00000000 00000000
    6, // 7744,  00000010 01111000 00000000 00000000
    0, // 7745,  10000010 01111000 00000000 00000000
    1, // 7746,  01000010 01111000 00000000 00000000
    0, // 7747,  11000010 01111000 00000000 00000000
    2, // 7748,  00100010 01111000 00000000 00000000
    0, // 7749,  10100010 01111000 00000000 00000000
    1, // 7750,  01100010 01111000 00000000 00000000
    0, // 7751,  11100010 01111000 00000000 00000000
    3, // 7752,  00010010 01111000 00000000 00000000
    0, // 7753,  10010010 01111000 00000000 00000000
    1, // 7754,  01010010 01111000 00000000 00000000
    0, // 7755,  11010010 01111000 00000000 00000000
    2, // 7756,  00110010 01111000 00000000 00000000
    0, // 7757,  10110010 01111000 00000000 00000000
    1, // 7758,  01110010 01111000 00000000 00000000
    0, // 7759,  11110010 01111000 00000000 00000000
    4, // 7760,  00001010 01111000 00000000 00000000
    0, // 7761,  10001010 01111000 00000000 00000000
    1, // 7762,  01001010 01111000 00000000 00000000
    0, // 7763,  11001010 01111000 00000000 00000000
    2, // 7764,  00101010 01111000 00000000 00000000
    0, // 7765,  10101010 01111000 00000000 00000000
    1, // 7766,  01101010 01111000 00000000 00000000
    0, // 7767,  11101010 01111000 00000000 00000000
    3, // 7768,  00011010 01111000 00000000 00000000
    0, // 7769,  10011010 01111000 00000000 00000000
    1, // 7770,  01011010 01111000 00000000 00000000
    0, // 7771,  11011010 01111000 00000000 00000000
    2, // 7772,  00111010 01111000 00000000 00000000
    0, // 7773,  10111010 01111000 00000000 00000000
    1, // 7774,  01111010 01111000 00000000 00000000
    0, // 7775,  11111010 01111000 00000000 00000000
    5, // 7776,  00000110 01111000 00000000 00000000
    0, // 7777,  10000110 01111000 00000000 00000000
    1, // 7778,  01000110 01111000 00000000 00000000
    0, // 7779,  11000110 01111000 00000000 00000000
    2, // 7780,  00100110 01111000 00000000 00000000
    0, // 7781,  10100110 01111000 00000000 00000000
    1, // 7782,  01100110 01111000 00000000 00000000
    0, // 7783,  11100110 01111000 00000000 00000000
    3, // 7784,  00010110 01111000 00000000 00000000
    0, // 7785,  10010110 01111000 00000000 00000000
    1, // 7786,  01010110 01111000 00000000 00000000
    0, // 7787,  11010110 01111000 00000000 00000000
    2, // 7788,  00110110 01111000 00000000 00000000
    0, // 7789,  10110110 01111000 00000000 00000000
    1, // 7790,  01110110 01111000 00000000 00000000
    0, // 7791,  11110110 01111000 00000000 00000000
    4, // 7792,  00001110 01111000 00000000 00000000
    0, // 7793,  10001110 01111000 00000000 00000000
    1, // 7794,  01001110 01111000 00000000 00000000
    0, // 7795,  11001110 01111000 00000000 00000000
    2, // 7796,  00101110 01111000 00000000 00000000
    0, // 7797,  10101110 01111000 00000000 00000000
    1, // 7798,  01101110 01111000 00000000 00000000
    0, // 7799,  11101110 01111000 00000000 00000000
    3, // 7800,  00011110 01111000 00000000 00000000
    0, // 7801,  10011110 01111000 00000000 00000000
    1, // 7802,  01011110 01111000 00000000 00000000
    0, // 7803,  11011110 01111000 00000000 00000000
    2, // 7804,  00111110 01111000 00000000 00000000
    0, // 7805,  10111110 01111000 00000000 00000000
    1, // 7806,  01111110 01111000 00000000 00000000
    0, // 7807,  11111110 01111000 00000000 00000000
    7, // 7808,  00000001 01111000 00000000 00000000
    0, // 7809,  10000001 01111000 00000000 00000000
    1, // 7810,  01000001 01111000 00000000 00000000
    0, // 7811,  11000001 01111000 00000000 00000000
    2, // 7812,  00100001 01111000 00000000 00000000
    0, // 7813,  10100001 01111000 00000000 00000000
    1, // 7814,  01100001 01111000 00000000 00000000
    0, // 7815,  11100001 01111000 00000000 00000000
    3, // 7816,  00010001 01111000 00000000 00000000
    0, // 7817,  10010001 01111000 00000000 00000000
    1, // 7818,  01010001 01111000 00000000 00000000
    0, // 7819,  11010001 01111000 00000000 00000000
    2, // 7820,  00110001 01111000 00000000 00000000
    0, // 7821,  10110001 01111000 00000000 00000000
    1, // 7822,  01110001 01111000 00000000 00000000
    0, // 7823,  11110001 01111000 00000000 00000000
    4, // 7824,  00001001 01111000 00000000 00000000
    0, // 7825,  10001001 01111000 00000000 00000000
    1, // 7826,  01001001 01111000 00000000 00000000
    0, // 7827,  11001001 01111000 00000000 00000000
    2, // 7828,  00101001 01111000 00000000 00000000
    0, // 7829,  10101001 01111000 00000000 00000000
    1, // 7830,  01101001 01111000 00000000 00000000
    0, // 7831,  11101001 01111000 00000000 00000000
    3, // 7832,  00011001 01111000 00000000 00000000
    0, // 7833,  10011001 01111000 00000000 00000000
    1, // 7834,  01011001 01111000 00000000 00000000
    0, // 7835,  11011001 01111000 00000000 00000000
    2, // 7836,  00111001 01111000 00000000 00000000
    0, // 7837,  10111001 01111000 00000000 00000000
    1, // 7838,  01111001 01111000 00000000 00000000
    0, // 7839,  11111001 01111000 00000000 00000000
    5, // 7840,  00000101 01111000 00000000 00000000
    0, // 7841,  10000101 01111000 00000000 00000000
    1, // 7842,  01000101 01111000 00000000 00000000
    0, // 7843,  11000101 01111000 00000000 00000000
    2, // 7844,  00100101 01111000 00000000 00000000
    0, // 7845,  10100101 01111000 00000000 00000000
    1, // 7846,  01100101 01111000 00000000 00000000
    0, // 7847,  11100101 01111000 00000000 00000000
    3, // 7848,  00010101 01111000 00000000 00000000
    0, // 7849,  10010101 01111000 00000000 00000000
    1, // 7850,  01010101 01111000 00000000 00000000
    0, // 7851,  11010101 01111000 00000000 00000000
    2, // 7852,  00110101 01111000 00000000 00000000
    0, // 7853,  10110101 01111000 00000000 00000000
    1, // 7854,  01110101 01111000 00000000 00000000
    0, // 7855,  11110101 01111000 00000000 00000000
    4, // 7856,  00001101 01111000 00000000 00000000
    0, // 7857,  10001101 01111000 00000000 00000000
    1, // 7858,  01001101 01111000 00000000 00000000
    0, // 7859,  11001101 01111000 00000000 00000000
    2, // 7860,  00101101 01111000 00000000 00000000
    0, // 7861,  10101101 01111000 00000000 00000000
    1, // 7862,  01101101 01111000 00000000 00000000
    0, // 7863,  11101101 01111000 00000000 00000000
    3, // 7864,  00011101 01111000 00000000 00000000
    0, // 7865,  10011101 01111000 00000000 00000000
    1, // 7866,  01011101 01111000 00000000 00000000
    0, // 7867,  11011101 01111000 00000000 00000000
    2, // 7868,  00111101 01111000 00000000 00000000
    0, // 7869,  10111101 01111000 00000000 00000000
    1, // 7870,  01111101 01111000 00000000 00000000
    0, // 7871,  11111101 01111000 00000000 00000000
    6, // 7872,  00000011 01111000 00000000 00000000
    0, // 7873,  10000011 01111000 00000000 00000000
    1, // 7874,  01000011 01111000 00000000 00000000
    0, // 7875,  11000011 01111000 00000000 00000000
    2, // 7876,  00100011 01111000 00000000 00000000
    0, // 7877,  10100011 01111000 00000000 00000000
    1, // 7878,  01100011 01111000 00000000 00000000
    0, // 7879,  11100011 01111000 00000000 00000000
    3, // 7880,  00010011 01111000 00000000 00000000
    0, // 7881,  10010011 01111000 00000000 00000000
    1, // 7882,  01010011 01111000 00000000 00000000
    0, // 7883,  11010011 01111000 00000000 00000000
    2, // 7884,  00110011 01111000 00000000 00000000
    0, // 7885,  10110011 01111000 00000000 00000000
    1, // 7886,  01110011 01111000 00000000 00000000
    0, // 7887,  11110011 01111000 00000000 00000000
    4, // 7888,  00001011 01111000 00000000 00000000
    0, // 7889,  10001011 01111000 00000000 00000000
    1, // 7890,  01001011 01111000 00000000 00000000
    0, // 7891,  11001011 01111000 00000000 00000000
    2, // 7892,  00101011 01111000 00000000 00000000
    0, // 7893,  10101011 01111000 00000000 00000000
    1, // 7894,  01101011 01111000 00000000 00000000
    0, // 7895,  11101011 01111000 00000000 00000000
    3, // 7896,  00011011 01111000 00000000 00000000
    0, // 7897,  10011011 01111000 00000000 00000000
    1, // 7898,  01011011 01111000 00000000 00000000
    0, // 7899,  11011011 01111000 00000000 00000000
    2, // 7900,  00111011 01111000 00000000 00000000
    0, // 7901,  10111011 01111000 00000000 00000000
    1, // 7902,  01111011 01111000 00000000 00000000
    0, // 7903,  11111011 01111000 00000000 00000000
    5, // 7904,  00000111 01111000 00000000 00000000
    0, // 7905,  10000111 01111000 00000000 00000000
    1, // 7906,  01000111 01111000 00000000 00000000
    0, // 7907,  11000111 01111000 00000000 00000000
    2, // 7908,  00100111 01111000 00000000 00000000
    0, // 7909,  10100111 01111000 00000000 00000000
    1, // 7910,  01100111 01111000 00000000 00000000
    0, // 7911,  11100111 01111000 00000000 00000000
    3, // 7912,  00010111 01111000 00000000 00000000
    0, // 7913,  10010111 01111000 00000000 00000000
    1, // 7914,  01010111 01111000 00000000 00000000
    0, // 7915,  11010111 01111000 00000000 00000000
    2, // 7916,  00110111 01111000 00000000 00000000
    0, // 7917,  10110111 01111000 00000000 00000000
    1, // 7918,  01110111 01111000 00000000 00000000
    0, // 7919,  11110111 01111000 00000000 00000000
    4, // 7920,  00001111 01111000 00000000 00000000
    0, // 7921,  10001111 01111000 00000000 00000000
    1, // 7922,  01001111 01111000 00000000 00000000
    0, // 7923,  11001111 01111000 00000000 00000000
    2, // 7924,  00101111 01111000 00000000 00000000
    0, // 7925,  10101111 01111000 00000000 00000000
    1, // 7926,  01101111 01111000 00000000 00000000
    0, // 7927,  11101111 01111000 00000000 00000000
    3, // 7928,  00011111 01111000 00000000 00000000
    0, // 7929,  10011111 01111000 00000000 00000000
    1, // 7930,  01011111 01111000 00000000 00000000
    0, // 7931,  11011111 01111000 00000000 00000000
    2, // 7932,  00111111 01111000 00000000 00000000
    0, // 7933,  10111111 01111000 00000000 00000000
    1, // 7934,  01111111 01111000 00000000 00000000
    0, // 7935,  11111111 01111000 00000000 00000000
    8, // 7936,  00000000 11111000 00000000 00000000
    0, // 7937,  10000000 11111000 00000000 00000000
    1, // 7938,  01000000 11111000 00000000 00000000
    0, // 7939,  11000000 11111000 00000000 00000000
    2, // 7940,  00100000 11111000 00000000 00000000
    0, // 7941,  10100000 11111000 00000000 00000000
    1, // 7942,  01100000 11111000 00000000 00000000
    0, // 7943,  11100000 11111000 00000000 00000000
    3, // 7944,  00010000 11111000 00000000 00000000
    0, // 7945,  10010000 11111000 00000000 00000000
    1, // 7946,  01010000 11111000 00000000 00000000
    0, // 7947,  11010000 11111000 00000000 00000000
    2, // 7948,  00110000 11111000 00000000 00000000
    0, // 7949,  10110000 11111000 00000000 00000000
    1, // 7950,  01110000 11111000 00000000 00000000
    0, // 7951,  11110000 11111000 00000000 00000000
    4, // 7952,  00001000 11111000 00000000 00000000
    0, // 7953,  10001000 11111000 00000000 00000000
    1, // 7954,  01001000 11111000 00000000 00000000
    0, // 7955,  11001000 11111000 00000000 00000000
    2, // 7956,  00101000 11111000 00000000 00000000
    0, // 7957,  10101000 11111000 00000000 00000000
    1, // 7958,  01101000 11111000 00000000 00000000
    0, // 7959,  11101000 11111000 00000000 00000000
    3, // 7960,  00011000 11111000 00000000 00000000
    0, // 7961,  10011000 11111000 00000000 00000000
    1, // 7962,  01011000 11111000 00000000 00000000
    0, // 7963,  11011000 11111000 00000000 00000000
    2, // 7964,  00111000 11111000 00000000 00000000
    0, // 7965,  10111000 11111000 00000000 00000000
    1, // 7966,  01111000 11111000 00000000 00000000
    0, // 7967,  11111000 11111000 00000000 00000000
    5, // 7968,  00000100 11111000 00000000 00000000
    0, // 7969,  10000100 11111000 00000000 00000000
    1, // 7970,  01000100 11111000 00000000 00000000
    0, // 7971,  11000100 11111000 00000000 00000000
    2, // 7972,  00100100 11111000 00000000 00000000
    0, // 7973,  10100100 11111000 00000000 00000000
    1, // 7974,  01100100 11111000 00000000 00000000
    0, // 7975,  11100100 11111000 00000000 00000000
    3, // 7976,  00010100 11111000 00000000 00000000
    0, // 7977,  10010100 11111000 00000000 00000000
    1, // 7978,  01010100 11111000 00000000 00000000
    0, // 7979,  11010100 11111000 00000000 00000000
    2, // 7980,  00110100 11111000 00000000 00000000
    0, // 7981,  10110100 11111000 00000000 00000000
    1, // 7982,  01110100 11111000 00000000 00000000
    0, // 7983,  11110100 11111000 00000000 00000000
    4, // 7984,  00001100 11111000 00000000 00000000
    0, // 7985,  10001100 11111000 00000000 00000000
    1, // 7986,  01001100 11111000 00000000 00000000
    0, // 7987,  11001100 11111000 00000000 00000000
    2, // 7988,  00101100 11111000 00000000 00000000
    0, // 7989,  10101100 11111000 00000000 00000000
    1, // 7990,  01101100 11111000 00000000 00000000
    0, // 7991,  11101100 11111000 00000000 00000000
    3, // 7992,  00011100 11111000 00000000 00000000
    0, // 7993,  10011100 11111000 00000000 00000000
    1, // 7994,  01011100 11111000 00000000 00000000
    0, // 7995,  11011100 11111000 00000000 00000000
    2, // 7996,  00111100 11111000 00000000 00000000
    0, // 7997,  10111100 11111000 00000000 00000000
    1, // 7998,  01111100 11111000 00000000 00000000
    0, // 7999,  11111100 11111000 00000000 00000000
    6, // 8000,  00000010 11111000 00000000 00000000
    0, // 8001,  10000010 11111000 00000000 00000000
    1, // 8002,  01000010 11111000 00000000 00000000
    0, // 8003,  11000010 11111000 00000000 00000000
    2, // 8004,  00100010 11111000 00000000 00000000
    0, // 8005,  10100010 11111000 00000000 00000000
    1, // 8006,  01100010 11111000 00000000 00000000
    0, // 8007,  11100010 11111000 00000000 00000000
    3, // 8008,  00010010 11111000 00000000 00000000
    0, // 8009,  10010010 11111000 00000000 00000000
    1, // 8010,  01010010 11111000 00000000 00000000
    0, // 8011,  11010010 11111000 00000000 00000000
    2, // 8012,  00110010 11111000 00000000 00000000
    0, // 8013,  10110010 11111000 00000000 00000000
    1, // 8014,  01110010 11111000 00000000 00000000
    0, // 8015,  11110010 11111000 00000000 00000000
    4, // 8016,  00001010 11111000 00000000 00000000
    0, // 8017,  10001010 11111000 00000000 00000000
    1, // 8018,  01001010 11111000 00000000 00000000
    0, // 8019,  11001010 11111000 00000000 00000000
    2, // 8020,  00101010 11111000 00000000 00000000
    0, // 8021,  10101010 11111000 00000000 00000000
    1, // 8022,  01101010 11111000 00000000 00000000
    0, // 8023,  11101010 11111000 00000000 00000000
    3, // 8024,  00011010 11111000 00000000 00000000
    0, // 8025,  10011010 11111000 00000000 00000000
    1, // 8026,  01011010 11111000 00000000 00000000
    0, // 8027,  11011010 11111000 00000000 00000000
    2, // 8028,  00111010 11111000 00000000 00000000
    0, // 8029,  10111010 11111000 00000000 00000000
    1, // 8030,  01111010 11111000 00000000 00000000
    0, // 8031,  11111010 11111000 00000000 00000000
    5, // 8032,  00000110 11111000 00000000 00000000
    0, // 8033,  10000110 11111000 00000000 00000000
    1, // 8034,  01000110 11111000 00000000 00000000
    0, // 8035,  11000110 11111000 00000000 00000000
    2, // 8036,  00100110 11111000 00000000 00000000
    0, // 8037,  10100110 11111000 00000000 00000000
    1, // 8038,  01100110 11111000 00000000 00000000
    0, // 8039,  11100110 11111000 00000000 00000000
    3, // 8040,  00010110 11111000 00000000 00000000
    0, // 8041,  10010110 11111000 00000000 00000000
    1, // 8042,  01010110 11111000 00000000 00000000
    0, // 8043,  11010110 11111000 00000000 00000000
    2, // 8044,  00110110 11111000 00000000 00000000
    0, // 8045,  10110110 11111000 00000000 00000000
    1, // 8046,  01110110 11111000 00000000 00000000
    0, // 8047,  11110110 11111000 00000000 00000000
    4, // 8048,  00001110 11111000 00000000 00000000
    0, // 8049,  10001110 11111000 00000000 00000000
    1, // 8050,  01001110 11111000 00000000 00000000
    0, // 8051,  11001110 11111000 00000000 00000000
    2, // 8052,  00101110 11111000 00000000 00000000
    0, // 8053,  10101110 11111000 00000000 00000000
    1, // 8054,  01101110 11111000 00000000 00000000
    0, // 8055,  11101110 11111000 00000000 00000000
    3, // 8056,  00011110 11111000 00000000 00000000
    0, // 8057,  10011110 11111000 00000000 00000000
    1, // 8058,  01011110 11111000 00000000 00000000
    0, // 8059,  11011110 11111000 00000000 00000000
    2, // 8060,  00111110 11111000 00000000 00000000
    0, // 8061,  10111110 11111000 00000000 00000000
    1, // 8062,  01111110 11111000 00000000 00000000
    0, // 8063,  11111110 11111000 00000000 00000000
    7, // 8064,  00000001 11111000 00000000 00000000
    0, // 8065,  10000001 11111000 00000000 00000000
    1, // 8066,  01000001 11111000 00000000 00000000
    0, // 8067,  11000001 11111000 00000000 00000000
    2, // 8068,  00100001 11111000 00000000 00000000
    0, // 8069,  10100001 11111000 00000000 00000000
    1, // 8070,  01100001 11111000 00000000 00000000
    0, // 8071,  11100001 11111000 00000000 00000000
    3, // 8072,  00010001 11111000 00000000 00000000
    0, // 8073,  10010001 11111000 00000000 00000000
    1, // 8074,  01010001 11111000 00000000 00000000
    0, // 8075,  11010001 11111000 00000000 00000000
    2, // 8076,  00110001 11111000 00000000 00000000
    0, // 8077,  10110001 11111000 00000000 00000000
    1, // 8078,  01110001 11111000 00000000 00000000
    0, // 8079,  11110001 11111000 00000000 00000000
    4, // 8080,  00001001 11111000 00000000 00000000
    0, // 8081,  10001001 11111000 00000000 00000000
    1, // 8082,  01001001 11111000 00000000 00000000
    0, // 8083,  11001001 11111000 00000000 00000000
    2, // 8084,  00101001 11111000 00000000 00000000
    0, // 8085,  10101001 11111000 00000000 00000000
    1, // 8086,  01101001 11111000 00000000 00000000
    0, // 8087,  11101001 11111000 00000000 00000000
    3, // 8088,  00011001 11111000 00000000 00000000
    0, // 8089,  10011001 11111000 00000000 00000000
    1, // 8090,  01011001 11111000 00000000 00000000
    0, // 8091,  11011001 11111000 00000000 00000000
    2, // 8092,  00111001 11111000 00000000 00000000
    0, // 8093,  10111001 11111000 00000000 00000000
    1, // 8094,  01111001 11111000 00000000 00000000
    0, // 8095,  11111001 11111000 00000000 00000000
    5, // 8096,  00000101 11111000 00000000 00000000
    0, // 8097,  10000101 11111000 00000000 00000000
    1, // 8098,  01000101 11111000 00000000 00000000
    0, // 8099,  11000101 11111000 00000000 00000000
    2, // 8100,  00100101 11111000 00000000 00000000
    0, // 8101,  10100101 11111000 00000000 00000000
    1, // 8102,  01100101 11111000 00000000 00000000
    0, // 8103,  11100101 11111000 00000000 00000000
    3, // 8104,  00010101 11111000 00000000 00000000
    0, // 8105,  10010101 11111000 00000000 00000000
    1, // 8106,  01010101 11111000 00000000 00000000
    0, // 8107,  11010101 11111000 00000000 00000000
    2, // 8108,  00110101 11111000 00000000 00000000
    0, // 8109,  10110101 11111000 00000000 00000000
    1, // 8110,  01110101 11111000 00000000 00000000
    0, // 8111,  11110101 11111000 00000000 00000000
    4, // 8112,  00001101 11111000 00000000 00000000
    0, // 8113,  10001101 11111000 00000000 00000000
    1, // 8114,  01001101 11111000 00000000 00000000
    0, // 8115,  11001101 11111000 00000000 00000000
    2, // 8116,  00101101 11111000 00000000 00000000
    0, // 8117,  10101101 11111000 00000000 00000000
    1, // 8118,  01101101 11111000 00000000 00000000
    0, // 8119,  11101101 11111000 00000000 00000000
    3, // 8120,  00011101 11111000 00000000 00000000
    0, // 8121,  10011101 11111000 00000000 00000000
    1, // 8122,  01011101 11111000 00000000 00000000
    0, // 8123,  11011101 11111000 00000000 00000000
    2, // 8124,  00111101 11111000 00000000 00000000
    0, // 8125,  10111101 11111000 00000000 00000000
    1, // 8126,  01111101 11111000 00000000 00000000
    0, // 8127,  11111101 11111000 00000000 00000000
    6, // 8128,  00000011 11111000 00000000 00000000
    0, // 8129,  10000011 11111000 00000000 00000000
    1, // 8130,  01000011 11111000 00000000 00000000
    0, // 8131,  11000011 11111000 00000000 00000000
    2, // 8132,  00100011 11111000 00000000 00000000
    0, // 8133,  10100011 11111000 00000000 00000000
    1, // 8134,  01100011 11111000 00000000 00000000
    0, // 8135,  11100011 11111000 00000000 00000000
    3, // 8136,  00010011 11111000 00000000 00000000
    0, // 8137,  10010011 11111000 00000000 00000000
    1, // 8138,  01010011 11111000 00000000 00000000
    0, // 8139,  11010011 11111000 00000000 00000000
    2, // 8140,  00110011 11111000 00000000 00000000
    0, // 8141,  10110011 11111000 00000000 00000000
    1, // 8142,  01110011 11111000 00000000 00000000
    0, // 8143,  11110011 11111000 00000000 00000000
    4, // 8144,  00001011 11111000 00000000 00000000
    0, // 8145,  10001011 11111000 00000000 00000000
    1, // 8146,  01001011 11111000 00000000 00000000
    0, // 8147,  11001011 11111000 00000000 00000000
    2, // 8148,  00101011 11111000 00000000 00000000
    0, // 8149,  10101011 11111000 00000000 00000000
    1, // 8150,  01101011 11111000 00000000 00000000
    0, // 8151,  11101011 11111000 00000000 00000000
    3, // 8152,  00011011 11111000 00000000 00000000
    0, // 8153,  10011011 11111000 00000000 00000000
    1, // 8154,  01011011 11111000 00000000 00000000
    0, // 8155,  11011011 11111000 00000000 00000000
    2, // 8156,  00111011 11111000 00000000 00000000
    0, // 8157,  10111011 11111000 00000000 00000000
    1, // 8158,  01111011 11111000 00000000 00000000
    0, // 8159,  11111011 11111000 00000000 00000000
    5, // 8160,  00000111 11111000 00000000 00000000
    0, // 8161,  10000111 11111000 00000000 00000000
    1, // 8162,  01000111 11111000 00000000 00000000
    0, // 8163,  11000111 11111000 00000000 00000000
    2, // 8164,  00100111 11111000 00000000 00000000
    0, // 8165,  10100111 11111000 00000000 00000000
    1, // 8166,  01100111 11111000 00000000 00000000
    0, // 8167,  11100111 11111000 00000000 00000000
    3, // 8168,  00010111 11111000 00000000 00000000
    0, // 8169,  10010111 11111000 00000000 00000000
    1, // 8170,  01010111 11111000 00000000 00000000
    0, // 8171,  11010111 11111000 00000000 00000000
    2, // 8172,  00110111 11111000 00000000 00000000
    0, // 8173,  10110111 11111000 00000000 00000000
    1, // 8174,  01110111 11111000 00000000 00000000
    0, // 8175,  11110111 11111000 00000000 00000000
    4, // 8176,  00001111 11111000 00000000 00000000
    0, // 8177,  10001111 11111000 00000000 00000000
    1, // 8178,  01001111 11111000 00000000 00000000
    0, // 8179,  11001111 11111000 00000000 00000000
    2, // 8180,  00101111 11111000 00000000 00000000
    0, // 8181,  10101111 11111000 00000000 00000000
    1, // 8182,  01101111 11111000 00000000 00000000
    0, // 8183,  11101111 11111000 00000000 00000000
    3, // 8184,  00011111 11111000 00000000 00000000
    0, // 8185,  10011111 11111000 00000000 00000000
    1, // 8186,  01011111 11111000 00000000 00000000
    0, // 8187,  11011111 11111000 00000000 00000000
    2, // 8188,  00111111 11111000 00000000 00000000
    0, // 8189,  10111111 11111000 00000000 00000000
    1, // 8190,  01111111 11111000 00000000 00000000
    0, // 8191,  11111111 11111000 00000000 00000000
};

const int8_t straightTable[] =
{
    0,  0,  0,  0,  0,  0,  0, -3,  0,  0,  0, -3,  0, -3, -3,
    -2,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3, -1, -3, -1,
    -2,  4,  0,  0,  0,  0,  0,  0, -3, -3,  0,  0, -3, -3, -3,
    -3, -1, -2,  0,  0, -3, -3, -3, -3, -1, -1, -3, -3, -1, -1,
    -2, -2,  5,  5,  0,  0,  0,  0,  0,  0,  0, -3,  0,  0,  0,
    -3, -3, -3, -3, -2,  0,  0,  0, -3, -3, -3, -3, -1, -3, -3,
    -3, -1, -1, -2, -2,  4,  0,  0,  0,  0, -3, -3, -3, -3, -3,
    -3, -3, -3, -1, -1, -1, -2, -3, -3, -3, -3, -1, -1, -1, -1,
    -2, -2, -2, -2,  6,  6,  6,  6,  0,  0,  0,  0,  0,  0,  0,
    -3,  0,  0,  0, -3,  0, -3, -3, -2,  0,  0,  0, -3,  0, -3,
    -3, -1, -3, -3, -3, -1, -3, -1, -2,  4,  0,  0,  0,  0,  0,
    0, -3, -3, -3, -3, -3, -3, -3, -3, -1, -2, -3, -3, -3, -3,
    -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,  5,  5,  0,  0,  0,
    0,  0,  0,  0, -3, -3, -3, -3, -3, -3, -3, -3, -2, -3, -3,
    -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -1, -2, -2,  4, -3,
    -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -1, -1, -1, -2,
    -2, -2, -2, -2, -2, -2, -2, -2,  7,  7,  7,  7,  7,  7,  7,
    7,  0,  0,  0,  0,  0,  0,  0, -3,  0,  0,  0, -3,  0, -3,
    -3, -2,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3, -1, -3,
    -1, -2,  4,  0,  0,  0,  0,  0,  0, -3, -3,  0,  0, -3, -3,
    -3, -3, -1, -2, -3, -3, -3, -3, -3, -3, -1, -1, -3, -3, -1,
    -1, -2, -2,  5,  5,  0,  0,  0,  0,  0,  0,  0, -3,  0,  0,
    0, -3, -3, -3, -3, -2, -3, -3, -3, -3, -3, -3, -3, -1, -3,
    -3, -3, -1, -1, -2, -2,  4, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -1, -1, -1, -2, -1, -1, -1, -1, -2, -2, -2,
    -2, -2, -2, -2, -2,  6,  6,  6,  6,  0,  0,  0,  0,  0,  0,
    0, -3,  0,  0,  0, -3,  0, -3, -3, -2, -3, -3, -3, -3, -3,
    -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -1, -2, -1, -1, -1,
    -1, -1, -1, -2, -2, -1, -1, -2, -2, -2, -2,  5,  5, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -2, -1,
    -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2, -2,  4,
    -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
    0,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,
    8,  8,  0,  0,  0,  0,  0,  0,  0, -3,  0,  0,  0, -3,  0,
    -3, -3, -2,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3, -1,
    -3, -1, -2,  4,  0,  0,  0,  0,  0,  0, -3, -3,  0,  0, -3,
    -3, -3, -3, -1, -2,  0,  0, -3, -3, -3, -3, -1, -1, -3, -3,
    -1, -1, -2, -2,  5,  5,  0,  0,  0,  0,  0,  0,  0, -3,  0,
    0,  0, -3, -3, -3, -3, -2,  0,  0,  0, -3, -3, -3, -3, -1,
    -3, -3, -3, -1, -1, -2, -2,  4, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -1, -1, -1, -2, -3, -3, -3, -3, -1, -1,
    -1, -1, -2, -2, -2, -2,  6,  6,  6,  6,  0,  0,  0,  0,  0,
    0,  0, -3,  0,  0,  0, -3,  0, -3, -3, -2,  0,  0,  0, -3,
    0, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -1, -2, -3, -3,
    -3, -3, -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,  5,  5, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -2,
    -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -1, -2, -2,
    4, -1, -1, -1, -1, -1, -1, -1, -1, -2, -2, -2, -2, -2, -2,
    -2,  0, -2, -2, -2, -2, -2, -2, -2, -2,  7,  7,  7,  7,  7,
    7,  7,  7,  0,  0,  0,  0,  0,  0,  0, -3,  0,  0,  0, -3,
    0, -3, -3, -2,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3,
    -1, -3, -1, -2,  4, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -1, -2, -3, -3, -3, -3, -3, -3, -1, -1, -3,
    -3, -1, -1, -2, -2,  5,  5, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -2, -3, -3, -3, -3, -3, -3, -3,
    -1, -3, -3, -3, -1, -1, -2, -2,  4, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -2, -2, -2,  0, -1, -1, -1, -1, -2,
    -2, -2, -2, -2, -2, -2, -2,  6,  6,  6,  6, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -2, -3, -3, -3,
    -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -2,  0, -1,
    -1, -1, -1, -1, -1, -2, -2, -1, -1, -2, -2, -2, -2,  5,  5,
    -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
    0, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,  0,
    0,  4,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,
    9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,
    9,  9,  9,  9,  0,  0,  0,  0,  0,  0,  0, -3,  0,  0,  0,
    -3,  0, -3, -3, -2,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3,
    -3, -1, -3, -1, -2,  4,  0,  0,  0,  0,  0,  0, -3, -3,  0,
    0, -3, -3, -3, -3, -1, -2,  0,  0, -3, -3, -3, -3, -1, -1,
    -3, -3, -1, -1, -2, -2,  5,  5,  0,  0,  0,  0,  0,  0,  0,
    -3,  0,  0,  0, -3, -3, -3, -3, -2,  0,  0,  0, -3, -3, -3,
    -3, -1, -3, -3, -3, -1, -1, -2, -2,  4,  0,  0,  0,  0, -3,
    -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -2, -3, -3, -3, -3,
    -1, -1, -1, -1, -2, -2, -2, -2,  6,  6,  6,  6,  0,  0,  0,
    0,  0,  0,  0, -3,  0,  0,  0, -3,  0, -3, -3, -2,  0,  0,
    0, -3,  0, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4,  0,
    0,  0,  0,  0,  0, -3, -3, -3, -3, -3, -3, -3, -3, -1, -2,
    -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,  5,
    5, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -2, -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -1,
    -2, -2,  4, -3, -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1,
    -1, -1, -1, -2, -2, -2, -2, -2, -2, -2, -2, -2,  7,  7,  7,
    7,  7,  7,  7,  7,  0,  0,  0,  0,  0,  0,  0, -3,  0,  0,
    0, -3,  0, -3, -3, -2,  0,  0,  0, -3,  0, -3, -3, -1,  0,
    -3, -3, -1, -3, -1, -2,  4,  0,  0,  0,  0,  0,  0, -3, -3,
    0,  0, -3, -3, -3, -3, -1, -2, -3, -3, -3, -3, -3, -3, -1,
    -1, -3, -3, -1, -1, -2, -2,  5,  5, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -2, -3, -3, -3, -3, -3,
    -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  4, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -2, -1, -1, -1,
    -1, -2, -2, -2, -2, -2, -2, -2, -2,  6,  6,  6,  6, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -2, -3,
    -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -1,
    -2, -1, -1, -1, -1, -1, -1, -2, -2, -1, -1, -2, -2, -2, -2,
    5,  5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1,  0, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0,
    -2,  0,  0,  4, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
    -2, -2, -2, -2,  0,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,
    8,  8,  8,  8,  8,  8,  0,  0,  0,  0,  0,  0,  0, -3,  0,
    0,  0, -3,  0, -3, -3, -2,  0,  0,  0, -3,  0, -3, -3, -1,
    0, -3, -3, -1, -3, -1, -2,  4,  0,  0,  0,  0,  0,  0, -3,
    -3,  0,  0, -3, -3, -3, -3, -1, -2,  0,  0, -3, -3, -3, -3,
    -1, -1, -3, -3, -1, -1, -2, -2,  5,  5, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -2, -3, -3, -3, -3,
    -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  4, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -2, -3, -3,
    -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,  6,  6,  6,  6, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -2,
    -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,
    4, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -1, -2, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -2, -2,
    -2,  5,  5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1,  0, -1, -1, -1, -1, -1, -1, -1, -2, -2, -2, -2,
    0, -2,  0,  0,  4, -1, -1, -1, -1, -1, -1, -1, -1, -2, -2,
    -2, -2, -2, -2, -2,  0, -2, -2, -2, -2, -2, -2, -2, -2,  7,
    7,  7,  7,  7,  7,  7,  7, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -2, -3, -3, -3, -3, -3, -3, -3,
    -1, -3, -3, -3, -1, -3, -1, -2,  4, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -1, -2, -3, -3, -3, -3, -3,
    -3, -1, -1, -3, -3, -1, -1, -2, -2,  5,  5, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  0, -1, -1, -1,
    -1, -1, -1, -1, -2, -1, -1, -1, -2, -2,  0,  0,  4, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -2, -2, -2,  0, -1,
    -1, -1, -1, -2, -2, -2, -2, -2, -2, -2, -2,  6,  6,  6,  6,
    -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
    0, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,  0,
    0,  4, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
    -2,  0,  0, -2, -2, -2, -2, -2, -2,  0,  0, -2, -2,  0,  0,
    0,  0,  5,  5, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10,  0,  0,  0,  0,  0,  0,  0,
    -3,  0,  0,  0, -3,  0, -3, -3, -2,  0,  0,  0, -3,  0, -3,
    -3, -1,  0, -3, -3, -1, -3, -1, -2,  4,  0,  0,  0,  0,  0,
    0, -3, -3,  0,  0, -3, -3, -3, -3, -1, -2,  0,  0, -3, -3,
    -3, -3, -1, -1, -3, -3, -1, -1, -2, -2,  5,  5,  0,  0,  0,
    0,  0,  0,  0, -3,  0,  0,  0, -3, -3, -3, -3, -2,  0,  0,
    0, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  4,  0,
    0,  0,  0, -3, -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -2,
    -3, -3, -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,  6,  6,  6,
    6,  0,  0,  0,  0,  0,  0,  0, -3,  0,  0,  0, -3,  0, -3,
    -3, -2,  0,  0,  0, -3,  0, -3, -3, -1, -3, -3, -3, -1, -3,
    -1, -2,  4,  0,  0,  0,  0,  0,  0, -3, -3, -3, -3, -3, -3,
    -3, -3, -1, -2, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2,
    -2, -2, -2,  5,  5,  0,  0,  0,  0,  0,  0,  0, -3, -3, -3,
    -3, -3, -3, -3, -3, -2, -3, -3, -3, -3, -3, -3, -3, -1, -1,
    -1, -1, -2, -1, -2, -2,  4, -3, -3, -3, -3, -3, -3, -3, -3,
    -1, -1, -1, -1, -1, -1, -1, -2, -2, -2, -2, -2, -2, -2, -2,
    -2,  7,  7,  7,  7,  7,  7,  7,  7,  0,  0,  0,  0,  0,  0,
    0, -3,  0,  0,  0, -3,  0, -3, -3, -2,  0,  0,  0, -3,  0,
    -3, -3, -1,  0, -3, -3, -1, -3, -1, -2,  4,  0,  0,  0,  0,
    0,  0, -3, -3,  0,  0, -3, -3, -3, -3, -1, -2, -3, -3, -3,
    -3, -3, -3, -1, -1, -3, -3, -1, -1, -2, -2,  5,  5,  0,  0,
    0,  0,  0,  0,  0, -3,  0,  0,  0, -3, -3, -3, -3, -2, -3,
    -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  4,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -1, -1, -1,
    -2, -1, -1, -1, -1, -2, -2, -2, -2, -2, -2, -2, -2,  6,  6,
    6,  6, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -2, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1,
    -3, -1, -2,  4, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -1, -2, -1, -1, -1, -1, -1, -1, -2, -2, -1, -1,
    -2, -2, -2, -2,  5,  5, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -2, -1, -1, -1, -1, -1, -1, -1, -2,
    -1, -1, -1, -2, -1, -2, -2,  4, -2, -2, -2, -2, -2, -2, -2,
    -2, -2, -2, -2, -2, -2, -2, -2,  0,  8,  8,  8,  8,  8,  8,
    8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  0,  0,  0,  0,  0,
    0,  0, -3,  0,  0,  0, -3,  0, -3, -3, -2,  0,  0,  0, -3,
    0, -3, -3, -1,  0, -3, -3, -1, -3, -1, -2,  4,  0,  0,  0,
    0,  0,  0, -3, -3,  0,  0, -3, -3, -3, -3, -1, -2,  0,  0,
    -3, -3, -3, -3, -1, -1, -3, -3, -1, -1, -2, -2,  5,  5,  0,
    0,  0,  0,  0,  0,  0, -3,  0,  0,  0, -3, -3, -3, -3, -2,
    0,  0,  0, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,
    4, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -1, -1,
    -1, -2, -3, -3, -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,  6,
    6,  6,  6, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -2, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3,
    -1, -3, -1, -2,  4, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -1, -2, -3, -3, -3, -3, -3, -3, -1, -1, -1,
    -1, -2, -2, -2, -2,  5,  5, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -2, -3, -3, -3, -3, -3, -3, -3,
    -1, -1, -1, -1, -2, -1, -2, -2,  4, -1, -1, -1, -1, -1, -1,
    -1, -1, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2, -2, -2,
    -2, -2, -2,  7,  7,  7,  7,  7,  7,  7,  7, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -2, -3, -3, -3,
    -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -1, -2, -3,
    -3, -3, -3, -3, -3, -1, -1, -3, -3, -1, -1, -2, -2,  5,  5,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -2, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2,
    -2,  4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -2,
    -2, -2,  0, -1, -1, -1, -1, -2, -2, -2, -2, -2, -2, -2, -2,
    6,  6,  6,  6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1,  0, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1,
    -1, -2, -1, -2,  0,  4, -2, -2, -2, -2, -2, -2, -2, -2, -2,
    -2, -2, -2, -2, -2,  0,  0, -2, -2, -2, -2, -2, -2,  0,  0,
    -2, -2,  0,  0,  0,  0,  5,  5, -2, -2, -2, -2, -2, -2, -2,
    -2, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2, -2, -2, -2,
    -2,  0, -2, -2, -2,  0, -2,  0,  0,  4,  9,  9,  9,  9,  9,
    9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,
    9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  0,  0,  0,
    0,  0,  0,  0, -3,  0,  0,  0, -3,  0, -3, -3, -2,  0,  0,
    0, -3,  0, -3, -3, -1,  0, -3, -3, -1, -3, -1, -2,  4,  0,
    0,  0,  0,  0,  0, -3, -3,  0,  0, -3, -3, -3, -3, -1, -2,
    0,  0, -3, -3, -3, -3, -1, -1, -3, -3, -1, -1, -2, -2,  5,
    5,  0,  0,  0,  0,  0,  0,  0, -3,  0,  0,  0, -3, -3, -3,
    -3, -2,  0,  0,  0, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1,
    -2, -2,  4,  0,  0,  0,  0, -3, -3, -3, -3, -3, -3, -3, -3,
    -1, -1, -1, -2, -3, -3, -3, -3, -1, -1, -1, -1, -2, -2, -2,
    -2,  6,  6,  6,  6, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -2, -3, -3, -3, -3, -3, -3, -3, -1, -3,
    -3, -3, -1, -3, -1, -2,  4, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -1, -2, -3, -3, -3, -3, -3, -3, -1,
    -1, -1, -1, -2, -2, -2, -2,  5,  5, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -2, -3, -3, -3, -3, -3,
    -3, -3, -1, -1, -1, -1, -2, -1, -2, -2,  4, -3, -3, -3, -3,
    -3, -3, -3, -3, -1, -1, -1, -1, -1, -1, -1, -2, -2, -2, -2,
    -2, -2, -2, -2, -2,  7,  7,  7,  7,  7,  7,  7,  7, -3, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -2, -3,
    -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -1,
    -2, -3, -3, -3, -3, -3, -3, -1, -1, -3, -3, -1, -1, -2, -2,
    5,  5, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -2, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1,
    -1, -2, -2,  4, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -1, -1, -1, -2, -1, -1, -1, -1, -2, -2, -2, -2, -2, -2,
    -2, -2,  6,  6,  6,  6, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1,  0, -1, -1, -1, -1, -1, -1, -1, -2,
    -1, -1, -1, -2, -1, -2,  0,  4, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -2,  0, -2, -2, -2, -2, -2, -2,
    0,  0, -2, -2,  0,  0,  0,  0,  5,  5, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  0, -2, -2, -2, -2,
    -2, -2, -2,  0, -2, -2, -2,  0, -2,  0,  0,  4, -2, -2, -2,
    -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,  0,  8,  8,
    8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8, -3,
    -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -2,
    -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,
    4, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -1, -2, -3, -3, -3, -3, -3, -3, -1, -1, -3, -3, -1, -1, -2,
    -2,  5,  5, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -3, -2, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3,
    -1, -1, -2, -2,  4, -3, -3, -3, -3, -3, -3, -3, -3, -3, -3,
    -3, -3, -1, -1, -1, -2, -3, -3, -3, -3, -1, -1, -1, -1, -2,
    -2, -2, -2,  6,  6,  6,  6, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1,  0, -1, -1, -1, -1, -1, -1, -1,
    -2, -1, -1, -1, -2, -1, -2,  0,  4, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -2,  0, -1, -1, -1, -1, -1,
    -1, -2, -2, -2, -2,  0,  0,  0,  0,  5,  5, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  0, -1, -1, -1,
    -1, -1, -1, -1, -2, -2, -2, -2,  0, -2,  0,  0,  4, -1, -1,
    -1, -1, -1, -1, -1, -1, -2, -2, -2, -2, -2, -2, -2,  0, -2,
    -2, -2, -2, -2, -2, -2, -2,  7,  7,  7,  7,  7,  7,  7,  7,
    -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
    0, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,  0,
    0,  4, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
    -2,  0,  0, -2, -2, -2, -2, -2, -2,  0,  0, -2, -2,  0,  0,
    0,  0,  5,  5, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
    -2, -2, -2, -2,  0, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2,
    -2,  0,  0,  0,  0,  4, -2, -2, -2, -2, -2, -2, -2, -2, -2,
    -2, -2, -2,  0,  0,  0,  0, -2, -2, -2, -2,  0,  0,  0,  0,
    0,  0,  0,  0,  6,  6,  6,  6, 11, 11, 11, 11, 11, 11, 11,
    11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
    11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
    11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
    11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
    11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
    11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
    11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
    11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
    11,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3, -1, -3, -1,
    -1,  3,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3, -1, -3,
    -1, -2,  4,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3, -1,
    -3, -1, -2,  3,  0,  0, -3, -3, -3, -3, -1, -1, -3, -3, -1,
    -1, -2, -2,  5,  5,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3,
    -3, -1, -3, -1, -1,  3,  0,  0,  0, -3, -3, -3, -3, -1, -3,
    -3, -3, -1, -1, -2, -2,  4,  0,  0,  0, -3, -3, -3, -3, -1,
    -3, -3, -3, -1, -1, -2, -2,  3, -3, -3, -3, -3, -1, -1, -1,
    -1, -2, -2, -2, -2,  6,  6,  6,  6,  0,  0,  0, -3,  0, -3,
    -3, -1,  0, -3, -3, -1, -3, -1, -1,  3,  0,  0,  0, -3,  0,
    -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4,  0,  0,  0, -3,
    0, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  3, -3, -3, -3,
    -3, -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,  5,  5,  0,  0,
    0, -3,  0, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3, -3,
    -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -1, -2, -2,  4,
    -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -1, -2, -2,
    3, -2, -2, -2, -2, -2, -2, -2, -2,  7,  7,  7,  7,  7,  7,
    7,  7,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3, -1, -3,
    -1, -1,  3,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3, -1,
    -3, -1, -2,  4,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3,
    -1, -3, -1, -2,  3, -3, -3, -3, -3, -3, -3, -1, -1, -3, -3,
    -1, -1, -2, -2,  5,  5,  0,  0,  0, -3,  0, -3, -3, -1,  0,
    -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3, -3, -3, -3, -1,
    -3, -3, -3, -1, -1, -2, -2,  4, -3, -3, -3, -3, -3, -3, -3,
    -1, -3, -3, -3, -1, -1, -2, -2,  3, -1, -1, -1, -1, -2, -2,
    -2, -2, -2, -2, -2, -2,  6,  6,  6,  6,  0,  0,  0, -3,  0,
    -3, -3, -1,  0, -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3,
    -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4, -3, -3, -3,
    -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  3, -1, -1,
    -1, -1, -1, -1, -2, -2, -1, -1, -2, -2, -2, -2,  5,  5, -3,
    -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3,
    -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2, -2,
    4, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,  0,
    0,  3,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,
    8,  8,  8,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3, -1,
    -3, -1, -1,  3,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3,
    -1, -3, -1, -2,  4,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3,
    -3, -1, -3, -1, -2,  3,  0,  0, -3, -3, -3, -3, -1, -1, -3,
    -3, -1, -1, -2, -2,  5,  5,  0,  0,  0, -3,  0, -3, -3, -1,
    0, -3, -3, -1, -3, -1, -1,  3,  0,  0,  0, -3, -3, -3, -3,
    -1, -3, -3, -3, -1, -1, -2, -2,  4, -3, -3, -3, -3, -3, -3,
    -3, -1, -3, -3, -3, -1, -1, -2, -2,  3, -3, -3, -3, -3, -1,
    -1, -1, -1, -2, -2, -2, -2,  6,  6,  6,  6,  0,  0,  0, -3,
    0, -3, -3, -1,  0, -3, -3, -1, -3, -1, -1,  3,  0,  0,  0,
    -3,  0, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4, -3, -3,
    -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  3, -3,
    -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,  5,  5,
    -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,
    3, -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -1, -2,
    -2,  4, -1, -1, -1, -1, -1, -1, -1, -2, -2, -2, -2,  0, -2,
    0,  0,  3, -2, -2, -2, -2, -2, -2, -2, -2,  7,  7,  7,  7,
    7,  7,  7,  7, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3,
    -1, -3, -1, -1,  3, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3,
    -3, -1, -3, -1, -2,  4, -3, -3, -3, -3, -3, -3, -3, -1, -3,
    -3, -3, -1, -3, -1, -2,  3, -3, -3, -3, -3, -3, -3, -1, -1,
    -3, -3, -1, -1, -2, -2,  5,  5, -3, -3, -3, -3, -3, -3, -3,
    -1, -3, -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3, -3, -3,
    -3, -1, -3, -3, -3, -1, -1, -2, -2,  4, -1, -1, -1, -1, -1,
    -1, -1, -2, -1, -1, -1, -2, -2,  0,  0,  3, -1, -1, -1, -1,
    -2, -2, -2, -2, -2, -2, -2, -2,  6,  6,  6,  6, -3, -3, -3,
    -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3, -3, -3,
    -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4, -1,
    -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2,  0,  3,
    -1, -1, -1, -1, -1, -1, -2, -2, -1, -1, -2, -2, -2, -2,  5,
    5, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,  0,
    0,  3, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,
    0,  0,  4,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,
    9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,
    9,  9,  9,  9,  9,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3,
    -3, -1, -3, -1, -1,  3,  0,  0,  0, -3,  0, -3, -3, -1,  0,
    -3, -3, -1, -3, -1, -2,  4,  0,  0,  0, -3,  0, -3, -3, -1,
    0, -3, -3, -1, -3, -1, -2,  3,  0,  0, -3, -3, -3, -3, -1,
    -1, -3, -3, -1, -1, -2, -2,  5,  5,  0,  0,  0, -3,  0, -3,
    -3, -1,  0, -3, -3, -1, -3, -1, -1,  3,  0,  0,  0, -3, -3,
    -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  4,  0,  0,  0, -3,
    -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  3, -3, -3, -3,
    -3, -1, -1, -1, -1, -2, -2, -2, -2,  6,  6,  6,  6,  0,  0,
    0, -3,  0, -3, -3, -1,  0, -3, -3, -1, -3, -1, -1,  3,  0,
    0,  0, -3,  0, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4,
    0,  0,  0, -3,  0, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,
    3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,
    5,  5, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3,
    -1, -1,  3, -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2,
    -1, -2, -2,  4, -3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1,
    -2, -1, -2, -2,  3, -2, -2, -2, -2, -2, -2, -2, -2,  7,  7,
    7,  7,  7,  7,  7,  7, -3, -3, -3, -3, -3, -3, -3, -1, -3,
    -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3, -3, -3, -3, -1,
    -3, -3, -3, -1, -3, -1, -2,  4, -3, -3, -3, -3, -3, -3, -3,
    -1, -3, -3, -3, -1, -3, -1, -2,  3, -3, -3, -3, -3, -3, -3,
    -1, -1, -3, -3, -1, -1, -2, -2,  5,  5, -3, -3, -3, -3, -3,
    -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3,
    -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  4, -3, -3, -3,
    -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  3, -1, -1,
    -1, -1, -2, -2, -2, -2, -2, -2, -2, -2,  6,  6,  6,  6, -3,
    -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3,
    -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,
    4, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1,
    -2,  3, -1, -1, -1, -1, -1, -1, -2, -2, -1, -1, -2, -2, -2,
    -2,  5,  5, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2,
    -1, -2, -2,  3, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,
    0, -2,  0,  0,  4, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2,
    -2,  0, -2,  0,  0,  3,  8,  8,  8,  8,  8,  8,  8,  8,  8,
    8,  8,  8,  8,  8,  8,  8, -3, -3, -3, -3, -3, -3, -3, -1,
    -3, -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3, -3, -3, -3,
    -1, -3, -3, -3, -1, -3, -1, -2,  4, -3, -3, -3, -3, -3, -3,
    -3, -1, -3, -3, -3, -1, -3, -1, -2,  3, -3, -3, -3, -3, -3,
    -3, -1, -1, -3, -3, -1, -1, -2, -2,  5,  5, -3, -3, -3, -3,
    -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3, -3, -3, -3,
    -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  4, -3, -3,
    -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  3, -3,
    -3, -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,  6,  6,  6,  6,
    -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,
    3, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1,
    -2,  4, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3,
    -1, -2,  3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -2,
    -2, -2,  5,  5, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1,
    -2, -1, -2, -2,  3, -1, -1, -1, -1, -1, -1, -1, -2, -2, -2,
    -2,  0, -2,  0,  0,  4, -1, -1, -1, -1, -1, -1, -1, -2, -2,
    -2, -2,  0, -2,  0,  0,  3, -2, -2, -2, -2, -2, -2, -2, -2,
    7,  7,  7,  7,  7,  7,  7,  7, -1, -1, -1, -1, -1, -1, -1,
    -2, -1, -1, -1, -2, -1, -2, -2,  3, -1, -1, -1, -1, -1, -1,
    -1, -2, -1, -1, -1, -2, -1, -2,  0,  4, -1, -1, -1, -1, -1,
    -1, -1, -2, -1, -1, -1, -2, -1, -2,  0,  3, -1, -1, -1, -1,
    -1, -1, -2, -2, -1, -1, -2, -2,  0,  0,  5,  5, -2, -2, -2,
    -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,  0,  0,  3, -2, -2,
    -2, -2, -2, -2, -2,  0, -2, -2, -2,  0,  0,  0,  0,  4, -2,
    -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0,  0,  0,  0,  3,
    -2, -2, -2, -2,  0,  0,  0,  0,  0,  0,  0,  0,  6,  6,  6,
    6, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,  0,
    0,  3, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,
    0,  0,  4, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0,
    -2,  0,  0,  3, -2, -2, -2, -2, -2, -2,  0,  0, -2, -2,  0,
    0,  0,  0,  5,  5, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10,  0,  0,  0, -3,  0, -3,
    -3, -1,  0, -3, -3, -1, -3, -1, -1,  3,  0,  0,  0, -3,  0,
    -3, -3, -1,  0, -3, -3, -1, -3, -1, -2,  4,  0,  0,  0, -3,
    0, -3, -3, -1,  0, -3, -3, -1, -3, -1, -2,  3,  0,  0, -3,
    -3, -3, -3, -1, -1, -3, -3, -1, -1, -2, -2,  5,  5,  0,  0,
    0, -3,  0, -3, -3, -1,  0, -3, -3, -1, -3, -1, -1,  3,  0,
    0,  0, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,  4,
    0,  0,  0, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,
    3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,  6,  6,
    6,  6,  0,  0,  0, -3,  0, -3, -3, -1,  0, -3, -3, -1, -3,
    -1, -1,  3,  0,  0,  0, -3,  0, -3, -3, -1, -3, -3, -3, -1,
    -3, -1, -2,  4,  0,  0,  0, -3,  0, -3, -3, -1, -3, -3, -3,
    -1, -3, -1, -2,  3, -3, -3, -3, -3, -3, -3, -1, -1, -1, -1,
    -2, -2, -2, -2,  5,  5,  0,  0,  0, -3,  0, -3, -3, -1, -3,
    -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3, -3, -3, -3, -1,
    -1, -1, -1, -2, -1, -2, -2,  4, -3, -3, -3, -3, -3, -3, -3,
    -1, -1, -1, -1, -2, -1, -2, -2,  3, -2, -2, -2, -2, -2, -2,
    -2, -2,  7,  7,  7,  7,  7,  7,  7,  7, -3, -3, -3, -3, -3,
    -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3,
    -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4, -3, -3, -3,
    -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  3, -3, -3,
    -3, -3, -3, -3, -1, -1, -3, -3, -1, -1, -2, -2,  5,  5, -3,
    -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3,
    -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2, -2,
    4, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2,
    -2,  3, -1, -1, -1, -1, -2, -2, -2, -2, -2, -2, -2, -2,  6,
    6,  6,  6, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1,
    -3, -1, -1,  3, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3,
    -1, -3, -1, -2,  4, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3,
    -3, -1, -3, -1, -2,  3, -1, -1, -1, -1, -1, -1, -2, -2, -1,
    -1, -2, -2, -2, -2,  5,  5, -3, -3, -3, -3, -3, -3, -3, -1,
    -3, -3, -3, -1, -3, -1, -1,  3, -1, -1, -1, -1, -1, -1, -1,
    -2, -1, -1, -1, -2, -1, -2, -2,  4, -2, -2, -2, -2, -2, -2,
    -2,  0, -2, -2, -2,  0, -2,  0,  0,  3,  8,  8,  8,  8,  8,
    8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8, -3, -3, -3, -3,
    -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3, -3, -3, -3,
    -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4, -3, -3,
    -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  3, -3,
    -3, -3, -3, -3, -3, -1, -1, -3, -3, -1, -1, -2, -2,  5,  5,
    -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,
    3, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1, -2,
    -2,  4, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -1,
    -2, -2,  3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -2, -2, -2,
    6,  6,  6,  6, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3,
    -1, -3, -1, -1,  3, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3,
    -3, -1, -3, -1, -2,  4, -3, -3, -3, -3, -3, -3, -3, -1, -3,
    -3, -3, -1, -3, -1, -2,  3, -3, -3, -3, -3, -3, -3, -1, -1,
    -1, -1, -2, -2, -2, -2,  5,  5, -3, -3, -3, -3, -3, -3, -3,
    -1, -3, -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3, -3, -3,
    -3, -1, -1, -1, -1, -2, -1, -2, -2,  4, -1, -1, -1, -1, -1,
    -1, -1, -2, -2, -2, -2,  0, -2,  0,  0,  3, -2, -2, -2, -2,
    -2, -2, -2, -2,  7,  7,  7,  7,  7,  7,  7,  7, -1, -1, -1,
    -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2, -2,  3, -1, -1,
    -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2,  0,  4, -1,
    -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2,  0,  3,
    -1, -1, -1, -1, -1, -1, -2, -2, -1, -1, -2, -2,  0,  0,  5,
    5, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2,
    -2,  3, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -2,
    0,  0,  4, -2, -2, -2, -2, -2, -2, -2,  0, -2, -2, -2,  0,
    0,  0,  0,  3, -2, -2, -2, -2,  0,  0,  0,  0,  0,  0,  0,
    0,  6,  6,  6,  6, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1,
    -1, -2, -1, -2, -2,  3, -1, -1, -1, -1, -1, -1, -1, -2, -1,
    -1, -1, -2, -1, -2,  0,  4, -2, -2, -2, -2, -2, -2, -2,  0,
    -2, -2, -2,  0, -2,  0,  0,  3, -2, -2, -2, -2, -2, -2,  0,
    0, -2, -2,  0,  0,  0,  0,  5,  5, -2, -2, -2, -2, -2, -2,
    -2,  0, -2, -2, -2,  0, -2,  0,  0,  3, -2, -2, -2, -2, -2,
    -2, -2,  0, -2, -2, -2,  0, -2,  0,  0,  4,  9,  9,  9,  9,
    9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,
    9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9,  9, -3, -3,
    -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3, -3,
    -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,  4,
    -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3, -1, -2,
    3, -3, -3, -3, -3, -3, -3, -1, -1, -3, -3, -1, -1, -2, -2,
    5,  5, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1, -3,
    -1, -1,  3, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3, -1,
    -1, -2, -2,  4, -3, -3, -3, -3, -3, -3, -3, -1, -3, -3, -3,
    -1, -1, -2, -2,  3, -3, -3, -3, -3, -1, -1, -1, -1, -2, -2,
    -2, -2,  6,  6,  6,  6, -3, -3, -3, -3, -3, -3, -3, -1, -3,
    -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3, -3, -3, -3, -1,
    -3, -3, -3, -1, -3, -1, -2,  4, -3, -3, -3, -3, -3, -3, -3,
    -1, -3, -3, -3, -1, -3, -1, -2,  3, -3, -3, -3, -3, -3, -3,
    -1, -1, -1, -1, -2, -2, -2, -2,  5,  5, -3, -3, -3, -3, -3,
    -3, -3, -1, -3, -3, -3, -1, -3, -1, -1,  3, -3, -3, -3, -3,
    -3, -3, -3, -1, -1, -1, -1, -2, -1, -2, -2,  4, -3, -3, -3,
    -3, -3, -3, -3, -1, -1, -1, -1, -2, -1, -2, -2,  3, -2, -2,
    -2, -2, -2, -2, -2, -2,  7,  7,  7,  7,  7,  7,  7,  7, -1,
    -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2, -2,  3,
    -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2,  0,
    4, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2,
    0,  3, -1, -1, -1, -1, -1, -1, -2, -2, -1, -1, -2, -2,  0,
    0,  5,  5, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2,
    -1, -2, -2,  3, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1,
    -2, -2,  0,  0,  4, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1,
    -1, -2, -2,  0,  0,  3, -2, -2, -2, -2,  0,  0,  0,  0,  0,
    0,  0,  0,  6,  6,  6,  6, -1, -1, -1, -1, -1, -1, -1, -2,
    -1, -1, -1, -2, -1, -2, -2,  3, -1, -1, -1, -1, -1, -1, -1,
    -2, -1, -1, -1, -2, -1, -2,  0,  4, -1, -1, -1, -1, -1, -1,
    -1, -2, -1, -1, -1, -2, -1, -2,  0,  3, -2, -2, -2, -2, -2,
    -2,  0,  0, -2, -2,  0,  0,  0,  0,  5,  5, -1, -1, -1, -1,
    -1, -1, -1, -2, -1, -1, -1, -2, -1, -2, -2,  3, -2, -2, -2,
    -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,  0,  0,  4, -2, -2,
    -2, -2, -2, -2, -2,  0, -2, -2, -2,  0, -2,  0,  0,  3,  8,
    8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,
    -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2, -2,
    3, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2,
    0,  4, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1, -2, -1,
    -2,  0,  3, -1, -1, -1, -1, -1, -1, -2, -2, -1, -1, -2, -2,
    0,  0,  5,  5, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -1,
    -2, -1, -2, -2,  3, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1,
    -1, -2, -2,  0,  0,  4, -1, -1, -1, -1, -1, -1, -1, -2, -1,
    -1, -1, -2, -2,  0,  0,  3, -1, -1, -1, -1, -2, -2, -2, -2,
    0,  0,  0,  0,  6,  6,  6,  6, -1, -1, -1, -1, -1, -1, -1,
    -2, -1, -1, -1, -2, -1, -2, -2,  3, -1, -1, -1, -1, -1, -1,
    -1, -2, -1, -1, -1, -2, -1, -2,  0,  4, -1, -1, -1, -1, -1,
    -1, -1, -2, -1, -1, -1, -2, -1, -2,  0,  3, -1, -1, -1, -1,
    -1, -1, -2, -2, -2, -2,  0,  0,  0,  0,  5,  5, -1, -1, -1,
    -1, -1, -1, -1, -2, -1, -1, -1, -2, -1, -2, -2,  3, -1, -1,
    -1, -1, -1, -1, -1, -2, -2, -2, -2,  0, -2,  0,  0,  4, -1,
    -1, -1, -1, -1, -1, -1, -2, -2, -2, -2,  0, -2,  0,  0,  3,
    -2, -2, -2, -2, -2, -2, -2, -2,  7,  7,  7,  7,  7,  7,  7,
    7, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12,
};


const uint8_t nRanksTable[] =
{
    0,  1,  1,  2,  1,  2,  2,  3,  1,  2,  2,  3,  2,  3,  3,
    4,  1,  2,  2,  3,  2,  3,  3,  4,  2,  3,  3,  4,  3,  4,
    4,  5,  1,  2,  2,  3,  2,  3,  3,  4,  2,  3,  3,  4,  3,
    4,  4,  5,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,  5,
    4,  5,  5,  6,  1,  2,  2,  3,  2,  3,  3,  4,  2,  3,  3,
    4,  3,  4,  4,  5,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,
    4,  5,  4,  5,  5,  6,  2,  3,  3,  4,  3,  4,  4,  5,  3,
    4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  1,  2,  2,  3,  2,  3,  3,
    4,  2,  3,  3,  4,  3,  4,  4,  5,  2,  3,  3,  4,  3,  4,
    4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  2,  3,  3,  4,  3,
    4,  4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  2,  3,  3,
    4,  3,  4,  4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,
    4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  1,  2,  2,  3,  2,  3,  3,  4,  2,  3,  3,  4,  3,  4,
    4,  5,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,  5,  4,
    5,  5,  6,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,  5,
    4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,
    6,  5,  6,  6,  7,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,
    4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,
    5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  2,  3,  3,  4,  3,  4,
    4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,
    5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  1,  2,  2,  3,  2,  3,  3,  4,  2,  3,  3,  4,  3,
    4,  4,  5,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,  5,
    4,  5,  5,  6,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,
    5,  4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,
    5,  6,  5,  6,  6,  7,  2,  3,  3,  4,  3,  4,  4,  5,  3,
    4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,  4,  5,  5,
    6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,
    6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  2,  3,  3,  4,  3,
    4,  4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,  4,  4,
    5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,
    5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  3,
    4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,
    7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,  5,
    4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,
    6,  5,  6,  6,  7,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,
    5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,
    6,  6,  7,  6,  7,  7,  8,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,  6,  5,  6,
    6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,
    7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,
    5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,
    6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  1,  2,  2,  3,  2,  3,  3,  4,  2,  3,  3,
    4,  3,  4,  4,  5,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,
    4,  5,  4,  5,  5,  6,  2,  3,  3,  4,  3,  4,  4,  5,  3,
    4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  2,  3,  3,  4,  3,  4,  4,
    5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,  5,
    5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,  4,
    5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,
    5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  2,  3,  3,
    4,  3,  4,  4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,
    4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,
    6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,
    7,  7,  8,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,
    6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,
    4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,
    5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  3,  4,  4,  5,  4,  5,
    5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,
    6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,  6,
    5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,
    7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,
    7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,
    7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  2,  3,  3,  4,  3,  4,  4,  5,  3,
    4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,  4,  5,  5,
    6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,
    6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  3,  4,  4,  5,  4,
    5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,
    5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,
    6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  3,
    4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,
    7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,
    6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,
    7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,
    8,  8,  9,  8,  9,  9, 10,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,  6,  5,  6,
    6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,
    7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  4,  5,  5,  6,
    5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,
    7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  5,  6,
    6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,
    7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,  7,
    8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,
    8,  9,  9, 10,  9, 10, 10, 11,  1,  2,  2,  3,  2,  3,  3,
    4,  2,  3,  3,  4,  3,  4,  4,  5,  2,  3,  3,  4,  3,  4,
    4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  2,  3,  3,  4,  3,
    4,  4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  2,  3,  3,
    4,  3,  4,  4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,
    4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,  5,  4,  5,
    5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,
    6,  6,  7,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,
    5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,
    7,  6,  7,  7,  8,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,
    5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,
    6,  6,  7,  6,  7,  7,  8,  4,  5,  5,  6,  5,  6,  6,  7,
    5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  2,  3,  3,  4,  3,  4,
    4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,
    5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,
    6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,
    6,  7,  7,  8,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,
    7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,
    7,  8,  7,  8,  8,  9,  4,  5,  5,  6,  5,  6,  6,  7,  5,
    6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,
    6,  7,  7,  8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  2,  3,  3,  4,  3,
    4,  4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,  4,  4,
    5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,
    5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  3,
    4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,
    7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,
    5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,
    7,  6,  7,  7,  8,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,
    6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,
    7,  7,  8,  7,  8,  8,  9,  4,  5,  5,  6,  5,  6,  6,  7,
    5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,
    7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,
    8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,
    5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,
    6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,
    7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,
    7,  8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,
    7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,
    7,  8,  8,  9,  8,  9,  9, 10,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,
    8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,
    8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  2,  3,  3,
    4,  3,  4,  4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,
    4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,
    6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,
    7,  7,  8,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,
    6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,
    5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,
    6,  6,  7,  6,  7,  7,  8,  4,  5,  5,  6,  5,  6,  6,  7,
    5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  4,  5,  5,  6,  5,  6,
    6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,
    7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  5,  6,  6,  7,
    6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,
    8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,
    7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,
    7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  4,  5,  5,  6,  5,  6,  6,  7,  5,
    6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,
    6,  7,  7,  8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  5,  6,  6,  7,  6,
    7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,
    7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  6,  7,  7,
    8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,
    8,  9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  3,
    4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,
    7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,
    6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,
    7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,
    8,  8,  9,  8,  9,  9, 10,  4,  5,  5,  6,  5,  6,  6,  7,
    5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,
    7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,
    8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  5,  6,  6,  7,
    6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,
    8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  6,  7,
    7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,
    8,  8,  9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,  7,
    8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,
    8,  9,  9, 10,  9, 10, 10, 11,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,
    8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,
    8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  6,  7,  7,
    8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,
    8,  9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  7,
    8,  8,  9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,
    8,  9,  9, 10,  9, 10, 10, 11,  9, 10, 10, 11, 10, 11, 11,
    12,  1,  2,  2,  3,  2,  3,  3,  4,  2,  3,  3,  4,  3,  4,
    4,  5,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,  5,  4,
    5,  5,  6,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,  5,
    4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,
    6,  5,  6,  6,  7,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,
    4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,
    5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  2,  3,  3,  4,  3,  4,
    4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,
    5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,  5,  4,
    5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,
    5,  6,  6,  7,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,
    6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,
    6,  7,  6,  7,  7,  8,  3,  4,  4,  5,  4,  5,  5,  6,  4,
    5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,
    5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,
    7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  3,  4,  4,  5,  4,
    5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,
    5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,
    6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,
    9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,
    9,  9, 10,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,  4,  5,
    4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,
    6,  5,  6,  6,  7,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,
    5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,
    6,  6,  7,  6,  7,  7,  8,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,  6,  5,  6,
    6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,
    7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,
    5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,
    6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,
    6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,
    6,  7,  6,  7,  7,  8,  4,  5,  5,  6,  5,  6,  6,  7,  5,
    6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,
    6,  7,  7,  8,  7,  8,  8,  9,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,
    7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  5,  6,  6,  7,  6,
    7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,
    7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,
    6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  5,
    6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,
    6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9,
    10,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,
    9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,  8,  9,  9,
    10,  9, 10, 10, 11,  2,  3,  3,  4,  3,  4,  4,  5,  3,  4,
    4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,  5,  5,  6,  4,
    5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  3,  4,  4,  5,  4,  5,
    5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,
    6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,  6,
    5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,
    7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,
    7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,
    7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  3,  4,  4,  5,  4,  5,  5,  6,  4,
    5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,
    5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,
    7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  4,  5,  5,  6,  5,
    6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,
    6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  5,  6,  6,
    7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,
    7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,
    9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,
    9,  9, 10,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,
    7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,
    9,  8,  9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,  8,
    9,  9, 10,  9, 10, 10, 11,  3,  4,  4,  5,  4,  5,  5,  6,
    4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,  6,  5,  6,
    6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,
    7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  4,  5,  5,  6,
    5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,
    7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  5,  6,
    6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,
    7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,  7,
    8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,
    8,  9,  9, 10,  9, 10, 10, 11,  4,  5,  5,  6,  5,  6,  6,
    7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,
    7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  5,  6,  6,  7,  6,
    7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,
    7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  5,  6,  6,
    7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,
    7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  6,
    7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,
    7,  8,  8,  9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10,
    11,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,
    9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,  8,  9,  9,
    10,  9, 10, 10, 11,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,  8,
    9,  9, 10,  9, 10, 10, 11,  7,  8,  8,  9,  8,  9,  9, 10,
    8,  9,  9, 10,  9, 10, 10, 11,  8,  9,  9, 10,  9, 10, 10,
    11,  9, 10, 10, 11, 10, 11, 11, 12,  2,  3,  3,  4,  3,  4,
    4,  5,  3,  4,  4,  5,  4,  5,  5,  6,  3,  4,  4,  5,  4,
    5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  3,  4,  4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,
    6,  6,  7,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,
    6,  7,  7,  8,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,
    7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,
    7,  8,  7,  8,  8,  9,  4,  5,  5,  6,  5,  6,  6,  7,  5,
    6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,
    6,  7,  7,  8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  3,  4,  4,  5,  4,
    5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,  6,
    5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,
    6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,
    9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,
    9,  9, 10,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,
    6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,
    7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,
    8,  8,  9,  8,  9,  9, 10,  5,  6,  6,  7,  6,  7,  7,  8,
    6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,
    9,  7,  8,  8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,
    9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  3,  4,  4,  5,
    4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  4,  5,
    5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,
    6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,
    7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,
    7,  8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,
    7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,
    7,  8,  8,  9,  8,  9,  9, 10,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,
    8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,
    8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  4,  5,  5,
    6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,  5,  6,
    6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  5,
    6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,
    6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9,
    10,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,
    9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,  8,  9,  9,
    10,  9, 10, 10, 11,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,
    7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,
    8,  8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,
    7,  8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9,
    10,  8,  9,  9, 10,  9, 10, 10, 11,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,
    9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  7,  8,  8,  9,
    8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  8,  9,  9,
    10,  9, 10, 10, 11,  9, 10, 10, 11, 10, 11, 11, 12,  3,  4,
    4,  5,  4,  5,  5,  6,  4,  5,  5,  6,  5,  6,  6,  7,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,
    7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,
    7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  4,  5,  5,  6,  5,  6,  6,  7,  5,
    6,  6,  7,  6,  7,  7,  8,  5,  6,  6,  7,  6,  7,  7,  8,
    6,  7,  7,  8,  7,  8,  8,  9,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  5,  6,  6,  7,  6,
    7,  7,  8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,
    7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  6,  7,  7,
    8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,
    8,  9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  4,
    5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,  8,
    5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,  8,
    9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,
    9,  9, 10,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,
    7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,
    9,  8,  9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,  8,
    9,  9, 10,  9, 10, 10, 11,  5,  6,  6,  7,  6,  7,  7,  8,
    6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,
    9,  7,  8,  8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,
    9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  6,  7,  7,  8,
    7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,  8,
    9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  7,  8,
    8,  9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  8,
    9,  9, 10,  9, 10, 10, 11,  9, 10, 10, 11, 10, 11, 11, 12,
    4,  5,  5,  6,  5,  6,  6,  7,  5,  6,  6,  7,  6,  7,  7,
    8,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,
    8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,
    8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,  7,
    8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,
    8,  9,  9, 10,  9, 10, 10, 11,  5,  6,  6,  7,  6,  7,  7,
    8,  6,  7,  7,  8,  7,  8,  8,  9,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  6,  7,  7,  8,  7,
    8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,
    8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  6,  7,  7,
    8,  7,  8,  8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,
    8,  9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  7,
    8,  8,  9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,
    8,  9,  9, 10,  9, 10, 10, 11,  9, 10, 10, 11, 10, 11, 11,
    12,  5,  6,  6,  7,  6,  7,  7,  8,  6,  7,  7,  8,  7,  8,
    8,  9,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,  8,
    9,  9, 10,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,  8,  9,
    8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,  8,  9,  9,
    10,  9, 10, 10, 11,  6,  7,  7,  8,  7,  8,  8,  9,  7,  8,
    8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,  9,  9, 10,  8,
    9,  9, 10,  9, 10, 10, 11,  7,  8,  8,  9,  8,  9,  9, 10,
    8,  9,  9, 10,  9, 10, 10, 11,  8,  9,  9, 10,  9, 10, 10,
    11,  9, 10, 10, 11, 10, 11, 11, 12,  6,  7,  7,  8,  7,  8,
    8,  9,  7,  8,  8,  9,  8,  9,  9, 10,  7,  8,  8,  9,  8,
    9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  7,  8,  8,  9,
    8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  8,  9,  9,
    10,  9, 10, 10, 11,  9, 10, 10, 11, 10, 11, 11, 12,  7,  8,
    8,  9,  8,  9,  9, 10,  8,  9,  9, 10,  9, 10, 10, 11,  8,
    9,  9, 10,  9, 10, 10, 11,  9, 10, 10, 11, 10, 11, 11, 12,
    8,  9,  9, 10,  9, 10, 10, 11,  9, 10, 10, 11, 10, 11, 11,
    12,  9, 10, 10, 11, 10, 11, 11, 12, 10, 11, 11, 12, 11, 12,
    12, 13,
};

const int8_t flushTable[] =
{
    0,   0,   0, -16,  -1,   1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,
    1,   0,   0,   0, -16,  -1,   1,   1,   1, -32, -32, -32, -16,  -1,   1,
    1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,   1,   0,   0,   0,
    -16,  -1,   1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,   1, -32, -32,
    -32, -16,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,
    1,   0,   0,   0, -16,  -1,   1,   1,   1,   0,   0,   0, -16,  -1,   1,
    1,   1, -32, -32, -32, -16,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1, -48, -48, -48,
    -16,  -1,   1,   1,   1, -48, -48, -48, -16,  -1,   1,   1,   1, -48, -48,
    -48, -16,  -1,   1,   1,   1, -32, -32, -32, -16,  -1,   1,   1,   1,  -2,
    -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,  -3,  -3,  -3,  -3,  -1,   1,   1,   1,  -3,  -3,  -3,  -3,  -1,   1,
    1,   1,  -3,  -3,  -3,  -3,  -1,   1,   1,   1,  -3,  -3,  -3,  -3,  -1,
    1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,
    3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,
    3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   3,   3,   3,   3,   3,   1,
    1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,
    1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,
    3,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   3,   3,
    3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,
    3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,
    3,   3,   3,   3,   3,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   0,   0,   0, -16,  -1,   1,   1,   1,   0,   0,   0, -16,  -1,
    1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,   1, -32, -32, -32, -16,
    -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,   1,   0,
    0,   0, -16,  -1,   1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,   1,
    -32, -32, -32, -16,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   0,   0,   0, -16,  -1,
    1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,   1,   0,   0,   0, -16,
    -1,   1,   1,   1, -32, -32, -32, -16,  -1,   1,   1,   1,  -2,  -2,  -2,
    -2,  -1,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1, -48,
    -48, -48, -16,  -1,   1,   1,   1, -48, -48, -48, -16,  -1,   1,   1,   1,
    -48, -48, -48, -16,  -1,   1,   1,   1, -32, -32, -32, -16,  -1,   1,   1,
    1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,  -3,  -3,  -3,  -3,  -1,   1,   1,   1,  -3,  -3,  -3,  -3,
    -1,   1,   1,   1,  -3,  -3,  -3,  -3,  -1,   1,   1,   1,  -3,  -3,  -3,
    -3,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,
    3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,
    1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   3,   3,   3,   3,
    3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,
    3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,
    3,   3,   3,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,
    1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,
    1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,   1,   0,   0,   0,
    -16,  -1,   1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,   1, -32, -32,
    -32, -16,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,
    1,   0,   0,   0, -16,  -1,   1,   1,   1,   0,   0,   0, -16,  -1,   1,
    1,   1, -32, -32, -32, -16,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   0,   0,   0,
    -16,  -1,   1,   1,   1,   0,   0,   0, -16,  -1,   1,   1,   1,   0,   0,
    0, -16,  -1,   1,   1,   1, -32, -32, -32, -16,  -1,   1,   1,   1,  -2,
    -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1, -48, -48, -48, -16,  -1,   1,   1,   1, -48, -48, -48, -16,  -1,   1,
    1,   1, -48, -48, -48, -16,  -1,   1,   1,   1, -32, -32, -32, -16,  -1,
    1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,  -3,  -3,  -3,  -3,  -1,   1,   1,   1,  -3,  -3,
    -3,  -3,  -1,   1,   1,   1,  -3,  -3,  -3,  -3,  -1,   1,   1,   1,  -3,
    -3,  -3,  -3,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   3,   3,   3,   3,   3,   1,
    1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,
    1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,
    3,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   3,   3,
    3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,
    3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,
    3,   3,   3,   3,   3,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,
    1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,
    3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1, -64, -64, -64, -16,  -1,   1,   1,   1, -64,
    -64, -64, -16,  -1,   1,   1,   1, -64, -64, -64, -16,  -1,   1,   1,   1,
    -32, -32, -32, -16,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1, -64, -64, -64, -16,  -1,
    1,   1,   1, -64, -64, -64, -16,  -1,   1,   1,   1, -64, -64, -64, -16,
    -1,   1,   1,   1, -32, -32, -32, -16,  -1,   1,   1,   1,  -2,  -2,  -2,
    -2,  -1,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1, -64,
    -64, -64, -16,  -1,   1,   1,   1, -64, -64, -64, -16,  -1,   1,   1,   1,
    -64, -64, -64, -16,  -1,   1,   1,   1, -32, -32, -32, -16,  -1,   1,   1,
    1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1, -48, -48, -48, -16,  -1,   1,   1,   1, -48, -48, -48, -16,
    -1,   1,   1,   1, -48, -48, -48, -16,  -1,   1,   1,   1, -32, -32, -32,
    -16,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,  -3,  -3,  -3,  -3,  -1,   1,   1,   1,
    -3,  -3,  -3,  -3,  -1,   1,   1,   1,  -3,  -3,  -3,  -3,  -1,   1,   1,
    1,  -3,  -3,  -3,  -3,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   3,   3,   3,   3,
    3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,
    3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,
    3,   3,   3,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,
    1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,
    1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,
    3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,
    3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,  -4,  -4,  -4,  -4,  -1,   1,   1,
    1,  -4,  -4,  -4,  -4,  -1,   1,   1,   1,  -4,  -4,  -4,  -4,  -1,   1,
    1,   1,  -4,  -4,  -4,  -4,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,  -4,  -4,  -4,
    -4,  -1,   1,   1,   1,  -4,  -4,  -4,  -4,  -1,   1,   1,   1,  -4,  -4,
    -4,  -4,  -1,   1,   1,   1,  -4,  -4,  -4,  -4,  -1,   1,   1,   1,  -2,
    -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,  -4,  -4,  -4,  -4,  -1,   1,   1,   1,  -4,  -4,  -4,  -4,  -1,   1,
    1,   1,  -4,  -4,  -4,  -4,  -1,   1,   1,   1,  -4,  -4,  -4,  -4,  -1,
    1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,  -4,  -4,  -4,  -4,  -1,   1,   1,   1,  -4,  -4,
    -4,  -4,  -1,   1,   1,   1,  -4,  -4,  -4,  -4,  -1,   1,   1,   1,  -4,
    -4,  -4,  -4,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,  -1,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,  -3,  -3,  -3,  -3,  -1,   1,
    1,   1,  -3,  -3,  -3,  -3,  -1,   1,   1,   1,  -3,  -3,  -3,  -3,  -1,
    1,   1,   1,  -3,  -3,  -3,  -3,  -1,   1,   1,   1,  -2,  -2,  -2,  -2,
    -1,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   3,   3,
    3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,
    3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,
    3,   3,   3,   3,   3,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,
    1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,
    3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,
    3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,
    3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   4,   4,   4,   4,   4,
    1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,
    4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,
    4,   4,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   4,
    4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,
    4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,
    1,   4,   4,   4,   4,   4,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,
    4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,
    4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,
    4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,
    1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   4,   4,   4,   4,
    4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,
    4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,
    4,   4,   4,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,
    1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,
    1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,
    3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,
    3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,
    1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,
    1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   4,   4,   4,
    4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,
    4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,
    4,   4,   4,   4,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,
    1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,
    1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,
    4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,
    4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   4,   4,   4,   4,   4,   1,
    1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,
    1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,
    4,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   4,   4,
    4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,
    4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,
    4,   4,   4,   4,   4,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,
    1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,
    3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   2,   2,   2,
    2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,
    3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,
    3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,
    1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   3,   3,   3,   3,   3,
    1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,
    3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,
    3,   3,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   4,
    4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,
    4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,
    1,   4,   4,   4,   4,   4,   1,   1,   1,   2,   2,   2,   2,   2,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,
    4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,
    4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   2,   2,
    2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,
    4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,
    1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,
    1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   4,   4,   4,   4,
    4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,
    4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,
    4,   4,   4,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,   1,
    1,   4,   4,   4,   4,   4,   1,   1,   1,   4,   4,   4,   4,   4,   1,
    1,   1,   4,   4,   4,   4,   4,   1,   1,   1,   2,   2,   2,   2,   2,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,
    3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,
    3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   2,
    2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,
    1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,
    1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,
    1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,
    2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,   3,   3,   3,
    3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,   3,
    3,   3,   3,   1,   1,   1,   3,   3,   3,   3,   3,   1,   1,   1,   3,
    3,   3,   3,   3,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,   1,
    2,   2,   2,   2,   2,   1,   1,   1,   2,   2,   2,   2,   2,   1,   1,
    1,
};

const uint16_t topFiveRanksTable[] =
{
    0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
    15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
    30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44,
    45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
    60, 61, 62, 62, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74,
    75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89,
    90, 91, 92, 93, 94, 94, 96, 97, 98, 99,100,101,102,103,104,
    105,106,107,108,109,110,110,112,113,114,115,116,117,118,118,
    120,121,122,122,124,124,124,124,128,129,130,131,132,133,134,
    135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,
    150,151,152,153,154,155,156,157,158,158,160,161,162,163,164,
    165,166,167,168,169,170,171,172,173,174,174,176,177,178,179,
    180,181,182,182,184,185,186,186,188,188,188,188,192,193,194,
    195,196,197,198,199,200,201,202,203,204,205,206,206,208,209,
    210,211,212,213,214,214,216,217,218,218,220,220,220,220,224,
    225,226,227,228,229,230,230,232,233,234,234,236,236,236,236,
    240,241,242,242,244,244,244,244,248,248,248,248,248,248,248,
    248,256,257,258,259,260,261,262,263,264,265,266,267,268,269,
    270,271,272,273,274,275,276,277,278,279,280,281,282,283,284,
    285,286,286,288,289,290,291,292,293,294,295,296,297,298,299,
    300,301,302,302,304,305,306,307,308,309,310,310,312,313,314,
    314,316,316,316,316,320,321,322,323,324,325,326,327,328,329,
    330,331,332,333,334,334,336,337,338,339,340,341,342,342,344,
    345,346,346,348,348,348,348,352,353,354,355,356,357,358,358,
    360,361,362,362,364,364,364,364,368,369,370,370,372,372,372,
    372,376,376,376,376,376,376,376,376,384,385,386,387,388,389,
    390,391,392,393,394,395,396,397,398,398,400,401,402,403,404,
    405,406,406,408,409,410,410,412,412,412,412,416,417,418,419,
    420,421,422,422,424,425,426,426,428,428,428,428,432,433,434,
    434,436,436,436,436,440,440,440,440,440,440,440,440,448,449,
    450,451,452,453,454,454,456,457,458,458,460,460,460,460,464,
    465,466,466,468,468,468,468,472,472,472,472,472,472,472,472,
    480,481,482,482,484,484,484,484,488,488,488,488,488,488,488,
    488,496,496,496,496,496,496,496,496,496,496,496,496,496,496,
    496,496,512,513,514,515,516,517,518,519,520,521,522,523,524,
    525,526,527,528,529,530,531,532,533,534,535,536,537,538,539,
    540,541,542,542,544,545,546,547,548,549,550,551,552,553,554,
    555,556,557,558,558,560,561,562,563,564,565,566,566,568,569,
    570,570,572,572,572,572,576,577,578,579,580,581,582,583,584,
    585,586,587,588,589,590,590,592,593,594,595,596,597,598,598,
    600,601,602,602,604,604,604,604,608,609,610,611,612,613,614,
    614,616,617,618,618,620,620,620,620,624,625,626,626,628,628,
    628,628,632,632,632,632,632,632,632,632,640,641,642,643,644,
    645,646,647,648,649,650,651,652,653,654,654,656,657,658,659,
    660,661,662,662,664,665,666,666,668,668,668,668,672,673,674,
    675,676,677,678,678,680,681,682,682,684,684,684,684,688,689,
    690,690,692,692,692,692,696,696,696,696,696,696,696,696,704,
    705,706,707,708,709,710,710,712,713,714,714,716,716,716,716,
    720,721,722,722,724,724,724,724,728,728,728,728,728,728,728,
    728,736,737,738,738,740,740,740,740,744,744,744,744,744,744,
    744,744,752,752,752,752,752,752,752,752,752,752,752,752,752,
    752,752,752,768,769,770,771,772,773,774,775,776,777,778,779,
    780,781,782,782,784,785,786,787,788,789,790,790,792,793,794,
    794,796,796,796,796,800,801,802,803,804,805,806,806,808,809,
    810,810,812,812,812,812,816,817,818,818,820,820,820,820,824,
    824,824,824,824,824,824,824,832,833,834,835,836,837,838,838,
    840,841,842,842,844,844,844,844,848,849,850,850,852,852,852,
    852,856,856,856,856,856,856,856,856,864,865,866,866,868,868,
    868,868,872,872,872,872,872,872,872,872,880,880,880,880,880,
    880,880,880,880,880,880,880,880,880,880,880,896,897,898,899,
    900,901,902,902,904,905,906,906,908,908,908,908,912,913,914,
    914,916,916,916,916,920,920,920,920,920,920,920,920,928,929,
    930,930,932,932,932,932,936,936,936,936,936,936,936,936,944,
    944,944,944,944,944,944,944,944,944,944,944,944,944,944,944,
    960,961,962,962,964,964,964,964,968,968,968,968,968,968,968,
    968,976,976,976,976,976,976,976,976,976,976,976,976,976,976,
    976,976,992,992,992,992,992,992,992,992,992,992,992,992,992,
    992,992,992,992,992,992,992,992,992,992,992,992,992,992,992,
    992,992,992,992,1024,1025,1026,1027,1028,1029,1030,1031,1032,1033,1034,
    1035,1036,1037,1038,1039,1040,1041,1042,1043,1044,1045,1046,1047,1048,1049,
    1050,1051,1052,1053,1054,1054,1056,1057,1058,1059,1060,1061,1062,1063,1064,
    1065,1066,1067,1068,1069,1070,1070,1072,1073,1074,1075,1076,1077,1078,1078,
    1080,1081,1082,1082,1084,1084,1084,1084,1088,1089,1090,1091,1092,1093,1094,
    1095,1096,1097,1098,1099,1100,1101,1102,1102,1104,1105,1106,1107,1108,1109,
    1110,1110,1112,1113,1114,1114,1116,1116,1116,1116,1120,1121,1122,1123,1124,
    1125,1126,1126,1128,1129,1130,1130,1132,1132,1132,1132,1136,1137,1138,1138,
    1140,1140,1140,1140,1144,1144,1144,1144,1144,1144,1144,1144,1152,1153,1154,
    1155,1156,1157,1158,1159,1160,1161,1162,1163,1164,1165,1166,1166,1168,1169,
    1170,1171,1172,1173,1174,1174,1176,1177,1178,1178,1180,1180,1180,1180,1184,
    1185,1186,1187,1188,1189,1190,1190,1192,1193,1194,1194,1196,1196,1196,1196,
    1200,1201,1202,1202,1204,1204,1204,1204,1208,1208,1208,1208,1208,1208,1208,
    1208,1216,1217,1218,1219,1220,1221,1222,1222,1224,1225,1226,1226,1228,1228,
    1228,1228,1232,1233,1234,1234,1236,1236,1236,1236,1240,1240,1240,1240,1240,
    1240,1240,1240,1248,1249,1250,1250,1252,1252,1252,1252,1256,1256,1256,1256,
    1256,1256,1256,1256,1264,1264,1264,1264,1264,1264,1264,1264,1264,1264,1264,
    1264,1264,1264,1264,1264,1280,1281,1282,1283,1284,1285,1286,1287,1288,1289,
    1290,1291,1292,1293,1294,1294,1296,1297,1298,1299,1300,1301,1302,1302,1304,
    1305,1306,1306,1308,1308,1308,1308,1312,1313,1314,1315,1316,1317,1318,1318,
    1320,1321,1322,1322,1324,1324,1324,1324,1328,1329,1330,1330,1332,1332,1332,
    1332,1336,1336,1336,1336,1336,1336,1336,1336,1344,1345,1346,1347,1348,1349,
    1350,1350,1352,1353,1354,1354,1356,1356,1356,1356,1360,1361,1362,1362,1364,
    1364,1364,1364,1368,1368,1368,1368,1368,1368,1368,1368,1376,1377,1378,1378,
    1380,1380,1380,1380,1384,1384,1384,1384,1384,1384,1384,1384,1392,1392,1392,
    1392,1392,1392,1392,1392,1392,1392,1392,1392,1392,1392,1392,1392,1408,1409,
    1410,1411,1412,1413,1414,1414,1416,1417,1418,1418,1420,1420,1420,1420,1424,
    1425,1426,1426,1428,1428,1428,1428,1432,1432,1432,1432,1432,1432,1432,1432,
    1440,1441,1442,1442,1444,1444,1444,1444,1448,1448,1448,1448,1448,1448,1448,
    1448,1456,1456,1456,1456,1456,1456,1456,1456,1456,1456,1456,1456,1456,1456,
    1456,1456,1472,1473,1474,1474,1476,1476,1476,1476,1480,1480,1480,1480,1480,
    1480,1480,1480,1488,1488,1488,1488,1488,1488,1488,1488,1488,1488,1488,1488,
    1488,1488,1488,1488,1504,1504,1504,1504,1504,1504,1504,1504,1504,1504,1504,
    1504,1504,1504,1504,1504,1504,1504,1504,1504,1504,1504,1504,1504,1504,1504,
    1504,1504,1504,1504,1504,1504,1536,1537,1538,1539,1540,1541,1542,1543,1544,
    1545,1546,1547,1548,1549,1550,1550,1552,1553,1554,1555,1556,1557,1558,1558,
    1560,1561,1562,1562,1564,1564,1564,1564,1568,1569,1570,1571,1572,1573,1574,
    1574,1576,1577,1578,1578,1580,1580,1580,1580,1584,1585,1586,1586,1588,1588,
    1588,1588,1592,1592,1592,1592,1592,1592,1592,1592,1600,1601,1602,1603,1604,
    1605,1606,1606,1608,1609,1610,1610,1612,1612,1612,1612,1616,1617,1618,1618,
    1620,1620,1620,1620,1624,1624,1624,1624,1624,1624,1624,1624,1632,1633,1634,
    1634,1636,1636,1636,1636,1640,1640,1640,1640,1640,1640,1640,1640,1648,1648,
    1648,1648,1648,1648,1648,1648,1648,1648,1648,1648,1648,1648,1648,1648,1664,
    1665,1666,1667,1668,1669,1670,1670,1672,1673,1674,1674,1676,1676,1676,1676,
    1680,1681,1682,1682,1684,1684,1684,1684,1688,1688,1688,1688,1688,1688,1688,
    1688,1696,1697,1698,1698,1700,1700,1700,1700,1704,1704,1704,1704,1704,1704,
    1704,1704,1712,1712,1712,1712,1712,1712,1712,1712,1712,1712,1712,1712,1712,
    1712,1712,1712,1728,1729,1730,1730,1732,1732,1732,1732,1736,1736,1736,1736,
    1736,1736,1736,1736,1744,1744,1744,1744,1744,1744,1744,1744,1744,1744,1744,
    1744,1744,1744,1744,1744,1760,1760,1760,1760,1760,1760,1760,1760,1760,1760,
    1760,1760,1760,1760,1760,1760,1760,1760,1760,1760,1760,1760,1760,1760,1760,
    1760,1760,1760,1760,1760,1760,1760,1792,1793,1794,1795,1796,1797,1798,1798,
    1800,1801,1802,1802,1804,1804,1804,1804,1808,1809,1810,1810,1812,1812,1812,
    1812,1816,1816,1816,1816,1816,1816,1816,1816,1824,1825,1826,1826,1828,1828,
    1828,1828,1832,1832,1832,1832,1832,1832,1832,1832,1840,1840,1840,1840,1840,
    1840,1840,1840,1840,1840,1840,1840,1840,1840,1840,1840,1856,1857,1858,1858,
    1860,1860,1860,1860,1864,1864,1864,1864,1864,1864,1864,1864,1872,1872,1872,
    1872,1872,1872,1872,1872,1872,1872,1872,1872,1872,1872,1872,1872,1888,1888,
    1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,
    1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,1888,
    1920,1921,1922,1922,1924,1924,1924,1924,1928,1928,1928,1928,1928,1928,1928,
    1928,1936,1936,1936,1936,1936,1936,1936,1936,1936,1936,1936,1936,1936,1936,
    1936,1936,1952,1952,1952,1952,1952,1952,1952,1952,1952,1952,1952,1952,1952,
    1952,1952,1952,1952,1952,1952,1952,1952,1952,1952,1952,1952,1952,1952,1952,
    1952,1952,1952,1952,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,
    1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,
    1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,
    1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,1984,
    1984,1984,1984,1984,1984,1984,1984,1984,2048,2049,2050,2051,2052,2053,2054,
    2055,2056,2057,2058,2059,2060,2061,2062,2063,2064,2065,2066,2067,2068,2069,
    2070,2071,2072,2073,2074,2075,2076,2077,2078,2078,2080,2081,2082,2083,2084,
    2085,2086,2087,2088,2089,2090,2091,2092,2093,2094,2094,2096,2097,2098,2099,
    2100,2101,2102,2102,2104,2105,2106,2106,2108,2108,2108,2108,2112,2113,2114,
    2115,2116,2117,2118,2119,2120,2121,2122,2123,2124,2125,2126,2126,2128,2129,
    2130,2131,2132,2133,2134,2134,2136,2137,2138,2138,2140,2140,2140,2140,2144,
    2145,2146,2147,2148,2149,2150,2150,2152,2153,2154,2154,2156,2156,2156,2156,
    2160,2161,2162,2162,2164,2164,2164,2164,2168,2168,2168,2168,2168,2168,2168,
    2168,2176,2177,2178,2179,2180,2181,2182,2183,2184,2185,2186,2187,2188,2189,
    2190,2190,2192,2193,2194,2195,2196,2197,2198,2198,2200,2201,2202,2202,2204,
    2204,2204,2204,2208,2209,2210,2211,2212,2213,2214,2214,2216,2217,2218,2218,
    2220,2220,2220,2220,2224,2225,2226,2226,2228,2228,2228,2228,2232,2232,2232,
    2232,2232,2232,2232,2232,2240,2241,2242,2243,2244,2245,2246,2246,2248,2249,
    2250,2250,2252,2252,2252,2252,2256,2257,2258,2258,2260,2260,2260,2260,2264,
    2264,2264,2264,2264,2264,2264,2264,2272,2273,2274,2274,2276,2276,2276,2276,
    2280,2280,2280,2280,2280,2280,2280,2280,2288,2288,2288,2288,2288,2288,2288,
    2288,2288,2288,2288,2288,2288,2288,2288,2288,2304,2305,2306,2307,2308,2309,
    2310,2311,2312,2313,2314,2315,2316,2317,2318,2318,2320,2321,2322,2323,2324,
    2325,2326,2326,2328,2329,2330,2330,2332,2332,2332,2332,2336,2337,2338,2339,
    2340,2341,2342,2342,2344,2345,2346,2346,2348,2348,2348,2348,2352,2353,2354,
    2354,2356,2356,2356,2356,2360,2360,2360,2360,2360,2360,2360,2360,2368,2369,
    2370,2371,2372,2373,2374,2374,2376,2377,2378,2378,2380,2380,2380,2380,2384,
    2385,2386,2386,2388,2388,2388,2388,2392,2392,2392,2392,2392,2392,2392,2392,
    2400,2401,2402,2402,2404,2404,2404,2404,2408,2408,2408,2408,2408,2408,2408,
    2408,2416,2416,2416,2416,2416,2416,2416,2416,2416,2416,2416,2416,2416,2416,
    2416,2416,2432,2433,2434,2435,2436,2437,2438,2438,2440,2441,2442,2442,2444,
    2444,2444,2444,2448,2449,2450,2450,2452,2452,2452,2452,2456,2456,2456,2456,
    2456,2456,2456,2456,2464,2465,2466,2466,2468,2468,2468,2468,2472,2472,2472,
    2472,2472,2472,2472,2472,2480,2480,2480,2480,2480,2480,2480,2480,2480,2480,
    2480,2480,2480,2480,2480,2480,2496,2497,2498,2498,2500,2500,2500,2500,2504,
    2504,2504,2504,2504,2504,2504,2504,2512,2512,2512,2512,2512,2512,2512,2512,
    2512,2512,2512,2512,2512,2512,2512,2512,2528,2528,2528,2528,2528,2528,2528,
    2528,2528,2528,2528,2528,2528,2528,2528,2528,2528,2528,2528,2528,2528,2528,
    2528,2528,2528,2528,2528,2528,2528,2528,2528,2528,2560,2561,2562,2563,2564,
    2565,2566,2567,2568,2569,2570,2571,2572,2573,2574,2574,2576,2577,2578,2579,
    2580,2581,2582,2582,2584,2585,2586,2586,2588,2588,2588,2588,2592,2593,2594,
    2595,2596,2597,2598,2598,2600,2601,2602,2602,2604,2604,2604,2604,2608,2609,
    2610,2610,2612,2612,2612,2612,2616,2616,2616,2616,2616,2616,2616,2616,2624,
    2625,2626,2627,2628,2629,2630,2630,2632,2633,2634,2634,2636,2636,2636,2636,
    2640,2641,2642,2642,2644,2644,2644,2644,2648,2648,2648,2648,2648,2648,2648,
    2648,2656,2657,2658,2658,2660,2660,2660,2660,2664,2664,2664,2664,2664,2664,
    2664,2664,2672,2672,2672,2672,2672,2672,2672,2672,2672,2672,2672,2672,2672,
    2672,2672,2672,2688,2689,2690,2691,2692,2693,2694,2694,2696,2697,2698,2698,
    2700,2700,2700,2700,2704,2705,2706,2706,2708,2708,2708,2708,2712,2712,2712,
    2712,2712,2712,2712,2712,2720,2721,2722,2722,2724,2724,2724,2724,2728,2728,
    2728,2728,2728,2728,2728,2728,2736,2736,2736,2736,2736,2736,2736,2736,2736,
    2736,2736,2736,2736,2736,2736,2736,2752,2753,2754,2754,2756,2756,2756,2756,
    2760,2760,2760,2760,2760,2760,2760,2760,2768,2768,2768,2768,2768,2768,2768,
    2768,2768,2768,2768,2768,2768,2768,2768,2768,2784,2784,2784,2784,2784,2784,
    2784,2784,2784,2784,2784,2784,2784,2784,2784,2784,2784,2784,2784,2784,2784,
    2784,2784,2784,2784,2784,2784,2784,2784,2784,2784,2784,2816,2817,2818,2819,
    2820,2821,2822,2822,2824,2825,2826,2826,2828,2828,2828,2828,2832,2833,2834,
    2834,2836,2836,2836,2836,2840,2840,2840,2840,2840,2840,2840,2840,2848,2849,
    2850,2850,2852,2852,2852,2852,2856,2856,2856,2856,2856,2856,2856,2856,2864,
    2864,2864,2864,2864,2864,2864,2864,2864,2864,2864,2864,2864,2864,2864,2864,
    2880,2881,2882,2882,2884,2884,2884,2884,2888,2888,2888,2888,2888,2888,2888,
    2888,2896,2896,2896,2896,2896,2896,2896,2896,2896,2896,2896,2896,2896,2896,
    2896,2896,2912,2912,2912,2912,2912,2912,2912,2912,2912,2912,2912,2912,2912,
    2912,2912,2912,2912,2912,2912,2912,2912,2912,2912,2912,2912,2912,2912,2912,
    2912,2912,2912,2912,2944,2945,2946,2946,2948,2948,2948,2948,2952,2952,2952,
    2952,2952,2952,2952,2952,2960,2960,2960,2960,2960,2960,2960,2960,2960,2960,
    2960,2960,2960,2960,2960,2960,2976,2976,2976,2976,2976,2976,2976,2976,2976,
    2976,2976,2976,2976,2976,2976,2976,2976,2976,2976,2976,2976,2976,2976,2976,
    2976,2976,2976,2976,2976,2976,2976,2976,3008,3008,3008,3008,3008,3008,3008,
    3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,
    3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,
    3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,
    3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3008,3072,3073,3074,
    3075,3076,3077,3078,3079,3080,3081,3082,3083,3084,3085,3086,3086,3088,3089,
    3090,3091,3092,3093,3094,3094,3096,3097,3098,3098,3100,3100,3100,3100,3104,
    3105,3106,3107,3108,3109,3110,3110,3112,3113,3114,3114,3116,3116,3116,3116,
    3120,3121,3122,3122,3124,3124,3124,3124,3128,3128,3128,3128,3128,3128,3128,
    3128,3136,3137,3138,3139,3140,3141,3142,3142,3144,3145,3146,3146,3148,3148,
    3148,3148,3152,3153,3154,3154,3156,3156,3156,3156,3160,3160,3160,3160,3160,
    3160,3160,3160,3168,3169,3170,3170,3172,3172,3172,3172,3176,3176,3176,3176,
    3176,3176,3176,3176,3184,3184,3184,3184,3184,3184,3184,3184,3184,3184,3184,
    3184,3184,3184,3184,3184,3200,3201,3202,3203,3204,3205,3206,3206,3208,3209,
    3210,3210,3212,3212,3212,3212,3216,3217,3218,3218,3220,3220,3220,3220,3224,
    3224,3224,3224,3224,3224,3224,3224,3232,3233,3234,3234,3236,3236,3236,3236,
    3240,3240,3240,3240,3240,3240,3240,3240,3248,3248,3248,3248,3248,3248,3248,
    3248,3248,3248,3248,3248,3248,3248,3248,3248,3264,3265,3266,3266,3268,3268,
    3268,3268,3272,3272,3272,3272,3272,3272,3272,3272,3280,3280,3280,3280,3280,
    3280,3280,3280,3280,3280,3280,3280,3280,3280,3280,3280,3296,3296,3296,3296,
    3296,3296,3296,3296,3296,3296,3296,3296,3296,3296,3296,3296,3296,3296,3296,
    3296,3296,3296,3296,3296,3296,3296,3296,3296,3296,3296,3296,3296,3328,3329,
    3330,3331,3332,3333,3334,3334,3336,3337,3338,3338,3340,3340,3340,3340,3344,
    3345,3346,3346,3348,3348,3348,3348,3352,3352,3352,3352,3352,3352,3352,3352,
    3360,3361,3362,3362,3364,3364,3364,3364,3368,3368,3368,3368,3368,3368,3368,
    3368,3376,3376,3376,3376,3376,3376,3376,3376,3376,3376,3376,3376,3376,3376,
    3376,3376,3392,3393,3394,3394,3396,3396,3396,3396,3400,3400,3400,3400,3400,
    3400,3400,3400,3408,3408,3408,3408,3408,3408,3408,3408,3408,3408,3408,3408,
    3408,3408,3408,3408,3424,3424,3424,3424,3424,3424,3424,3424,3424,3424,3424,
    3424,3424,3424,3424,3424,3424,3424,3424,3424,3424,3424,3424,3424,3424,3424,
    3424,3424,3424,3424,3424,3424,3456,3457,3458,3458,3460,3460,3460,3460,3464,
    3464,3464,3464,3464,3464,3464,3464,3472,3472,3472,3472,3472,3472,3472,3472,
    3472,3472,3472,3472,3472,3472,3472,3472,3488,3488,3488,3488,3488,3488,3488,
    3488,3488,3488,3488,3488,3488,3488,3488,3488,3488,3488,3488,3488,3488,3488,
    3488,3488,3488,3488,3488,3488,3488,3488,3488,3488,3520,3520,3520,3520,3520,
    3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,
    3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,
    3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,
    3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3520,3584,
    3585,3586,3587,3588,3589,3590,3590,3592,3593,3594,3594,3596,3596,3596,3596,
    3600,3601,3602,3602,3604,3604,3604,3604,3608,3608,3608,3608,3608,3608,3608,
    3608,3616,3617,3618,3618,3620,3620,3620,3620,3624,3624,3624,3624,3624,3624,
    3624,3624,3632,3632,3632,3632,3632,3632,3632,3632,3632,3632,3632,3632,3632,
    3632,3632,3632,3648,3649,3650,3650,3652,3652,3652,3652,3656,3656,3656,3656,
    3656,3656,3656,3656,3664,3664,3664,3664,3664,3664,3664,3664,3664,3664,3664,
    3664,3664,3664,3664,3664,3680,3680,3680,3680,3680,3680,3680,3680,3680,3680,
    3680,3680,3680,3680,3680,3680,3680,3680,3680,3680,3680,3680,3680,3680,3680,
    3680,3680,3680,3680,3680,3680,3680,3712,3713,3714,3714,3716,3716,3716,3716,
    3720,3720,3720,3720,3720,3720,3720,3720,3728,3728,3728,3728,3728,3728,3728,
    3728,3728,3728,3728,3728,3728,3728,3728,3728,3744,3744,3744,3744,3744,3744,
    3744,3744,3744,3744,3744,3744,3744,3744,3744,3744,3744,3744,3744,3744,3744,
    3744,3744,3744,3744,3744,3744,3744,3744,3744,3744,3744,3776,3776,3776,3776,
    3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,
    3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,
    3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,
    3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,3776,
    3840,3841,3842,3842,3844,3844,3844,3844,3848,3848,3848,3848,3848,3848,3848,
    3848,3856,3856,3856,3856,3856,3856,3856,3856,3856,3856,3856,3856,3856,3856,
    3856,3856,3872,3872,3872,3872,3872,3872,3872,3872,3872,3872,3872,3872,3872,
    3872,3872,3872,3872,3872,3872,3872,3872,3872,3872,3872,3872,3872,3872,3872,
    3872,3872,3872,3872,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,
    3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,
    3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,
    3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,3904,
    3904,3904,3904,3904,3904,3904,3904,3904,3968,3968,3968,3968,3968,3968,3968,
    3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,
    3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,
    3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,
    3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,
    3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,
    3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,
    3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,
    3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,3968,
    3968,4096,4097,4098,4099,4100,4101,4102,4103,4104,4105,4106,4107,4108,4109,
    4110,4111,4112,4113,4114,4115,4116,4117,4118,4119,4120,4121,4122,4123,4124,
    4125,4126,4126,4128,4129,4130,4131,4132,4133,4134,4135,4136,4137,4138,4139,
    4140,4141,4142,4142,4144,4145,4146,4147,4148,4149,4150,4150,4152,4153,4154,
    4154,4156,4156,4156,4156,4160,4161,4162,4163,4164,4165,4166,4167,4168,4169,
    4170,4171,4172,4173,4174,4174,4176,4177,4178,4179,4180,4181,4182,4182,4184,
    4185,4186,4186,4188,4188,4188,4188,4192,4193,4194,4195,4196,4197,4198,4198,
    4200,4201,4202,4202,4204,4204,4204,4204,4208,4209,4210,4210,4212,4212,4212,
    4212,4216,4216,4216,4216,4216,4216,4216,4216,4224,4225,4226,4227,4228,4229,
    4230,4231,4232,4233,4234,4235,4236,4237,4238,4238,4240,4241,4242,4243,4244,
    4245,4246,4246,4248,4249,4250,4250,4252,4252,4252,4252,4256,4257,4258,4259,
    4260,4261,4262,4262,4264,4265,4266,4266,4268,4268,4268,4268,4272,4273,4274,
    4274,4276,4276,4276,4276,4280,4280,4280,4280,4280,4280,4280,4280,4288,4289,
    4290,4291,4292,4293,4294,4294,4296,4297,4298,4298,4300,4300,4300,4300,4304,
    4305,4306,4306,4308,4308,4308,4308,4312,4312,4312,4312,4312,4312,4312,4312,
    4320,4321,4322,4322,4324,4324,4324,4324,4328,4328,4328,4328,4328,4328,4328,
    4328,4336,4336,4336,4336,4336,4336,4336,4336,4336,4336,4336,4336,4336,4336,
    4336,4336,4352,4353,4354,4355,4356,4357,4358,4359,4360,4361,4362,4363,4364,
    4365,4366,4366,4368,4369,4370,4371,4372,4373,4374,4374,4376,4377,4378,4378,
    4380,4380,4380,4380,4384,4385,4386,4387,4388,4389,4390,4390,4392,4393,4394,
    4394,4396,4396,4396,4396,4400,4401,4402,4402,4404,4404,4404,4404,4408,4408,
    4408,4408,4408,4408,4408,4408,4416,4417,4418,4419,4420,4421,4422,4422,4424,
    4425,4426,4426,4428,4428,4428,4428,4432,4433,4434,4434,4436,4436,4436,4436,
    4440,4440,4440,4440,4440,4440,4440,4440,4448,4449,4450,4450,4452,4452,4452,
    4452,4456,4456,4456,4456,4456,4456,4456,4456,4464,4464,4464,4464,4464,4464,
    4464,4464,4464,4464,4464,4464,4464,4464,4464,4464,4480,4481,4482,4483,4484,
    4485,4486,4486,4488,4489,4490,4490,4492,4492,4492,4492,4496,4497,4498,4498,
    4500,4500,4500,4500,4504,4504,4504,4504,4504,4504,4504,4504,4512,4513,4514,
    4514,4516,4516,4516,4516,4520,4520,4520,4520,4520,4520,4520,4520,4528,4528,
    4528,4528,4528,4528,4528,4528,4528,4528,4528,4528,4528,4528,4528,4528,4544,
    4545,4546,4546,4548,4548,4548,4548,4552,4552,4552,4552,4552,4552,4552,4552,
    4560,4560,4560,4560,4560,4560,4560,4560,4560,4560,4560,4560,4560,4560,4560,
    4560,4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,
    4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,4576,
    4576,4576,4576,4608,4609,4610,4611,4612,4613,4614,4615,4616,4617,4618,4619,
    4620,4621,4622,4622,4624,4625,4626,4627,4628,4629,4630,4630,4632,4633,4634,
    4634,4636,4636,4636,4636,4640,4641,4642,4643,4644,4645,4646,4646,4648,4649,
    4650,4650,4652,4652,4652,4652,4656,4657,4658,4658,4660,4660,4660,4660,4664,
    4664,4664,4664,4664,4664,4664,4664,4672,4673,4674,4675,4676,4677,4678,4678,
    4680,4681,4682,4682,4684,4684,4684,4684,4688,4689,4690,4690,4692,4692,4692,
    4692,4696,4696,4696,4696,4696,4696,4696,4696,4704,4705,4706,4706,4708,4708,
    4708,4708,4712,4712,4712,4712,4712,4712,4712,4712,4720,4720,4720,4720,4720,
    4720,4720,4720,4720,4720,4720,4720,4720,4720,4720,4720,4736,4737,4738,4739,
    4740,4741,4742,4742,4744,4745,4746,4746,4748,4748,4748,4748,4752,4753,4754,
    4754,4756,4756,4756,4756,4760,4760,4760,4760,4760,4760,4760,4760,4768,4769,
    4770,4770,4772,4772,4772,4772,4776,4776,4776,4776,4776,4776,4776,4776,4784,
    4784,4784,4784,4784,4784,4784,4784,4784,4784,4784,4784,4784,4784,4784,4784,
    4800,4801,4802,4802,4804,4804,4804,4804,4808,4808,4808,4808,4808,4808,4808,
    4808,4816,4816,4816,4816,4816,4816,4816,4816,4816,4816,4816,4816,4816,4816,
    4816,4816,4832,4832,4832,4832,4832,4832,4832,4832,4832,4832,4832,4832,4832,
    4832,4832,4832,4832,4832,4832,4832,4832,4832,4832,4832,4832,4832,4832,4832,
    4832,4832,4832,4832,4864,4865,4866,4867,4868,4869,4870,4870,4872,4873,4874,
    4874,4876,4876,4876,4876,4880,4881,4882,4882,4884,4884,4884,4884,4888,4888,
    4888,4888,4888,4888,4888,4888,4896,4897,4898,4898,4900,4900,4900,4900,4904,
    4904,4904,4904,4904,4904,4904,4904,4912,4912,4912,4912,4912,4912,4912,4912,
    4912,4912,4912,4912,4912,4912,4912,4912,4928,4929,4930,4930,4932,4932,4932,
    4932,4936,4936,4936,4936,4936,4936,4936,4936,4944,4944,4944,4944,4944,4944,
    4944,4944,4944,4944,4944,4944,4944,4944,4944,4944,4960,4960,4960,4960,4960,
    4960,4960,4960,4960,4960,4960,4960,4960,4960,4960,4960,4960,4960,4960,4960,
    4960,4960,4960,4960,4960,4960,4960,4960,4960,4960,4960,4960,4992,4993,4994,
    4994,4996,4996,4996,4996,5000,5000,5000,5000,5000,5000,5000,5000,5008,5008,
    5008,5008,5008,5008,5008,5008,5008,5008,5008,5008,5008,5008,5008,5008,5024,
    5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,
    5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,5024,
    5024,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,
    5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,
    5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,
    5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,5056,
    5056,5056,5056,5056,5056,5120,5121,5122,5123,5124,5125,5126,5127,5128,5129,
    5130,5131,5132,5133,5134,5134,5136,5137,5138,5139,5140,5141,5142,5142,5144,
    5145,5146,5146,5148,5148,5148,5148,5152,5153,5154,5155,5156,5157,5158,5158,
    5160,5161,5162,5162,5164,5164,5164,5164,5168,5169,5170,5170,5172,5172,5172,
    5172,5176,5176,5176,5176,5176,5176,5176,5176,5184,5185,5186,5187,5188,5189,
    5190,5190,5192,5193,5194,5194,5196,5196,5196,5196,5200,5201,5202,5202,5204,
    5204,5204,5204,5208,5208,5208,5208,5208,5208,5208,5208,5216,5217,5218,5218,
    5220,5220,5220,5220,5224,5224,5224,5224,5224,5224,5224,5224,5232,5232,5232,
    5232,5232,5232,5232,5232,5232,5232,5232,5232,5232,5232,5232,5232,5248,5249,
    5250,5251,5252,5253,5254,5254,5256,5257,5258,5258,5260,5260,5260,5260,5264,
    5265,5266,5266,5268,5268,5268,5268,5272,5272,5272,5272,5272,5272,5272,5272,
    5280,5281,5282,5282,5284,5284,5284,5284,5288,5288,5288,5288,5288,5288,5288,
    5288,5296,5296,5296,5296,5296,5296,5296,5296,5296,5296,5296,5296,5296,5296,
    5296,5296,5312,5313,5314,5314,5316,5316,5316,5316,5320,5320,5320,5320,5320,
    5320,5320,5320,5328,5328,5328,5328,5328,5328,5328,5328,5328,5328,5328,5328,
    5328,5328,5328,5328,5344,5344,5344,5344,5344,5344,5344,5344,5344,5344,5344,
    5344,5344,5344,5344,5344,5344,5344,5344,5344,5344,5344,5344,5344,5344,5344,
    5344,5344,5344,5344,5344,5344,5376,5377,5378,5379,5380,5381,5382,5382,5384,
    5385,5386,5386,5388,5388,5388,5388,5392,5393,5394,5394,5396,5396,5396,5396,
    5400,5400,5400,5400,5400,5400,5400,5400,5408,5409,5410,5410,5412,5412,5412,
    5412,5416,5416,5416,5416,5416,5416,5416,5416,5424,5424,5424,5424,5424,5424,
    5424,5424,5424,5424,5424,5424,5424,5424,5424,5424,5440,5441,5442,5442,5444,
    5444,5444,5444,5448,5448,5448,5448,5448,5448,5448,5448,5456,5456,5456,5456,
    5456,5456,5456,5456,5456,5456,5456,5456,5456,5456,5456,5456,5472,5472,5472,
    5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,
    5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,5472,5504,
    5505,5506,5506,5508,5508,5508,5508,5512,5512,5512,5512,5512,5512,5512,5512,
    5520,5520,5520,5520,5520,5520,5520,5520,5520,5520,5520,5520,5520,5520,5520,
    5520,5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,
    5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,5536,
    5536,5536,5536,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,
    5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,
    5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,
    5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,5568,
    5568,5568,5568,5568,5568,5568,5568,5632,5633,5634,5635,5636,5637,5638,5638,
    5640,5641,5642,5642,5644,5644,5644,5644,5648,5649,5650,5650,5652,5652,5652,
    5652,5656,5656,5656,5656,5656,5656,5656,5656,5664,5665,5666,5666,5668,5668,
    5668,5668,5672,5672,5672,5672,5672,5672,5672,5672,5680,5680,5680,5680,5680,
    5680,5680,5680,5680,5680,5680,5680,5680,5680,5680,5680,5696,5697,5698,5698,
    5700,5700,5700,5700,5704,5704,5704,5704,5704,5704,5704,5704,5712,5712,5712,
    5712,5712,5712,5712,5712,5712,5712,5712,5712,5712,5712,5712,5712,5728,5728,
    5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,
    5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,5728,
    5760,5761,5762,5762,5764,5764,5764,5764,5768,5768,5768,5768,5768,5768,5768,
    5768,5776,5776,5776,5776,5776,5776,5776,5776,5776,5776,5776,5776,5776,5776,
    5776,5776,5792,5792,5792,5792,5792,5792,5792,5792,5792,5792,5792,5792,5792,
    5792,5792,5792,5792,5792,5792,5792,5792,5792,5792,5792,5792,5792,5792,5792,
    5792,5792,5792,5792,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,
    5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,
    5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,
    5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,5824,
    5824,5824,5824,5824,5824,5824,5824,5824,5888,5889,5890,5890,5892,5892,5892,
    5892,5896,5896,5896,5896,5896,5896,5896,5896,5904,5904,5904,5904,5904,5904,
    5904,5904,5904,5904,5904,5904,5904,5904,5904,5904,5920,5920,5920,5920,5920,
    5920,5920,5920,5920,5920,5920,5920,5920,5920,5920,5920,5920,5920,5920,5920,
    5920,5920,5920,5920,5920,5920,5920,5920,5920,5920,5920,5920,5952,5952,5952,
    5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,
    5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,
    5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,
    5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,5952,
    5952,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,
    6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,
    6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,
    6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,
    6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,
    6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,
    6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,
    6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,6016,
    6016,6016,6016,6016,6016,6016,6016,6016,6016,6144,6145,6146,6147,6148,6149,
    6150,6151,6152,6153,6154,6155,6156,6157,6158,6158,6160,6161,6162,6163,6164,
    6165,6166,6166,6168,6169,6170,6170,6172,6172,6172,6172,6176,6177,6178,6179,
    6180,6181,6182,6182,6184,6185,6186,6186,6188,6188,6188,6188,6192,6193,6194,
    6194,6196,6196,6196,6196,6200,6200,6200,6200,6200,6200,6200,6200,6208,6209,
    6210,6211,6212,6213,6214,6214,6216,6217,6218,6218,6220,6220,6220,6220,6224,
    6225,6226,6226,6228,6228,6228,6228,6232,6232,6232,6232,6232,6232,6232,6232,
    6240,6241,6242,6242,6244,6244,6244,6244,6248,6248,6248,6248,6248,6248,6248,
    6248,6256,6256,6256,6256,6256,6256,6256,6256,6256,6256,6256,6256,6256,6256,
    6256,6256,6272,6273,6274,6275,6276,6277,6278,6278,6280,6281,6282,6282,6284,
    6284,6284,6284,6288,6289,6290,6290,6292,6292,6292,6292,6296,6296,6296,6296,
    6296,6296,6296,6296,6304,6305,6306,6306,6308,6308,6308,6308,6312,6312,6312,
    6312,6312,6312,6312,6312,6320,6320,6320,6320,6320,6320,6320,6320,6320,6320,
    6320,6320,6320,6320,6320,6320,6336,6337,6338,6338,6340,6340,6340,6340,6344,
    6344,6344,6344,6344,6344,6344,6344,6352,6352,6352,6352,6352,6352,6352,6352,
    6352,6352,6352,6352,6352,6352,6352,6352,6368,6368,6368,6368,6368,6368,6368,
    6368,6368,6368,6368,6368,6368,6368,6368,6368,6368,6368,6368,6368,6368,6368,
    6368,6368,6368,6368,6368,6368,6368,6368,6368,6368,6400,6401,6402,6403,6404,
    6405,6406,6406,6408,6409,6410,6410,6412,6412,6412,6412,6416,6417,6418,6418,
    6420,6420,6420,6420,6424,6424,6424,6424,6424,6424,6424,6424,6432,6433,6434,
    6434,6436,6436,6436,6436,6440,6440,6440,6440,6440,6440,6440,6440,6448,6448,
    6448,6448,6448,6448,6448,6448,6448,6448,6448,6448,6448,6448,6448,6448,6464,
    6465,6466,6466,6468,6468,6468,6468,6472,6472,6472,6472,6472,6472,6472,6472,
    6480,6480,6480,6480,6480,6480,6480,6480,6480,6480,6480,6480,6480,6480,6480,
    6480,6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,
    6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,6496,
    6496,6496,6496,6528,6529,6530,6530,6532,6532,6532,6532,6536,6536,6536,6536,
    6536,6536,6536,6536,6544,6544,6544,6544,6544,6544,6544,6544,6544,6544,6544,
    6544,6544,6544,6544,6544,6560,6560,6560,6560,6560,6560,6560,6560,6560,6560,
    6560,6560,6560,6560,6560,6560,6560,6560,6560,6560,6560,6560,6560,6560,6560,
    6560,6560,6560,6560,6560,6560,6560,6592,6592,6592,6592,6592,6592,6592,6592,
    6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,
    6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,
    6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,
    6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6592,6656,6657,6658,6659,
    6660,6661,6662,6662,6664,6665,6666,6666,6668,6668,6668,6668,6672,6673,6674,
    6674,6676,6676,6676,6676,6680,6680,6680,6680,6680,6680,6680,6680,6688,6689,
    6690,6690,6692,6692,6692,6692,6696,6696,6696,6696,6696,6696,6696,6696,6704,
    6704,6704,6704,6704,6704,6704,6704,6704,6704,6704,6704,6704,6704,6704,6704,
    6720,6721,6722,6722,6724,6724,6724,6724,6728,6728,6728,6728,6728,6728,6728,
    6728,6736,6736,6736,6736,6736,6736,6736,6736,6736,6736,6736,6736,6736,6736,
    6736,6736,6752,6752,6752,6752,6752,6752,6752,6752,6752,6752,6752,6752,6752,
    6752,6752,6752,6752,6752,6752,6752,6752,6752,6752,6752,6752,6752,6752,6752,
    6752,6752,6752,6752,6784,6785,6786,6786,6788,6788,6788,6788,6792,6792,6792,
    6792,6792,6792,6792,6792,6800,6800,6800,6800,6800,6800,6800,6800,6800,6800,
    6800,6800,6800,6800,6800,6800,6816,6816,6816,6816,6816,6816,6816,6816,6816,
    6816,6816,6816,6816,6816,6816,6816,6816,6816,6816,6816,6816,6816,6816,6816,
    6816,6816,6816,6816,6816,6816,6816,6816,6848,6848,6848,6848,6848,6848,6848,
    6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,
    6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,
    6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,
    6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6848,6912,6913,6914,
    6914,6916,6916,6916,6916,6920,6920,6920,6920,6920,6920,6920,6920,6928,6928,
    6928,6928,6928,6928,6928,6928,6928,6928,6928,6928,6928,6928,6928,6928,6944,
    6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,
    6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,6944,
    6944,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,
    6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,
    6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,
    6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,6976,
    6976,6976,6976,6976,6976,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,
    7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,
    7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,
    7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,
    7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,
    7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,
    7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,
    7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,
    7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7040,7168,7169,
    7170,7171,7172,7173,7174,7174,7176,7177,7178,7178,7180,7180,7180,7180,7184,
    7185,7186,7186,7188,7188,7188,7188,7192,7192,7192,7192,7192,7192,7192,7192,
    7200,7201,7202,7202,7204,7204,7204,7204,7208,7208,7208,7208,7208,7208,7208,
    7208,7216,7216,7216,7216,7216,7216,7216,7216,7216,7216,7216,7216,7216,7216,
    7216,7216,7232,7233,7234,7234,7236,7236,7236,7236,7240,7240,7240,7240,7240,
    7240,7240,7240,7248,7248,7248,7248,7248,7248,7248,7248,7248,7248,7248,7248,
    7248,7248,7248,7248,7264,7264,7264,7264,7264,7264,7264,7264,7264,7264,7264,
    7264,7264,7264,7264,7264,7264,7264,7264,7264,7264,7264,7264,7264,7264,7264,
    7264,7264,7264,7264,7264,7264,7296,7297,7298,7298,7300,7300,7300,7300,7304,
    7304,7304,7304,7304,7304,7304,7304,7312,7312,7312,7312,7312,7312,7312,7312,
    7312,7312,7312,7312,7312,7312,7312,7312,7328,7328,7328,7328,7328,7328,7328,
    7328,7328,7328,7328,7328,7328,7328,7328,7328,7328,7328,7328,7328,7328,7328,
    7328,7328,7328,7328,7328,7328,7328,7328,7328,7328,7360,7360,7360,7360,7360,
    7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,
    7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,
    7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,
    7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7360,7424,
    7425,7426,7426,7428,7428,7428,7428,7432,7432,7432,7432,7432,7432,7432,7432,
    7440,7440,7440,7440,7440,7440,7440,7440,7440,7440,7440,7440,7440,7440,7440,
    7440,7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,
    7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,7456,
    7456,7456,7456,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,
    7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,
    7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,
    7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,7488,
    7488,7488,7488,7488,7488,7488,7488,7552,7552,7552,7552,7552,7552,7552,7552,
    7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,
    7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,
    7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,
    7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,
    7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,
    7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,
    7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,
    7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,7552,
    7680,7681,7682,7682,7684,7684,7684,7684,7688,7688,7688,7688,7688,7688,7688,
    7688,7696,7696,7696,7696,7696,7696,7696,7696,7696,7696,7696,7696,7696,7696,
    7696,7696,7712,7712,7712,7712,7712,7712,7712,7712,7712,7712,7712,7712,7712,
    7712,7712,7712,7712,7712,7712,7712,7712,7712,7712,7712,7712,7712,7712,7712,
    7712,7712,7712,7712,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,
    7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,
    7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,
    7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,7744,
    7744,7744,7744,7744,7744,7744,7744,7744,7808,7808,7808,7808,7808,7808,7808,
    7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,
    7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,
    7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,
    7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,
    7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,
    7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,
    7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,
    7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,7808,
    7808,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,7936,
    7936,7936,
};

const uint16_t topThreeRanksTable[] =
{
    0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
    14, 16, 17, 18, 19, 20, 21, 22, 22, 24, 25, 26, 26, 28, 28,
    28, 28, 32, 33, 34, 35, 36, 37, 38, 38, 40, 41, 42, 42, 44,
    44, 44, 44, 48, 49, 50, 50, 52, 52, 52, 52, 56, 56, 56, 56,
    56, 56, 56, 56, 64, 65, 66, 67, 68, 69, 70, 70, 72, 73, 74,
    74, 76, 76, 76, 76, 80, 81, 82, 82, 84, 84, 84, 84, 88, 88,
    88, 88, 88, 88, 88, 88, 96, 97, 98, 98,100,100,100,100,104,
    104,104,104,104,104,104,104,112,112,112,112,112,112,112,112,
    112,112,112,112,112,112,112,112,128,129,130,131,132,133,134,
    134,136,137,138,138,140,140,140,140,144,145,146,146,148,148,
    148,148,152,152,152,152,152,152,152,152,160,161,162,162,164,
    164,164,164,168,168,168,168,168,168,168,168,176,176,176,176,
    176,176,176,176,176,176,176,176,176,176,176,176,192,193,194,
    194,196,196,196,196,200,200,200,200,200,200,200,200,208,208,
    208,208,208,208,208,208,208,208,208,208,208,208,208,208,224,
    224,224,224,224,224,224,224,224,224,224,224,224,224,224,224,
    224,224,224,224,224,224,224,224,224,224,224,224,224,224,224,
    224,256,257,258,259,260,261,262,262,264,265,266,266,268,268,
    268,268,272,273,274,274,276,276,276,276,280,280,280,280,280,
    280,280,280,288,289,290,290,292,292,292,292,296,296,296,296,
    296,296,296,296,304,304,304,304,304,304,304,304,304,304,304,
    304,304,304,304,304,320,321,322,322,324,324,324,324,328,328,
    328,328,328,328,328,328,336,336,336,336,336,336,336,336,336,
    336,336,336,336,336,336,336,352,352,352,352,352,352,352,352,
    352,352,352,352,352,352,352,352,352,352,352,352,352,352,352,
    352,352,352,352,352,352,352,352,352,384,385,386,386,388,388,
    388,388,392,392,392,392,392,392,392,392,400,400,400,400,400,
    400,400,400,400,400,400,400,400,400,400,400,416,416,416,416,
    416,416,416,416,416,416,416,416,416,416,416,416,416,416,416,
    416,416,416,416,416,416,416,416,416,416,416,416,416,448,448,
    448,448,448,448,448,448,448,448,448,448,448,448,448,448,448,
    448,448,448,448,448,448,448,448,448,448,448,448,448,448,448,
    448,448,448,448,448,448,448,448,448,448,448,448,448,448,448,
    448,448,448,448,448,448,448,448,448,448,448,448,448,448,448,
    448,448,512,513,514,515,516,517,518,518,520,521,522,522,524,
    524,524,524,528,529,530,530,532,532,532,532,536,536,536,536,
    536,536,536,536,544,545,546,546,548,548,548,548,552,552,552,
    552,552,552,552,552,560,560,560,560,560,560,560,560,560,560,
    560,560,560,560,560,560,576,577,578,578,580,580,580,580,584,
    584,584,584,584,584,584,584,592,592,592,592,592,592,592,592,
    592,592,592,592,592,592,592,592,608,608,608,608,608,608,608,
    608,608,608,608,608,608,608,608,608,608,608,608,608,608,608,
    608,608,608,608,608,608,608,608,608,608,640,641,642,642,644,
    644,644,644,648,648,648,648,648,648,648,648,656,656,656,656,
    656,656,656,656,656,656,656,656,656,656,656,656,672,672,672,
    672,672,672,672,672,672,672,672,672,672,672,672,672,672,672,
    672,672,672,672,672,672,672,672,672,672,672,672,672,672,704,
    704,704,704,704,704,704,704,704,704,704,704,704,704,704,704,
    704,704,704,704,704,704,704,704,704,704,704,704,704,704,704,
    704,704,704,704,704,704,704,704,704,704,704,704,704,704,704,
    704,704,704,704,704,704,704,704,704,704,704,704,704,704,704,
    704,704,704,768,769,770,770,772,772,772,772,776,776,776,776,
    776,776,776,776,784,784,784,784,784,784,784,784,784,784,784,
    784,784,784,784,784,800,800,800,800,800,800,800,800,800,800,
    800,800,800,800,800,800,800,800,800,800,800,800,800,800,800,
    800,800,800,800,800,800,800,832,832,832,832,832,832,832,832,
    832,832,832,832,832,832,832,832,832,832,832,832,832,832,832,
    832,832,832,832,832,832,832,832,832,832,832,832,832,832,832,
    832,832,832,832,832,832,832,832,832,832,832,832,832,832,832,
    832,832,832,832,832,832,832,832,832,832,832,896,896,896,896,
    896,896,896,896,896,896,896,896,896,896,896,896,896,896,896,
    896,896,896,896,896,896,896,896,896,896,896,896,896,896,896,
    896,896,896,896,896,896,896,896,896,896,896,896,896,896,896,
    896,896,896,896,896,896,896,896,896,896,896,896,896,896,896,
    896,896,896,896,896,896,896,896,896,896,896,896,896,896,896,
    896,896,896,896,896,896,896,896,896,896,896,896,896,896,896,
    896,896,896,896,896,896,896,896,896,896,896,896,896,896,896,
    896,896,896,896,896,896,896,896,896,896,896,896,896,896,896,
    896,896,896,896,1024,1025,1026,1027,1028,1029,1030,1030,1032,1033,1034,
    1034,1036,1036,1036,1036,1040,1041,1042,1042,1044,1044,1044,1044,1048,1048,
    1048,1048,1048,1048,1048,1048,1056,1057,1058,1058,1060,1060,1060,1060,1064,
    1064,1064,1064,1064,1064,1064,1064,1072,1072,1072,1072,1072,1072,1072,1072,
    1072,1072,1072,1072,1072,1072,1072,1072,1088,1089,1090,1090,1092,1092,1092,
    1092,1096,1096,1096,1096,1096,1096,1096,1096,1104,1104,1104,1104,1104,1104,
    1104,1104,1104,1104,1104,1104,1104,1104,1104,1104,1120,1120,1120,1120,1120,
    1120,1120,1120,1120,1120,1120,1120,1120,1120,1120,1120,1120,1120,1120,1120,
    1120,1120,1120,1120,1120,1120,1120,1120,1120,1120,1120,1120,1152,1153,1154,
    1154,1156,1156,1156,1156,1160,1160,1160,1160,1160,1160,1160,1160,1168,1168,
    1168,1168,1168,1168,1168,1168,1168,1168,1168,1168,1168,1168,1168,1168,1184,
    1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,
    1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,1184,
    1184,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,
    1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,
    1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,
    1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,1216,
    1216,1216,1216,1216,1216,1280,1281,1282,1282,1284,1284,1284,1284,1288,1288,
    1288,1288,1288,1288,1288,1288,1296,1296,1296,1296,1296,1296,1296,1296,1296,
    1296,1296,1296,1296,1296,1296,1296,1312,1312,1312,1312,1312,1312,1312,1312,
    1312,1312,1312,1312,1312,1312,1312,1312,1312,1312,1312,1312,1312,1312,1312,
    1312,1312,1312,1312,1312,1312,1312,1312,1312,1344,1344,1344,1344,1344,1344,
    1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,
    1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,
    1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,
    1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1344,1408,1408,
    1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,
    1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,
    1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,
    1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,
    1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,
    1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,
    1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,
    1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,1408,
    1408,1408,1408,1408,1408,1408,1536,1537,1538,1538,1540,1540,1540,1540,1544,
    1544,1544,1544,1544,1544,1544,1544,1552,1552,1552,1552,1552,1552,1552,1552,
    1552,1552,1552,1552,1552,1552,1552,1552,1568,1568,1568,1568,1568,1568,1568,
    1568,1568,1568,1568,1568,1568,1568,1568,1568,1568,1568,1568,1568,1568,1568,
    1568,1568,1568,1568,1568,1568,1568,1568,1568,1568,1600,1600,1600,1600,1600,
    1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,
    1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,
    1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,
    1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1600,1664,
    1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,
    1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,
    1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,
    1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,
    1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,
    1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,
    1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,
    1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,1664,
    1664,1664,1664,1664,1664,1664,1664,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,1792,
    1792,1792,1792,1792,1792,1792,1792,1792,2048,2049,2050,2051,2052,2053,2054,
    2054,2056,2057,2058,2058,2060,2060,2060,2060,2064,2065,2066,2066,2068,2068,
    2068,2068,2072,2072,2072,2072,2072,2072,2072,2072,2080,2081,2082,2082,2084,
    2084,2084,2084,2088,2088,2088,2088,2088,2088,2088,2088,2096,2096,2096,2096,
    2096,2096,2096,2096,2096,2096,2096,2096,2096,2096,2096,2096,2112,2113,2114,
    2114,2116,2116,2116,2116,2120,2120,2120,2120,2120,2120,2120,2120,2128,2128,
    2128,2128,2128,2128,2128,2128,2128,2128,2128,2128,2128,2128,2128,2128,2144,
    2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,
    2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,2144,
    2144,2176,2177,2178,2178,2180,2180,2180,2180,2184,2184,2184,2184,2184,2184,
    2184,2184,2192,2192,2192,2192,2192,2192,2192,2192,2192,2192,2192,2192,2192,
    2192,2192,2192,2208,2208,2208,2208,2208,2208,2208,2208,2208,2208,2208,2208,
    2208,2208,2208,2208,2208,2208,2208,2208,2208,2208,2208,2208,2208,2208,2208,
    2208,2208,2208,2208,2208,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,
    2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,
    2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,
    2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,2240,
    2240,2240,2240,2240,2240,2240,2240,2240,2240,2304,2305,2306,2306,2308,2308,
    2308,2308,2312,2312,2312,2312,2312,2312,2312,2312,2320,2320,2320,2320,2320,
    2320,2320,2320,2320,2320,2320,2320,2320,2320,2320,2320,2336,2336,2336,2336,
    2336,2336,2336,2336,2336,2336,2336,2336,2336,2336,2336,2336,2336,2336,2336,
    2336,2336,2336,2336,2336,2336,2336,2336,2336,2336,2336,2336,2336,2368,2368,
    2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,
    2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,
    2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,
    2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,2368,
    2368,2368,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,
    2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,
    2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,
    2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,
    2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,
    2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,
    2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,
    2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,
    2432,2432,2432,2432,2432,2432,2432,2432,2432,2432,2560,2561,2562,2562,2564,
    2564,2564,2564,2568,2568,2568,2568,2568,2568,2568,2568,2576,2576,2576,2576,
    2576,2576,2576,2576,2576,2576,2576,2576,2576,2576,2576,2576,2592,2592,2592,
    2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,
    2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,2592,2624,
    2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,
    2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,
    2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,
    2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,2624,
    2624,2624,2624,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,
    2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,
    2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,
    2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,
    2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,
    2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,
    2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,
    2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,
    2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2688,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,
    2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,2816,3072,3073,3074,
    3074,3076,3076,3076,3076,3080,3080,3080,3080,3080,3080,3080,3080,3088,3088,
    3088,3088,3088,3088,3088,3088,3088,3088,3088,3088,3088,3088,3088,3088,3104,
    3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,
    3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,3104,
    3104,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,
    3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,
    3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,
    3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,3136,
    3136,3136,3136,3136,3136,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,
    3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,
    3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,
    3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,
    3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,
    3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,
    3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,
    3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,
    3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3200,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,
    3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3328,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,3584,
    3584,4096,4097,4098,4099,4100,4101,4102,4102,4104,4105,4106,4106,4108,4108,
    4108,4108,4112,4113,4114,4114,4116,4116,4116,4116,4120,4120,4120,4120,4120,
    4120,4120,4120,4128,4129,4130,4130,4132,4132,4132,4132,4136,4136,4136,4136,
    4136,4136,4136,4136,4144,4144,4144,4144,4144,4144,4144,4144,4144,4144,4144,
    4144,4144,4144,4144,4144,4160,4161,4162,4162,4164,4164,4164,4164,4168,4168,
    4168,4168,4168,4168,4168,4168,4176,4176,4176,4176,4176,4176,4176,4176,4176,
    4176,4176,4176,4176,4176,4176,4176,4192,4192,4192,4192,4192,4192,4192,4192,
    4192,4192,4192,4192,4192,4192,4192,4192,4192,4192,4192,4192,4192,4192,4192,
    4192,4192,4192,4192,4192,4192,4192,4192,4192,4224,4225,4226,4226,4228,4228,
    4228,4228,4232,4232,4232,4232,4232,4232,4232,4232,4240,4240,4240,4240,4240,
    4240,4240,4240,4240,4240,4240,4240,4240,4240,4240,4240,4256,4256,4256,4256,
    4256,4256,4256,4256,4256,4256,4256,4256,4256,4256,4256,4256,4256,4256,4256,
    4256,4256,4256,4256,4256,4256,4256,4256,4256,4256,4256,4256,4256,4288,4288,
    4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,
    4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,
    4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,
    4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,4288,
    4288,4288,4352,4353,4354,4354,4356,4356,4356,4356,4360,4360,4360,4360,4360,
    4360,4360,4360,4368,4368,4368,4368,4368,4368,4368,4368,4368,4368,4368,4368,
    4368,4368,4368,4368,4384,4384,4384,4384,4384,4384,4384,4384,4384,4384,4384,
    4384,4384,4384,4384,4384,4384,4384,4384,4384,4384,4384,4384,4384,4384,4384,
    4384,4384,4384,4384,4384,4384,4416,4416,4416,4416,4416,4416,4416,4416,4416,
    4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,
    4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,
    4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,
    4416,4416,4416,4416,4416,4416,4416,4416,4416,4416,4480,4480,4480,4480,4480,
    4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,
    4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,
    4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,
    4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,
    4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,
    4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,
    4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,
    4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,4480,
    4480,4480,4480,4608,4609,4610,4610,4612,4612,4612,4612,4616,4616,4616,4616,
    4616,4616,4616,4616,4624,4624,4624,4624,4624,4624,4624,4624,4624,4624,4624,
    4624,4624,4624,4624,4624,4640,4640,4640,4640,4640,4640,4640,4640,4640,4640,
    4640,4640,4640,4640,4640,4640,4640,4640,4640,4640,4640,4640,4640,4640,4640,
    4640,4640,4640,4640,4640,4640,4640,4672,4672,4672,4672,4672,4672,4672,4672,
    4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,
    4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,
    4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,
    4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4672,4736,4736,4736,4736,
    4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,
    4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,
    4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,
    4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,
    4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,
    4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,
    4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,
    4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,4736,
    4736,4736,4736,4736,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,4864,
    4864,4864,4864,4864,4864,5120,5121,5122,5122,5124,5124,5124,5124,5128,5128,
    5128,5128,5128,5128,5128,5128,5136,5136,5136,5136,5136,5136,5136,5136,5136,
    5136,5136,5136,5136,5136,5136,5136,5152,5152,5152,5152,5152,5152,5152,5152,
    5152,5152,5152,5152,5152,5152,5152,5152,5152,5152,5152,5152,5152,5152,5152,
    5152,5152,5152,5152,5152,5152,5152,5152,5152,5184,5184,5184,5184,5184,5184,
    5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,
    5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,
    5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,
    5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5184,5248,5248,
    5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,
    5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,
    5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,
    5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,
    5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,
    5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,
    5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,
    5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,5248,
    5248,5248,5248,5248,5248,5248,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,5376,
    5376,5376,5376,5376,5376,5376,5376,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,5632,
    5632,5632,5632,5632,5632,5632,5632,5632,5632,6144,6145,6146,6146,6148,6148,
    6148,6148,6152,6152,6152,6152,6152,6152,6152,6152,6160,6160,6160,6160,6160,
    6160,6160,6160,6160,6160,6160,6160,6160,6160,6160,6160,6176,6176,6176,6176,
    6176,6176,6176,6176,6176,6176,6176,6176,6176,6176,6176,6176,6176,6176,6176,
    6176,6176,6176,6176,6176,6176,6176,6176,6176,6176,6176,6176,6176,6208,6208,
    6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,
    6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,
    6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,
    6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,6208,
    6208,6208,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,
    6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,
    6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,
    6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,
    6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,
    6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,
    6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,
    6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,
    6272,6272,6272,6272,6272,6272,6272,6272,6272,6272,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,
    6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6400,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,
    6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,6656,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,7168,
    7168,7168,
};

const int lowballA5Ranks[] =
{
    0, //  00000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    6, //  01100000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    10, //  01010000 00000000 00000000 00000000
    12, //  00110000 00000000 00000000 00000000
    14, //  01110000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    18, //  01001000 00000000 00000000 00000000
    20, //  00101000 00000000 00000000 00000000
    22, //  01101000 00000000 00000000 00000000
    24, //  00011000 00000000 00000000 00000000
    26, //  01011000 00000000 00000000 00000000
    28, //  00111000 00000000 00000000 00000000
    30, //  01111000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    34, //  01000100 00000000 00000000 00000000
    36, //  00100100 00000000 00000000 00000000
    38, //  01100100 00000000 00000000 00000000
    40, //  00010100 00000000 00000000 00000000
    42, //  01010100 00000000 00000000 00000000
    44, //  00110100 00000000 00000000 00000000
    46, //  01110100 00000000 00000000 00000000
    48, //  00001100 00000000 00000000 00000000
    50, //  01001100 00000000 00000000 00000000
    52, //  00101100 00000000 00000000 00000000
    54, //  01101100 00000000 00000000 00000000
    56, //  00011100 00000000 00000000 00000000
    58, //  01011100 00000000 00000000 00000000
    60, //  00111100 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    66, //  01000010 00000000 00000000 00000000
    68, //  00100010 00000000 00000000 00000000
    70, //  01100010 00000000 00000000 00000000
    72, //  00010010 00000000 00000000 00000000
    74, //  01010010 00000000 00000000 00000000
    76, //  00110010 00000000 00000000 00000000
    78, //  01110010 00000000 00000000 00000000
    80, //  00001010 00000000 00000000 00000000
    82, //  01001010 00000000 00000000 00000000
    84, //  00101010 00000000 00000000 00000000
    86, //  01101010 00000000 00000000 00000000
    88, //  00011010 00000000 00000000 00000000
    90, //  01011010 00000000 00000000 00000000
    92, //  00111010 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    96, //  00000110 00000000 00000000 00000000
    98, //  01000110 00000000 00000000 00000000
    100, //  00100110 00000000 00000000 00000000
    102, //  01100110 00000000 00000000 00000000
    104, //  00010110 00000000 00000000 00000000
    106, //  01010110 00000000 00000000 00000000
    108, //  00110110 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    112, //  00001110 00000000 00000000 00000000
    114, //  01001110 00000000 00000000 00000000
    116, //  00101110 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    120, //  00011110 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    130, //  01000001 00000000 00000000 00000000
    132, //  00100001 00000000 00000000 00000000
    134, //  01100001 00000000 00000000 00000000
    136, //  00010001 00000000 00000000 00000000
    138, //  01010001 00000000 00000000 00000000
    140, //  00110001 00000000 00000000 00000000
    142, //  01110001 00000000 00000000 00000000
    144, //  00001001 00000000 00000000 00000000
    146, //  01001001 00000000 00000000 00000000
    148, //  00101001 00000000 00000000 00000000
    150, //  01101001 00000000 00000000 00000000
    152, //  00011001 00000000 00000000 00000000
    154, //  01011001 00000000 00000000 00000000
    156, //  00111001 00000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    160, //  00000101 00000000 00000000 00000000
    162, //  01000101 00000000 00000000 00000000
    164, //  00100101 00000000 00000000 00000000
    166, //  01100101 00000000 00000000 00000000
    168, //  00010101 00000000 00000000 00000000
    170, //  01010101 00000000 00000000 00000000
    172, //  00110101 00000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    176, //  00001101 00000000 00000000 00000000
    178, //  01001101 00000000 00000000 00000000
    180, //  00101101 00000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    184, //  00011101 00000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    192, //  00000011 00000000 00000000 00000000
    194, //  01000011 00000000 00000000 00000000
    196, //  00100011 00000000 00000000 00000000
    198, //  01100011 00000000 00000000 00000000
    200, //  00010011 00000000 00000000 00000000
    202, //  01010011 00000000 00000000 00000000
    204, //  00110011 00000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    208, //  00001011 00000000 00000000 00000000
    210, //  01001011 00000000 00000000 00000000
    212, //  00101011 00000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    216, //  00011011 00000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    224, //  00000111 00000000 00000000 00000000
    226, //  01000111 00000000 00000000 00000000
    228, //  00100111 00000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    232, //  00010111 00000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    240, //  00001111 00000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    258, //  01000000 10000000 00000000 00000000
    260, //  00100000 10000000 00000000 00000000
    262, //  01100000 10000000 00000000 00000000
    264, //  00010000 10000000 00000000 00000000
    266, //  01010000 10000000 00000000 00000000
    268, //  00110000 10000000 00000000 00000000
    270, //  01110000 10000000 00000000 00000000
    272, //  00001000 10000000 00000000 00000000
    274, //  01001000 10000000 00000000 00000000
    276, //  00101000 10000000 00000000 00000000
    278, //  01101000 10000000 00000000 00000000
    280, //  00011000 10000000 00000000 00000000
    282, //  01011000 10000000 00000000 00000000
    284, //  00111000 10000000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    288, //  00000100 10000000 00000000 00000000
    290, //  01000100 10000000 00000000 00000000
    292, //  00100100 10000000 00000000 00000000
    294, //  01100100 10000000 00000000 00000000
    296, //  00010100 10000000 00000000 00000000
    298, //  01010100 10000000 00000000 00000000
    300, //  00110100 10000000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    304, //  00001100 10000000 00000000 00000000
    306, //  01001100 10000000 00000000 00000000
    308, //  00101100 10000000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    312, //  00011100 10000000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    320, //  00000010 10000000 00000000 00000000
    322, //  01000010 10000000 00000000 00000000
    324, //  00100010 10000000 00000000 00000000
    326, //  01100010 10000000 00000000 00000000
    328, //  00010010 10000000 00000000 00000000
    330, //  01010010 10000000 00000000 00000000
    332, //  00110010 10000000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    336, //  00001010 10000000 00000000 00000000
    338, //  01001010 10000000 00000000 00000000
    340, //  00101010 10000000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    344, //  00011010 10000000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    352, //  00000110 10000000 00000000 00000000
    354, //  01000110 10000000 00000000 00000000
    356, //  00100110 10000000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    360, //  00010110 10000000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    368, //  00001110 10000000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    384, //  00000001 10000000 00000000 00000000
    386, //  01000001 10000000 00000000 00000000
    388, //  00100001 10000000 00000000 00000000
    390, //  01100001 10000000 00000000 00000000
    392, //  00010001 10000000 00000000 00000000
    394, //  01010001 10000000 00000000 00000000
    396, //  00110001 10000000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    400, //  00001001 10000000 00000000 00000000
    402, //  01001001 10000000 00000000 00000000
    404, //  00101001 10000000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    408, //  00011001 10000000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    416, //  00000101 10000000 00000000 00000000
    418, //  01000101 10000000 00000000 00000000
    420, //  00100101 10000000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    424, //  00010101 10000000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    432, //  00001101 10000000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    448, //  00000011 10000000 00000000 00000000
    450, //  01000011 10000000 00000000 00000000
    452, //  00100011 10000000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    456, //  00010011 10000000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    464, //  00001011 10000000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    480, //  00000111 10000000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    512, //  00000000 01000000 00000000 00000000
    514, //  01000000 01000000 00000000 00000000
    516, //  00100000 01000000 00000000 00000000
    518, //  01100000 01000000 00000000 00000000
    520, //  00010000 01000000 00000000 00000000
    522, //  01010000 01000000 00000000 00000000
    524, //  00110000 01000000 00000000 00000000
    526, //  01110000 01000000 00000000 00000000
    528, //  00001000 01000000 00000000 00000000
    530, //  01001000 01000000 00000000 00000000
    532, //  00101000 01000000 00000000 00000000
    534, //  01101000 01000000 00000000 00000000
    536, //  00011000 01000000 00000000 00000000
    538, //  01011000 01000000 00000000 00000000
    540, //  00111000 01000000 00000000 00000000
    542, //  01111000 01000000 00000000 00000000
    544, //  00000100 01000000 00000000 00000000
    546, //  01000100 01000000 00000000 00000000
    548, //  00100100 01000000 00000000 00000000
    550, //  01100100 01000000 00000000 00000000
    552, //  00010100 01000000 00000000 00000000
    554, //  01010100 01000000 00000000 00000000
    556, //  00110100 01000000 00000000 00000000
    558, //  01110100 01000000 00000000 00000000
    560, //  00001100 01000000 00000000 00000000
    562, //  01001100 01000000 00000000 00000000
    564, //  00101100 01000000 00000000 00000000
    566, //  01101100 01000000 00000000 00000000
    568, //  00011100 01000000 00000000 00000000
    570, //  01011100 01000000 00000000 00000000
    572, //  00111100 01000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    576, //  00000010 01000000 00000000 00000000
    578, //  01000010 01000000 00000000 00000000
    580, //  00100010 01000000 00000000 00000000
    582, //  01100010 01000000 00000000 00000000
    584, //  00010010 01000000 00000000 00000000
    586, //  01010010 01000000 00000000 00000000
    588, //  00110010 01000000 00000000 00000000
    590, //  01110010 01000000 00000000 00000000
    592, //  00001010 01000000 00000000 00000000
    594, //  01001010 01000000 00000000 00000000
    596, //  00101010 01000000 00000000 00000000
    598, //  01101010 01000000 00000000 00000000
    600, //  00011010 01000000 00000000 00000000
    602, //  01011010 01000000 00000000 00000000
    604, //  00111010 01000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    608, //  00000110 01000000 00000000 00000000
    610, //  01000110 01000000 00000000 00000000
    612, //  00100110 01000000 00000000 00000000
    614, //  01100110 01000000 00000000 00000000
    616, //  00010110 01000000 00000000 00000000
    618, //  01010110 01000000 00000000 00000000
    620, //  00110110 01000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    624, //  00001110 01000000 00000000 00000000
    626, //  01001110 01000000 00000000 00000000
    628, //  00101110 01000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    632, //  00011110 01000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    640, //  00000001 01000000 00000000 00000000
    642, //  01000001 01000000 00000000 00000000
    644, //  00100001 01000000 00000000 00000000
    646, //  01100001 01000000 00000000 00000000
    648, //  00010001 01000000 00000000 00000000
    650, //  01010001 01000000 00000000 00000000
    652, //  00110001 01000000 00000000 00000000
    654, //  01110001 01000000 00000000 00000000
    656, //  00001001 01000000 00000000 00000000
    658, //  01001001 01000000 00000000 00000000
    660, //  00101001 01000000 00000000 00000000
    662, //  01101001 01000000 00000000 00000000
    664, //  00011001 01000000 00000000 00000000
    666, //  01011001 01000000 00000000 00000000
    668, //  00111001 01000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    672, //  00000101 01000000 00000000 00000000
    674, //  01000101 01000000 00000000 00000000
    676, //  00100101 01000000 00000000 00000000
    678, //  01100101 01000000 00000000 00000000
    680, //  00010101 01000000 00000000 00000000
    682, //  01010101 01000000 00000000 00000000
    684, //  00110101 01000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    688, //  00001101 01000000 00000000 00000000
    690, //  01001101 01000000 00000000 00000000
    692, //  00101101 01000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    696, //  00011101 01000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    704, //  00000011 01000000 00000000 00000000
    706, //  01000011 01000000 00000000 00000000
    708, //  00100011 01000000 00000000 00000000
    710, //  01100011 01000000 00000000 00000000
    712, //  00010011 01000000 00000000 00000000
    714, //  01010011 01000000 00000000 00000000
    716, //  00110011 01000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    720, //  00001011 01000000 00000000 00000000
    722, //  01001011 01000000 00000000 00000000
    724, //  00101011 01000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    728, //  00011011 01000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    736, //  00000111 01000000 00000000 00000000
    738, //  01000111 01000000 00000000 00000000
    740, //  00100111 01000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    744, //  00010111 01000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    752, //  00001111 01000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    768, //  00000000 11000000 00000000 00000000
    770, //  01000000 11000000 00000000 00000000
    772, //  00100000 11000000 00000000 00000000
    774, //  01100000 11000000 00000000 00000000
    776, //  00010000 11000000 00000000 00000000
    778, //  01010000 11000000 00000000 00000000
    780, //  00110000 11000000 00000000 00000000
    782, //  01110000 11000000 00000000 00000000
    784, //  00001000 11000000 00000000 00000000
    786, //  01001000 11000000 00000000 00000000
    788, //  00101000 11000000 00000000 00000000
    790, //  01101000 11000000 00000000 00000000
    792, //  00011000 11000000 00000000 00000000
    794, //  01011000 11000000 00000000 00000000
    796, //  00111000 11000000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    800, //  00000100 11000000 00000000 00000000
    802, //  01000100 11000000 00000000 00000000
    804, //  00100100 11000000 00000000 00000000
    806, //  01100100 11000000 00000000 00000000
    808, //  00010100 11000000 00000000 00000000
    810, //  01010100 11000000 00000000 00000000
    812, //  00110100 11000000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    816, //  00001100 11000000 00000000 00000000
    818, //  01001100 11000000 00000000 00000000
    820, //  00101100 11000000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    824, //  00011100 11000000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    832, //  00000010 11000000 00000000 00000000
    834, //  01000010 11000000 00000000 00000000
    836, //  00100010 11000000 00000000 00000000
    838, //  01100010 11000000 00000000 00000000
    840, //  00010010 11000000 00000000 00000000
    842, //  01010010 11000000 00000000 00000000
    844, //  00110010 11000000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    848, //  00001010 11000000 00000000 00000000
    850, //  01001010 11000000 00000000 00000000
    852, //  00101010 11000000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    856, //  00011010 11000000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    864, //  00000110 11000000 00000000 00000000
    866, //  01000110 11000000 00000000 00000000
    868, //  00100110 11000000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    872, //  00010110 11000000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    880, //  00001110 11000000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    896, //  00000001 11000000 00000000 00000000
    898, //  01000001 11000000 00000000 00000000
    900, //  00100001 11000000 00000000 00000000
    902, //  01100001 11000000 00000000 00000000
    904, //  00010001 11000000 00000000 00000000
    906, //  01010001 11000000 00000000 00000000
    908, //  00110001 11000000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    912, //  00001001 11000000 00000000 00000000
    914, //  01001001 11000000 00000000 00000000
    916, //  00101001 11000000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    920, //  00011001 11000000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    928, //  00000101 11000000 00000000 00000000
    930, //  01000101 11000000 00000000 00000000
    932, //  00100101 11000000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    936, //  00010101 11000000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    944, //  00001101 11000000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    960, //  00000011 11000000 00000000 00000000
    962, //  01000011 11000000 00000000 00000000
    964, //  00100011 11000000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    968, //  00010011 11000000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    976, //  00001011 11000000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    992, //  00000111 11000000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1024, //  00000000 00100000 00000000 00000000
    1026, //  01000000 00100000 00000000 00000000
    1028, //  00100000 00100000 00000000 00000000
    1030, //  01100000 00100000 00000000 00000000
    1032, //  00010000 00100000 00000000 00000000
    1034, //  01010000 00100000 00000000 00000000
    1036, //  00110000 00100000 00000000 00000000
    1038, //  01110000 00100000 00000000 00000000
    1040, //  00001000 00100000 00000000 00000000
    1042, //  01001000 00100000 00000000 00000000
    1044, //  00101000 00100000 00000000 00000000
    1046, //  01101000 00100000 00000000 00000000
    1048, //  00011000 00100000 00000000 00000000
    1050, //  01011000 00100000 00000000 00000000
    1052, //  00111000 00100000 00000000 00000000
    1054, //  01111000 00100000 00000000 00000000
    1056, //  00000100 00100000 00000000 00000000
    1058, //  01000100 00100000 00000000 00000000
    1060, //  00100100 00100000 00000000 00000000
    1062, //  01100100 00100000 00000000 00000000
    1064, //  00010100 00100000 00000000 00000000
    1066, //  01010100 00100000 00000000 00000000
    1068, //  00110100 00100000 00000000 00000000
    1070, //  01110100 00100000 00000000 00000000
    1072, //  00001100 00100000 00000000 00000000
    1074, //  01001100 00100000 00000000 00000000
    1076, //  00101100 00100000 00000000 00000000
    1078, //  01101100 00100000 00000000 00000000
    1080, //  00011100 00100000 00000000 00000000
    1082, //  01011100 00100000 00000000 00000000
    1084, //  00111100 00100000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1088, //  00000010 00100000 00000000 00000000
    1090, //  01000010 00100000 00000000 00000000
    1092, //  00100010 00100000 00000000 00000000
    1094, //  01100010 00100000 00000000 00000000
    1096, //  00010010 00100000 00000000 00000000
    1098, //  01010010 00100000 00000000 00000000
    1100, //  00110010 00100000 00000000 00000000
    1102, //  01110010 00100000 00000000 00000000
    1104, //  00001010 00100000 00000000 00000000
    1106, //  01001010 00100000 00000000 00000000
    1108, //  00101010 00100000 00000000 00000000
    1110, //  01101010 00100000 00000000 00000000
    1112, //  00011010 00100000 00000000 00000000
    1114, //  01011010 00100000 00000000 00000000
    1116, //  00111010 00100000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1120, //  00000110 00100000 00000000 00000000
    1122, //  01000110 00100000 00000000 00000000
    1124, //  00100110 00100000 00000000 00000000
    1126, //  01100110 00100000 00000000 00000000
    1128, //  00010110 00100000 00000000 00000000
    1130, //  01010110 00100000 00000000 00000000
    1132, //  00110110 00100000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1136, //  00001110 00100000 00000000 00000000
    1138, //  01001110 00100000 00000000 00000000
    1140, //  00101110 00100000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    1144, //  00011110 00100000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1152, //  00000001 00100000 00000000 00000000
    1154, //  01000001 00100000 00000000 00000000
    1156, //  00100001 00100000 00000000 00000000
    1158, //  01100001 00100000 00000000 00000000
    1160, //  00010001 00100000 00000000 00000000
    1162, //  01010001 00100000 00000000 00000000
    1164, //  00110001 00100000 00000000 00000000
    1166, //  01110001 00100000 00000000 00000000
    1168, //  00001001 00100000 00000000 00000000
    1170, //  01001001 00100000 00000000 00000000
    1172, //  00101001 00100000 00000000 00000000
    1174, //  01101001 00100000 00000000 00000000
    1176, //  00011001 00100000 00000000 00000000
    1178, //  01011001 00100000 00000000 00000000
    1180, //  00111001 00100000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    1184, //  00000101 00100000 00000000 00000000
    1186, //  01000101 00100000 00000000 00000000
    1188, //  00100101 00100000 00000000 00000000
    1190, //  01100101 00100000 00000000 00000000
    1192, //  00010101 00100000 00000000 00000000
    1194, //  01010101 00100000 00000000 00000000
    1196, //  00110101 00100000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    1200, //  00001101 00100000 00000000 00000000
    1202, //  01001101 00100000 00000000 00000000
    1204, //  00101101 00100000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    1208, //  00011101 00100000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1216, //  00000011 00100000 00000000 00000000
    1218, //  01000011 00100000 00000000 00000000
    1220, //  00100011 00100000 00000000 00000000
    1222, //  01100011 00100000 00000000 00000000
    1224, //  00010011 00100000 00000000 00000000
    1226, //  01010011 00100000 00000000 00000000
    1228, //  00110011 00100000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    1232, //  00001011 00100000 00000000 00000000
    1234, //  01001011 00100000 00000000 00000000
    1236, //  00101011 00100000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    1240, //  00011011 00100000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1248, //  00000111 00100000 00000000 00000000
    1250, //  01000111 00100000 00000000 00000000
    1252, //  00100111 00100000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    1256, //  00010111 00100000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1264, //  00001111 00100000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1280, //  00000000 10100000 00000000 00000000
    1282, //  01000000 10100000 00000000 00000000
    1284, //  00100000 10100000 00000000 00000000
    1286, //  01100000 10100000 00000000 00000000
    1288, //  00010000 10100000 00000000 00000000
    1290, //  01010000 10100000 00000000 00000000
    1292, //  00110000 10100000 00000000 00000000
    1294, //  01110000 10100000 00000000 00000000
    1296, //  00001000 10100000 00000000 00000000
    1298, //  01001000 10100000 00000000 00000000
    1300, //  00101000 10100000 00000000 00000000
    1302, //  01101000 10100000 00000000 00000000
    1304, //  00011000 10100000 00000000 00000000
    1306, //  01011000 10100000 00000000 00000000
    1308, //  00111000 10100000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    1312, //  00000100 10100000 00000000 00000000
    1314, //  01000100 10100000 00000000 00000000
    1316, //  00100100 10100000 00000000 00000000
    1318, //  01100100 10100000 00000000 00000000
    1320, //  00010100 10100000 00000000 00000000
    1322, //  01010100 10100000 00000000 00000000
    1324, //  00110100 10100000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    1328, //  00001100 10100000 00000000 00000000
    1330, //  01001100 10100000 00000000 00000000
    1332, //  00101100 10100000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    1336, //  00011100 10100000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1344, //  00000010 10100000 00000000 00000000
    1346, //  01000010 10100000 00000000 00000000
    1348, //  00100010 10100000 00000000 00000000
    1350, //  01100010 10100000 00000000 00000000
    1352, //  00010010 10100000 00000000 00000000
    1354, //  01010010 10100000 00000000 00000000
    1356, //  00110010 10100000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    1360, //  00001010 10100000 00000000 00000000
    1362, //  01001010 10100000 00000000 00000000
    1364, //  00101010 10100000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    1368, //  00011010 10100000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1376, //  00000110 10100000 00000000 00000000
    1378, //  01000110 10100000 00000000 00000000
    1380, //  00100110 10100000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    1384, //  00010110 10100000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1392, //  00001110 10100000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1408, //  00000001 10100000 00000000 00000000
    1410, //  01000001 10100000 00000000 00000000
    1412, //  00100001 10100000 00000000 00000000
    1414, //  01100001 10100000 00000000 00000000
    1416, //  00010001 10100000 00000000 00000000
    1418, //  01010001 10100000 00000000 00000000
    1420, //  00110001 10100000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    1424, //  00001001 10100000 00000000 00000000
    1426, //  01001001 10100000 00000000 00000000
    1428, //  00101001 10100000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    1432, //  00011001 10100000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    1440, //  00000101 10100000 00000000 00000000
    1442, //  01000101 10100000 00000000 00000000
    1444, //  00100101 10100000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    1448, //  00010101 10100000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    1456, //  00001101 10100000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1472, //  00000011 10100000 00000000 00000000
    1474, //  01000011 10100000 00000000 00000000
    1476, //  00100011 10100000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    1480, //  00010011 10100000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    1488, //  00001011 10100000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1504, //  00000111 10100000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1536, //  00000000 01100000 00000000 00000000
    1538, //  01000000 01100000 00000000 00000000
    1540, //  00100000 01100000 00000000 00000000
    1542, //  01100000 01100000 00000000 00000000
    1544, //  00010000 01100000 00000000 00000000
    1546, //  01010000 01100000 00000000 00000000
    1548, //  00110000 01100000 00000000 00000000
    1550, //  01110000 01100000 00000000 00000000
    1552, //  00001000 01100000 00000000 00000000
    1554, //  01001000 01100000 00000000 00000000
    1556, //  00101000 01100000 00000000 00000000
    1558, //  01101000 01100000 00000000 00000000
    1560, //  00011000 01100000 00000000 00000000
    1562, //  01011000 01100000 00000000 00000000
    1564, //  00111000 01100000 00000000 00000000
    542, //  01111000 01000000 00000000 00000000
    1568, //  00000100 01100000 00000000 00000000
    1570, //  01000100 01100000 00000000 00000000
    1572, //  00100100 01100000 00000000 00000000
    1574, //  01100100 01100000 00000000 00000000
    1576, //  00010100 01100000 00000000 00000000
    1578, //  01010100 01100000 00000000 00000000
    1580, //  00110100 01100000 00000000 00000000
    558, //  01110100 01000000 00000000 00000000
    1584, //  00001100 01100000 00000000 00000000
    1586, //  01001100 01100000 00000000 00000000
    1588, //  00101100 01100000 00000000 00000000
    566, //  01101100 01000000 00000000 00000000
    1592, //  00011100 01100000 00000000 00000000
    570, //  01011100 01000000 00000000 00000000
    572, //  00111100 01000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1600, //  00000010 01100000 00000000 00000000
    1602, //  01000010 01100000 00000000 00000000
    1604, //  00100010 01100000 00000000 00000000
    1606, //  01100010 01100000 00000000 00000000
    1608, //  00010010 01100000 00000000 00000000
    1610, //  01010010 01100000 00000000 00000000
    1612, //  00110010 01100000 00000000 00000000
    590, //  01110010 01000000 00000000 00000000
    1616, //  00001010 01100000 00000000 00000000
    1618, //  01001010 01100000 00000000 00000000
    1620, //  00101010 01100000 00000000 00000000
    598, //  01101010 01000000 00000000 00000000
    1624, //  00011010 01100000 00000000 00000000
    602, //  01011010 01000000 00000000 00000000
    604, //  00111010 01000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1632, //  00000110 01100000 00000000 00000000
    1634, //  01000110 01100000 00000000 00000000
    1636, //  00100110 01100000 00000000 00000000
    614, //  01100110 01000000 00000000 00000000
    1640, //  00010110 01100000 00000000 00000000
    618, //  01010110 01000000 00000000 00000000
    620, //  00110110 01000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1648, //  00001110 01100000 00000000 00000000
    626, //  01001110 01000000 00000000 00000000
    628, //  00101110 01000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    632, //  00011110 01000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1664, //  00000001 01100000 00000000 00000000
    1666, //  01000001 01100000 00000000 00000000
    1668, //  00100001 01100000 00000000 00000000
    1670, //  01100001 01100000 00000000 00000000
    1672, //  00010001 01100000 00000000 00000000
    1674, //  01010001 01100000 00000000 00000000
    1676, //  00110001 01100000 00000000 00000000
    654, //  01110001 01000000 00000000 00000000
    1680, //  00001001 01100000 00000000 00000000
    1682, //  01001001 01100000 00000000 00000000
    1684, //  00101001 01100000 00000000 00000000
    662, //  01101001 01000000 00000000 00000000
    1688, //  00011001 01100000 00000000 00000000
    666, //  01011001 01000000 00000000 00000000
    668, //  00111001 01000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    1696, //  00000101 01100000 00000000 00000000
    1698, //  01000101 01100000 00000000 00000000
    1700, //  00100101 01100000 00000000 00000000
    678, //  01100101 01000000 00000000 00000000
    1704, //  00010101 01100000 00000000 00000000
    682, //  01010101 01000000 00000000 00000000
    684, //  00110101 01000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    1712, //  00001101 01100000 00000000 00000000
    690, //  01001101 01000000 00000000 00000000
    692, //  00101101 01000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    696, //  00011101 01000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1728, //  00000011 01100000 00000000 00000000
    1730, //  01000011 01100000 00000000 00000000
    1732, //  00100011 01100000 00000000 00000000
    710, //  01100011 01000000 00000000 00000000
    1736, //  00010011 01100000 00000000 00000000
    714, //  01010011 01000000 00000000 00000000
    716, //  00110011 01000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    1744, //  00001011 01100000 00000000 00000000
    722, //  01001011 01000000 00000000 00000000
    724, //  00101011 01000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    728, //  00011011 01000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1760, //  00000111 01100000 00000000 00000000
    738, //  01000111 01000000 00000000 00000000
    740, //  00100111 01000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    744, //  00010111 01000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    752, //  00001111 01000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1792, //  00000000 11100000 00000000 00000000
    1794, //  01000000 11100000 00000000 00000000
    1796, //  00100000 11100000 00000000 00000000
    1798, //  01100000 11100000 00000000 00000000
    1800, //  00010000 11100000 00000000 00000000
    1802, //  01010000 11100000 00000000 00000000
    1804, //  00110000 11100000 00000000 00000000
    782, //  01110000 11000000 00000000 00000000
    1808, //  00001000 11100000 00000000 00000000
    1810, //  01001000 11100000 00000000 00000000
    1812, //  00101000 11100000 00000000 00000000
    790, //  01101000 11000000 00000000 00000000
    1816, //  00011000 11100000 00000000 00000000
    794, //  01011000 11000000 00000000 00000000
    796, //  00111000 11000000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    1824, //  00000100 11100000 00000000 00000000
    1826, //  01000100 11100000 00000000 00000000
    1828, //  00100100 11100000 00000000 00000000
    806, //  01100100 11000000 00000000 00000000
    1832, //  00010100 11100000 00000000 00000000
    810, //  01010100 11000000 00000000 00000000
    812, //  00110100 11000000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    1840, //  00001100 11100000 00000000 00000000
    818, //  01001100 11000000 00000000 00000000
    820, //  00101100 11000000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    824, //  00011100 11000000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1856, //  00000010 11100000 00000000 00000000
    1858, //  01000010 11100000 00000000 00000000
    1860, //  00100010 11100000 00000000 00000000
    838, //  01100010 11000000 00000000 00000000
    1864, //  00010010 11100000 00000000 00000000
    842, //  01010010 11000000 00000000 00000000
    844, //  00110010 11000000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    1872, //  00001010 11100000 00000000 00000000
    850, //  01001010 11000000 00000000 00000000
    852, //  00101010 11000000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    856, //  00011010 11000000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1888, //  00000110 11100000 00000000 00000000
    866, //  01000110 11000000 00000000 00000000
    868, //  00100110 11000000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    872, //  00010110 11000000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    880, //  00001110 11000000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1920, //  00000001 11100000 00000000 00000000
    1922, //  01000001 11100000 00000000 00000000
    1924, //  00100001 11100000 00000000 00000000
    902, //  01100001 11000000 00000000 00000000
    1928, //  00010001 11100000 00000000 00000000
    906, //  01010001 11000000 00000000 00000000
    908, //  00110001 11000000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    1936, //  00001001 11100000 00000000 00000000
    914, //  01001001 11000000 00000000 00000000
    916, //  00101001 11000000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    920, //  00011001 11000000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    1952, //  00000101 11100000 00000000 00000000
    930, //  01000101 11000000 00000000 00000000
    932, //  00100101 11000000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    936, //  00010101 11000000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    944, //  00001101 11000000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1984, //  00000011 11100000 00000000 00000000
    962, //  01000011 11000000 00000000 00000000
    964, //  00100011 11000000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    968, //  00010011 11000000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    976, //  00001011 11000000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    992, //  00000111 11000000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2048, //  00000000 00010000 00000000 00000000
    2050, //  01000000 00010000 00000000 00000000
    2052, //  00100000 00010000 00000000 00000000
    2054, //  01100000 00010000 00000000 00000000
    2056, //  00010000 00010000 00000000 00000000
    2058, //  01010000 00010000 00000000 00000000
    2060, //  00110000 00010000 00000000 00000000
    2062, //  01110000 00010000 00000000 00000000
    2064, //  00001000 00010000 00000000 00000000
    2066, //  01001000 00010000 00000000 00000000
    2068, //  00101000 00010000 00000000 00000000
    2070, //  01101000 00010000 00000000 00000000
    2072, //  00011000 00010000 00000000 00000000
    2074, //  01011000 00010000 00000000 00000000
    2076, //  00111000 00010000 00000000 00000000
    2078, //  01111000 00010000 00000000 00000000
    2080, //  00000100 00010000 00000000 00000000
    2082, //  01000100 00010000 00000000 00000000
    2084, //  00100100 00010000 00000000 00000000
    2086, //  01100100 00010000 00000000 00000000
    2088, //  00010100 00010000 00000000 00000000
    2090, //  01010100 00010000 00000000 00000000
    2092, //  00110100 00010000 00000000 00000000
    2094, //  01110100 00010000 00000000 00000000
    2096, //  00001100 00010000 00000000 00000000
    2098, //  01001100 00010000 00000000 00000000
    2100, //  00101100 00010000 00000000 00000000
    2102, //  01101100 00010000 00000000 00000000
    2104, //  00011100 00010000 00000000 00000000
    2106, //  01011100 00010000 00000000 00000000
    2108, //  00111100 00010000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2112, //  00000010 00010000 00000000 00000000
    2114, //  01000010 00010000 00000000 00000000
    2116, //  00100010 00010000 00000000 00000000
    2118, //  01100010 00010000 00000000 00000000
    2120, //  00010010 00010000 00000000 00000000
    2122, //  01010010 00010000 00000000 00000000
    2124, //  00110010 00010000 00000000 00000000
    2126, //  01110010 00010000 00000000 00000000
    2128, //  00001010 00010000 00000000 00000000
    2130, //  01001010 00010000 00000000 00000000
    2132, //  00101010 00010000 00000000 00000000
    2134, //  01101010 00010000 00000000 00000000
    2136, //  00011010 00010000 00000000 00000000
    2138, //  01011010 00010000 00000000 00000000
    2140, //  00111010 00010000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    2144, //  00000110 00010000 00000000 00000000
    2146, //  01000110 00010000 00000000 00000000
    2148, //  00100110 00010000 00000000 00000000
    2150, //  01100110 00010000 00000000 00000000
    2152, //  00010110 00010000 00000000 00000000
    2154, //  01010110 00010000 00000000 00000000
    2156, //  00110110 00010000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    2160, //  00001110 00010000 00000000 00000000
    2162, //  01001110 00010000 00000000 00000000
    2164, //  00101110 00010000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    2168, //  00011110 00010000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2176, //  00000001 00010000 00000000 00000000
    2178, //  01000001 00010000 00000000 00000000
    2180, //  00100001 00010000 00000000 00000000
    2182, //  01100001 00010000 00000000 00000000
    2184, //  00010001 00010000 00000000 00000000
    2186, //  01010001 00010000 00000000 00000000
    2188, //  00110001 00010000 00000000 00000000
    2190, //  01110001 00010000 00000000 00000000
    2192, //  00001001 00010000 00000000 00000000
    2194, //  01001001 00010000 00000000 00000000
    2196, //  00101001 00010000 00000000 00000000
    2198, //  01101001 00010000 00000000 00000000
    2200, //  00011001 00010000 00000000 00000000
    2202, //  01011001 00010000 00000000 00000000
    2204, //  00111001 00010000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    2208, //  00000101 00010000 00000000 00000000
    2210, //  01000101 00010000 00000000 00000000
    2212, //  00100101 00010000 00000000 00000000
    2214, //  01100101 00010000 00000000 00000000
    2216, //  00010101 00010000 00000000 00000000
    2218, //  01010101 00010000 00000000 00000000
    2220, //  00110101 00010000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    2224, //  00001101 00010000 00000000 00000000
    2226, //  01001101 00010000 00000000 00000000
    2228, //  00101101 00010000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    2232, //  00011101 00010000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2240, //  00000011 00010000 00000000 00000000
    2242, //  01000011 00010000 00000000 00000000
    2244, //  00100011 00010000 00000000 00000000
    2246, //  01100011 00010000 00000000 00000000
    2248, //  00010011 00010000 00000000 00000000
    2250, //  01010011 00010000 00000000 00000000
    2252, //  00110011 00010000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    2256, //  00001011 00010000 00000000 00000000
    2258, //  01001011 00010000 00000000 00000000
    2260, //  00101011 00010000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    2264, //  00011011 00010000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    2272, //  00000111 00010000 00000000 00000000
    2274, //  01000111 00010000 00000000 00000000
    2276, //  00100111 00010000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    2280, //  00010111 00010000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    2288, //  00001111 00010000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2304, //  00000000 10010000 00000000 00000000
    2306, //  01000000 10010000 00000000 00000000
    2308, //  00100000 10010000 00000000 00000000
    2310, //  01100000 10010000 00000000 00000000
    2312, //  00010000 10010000 00000000 00000000
    2314, //  01010000 10010000 00000000 00000000
    2316, //  00110000 10010000 00000000 00000000
    2318, //  01110000 10010000 00000000 00000000
    2320, //  00001000 10010000 00000000 00000000
    2322, //  01001000 10010000 00000000 00000000
    2324, //  00101000 10010000 00000000 00000000
    2326, //  01101000 10010000 00000000 00000000
    2328, //  00011000 10010000 00000000 00000000
    2330, //  01011000 10010000 00000000 00000000
    2332, //  00111000 10010000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    2336, //  00000100 10010000 00000000 00000000
    2338, //  01000100 10010000 00000000 00000000
    2340, //  00100100 10010000 00000000 00000000
    2342, //  01100100 10010000 00000000 00000000
    2344, //  00010100 10010000 00000000 00000000
    2346, //  01010100 10010000 00000000 00000000
    2348, //  00110100 10010000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    2352, //  00001100 10010000 00000000 00000000
    2354, //  01001100 10010000 00000000 00000000
    2356, //  00101100 10010000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    2360, //  00011100 10010000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2368, //  00000010 10010000 00000000 00000000
    2370, //  01000010 10010000 00000000 00000000
    2372, //  00100010 10010000 00000000 00000000
    2374, //  01100010 10010000 00000000 00000000
    2376, //  00010010 10010000 00000000 00000000
    2378, //  01010010 10010000 00000000 00000000
    2380, //  00110010 10010000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    2384, //  00001010 10010000 00000000 00000000
    2386, //  01001010 10010000 00000000 00000000
    2388, //  00101010 10010000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    2392, //  00011010 10010000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    2400, //  00000110 10010000 00000000 00000000
    2402, //  01000110 10010000 00000000 00000000
    2404, //  00100110 10010000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    2408, //  00010110 10010000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    2416, //  00001110 10010000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2432, //  00000001 10010000 00000000 00000000
    2434, //  01000001 10010000 00000000 00000000
    2436, //  00100001 10010000 00000000 00000000
    2438, //  01100001 10010000 00000000 00000000
    2440, //  00010001 10010000 00000000 00000000
    2442, //  01010001 10010000 00000000 00000000
    2444, //  00110001 10010000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    2448, //  00001001 10010000 00000000 00000000
    2450, //  01001001 10010000 00000000 00000000
    2452, //  00101001 10010000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    2456, //  00011001 10010000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    2464, //  00000101 10010000 00000000 00000000
    2466, //  01000101 10010000 00000000 00000000
    2468, //  00100101 10010000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    2472, //  00010101 10010000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    2480, //  00001101 10010000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2496, //  00000011 10010000 00000000 00000000
    2498, //  01000011 10010000 00000000 00000000
    2500, //  00100011 10010000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    2504, //  00010011 10010000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    2512, //  00001011 10010000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    2528, //  00000111 10010000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2560, //  00000000 01010000 00000000 00000000
    2562, //  01000000 01010000 00000000 00000000
    2564, //  00100000 01010000 00000000 00000000
    2566, //  01100000 01010000 00000000 00000000
    2568, //  00010000 01010000 00000000 00000000
    2570, //  01010000 01010000 00000000 00000000
    2572, //  00110000 01010000 00000000 00000000
    2574, //  01110000 01010000 00000000 00000000
    2576, //  00001000 01010000 00000000 00000000
    2578, //  01001000 01010000 00000000 00000000
    2580, //  00101000 01010000 00000000 00000000
    2582, //  01101000 01010000 00000000 00000000
    2584, //  00011000 01010000 00000000 00000000
    2586, //  01011000 01010000 00000000 00000000
    2588, //  00111000 01010000 00000000 00000000
    542, //  01111000 01000000 00000000 00000000
    2592, //  00000100 01010000 00000000 00000000
    2594, //  01000100 01010000 00000000 00000000
    2596, //  00100100 01010000 00000000 00000000
    2598, //  01100100 01010000 00000000 00000000
    2600, //  00010100 01010000 00000000 00000000
    2602, //  01010100 01010000 00000000 00000000
    2604, //  00110100 01010000 00000000 00000000
    558, //  01110100 01000000 00000000 00000000
    2608, //  00001100 01010000 00000000 00000000
    2610, //  01001100 01010000 00000000 00000000
    2612, //  00101100 01010000 00000000 00000000
    566, //  01101100 01000000 00000000 00000000
    2616, //  00011100 01010000 00000000 00000000
    570, //  01011100 01000000 00000000 00000000
    572, //  00111100 01000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2624, //  00000010 01010000 00000000 00000000
    2626, //  01000010 01010000 00000000 00000000
    2628, //  00100010 01010000 00000000 00000000
    2630, //  01100010 01010000 00000000 00000000
    2632, //  00010010 01010000 00000000 00000000
    2634, //  01010010 01010000 00000000 00000000
    2636, //  00110010 01010000 00000000 00000000
    590, //  01110010 01000000 00000000 00000000
    2640, //  00001010 01010000 00000000 00000000
    2642, //  01001010 01010000 00000000 00000000
    2644, //  00101010 01010000 00000000 00000000
    598, //  01101010 01000000 00000000 00000000
    2648, //  00011010 01010000 00000000 00000000
    602, //  01011010 01000000 00000000 00000000
    604, //  00111010 01000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    2656, //  00000110 01010000 00000000 00000000
    2658, //  01000110 01010000 00000000 00000000
    2660, //  00100110 01010000 00000000 00000000
    614, //  01100110 01000000 00000000 00000000
    2664, //  00010110 01010000 00000000 00000000
    618, //  01010110 01000000 00000000 00000000
    620, //  00110110 01000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    2672, //  00001110 01010000 00000000 00000000
    626, //  01001110 01000000 00000000 00000000
    628, //  00101110 01000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    632, //  00011110 01000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2688, //  00000001 01010000 00000000 00000000
    2690, //  01000001 01010000 00000000 00000000
    2692, //  00100001 01010000 00000000 00000000
    2694, //  01100001 01010000 00000000 00000000
    2696, //  00010001 01010000 00000000 00000000
    2698, //  01010001 01010000 00000000 00000000
    2700, //  00110001 01010000 00000000 00000000
    654, //  01110001 01000000 00000000 00000000
    2704, //  00001001 01010000 00000000 00000000
    2706, //  01001001 01010000 00000000 00000000
    2708, //  00101001 01010000 00000000 00000000
    662, //  01101001 01000000 00000000 00000000
    2712, //  00011001 01010000 00000000 00000000
    666, //  01011001 01000000 00000000 00000000
    668, //  00111001 01000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    2720, //  00000101 01010000 00000000 00000000
    2722, //  01000101 01010000 00000000 00000000
    2724, //  00100101 01010000 00000000 00000000
    678, //  01100101 01000000 00000000 00000000
    2728, //  00010101 01010000 00000000 00000000
    682, //  01010101 01000000 00000000 00000000
    684, //  00110101 01000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    2736, //  00001101 01010000 00000000 00000000
    690, //  01001101 01000000 00000000 00000000
    692, //  00101101 01000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    696, //  00011101 01000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2752, //  00000011 01010000 00000000 00000000
    2754, //  01000011 01010000 00000000 00000000
    2756, //  00100011 01010000 00000000 00000000
    710, //  01100011 01000000 00000000 00000000
    2760, //  00010011 01010000 00000000 00000000
    714, //  01010011 01000000 00000000 00000000
    716, //  00110011 01000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    2768, //  00001011 01010000 00000000 00000000
    722, //  01001011 01000000 00000000 00000000
    724, //  00101011 01000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    728, //  00011011 01000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    2784, //  00000111 01010000 00000000 00000000
    738, //  01000111 01000000 00000000 00000000
    740, //  00100111 01000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    744, //  00010111 01000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    752, //  00001111 01000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2816, //  00000000 11010000 00000000 00000000
    2818, //  01000000 11010000 00000000 00000000
    2820, //  00100000 11010000 00000000 00000000
    2822, //  01100000 11010000 00000000 00000000
    2824, //  00010000 11010000 00000000 00000000
    2826, //  01010000 11010000 00000000 00000000
    2828, //  00110000 11010000 00000000 00000000
    782, //  01110000 11000000 00000000 00000000
    2832, //  00001000 11010000 00000000 00000000
    2834, //  01001000 11010000 00000000 00000000
    2836, //  00101000 11010000 00000000 00000000
    790, //  01101000 11000000 00000000 00000000
    2840, //  00011000 11010000 00000000 00000000
    794, //  01011000 11000000 00000000 00000000
    796, //  00111000 11000000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    2848, //  00000100 11010000 00000000 00000000
    2850, //  01000100 11010000 00000000 00000000
    2852, //  00100100 11010000 00000000 00000000
    806, //  01100100 11000000 00000000 00000000
    2856, //  00010100 11010000 00000000 00000000
    810, //  01010100 11000000 00000000 00000000
    812, //  00110100 11000000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    2864, //  00001100 11010000 00000000 00000000
    818, //  01001100 11000000 00000000 00000000
    820, //  00101100 11000000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    824, //  00011100 11000000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2880, //  00000010 11010000 00000000 00000000
    2882, //  01000010 11010000 00000000 00000000
    2884, //  00100010 11010000 00000000 00000000
    838, //  01100010 11000000 00000000 00000000
    2888, //  00010010 11010000 00000000 00000000
    842, //  01010010 11000000 00000000 00000000
    844, //  00110010 11000000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    2896, //  00001010 11010000 00000000 00000000
    850, //  01001010 11000000 00000000 00000000
    852, //  00101010 11000000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    856, //  00011010 11000000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    2912, //  00000110 11010000 00000000 00000000
    866, //  01000110 11000000 00000000 00000000
    868, //  00100110 11000000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    872, //  00010110 11000000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    880, //  00001110 11000000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    2944, //  00000001 11010000 00000000 00000000
    2946, //  01000001 11010000 00000000 00000000
    2948, //  00100001 11010000 00000000 00000000
    902, //  01100001 11000000 00000000 00000000
    2952, //  00010001 11010000 00000000 00000000
    906, //  01010001 11000000 00000000 00000000
    908, //  00110001 11000000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    2960, //  00001001 11010000 00000000 00000000
    914, //  01001001 11000000 00000000 00000000
    916, //  00101001 11000000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    920, //  00011001 11000000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    2976, //  00000101 11010000 00000000 00000000
    930, //  01000101 11000000 00000000 00000000
    932, //  00100101 11000000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    936, //  00010101 11000000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    944, //  00001101 11000000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3008, //  00000011 11010000 00000000 00000000
    962, //  01000011 11000000 00000000 00000000
    964, //  00100011 11000000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    968, //  00010011 11000000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    976, //  00001011 11000000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    992, //  00000111 11000000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3072, //  00000000 00110000 00000000 00000000
    3074, //  01000000 00110000 00000000 00000000
    3076, //  00100000 00110000 00000000 00000000
    3078, //  01100000 00110000 00000000 00000000
    3080, //  00010000 00110000 00000000 00000000
    3082, //  01010000 00110000 00000000 00000000
    3084, //  00110000 00110000 00000000 00000000
    3086, //  01110000 00110000 00000000 00000000
    3088, //  00001000 00110000 00000000 00000000
    3090, //  01001000 00110000 00000000 00000000
    3092, //  00101000 00110000 00000000 00000000
    3094, //  01101000 00110000 00000000 00000000
    3096, //  00011000 00110000 00000000 00000000
    3098, //  01011000 00110000 00000000 00000000
    3100, //  00111000 00110000 00000000 00000000
    1054, //  01111000 00100000 00000000 00000000
    3104, //  00000100 00110000 00000000 00000000
    3106, //  01000100 00110000 00000000 00000000
    3108, //  00100100 00110000 00000000 00000000
    3110, //  01100100 00110000 00000000 00000000
    3112, //  00010100 00110000 00000000 00000000
    3114, //  01010100 00110000 00000000 00000000
    3116, //  00110100 00110000 00000000 00000000
    1070, //  01110100 00100000 00000000 00000000
    3120, //  00001100 00110000 00000000 00000000
    3122, //  01001100 00110000 00000000 00000000
    3124, //  00101100 00110000 00000000 00000000
    1078, //  01101100 00100000 00000000 00000000
    3128, //  00011100 00110000 00000000 00000000
    1082, //  01011100 00100000 00000000 00000000
    1084, //  00111100 00100000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3136, //  00000010 00110000 00000000 00000000
    3138, //  01000010 00110000 00000000 00000000
    3140, //  00100010 00110000 00000000 00000000
    3142, //  01100010 00110000 00000000 00000000
    3144, //  00010010 00110000 00000000 00000000
    3146, //  01010010 00110000 00000000 00000000
    3148, //  00110010 00110000 00000000 00000000
    1102, //  01110010 00100000 00000000 00000000
    3152, //  00001010 00110000 00000000 00000000
    3154, //  01001010 00110000 00000000 00000000
    3156, //  00101010 00110000 00000000 00000000
    1110, //  01101010 00100000 00000000 00000000
    3160, //  00011010 00110000 00000000 00000000
    1114, //  01011010 00100000 00000000 00000000
    1116, //  00111010 00100000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    3168, //  00000110 00110000 00000000 00000000
    3170, //  01000110 00110000 00000000 00000000
    3172, //  00100110 00110000 00000000 00000000
    1126, //  01100110 00100000 00000000 00000000
    3176, //  00010110 00110000 00000000 00000000
    1130, //  01010110 00100000 00000000 00000000
    1132, //  00110110 00100000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    3184, //  00001110 00110000 00000000 00000000
    1138, //  01001110 00100000 00000000 00000000
    1140, //  00101110 00100000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    1144, //  00011110 00100000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3200, //  00000001 00110000 00000000 00000000
    3202, //  01000001 00110000 00000000 00000000
    3204, //  00100001 00110000 00000000 00000000
    3206, //  01100001 00110000 00000000 00000000
    3208, //  00010001 00110000 00000000 00000000
    3210, //  01010001 00110000 00000000 00000000
    3212, //  00110001 00110000 00000000 00000000
    1166, //  01110001 00100000 00000000 00000000
    3216, //  00001001 00110000 00000000 00000000
    3218, //  01001001 00110000 00000000 00000000
    3220, //  00101001 00110000 00000000 00000000
    1174, //  01101001 00100000 00000000 00000000
    3224, //  00011001 00110000 00000000 00000000
    1178, //  01011001 00100000 00000000 00000000
    1180, //  00111001 00100000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    3232, //  00000101 00110000 00000000 00000000
    3234, //  01000101 00110000 00000000 00000000
    3236, //  00100101 00110000 00000000 00000000
    1190, //  01100101 00100000 00000000 00000000
    3240, //  00010101 00110000 00000000 00000000
    1194, //  01010101 00100000 00000000 00000000
    1196, //  00110101 00100000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    3248, //  00001101 00110000 00000000 00000000
    1202, //  01001101 00100000 00000000 00000000
    1204, //  00101101 00100000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    1208, //  00011101 00100000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3264, //  00000011 00110000 00000000 00000000
    3266, //  01000011 00110000 00000000 00000000
    3268, //  00100011 00110000 00000000 00000000
    1222, //  01100011 00100000 00000000 00000000
    3272, //  00010011 00110000 00000000 00000000
    1226, //  01010011 00100000 00000000 00000000
    1228, //  00110011 00100000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    3280, //  00001011 00110000 00000000 00000000
    1234, //  01001011 00100000 00000000 00000000
    1236, //  00101011 00100000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    1240, //  00011011 00100000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    3296, //  00000111 00110000 00000000 00000000
    1250, //  01000111 00100000 00000000 00000000
    1252, //  00100111 00100000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    1256, //  00010111 00100000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1264, //  00001111 00100000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3328, //  00000000 10110000 00000000 00000000
    3330, //  01000000 10110000 00000000 00000000
    3332, //  00100000 10110000 00000000 00000000
    3334, //  01100000 10110000 00000000 00000000
    3336, //  00010000 10110000 00000000 00000000
    3338, //  01010000 10110000 00000000 00000000
    3340, //  00110000 10110000 00000000 00000000
    1294, //  01110000 10100000 00000000 00000000
    3344, //  00001000 10110000 00000000 00000000
    3346, //  01001000 10110000 00000000 00000000
    3348, //  00101000 10110000 00000000 00000000
    1302, //  01101000 10100000 00000000 00000000
    3352, //  00011000 10110000 00000000 00000000
    1306, //  01011000 10100000 00000000 00000000
    1308, //  00111000 10100000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    3360, //  00000100 10110000 00000000 00000000
    3362, //  01000100 10110000 00000000 00000000
    3364, //  00100100 10110000 00000000 00000000
    1318, //  01100100 10100000 00000000 00000000
    3368, //  00010100 10110000 00000000 00000000
    1322, //  01010100 10100000 00000000 00000000
    1324, //  00110100 10100000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    3376, //  00001100 10110000 00000000 00000000
    1330, //  01001100 10100000 00000000 00000000
    1332, //  00101100 10100000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    1336, //  00011100 10100000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3392, //  00000010 10110000 00000000 00000000
    3394, //  01000010 10110000 00000000 00000000
    3396, //  00100010 10110000 00000000 00000000
    1350, //  01100010 10100000 00000000 00000000
    3400, //  00010010 10110000 00000000 00000000
    1354, //  01010010 10100000 00000000 00000000
    1356, //  00110010 10100000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    3408, //  00001010 10110000 00000000 00000000
    1362, //  01001010 10100000 00000000 00000000
    1364, //  00101010 10100000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    1368, //  00011010 10100000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    3424, //  00000110 10110000 00000000 00000000
    1378, //  01000110 10100000 00000000 00000000
    1380, //  00100110 10100000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    1384, //  00010110 10100000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1392, //  00001110 10100000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3456, //  00000001 10110000 00000000 00000000
    3458, //  01000001 10110000 00000000 00000000
    3460, //  00100001 10110000 00000000 00000000
    1414, //  01100001 10100000 00000000 00000000
    3464, //  00010001 10110000 00000000 00000000
    1418, //  01010001 10100000 00000000 00000000
    1420, //  00110001 10100000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    3472, //  00001001 10110000 00000000 00000000
    1426, //  01001001 10100000 00000000 00000000
    1428, //  00101001 10100000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    1432, //  00011001 10100000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    3488, //  00000101 10110000 00000000 00000000
    1442, //  01000101 10100000 00000000 00000000
    1444, //  00100101 10100000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    1448, //  00010101 10100000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    1456, //  00001101 10100000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3520, //  00000011 10110000 00000000 00000000
    1474, //  01000011 10100000 00000000 00000000
    1476, //  00100011 10100000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    1480, //  00010011 10100000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    1488, //  00001011 10100000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1504, //  00000111 10100000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3584, //  00000000 01110000 00000000 00000000
    3586, //  01000000 01110000 00000000 00000000
    3588, //  00100000 01110000 00000000 00000000
    3590, //  01100000 01110000 00000000 00000000
    3592, //  00010000 01110000 00000000 00000000
    3594, //  01010000 01110000 00000000 00000000
    3596, //  00110000 01110000 00000000 00000000
    1550, //  01110000 01100000 00000000 00000000
    3600, //  00001000 01110000 00000000 00000000
    3602, //  01001000 01110000 00000000 00000000
    3604, //  00101000 01110000 00000000 00000000
    1558, //  01101000 01100000 00000000 00000000
    3608, //  00011000 01110000 00000000 00000000
    1562, //  01011000 01100000 00000000 00000000
    1564, //  00111000 01100000 00000000 00000000
    542, //  01111000 01000000 00000000 00000000
    3616, //  00000100 01110000 00000000 00000000
    3618, //  01000100 01110000 00000000 00000000
    3620, //  00100100 01110000 00000000 00000000
    1574, //  01100100 01100000 00000000 00000000
    3624, //  00010100 01110000 00000000 00000000
    1578, //  01010100 01100000 00000000 00000000
    1580, //  00110100 01100000 00000000 00000000
    558, //  01110100 01000000 00000000 00000000
    3632, //  00001100 01110000 00000000 00000000
    1586, //  01001100 01100000 00000000 00000000
    1588, //  00101100 01100000 00000000 00000000
    566, //  01101100 01000000 00000000 00000000
    1592, //  00011100 01100000 00000000 00000000
    570, //  01011100 01000000 00000000 00000000
    572, //  00111100 01000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3648, //  00000010 01110000 00000000 00000000
    3650, //  01000010 01110000 00000000 00000000
    3652, //  00100010 01110000 00000000 00000000
    1606, //  01100010 01100000 00000000 00000000
    3656, //  00010010 01110000 00000000 00000000
    1610, //  01010010 01100000 00000000 00000000
    1612, //  00110010 01100000 00000000 00000000
    590, //  01110010 01000000 00000000 00000000
    3664, //  00001010 01110000 00000000 00000000
    1618, //  01001010 01100000 00000000 00000000
    1620, //  00101010 01100000 00000000 00000000
    598, //  01101010 01000000 00000000 00000000
    1624, //  00011010 01100000 00000000 00000000
    602, //  01011010 01000000 00000000 00000000
    604, //  00111010 01000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    3680, //  00000110 01110000 00000000 00000000
    1634, //  01000110 01100000 00000000 00000000
    1636, //  00100110 01100000 00000000 00000000
    614, //  01100110 01000000 00000000 00000000
    1640, //  00010110 01100000 00000000 00000000
    618, //  01010110 01000000 00000000 00000000
    620, //  00110110 01000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1648, //  00001110 01100000 00000000 00000000
    626, //  01001110 01000000 00000000 00000000
    628, //  00101110 01000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    632, //  00011110 01000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3712, //  00000001 01110000 00000000 00000000
    3714, //  01000001 01110000 00000000 00000000
    3716, //  00100001 01110000 00000000 00000000
    1670, //  01100001 01100000 00000000 00000000
    3720, //  00010001 01110000 00000000 00000000
    1674, //  01010001 01100000 00000000 00000000
    1676, //  00110001 01100000 00000000 00000000
    654, //  01110001 01000000 00000000 00000000
    3728, //  00001001 01110000 00000000 00000000
    1682, //  01001001 01100000 00000000 00000000
    1684, //  00101001 01100000 00000000 00000000
    662, //  01101001 01000000 00000000 00000000
    1688, //  00011001 01100000 00000000 00000000
    666, //  01011001 01000000 00000000 00000000
    668, //  00111001 01000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    3744, //  00000101 01110000 00000000 00000000
    1698, //  01000101 01100000 00000000 00000000
    1700, //  00100101 01100000 00000000 00000000
    678, //  01100101 01000000 00000000 00000000
    1704, //  00010101 01100000 00000000 00000000
    682, //  01010101 01000000 00000000 00000000
    684, //  00110101 01000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    1712, //  00001101 01100000 00000000 00000000
    690, //  01001101 01000000 00000000 00000000
    692, //  00101101 01000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    696, //  00011101 01000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3776, //  00000011 01110000 00000000 00000000
    1730, //  01000011 01100000 00000000 00000000
    1732, //  00100011 01100000 00000000 00000000
    710, //  01100011 01000000 00000000 00000000
    1736, //  00010011 01100000 00000000 00000000
    714, //  01010011 01000000 00000000 00000000
    716, //  00110011 01000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    1744, //  00001011 01100000 00000000 00000000
    722, //  01001011 01000000 00000000 00000000
    724, //  00101011 01000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    728, //  00011011 01000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1760, //  00000111 01100000 00000000 00000000
    738, //  01000111 01000000 00000000 00000000
    740, //  00100111 01000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    744, //  00010111 01000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    752, //  00001111 01000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3840, //  00000000 11110000 00000000 00000000
    3842, //  01000000 11110000 00000000 00000000
    3844, //  00100000 11110000 00000000 00000000
    1798, //  01100000 11100000 00000000 00000000
    3848, //  00010000 11110000 00000000 00000000
    1802, //  01010000 11100000 00000000 00000000
    1804, //  00110000 11100000 00000000 00000000
    782, //  01110000 11000000 00000000 00000000
    3856, //  00001000 11110000 00000000 00000000
    1810, //  01001000 11100000 00000000 00000000
    1812, //  00101000 11100000 00000000 00000000
    790, //  01101000 11000000 00000000 00000000
    1816, //  00011000 11100000 00000000 00000000
    794, //  01011000 11000000 00000000 00000000
    796, //  00111000 11000000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    3872, //  00000100 11110000 00000000 00000000
    1826, //  01000100 11100000 00000000 00000000
    1828, //  00100100 11100000 00000000 00000000
    806, //  01100100 11000000 00000000 00000000
    1832, //  00010100 11100000 00000000 00000000
    810, //  01010100 11000000 00000000 00000000
    812, //  00110100 11000000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    1840, //  00001100 11100000 00000000 00000000
    818, //  01001100 11000000 00000000 00000000
    820, //  00101100 11000000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    824, //  00011100 11000000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3904, //  00000010 11110000 00000000 00000000
    1858, //  01000010 11100000 00000000 00000000
    1860, //  00100010 11100000 00000000 00000000
    838, //  01100010 11000000 00000000 00000000
    1864, //  00010010 11100000 00000000 00000000
    842, //  01010010 11000000 00000000 00000000
    844, //  00110010 11000000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    1872, //  00001010 11100000 00000000 00000000
    850, //  01001010 11000000 00000000 00000000
    852, //  00101010 11000000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    856, //  00011010 11000000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1888, //  00000110 11100000 00000000 00000000
    866, //  01000110 11000000 00000000 00000000
    868, //  00100110 11000000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    872, //  00010110 11000000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    880, //  00001110 11000000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3968, //  00000001 11110000 00000000 00000000
    1922, //  01000001 11100000 00000000 00000000
    1924, //  00100001 11100000 00000000 00000000
    902, //  01100001 11000000 00000000 00000000
    1928, //  00010001 11100000 00000000 00000000
    906, //  01010001 11000000 00000000 00000000
    908, //  00110001 11000000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    1936, //  00001001 11100000 00000000 00000000
    914, //  01001001 11000000 00000000 00000000
    916, //  00101001 11000000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    920, //  00011001 11000000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    1952, //  00000101 11100000 00000000 00000000
    930, //  01000101 11000000 00000000 00000000
    932, //  00100101 11000000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    936, //  00010101 11000000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    944, //  00001101 11000000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1984, //  00000011 11100000 00000000 00000000
    962, //  01000011 11000000 00000000 00000000
    964, //  00100011 11000000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    968, //  00010011 11000000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    976, //  00001011 11000000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    992, //  00000111 11000000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4096, //  00000000 00001000 00000000 00000000
    4098, //  01000000 00001000 00000000 00000000
    4100, //  00100000 00001000 00000000 00000000
    4102, //  01100000 00001000 00000000 00000000
    4104, //  00010000 00001000 00000000 00000000
    4106, //  01010000 00001000 00000000 00000000
    4108, //  00110000 00001000 00000000 00000000
    4110, //  01110000 00001000 00000000 00000000
    4112, //  00001000 00001000 00000000 00000000
    4114, //  01001000 00001000 00000000 00000000
    4116, //  00101000 00001000 00000000 00000000
    4118, //  01101000 00001000 00000000 00000000
    4120, //  00011000 00001000 00000000 00000000
    4122, //  01011000 00001000 00000000 00000000
    4124, //  00111000 00001000 00000000 00000000
    4126, //  01111000 00001000 00000000 00000000
    4128, //  00000100 00001000 00000000 00000000
    4130, //  01000100 00001000 00000000 00000000
    4132, //  00100100 00001000 00000000 00000000
    4134, //  01100100 00001000 00000000 00000000
    4136, //  00010100 00001000 00000000 00000000
    4138, //  01010100 00001000 00000000 00000000
    4140, //  00110100 00001000 00000000 00000000
    4142, //  01110100 00001000 00000000 00000000
    4144, //  00001100 00001000 00000000 00000000
    4146, //  01001100 00001000 00000000 00000000
    4148, //  00101100 00001000 00000000 00000000
    4150, //  01101100 00001000 00000000 00000000
    4152, //  00011100 00001000 00000000 00000000
    4154, //  01011100 00001000 00000000 00000000
    4156, //  00111100 00001000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4160, //  00000010 00001000 00000000 00000000
    4162, //  01000010 00001000 00000000 00000000
    4164, //  00100010 00001000 00000000 00000000
    4166, //  01100010 00001000 00000000 00000000
    4168, //  00010010 00001000 00000000 00000000
    4170, //  01010010 00001000 00000000 00000000
    4172, //  00110010 00001000 00000000 00000000
    4174, //  01110010 00001000 00000000 00000000
    4176, //  00001010 00001000 00000000 00000000
    4178, //  01001010 00001000 00000000 00000000
    4180, //  00101010 00001000 00000000 00000000
    4182, //  01101010 00001000 00000000 00000000
    4184, //  00011010 00001000 00000000 00000000
    4186, //  01011010 00001000 00000000 00000000
    4188, //  00111010 00001000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    4192, //  00000110 00001000 00000000 00000000
    4194, //  01000110 00001000 00000000 00000000
    4196, //  00100110 00001000 00000000 00000000
    4198, //  01100110 00001000 00000000 00000000
    4200, //  00010110 00001000 00000000 00000000
    4202, //  01010110 00001000 00000000 00000000
    4204, //  00110110 00001000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    4208, //  00001110 00001000 00000000 00000000
    4210, //  01001110 00001000 00000000 00000000
    4212, //  00101110 00001000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    4216, //  00011110 00001000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4224, //  00000001 00001000 00000000 00000000
    4226, //  01000001 00001000 00000000 00000000
    4228, //  00100001 00001000 00000000 00000000
    4230, //  01100001 00001000 00000000 00000000
    4232, //  00010001 00001000 00000000 00000000
    4234, //  01010001 00001000 00000000 00000000
    4236, //  00110001 00001000 00000000 00000000
    4238, //  01110001 00001000 00000000 00000000
    4240, //  00001001 00001000 00000000 00000000
    4242, //  01001001 00001000 00000000 00000000
    4244, //  00101001 00001000 00000000 00000000
    4246, //  01101001 00001000 00000000 00000000
    4248, //  00011001 00001000 00000000 00000000
    4250, //  01011001 00001000 00000000 00000000
    4252, //  00111001 00001000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    4256, //  00000101 00001000 00000000 00000000
    4258, //  01000101 00001000 00000000 00000000
    4260, //  00100101 00001000 00000000 00000000
    4262, //  01100101 00001000 00000000 00000000
    4264, //  00010101 00001000 00000000 00000000
    4266, //  01010101 00001000 00000000 00000000
    4268, //  00110101 00001000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    4272, //  00001101 00001000 00000000 00000000
    4274, //  01001101 00001000 00000000 00000000
    4276, //  00101101 00001000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    4280, //  00011101 00001000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4288, //  00000011 00001000 00000000 00000000
    4290, //  01000011 00001000 00000000 00000000
    4292, //  00100011 00001000 00000000 00000000
    4294, //  01100011 00001000 00000000 00000000
    4296, //  00010011 00001000 00000000 00000000
    4298, //  01010011 00001000 00000000 00000000
    4300, //  00110011 00001000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    4304, //  00001011 00001000 00000000 00000000
    4306, //  01001011 00001000 00000000 00000000
    4308, //  00101011 00001000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    4312, //  00011011 00001000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    4320, //  00000111 00001000 00000000 00000000
    4322, //  01000111 00001000 00000000 00000000
    4324, //  00100111 00001000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    4328, //  00010111 00001000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    4336, //  00001111 00001000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4352, //  00000000 10001000 00000000 00000000
    4354, //  01000000 10001000 00000000 00000000
    4356, //  00100000 10001000 00000000 00000000
    4358, //  01100000 10001000 00000000 00000000
    4360, //  00010000 10001000 00000000 00000000
    4362, //  01010000 10001000 00000000 00000000
    4364, //  00110000 10001000 00000000 00000000
    4366, //  01110000 10001000 00000000 00000000
    4368, //  00001000 10001000 00000000 00000000
    4370, //  01001000 10001000 00000000 00000000
    4372, //  00101000 10001000 00000000 00000000
    4374, //  01101000 10001000 00000000 00000000
    4376, //  00011000 10001000 00000000 00000000
    4378, //  01011000 10001000 00000000 00000000
    4380, //  00111000 10001000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    4384, //  00000100 10001000 00000000 00000000
    4386, //  01000100 10001000 00000000 00000000
    4388, //  00100100 10001000 00000000 00000000
    4390, //  01100100 10001000 00000000 00000000
    4392, //  00010100 10001000 00000000 00000000
    4394, //  01010100 10001000 00000000 00000000
    4396, //  00110100 10001000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    4400, //  00001100 10001000 00000000 00000000
    4402, //  01001100 10001000 00000000 00000000
    4404, //  00101100 10001000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    4408, //  00011100 10001000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4416, //  00000010 10001000 00000000 00000000
    4418, //  01000010 10001000 00000000 00000000
    4420, //  00100010 10001000 00000000 00000000
    4422, //  01100010 10001000 00000000 00000000
    4424, //  00010010 10001000 00000000 00000000
    4426, //  01010010 10001000 00000000 00000000
    4428, //  00110010 10001000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    4432, //  00001010 10001000 00000000 00000000
    4434, //  01001010 10001000 00000000 00000000
    4436, //  00101010 10001000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    4440, //  00011010 10001000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    4448, //  00000110 10001000 00000000 00000000
    4450, //  01000110 10001000 00000000 00000000
    4452, //  00100110 10001000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    4456, //  00010110 10001000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    4464, //  00001110 10001000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4480, //  00000001 10001000 00000000 00000000
    4482, //  01000001 10001000 00000000 00000000
    4484, //  00100001 10001000 00000000 00000000
    4486, //  01100001 10001000 00000000 00000000
    4488, //  00010001 10001000 00000000 00000000
    4490, //  01010001 10001000 00000000 00000000
    4492, //  00110001 10001000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    4496, //  00001001 10001000 00000000 00000000
    4498, //  01001001 10001000 00000000 00000000
    4500, //  00101001 10001000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    4504, //  00011001 10001000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    4512, //  00000101 10001000 00000000 00000000
    4514, //  01000101 10001000 00000000 00000000
    4516, //  00100101 10001000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    4520, //  00010101 10001000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    4528, //  00001101 10001000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4544, //  00000011 10001000 00000000 00000000
    4546, //  01000011 10001000 00000000 00000000
    4548, //  00100011 10001000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    4552, //  00010011 10001000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    4560, //  00001011 10001000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    4576, //  00000111 10001000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4608, //  00000000 01001000 00000000 00000000
    4610, //  01000000 01001000 00000000 00000000
    4612, //  00100000 01001000 00000000 00000000
    4614, //  01100000 01001000 00000000 00000000
    4616, //  00010000 01001000 00000000 00000000
    4618, //  01010000 01001000 00000000 00000000
    4620, //  00110000 01001000 00000000 00000000
    4622, //  01110000 01001000 00000000 00000000
    4624, //  00001000 01001000 00000000 00000000
    4626, //  01001000 01001000 00000000 00000000
    4628, //  00101000 01001000 00000000 00000000
    4630, //  01101000 01001000 00000000 00000000
    4632, //  00011000 01001000 00000000 00000000
    4634, //  01011000 01001000 00000000 00000000
    4636, //  00111000 01001000 00000000 00000000
    542, //  01111000 01000000 00000000 00000000
    4640, //  00000100 01001000 00000000 00000000
    4642, //  01000100 01001000 00000000 00000000
    4644, //  00100100 01001000 00000000 00000000
    4646, //  01100100 01001000 00000000 00000000
    4648, //  00010100 01001000 00000000 00000000
    4650, //  01010100 01001000 00000000 00000000
    4652, //  00110100 01001000 00000000 00000000
    558, //  01110100 01000000 00000000 00000000
    4656, //  00001100 01001000 00000000 00000000
    4658, //  01001100 01001000 00000000 00000000
    4660, //  00101100 01001000 00000000 00000000
    566, //  01101100 01000000 00000000 00000000
    4664, //  00011100 01001000 00000000 00000000
    570, //  01011100 01000000 00000000 00000000
    572, //  00111100 01000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4672, //  00000010 01001000 00000000 00000000
    4674, //  01000010 01001000 00000000 00000000
    4676, //  00100010 01001000 00000000 00000000
    4678, //  01100010 01001000 00000000 00000000
    4680, //  00010010 01001000 00000000 00000000
    4682, //  01010010 01001000 00000000 00000000
    4684, //  00110010 01001000 00000000 00000000
    590, //  01110010 01000000 00000000 00000000
    4688, //  00001010 01001000 00000000 00000000
    4690, //  01001010 01001000 00000000 00000000
    4692, //  00101010 01001000 00000000 00000000
    598, //  01101010 01000000 00000000 00000000
    4696, //  00011010 01001000 00000000 00000000
    602, //  01011010 01000000 00000000 00000000
    604, //  00111010 01000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    4704, //  00000110 01001000 00000000 00000000
    4706, //  01000110 01001000 00000000 00000000
    4708, //  00100110 01001000 00000000 00000000
    614, //  01100110 01000000 00000000 00000000
    4712, //  00010110 01001000 00000000 00000000
    618, //  01010110 01000000 00000000 00000000
    620, //  00110110 01000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    4720, //  00001110 01001000 00000000 00000000
    626, //  01001110 01000000 00000000 00000000
    628, //  00101110 01000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    632, //  00011110 01000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4736, //  00000001 01001000 00000000 00000000
    4738, //  01000001 01001000 00000000 00000000
    4740, //  00100001 01001000 00000000 00000000
    4742, //  01100001 01001000 00000000 00000000
    4744, //  00010001 01001000 00000000 00000000
    4746, //  01010001 01001000 00000000 00000000
    4748, //  00110001 01001000 00000000 00000000
    654, //  01110001 01000000 00000000 00000000
    4752, //  00001001 01001000 00000000 00000000
    4754, //  01001001 01001000 00000000 00000000
    4756, //  00101001 01001000 00000000 00000000
    662, //  01101001 01000000 00000000 00000000
    4760, //  00011001 01001000 00000000 00000000
    666, //  01011001 01000000 00000000 00000000
    668, //  00111001 01000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    4768, //  00000101 01001000 00000000 00000000
    4770, //  01000101 01001000 00000000 00000000
    4772, //  00100101 01001000 00000000 00000000
    678, //  01100101 01000000 00000000 00000000
    4776, //  00010101 01001000 00000000 00000000
    682, //  01010101 01000000 00000000 00000000
    684, //  00110101 01000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    4784, //  00001101 01001000 00000000 00000000
    690, //  01001101 01000000 00000000 00000000
    692, //  00101101 01000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    696, //  00011101 01000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4800, //  00000011 01001000 00000000 00000000
    4802, //  01000011 01001000 00000000 00000000
    4804, //  00100011 01001000 00000000 00000000
    710, //  01100011 01000000 00000000 00000000
    4808, //  00010011 01001000 00000000 00000000
    714, //  01010011 01000000 00000000 00000000
    716, //  00110011 01000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    4816, //  00001011 01001000 00000000 00000000
    722, //  01001011 01000000 00000000 00000000
    724, //  00101011 01000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    728, //  00011011 01000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    4832, //  00000111 01001000 00000000 00000000
    738, //  01000111 01000000 00000000 00000000
    740, //  00100111 01000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    744, //  00010111 01000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    752, //  00001111 01000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4864, //  00000000 11001000 00000000 00000000
    4866, //  01000000 11001000 00000000 00000000
    4868, //  00100000 11001000 00000000 00000000
    4870, //  01100000 11001000 00000000 00000000
    4872, //  00010000 11001000 00000000 00000000
    4874, //  01010000 11001000 00000000 00000000
    4876, //  00110000 11001000 00000000 00000000
    782, //  01110000 11000000 00000000 00000000
    4880, //  00001000 11001000 00000000 00000000
    4882, //  01001000 11001000 00000000 00000000
    4884, //  00101000 11001000 00000000 00000000
    790, //  01101000 11000000 00000000 00000000
    4888, //  00011000 11001000 00000000 00000000
    794, //  01011000 11000000 00000000 00000000
    796, //  00111000 11000000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    4896, //  00000100 11001000 00000000 00000000
    4898, //  01000100 11001000 00000000 00000000
    4900, //  00100100 11001000 00000000 00000000
    806, //  01100100 11000000 00000000 00000000
    4904, //  00010100 11001000 00000000 00000000
    810, //  01010100 11000000 00000000 00000000
    812, //  00110100 11000000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    4912, //  00001100 11001000 00000000 00000000
    818, //  01001100 11000000 00000000 00000000
    820, //  00101100 11000000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    824, //  00011100 11000000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4928, //  00000010 11001000 00000000 00000000
    4930, //  01000010 11001000 00000000 00000000
    4932, //  00100010 11001000 00000000 00000000
    838, //  01100010 11000000 00000000 00000000
    4936, //  00010010 11001000 00000000 00000000
    842, //  01010010 11000000 00000000 00000000
    844, //  00110010 11000000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    4944, //  00001010 11001000 00000000 00000000
    850, //  01001010 11000000 00000000 00000000
    852, //  00101010 11000000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    856, //  00011010 11000000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    4960, //  00000110 11001000 00000000 00000000
    866, //  01000110 11000000 00000000 00000000
    868, //  00100110 11000000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    872, //  00010110 11000000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    880, //  00001110 11000000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    4992, //  00000001 11001000 00000000 00000000
    4994, //  01000001 11001000 00000000 00000000
    4996, //  00100001 11001000 00000000 00000000
    902, //  01100001 11000000 00000000 00000000
    5000, //  00010001 11001000 00000000 00000000
    906, //  01010001 11000000 00000000 00000000
    908, //  00110001 11000000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    5008, //  00001001 11001000 00000000 00000000
    914, //  01001001 11000000 00000000 00000000
    916, //  00101001 11000000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    920, //  00011001 11000000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    5024, //  00000101 11001000 00000000 00000000
    930, //  01000101 11000000 00000000 00000000
    932, //  00100101 11000000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    936, //  00010101 11000000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    944, //  00001101 11000000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5056, //  00000011 11001000 00000000 00000000
    962, //  01000011 11000000 00000000 00000000
    964, //  00100011 11000000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    968, //  00010011 11000000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    976, //  00001011 11000000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    992, //  00000111 11000000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5120, //  00000000 00101000 00000000 00000000
    5122, //  01000000 00101000 00000000 00000000
    5124, //  00100000 00101000 00000000 00000000
    5126, //  01100000 00101000 00000000 00000000
    5128, //  00010000 00101000 00000000 00000000
    5130, //  01010000 00101000 00000000 00000000
    5132, //  00110000 00101000 00000000 00000000
    5134, //  01110000 00101000 00000000 00000000
    5136, //  00001000 00101000 00000000 00000000
    5138, //  01001000 00101000 00000000 00000000
    5140, //  00101000 00101000 00000000 00000000
    5142, //  01101000 00101000 00000000 00000000
    5144, //  00011000 00101000 00000000 00000000
    5146, //  01011000 00101000 00000000 00000000
    5148, //  00111000 00101000 00000000 00000000
    1054, //  01111000 00100000 00000000 00000000
    5152, //  00000100 00101000 00000000 00000000
    5154, //  01000100 00101000 00000000 00000000
    5156, //  00100100 00101000 00000000 00000000
    5158, //  01100100 00101000 00000000 00000000
    5160, //  00010100 00101000 00000000 00000000
    5162, //  01010100 00101000 00000000 00000000
    5164, //  00110100 00101000 00000000 00000000
    1070, //  01110100 00100000 00000000 00000000
    5168, //  00001100 00101000 00000000 00000000
    5170, //  01001100 00101000 00000000 00000000
    5172, //  00101100 00101000 00000000 00000000
    1078, //  01101100 00100000 00000000 00000000
    5176, //  00011100 00101000 00000000 00000000
    1082, //  01011100 00100000 00000000 00000000
    1084, //  00111100 00100000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5184, //  00000010 00101000 00000000 00000000
    5186, //  01000010 00101000 00000000 00000000
    5188, //  00100010 00101000 00000000 00000000
    5190, //  01100010 00101000 00000000 00000000
    5192, //  00010010 00101000 00000000 00000000
    5194, //  01010010 00101000 00000000 00000000
    5196, //  00110010 00101000 00000000 00000000
    1102, //  01110010 00100000 00000000 00000000
    5200, //  00001010 00101000 00000000 00000000
    5202, //  01001010 00101000 00000000 00000000
    5204, //  00101010 00101000 00000000 00000000
    1110, //  01101010 00100000 00000000 00000000
    5208, //  00011010 00101000 00000000 00000000
    1114, //  01011010 00100000 00000000 00000000
    1116, //  00111010 00100000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    5216, //  00000110 00101000 00000000 00000000
    5218, //  01000110 00101000 00000000 00000000
    5220, //  00100110 00101000 00000000 00000000
    1126, //  01100110 00100000 00000000 00000000
    5224, //  00010110 00101000 00000000 00000000
    1130, //  01010110 00100000 00000000 00000000
    1132, //  00110110 00100000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    5232, //  00001110 00101000 00000000 00000000
    1138, //  01001110 00100000 00000000 00000000
    1140, //  00101110 00100000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    1144, //  00011110 00100000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5248, //  00000001 00101000 00000000 00000000
    5250, //  01000001 00101000 00000000 00000000
    5252, //  00100001 00101000 00000000 00000000
    5254, //  01100001 00101000 00000000 00000000
    5256, //  00010001 00101000 00000000 00000000
    5258, //  01010001 00101000 00000000 00000000
    5260, //  00110001 00101000 00000000 00000000
    1166, //  01110001 00100000 00000000 00000000
    5264, //  00001001 00101000 00000000 00000000
    5266, //  01001001 00101000 00000000 00000000
    5268, //  00101001 00101000 00000000 00000000
    1174, //  01101001 00100000 00000000 00000000
    5272, //  00011001 00101000 00000000 00000000
    1178, //  01011001 00100000 00000000 00000000
    1180, //  00111001 00100000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    5280, //  00000101 00101000 00000000 00000000
    5282, //  01000101 00101000 00000000 00000000
    5284, //  00100101 00101000 00000000 00000000
    1190, //  01100101 00100000 00000000 00000000
    5288, //  00010101 00101000 00000000 00000000
    1194, //  01010101 00100000 00000000 00000000
    1196, //  00110101 00100000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    5296, //  00001101 00101000 00000000 00000000
    1202, //  01001101 00100000 00000000 00000000
    1204, //  00101101 00100000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    1208, //  00011101 00100000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5312, //  00000011 00101000 00000000 00000000
    5314, //  01000011 00101000 00000000 00000000
    5316, //  00100011 00101000 00000000 00000000
    1222, //  01100011 00100000 00000000 00000000
    5320, //  00010011 00101000 00000000 00000000
    1226, //  01010011 00100000 00000000 00000000
    1228, //  00110011 00100000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    5328, //  00001011 00101000 00000000 00000000
    1234, //  01001011 00100000 00000000 00000000
    1236, //  00101011 00100000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    1240, //  00011011 00100000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    5344, //  00000111 00101000 00000000 00000000
    1250, //  01000111 00100000 00000000 00000000
    1252, //  00100111 00100000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    1256, //  00010111 00100000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1264, //  00001111 00100000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5376, //  00000000 10101000 00000000 00000000
    5378, //  01000000 10101000 00000000 00000000
    5380, //  00100000 10101000 00000000 00000000
    5382, //  01100000 10101000 00000000 00000000
    5384, //  00010000 10101000 00000000 00000000
    5386, //  01010000 10101000 00000000 00000000
    5388, //  00110000 10101000 00000000 00000000
    1294, //  01110000 10100000 00000000 00000000
    5392, //  00001000 10101000 00000000 00000000
    5394, //  01001000 10101000 00000000 00000000
    5396, //  00101000 10101000 00000000 00000000
    1302, //  01101000 10100000 00000000 00000000
    5400, //  00011000 10101000 00000000 00000000
    1306, //  01011000 10100000 00000000 00000000
    1308, //  00111000 10100000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    5408, //  00000100 10101000 00000000 00000000
    5410, //  01000100 10101000 00000000 00000000
    5412, //  00100100 10101000 00000000 00000000
    1318, //  01100100 10100000 00000000 00000000
    5416, //  00010100 10101000 00000000 00000000
    1322, //  01010100 10100000 00000000 00000000
    1324, //  00110100 10100000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    5424, //  00001100 10101000 00000000 00000000
    1330, //  01001100 10100000 00000000 00000000
    1332, //  00101100 10100000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    1336, //  00011100 10100000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5440, //  00000010 10101000 00000000 00000000
    5442, //  01000010 10101000 00000000 00000000
    5444, //  00100010 10101000 00000000 00000000
    1350, //  01100010 10100000 00000000 00000000
    5448, //  00010010 10101000 00000000 00000000
    1354, //  01010010 10100000 00000000 00000000
    1356, //  00110010 10100000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    5456, //  00001010 10101000 00000000 00000000
    1362, //  01001010 10100000 00000000 00000000
    1364, //  00101010 10100000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    1368, //  00011010 10100000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    5472, //  00000110 10101000 00000000 00000000
    1378, //  01000110 10100000 00000000 00000000
    1380, //  00100110 10100000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    1384, //  00010110 10100000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1392, //  00001110 10100000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5504, //  00000001 10101000 00000000 00000000
    5506, //  01000001 10101000 00000000 00000000
    5508, //  00100001 10101000 00000000 00000000
    1414, //  01100001 10100000 00000000 00000000
    5512, //  00010001 10101000 00000000 00000000
    1418, //  01010001 10100000 00000000 00000000
    1420, //  00110001 10100000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    5520, //  00001001 10101000 00000000 00000000
    1426, //  01001001 10100000 00000000 00000000
    1428, //  00101001 10100000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    1432, //  00011001 10100000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    5536, //  00000101 10101000 00000000 00000000
    1442, //  01000101 10100000 00000000 00000000
    1444, //  00100101 10100000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    1448, //  00010101 10100000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    1456, //  00001101 10100000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5568, //  00000011 10101000 00000000 00000000
    1474, //  01000011 10100000 00000000 00000000
    1476, //  00100011 10100000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    1480, //  00010011 10100000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    1488, //  00001011 10100000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1504, //  00000111 10100000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5632, //  00000000 01101000 00000000 00000000
    5634, //  01000000 01101000 00000000 00000000
    5636, //  00100000 01101000 00000000 00000000
    5638, //  01100000 01101000 00000000 00000000
    5640, //  00010000 01101000 00000000 00000000
    5642, //  01010000 01101000 00000000 00000000
    5644, //  00110000 01101000 00000000 00000000
    1550, //  01110000 01100000 00000000 00000000
    5648, //  00001000 01101000 00000000 00000000
    5650, //  01001000 01101000 00000000 00000000
    5652, //  00101000 01101000 00000000 00000000
    1558, //  01101000 01100000 00000000 00000000
    5656, //  00011000 01101000 00000000 00000000
    1562, //  01011000 01100000 00000000 00000000
    1564, //  00111000 01100000 00000000 00000000
    542, //  01111000 01000000 00000000 00000000
    5664, //  00000100 01101000 00000000 00000000
    5666, //  01000100 01101000 00000000 00000000
    5668, //  00100100 01101000 00000000 00000000
    1574, //  01100100 01100000 00000000 00000000
    5672, //  00010100 01101000 00000000 00000000
    1578, //  01010100 01100000 00000000 00000000
    1580, //  00110100 01100000 00000000 00000000
    558, //  01110100 01000000 00000000 00000000
    5680, //  00001100 01101000 00000000 00000000
    1586, //  01001100 01100000 00000000 00000000
    1588, //  00101100 01100000 00000000 00000000
    566, //  01101100 01000000 00000000 00000000
    1592, //  00011100 01100000 00000000 00000000
    570, //  01011100 01000000 00000000 00000000
    572, //  00111100 01000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5696, //  00000010 01101000 00000000 00000000
    5698, //  01000010 01101000 00000000 00000000
    5700, //  00100010 01101000 00000000 00000000
    1606, //  01100010 01100000 00000000 00000000
    5704, //  00010010 01101000 00000000 00000000
    1610, //  01010010 01100000 00000000 00000000
    1612, //  00110010 01100000 00000000 00000000
    590, //  01110010 01000000 00000000 00000000
    5712, //  00001010 01101000 00000000 00000000
    1618, //  01001010 01100000 00000000 00000000
    1620, //  00101010 01100000 00000000 00000000
    598, //  01101010 01000000 00000000 00000000
    1624, //  00011010 01100000 00000000 00000000
    602, //  01011010 01000000 00000000 00000000
    604, //  00111010 01000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    5728, //  00000110 01101000 00000000 00000000
    1634, //  01000110 01100000 00000000 00000000
    1636, //  00100110 01100000 00000000 00000000
    614, //  01100110 01000000 00000000 00000000
    1640, //  00010110 01100000 00000000 00000000
    618, //  01010110 01000000 00000000 00000000
    620, //  00110110 01000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1648, //  00001110 01100000 00000000 00000000
    626, //  01001110 01000000 00000000 00000000
    628, //  00101110 01000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    632, //  00011110 01000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5760, //  00000001 01101000 00000000 00000000
    5762, //  01000001 01101000 00000000 00000000
    5764, //  00100001 01101000 00000000 00000000
    1670, //  01100001 01100000 00000000 00000000
    5768, //  00010001 01101000 00000000 00000000
    1674, //  01010001 01100000 00000000 00000000
    1676, //  00110001 01100000 00000000 00000000
    654, //  01110001 01000000 00000000 00000000
    5776, //  00001001 01101000 00000000 00000000
    1682, //  01001001 01100000 00000000 00000000
    1684, //  00101001 01100000 00000000 00000000
    662, //  01101001 01000000 00000000 00000000
    1688, //  00011001 01100000 00000000 00000000
    666, //  01011001 01000000 00000000 00000000
    668, //  00111001 01000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    5792, //  00000101 01101000 00000000 00000000
    1698, //  01000101 01100000 00000000 00000000
    1700, //  00100101 01100000 00000000 00000000
    678, //  01100101 01000000 00000000 00000000
    1704, //  00010101 01100000 00000000 00000000
    682, //  01010101 01000000 00000000 00000000
    684, //  00110101 01000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    1712, //  00001101 01100000 00000000 00000000
    690, //  01001101 01000000 00000000 00000000
    692, //  00101101 01000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    696, //  00011101 01000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5824, //  00000011 01101000 00000000 00000000
    1730, //  01000011 01100000 00000000 00000000
    1732, //  00100011 01100000 00000000 00000000
    710, //  01100011 01000000 00000000 00000000
    1736, //  00010011 01100000 00000000 00000000
    714, //  01010011 01000000 00000000 00000000
    716, //  00110011 01000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    1744, //  00001011 01100000 00000000 00000000
    722, //  01001011 01000000 00000000 00000000
    724, //  00101011 01000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    728, //  00011011 01000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1760, //  00000111 01100000 00000000 00000000
    738, //  01000111 01000000 00000000 00000000
    740, //  00100111 01000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    744, //  00010111 01000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    752, //  00001111 01000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5888, //  00000000 11101000 00000000 00000000
    5890, //  01000000 11101000 00000000 00000000
    5892, //  00100000 11101000 00000000 00000000
    1798, //  01100000 11100000 00000000 00000000
    5896, //  00010000 11101000 00000000 00000000
    1802, //  01010000 11100000 00000000 00000000
    1804, //  00110000 11100000 00000000 00000000
    782, //  01110000 11000000 00000000 00000000
    5904, //  00001000 11101000 00000000 00000000
    1810, //  01001000 11100000 00000000 00000000
    1812, //  00101000 11100000 00000000 00000000
    790, //  01101000 11000000 00000000 00000000
    1816, //  00011000 11100000 00000000 00000000
    794, //  01011000 11000000 00000000 00000000
    796, //  00111000 11000000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    5920, //  00000100 11101000 00000000 00000000
    1826, //  01000100 11100000 00000000 00000000
    1828, //  00100100 11100000 00000000 00000000
    806, //  01100100 11000000 00000000 00000000
    1832, //  00010100 11100000 00000000 00000000
    810, //  01010100 11000000 00000000 00000000
    812, //  00110100 11000000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    1840, //  00001100 11100000 00000000 00000000
    818, //  01001100 11000000 00000000 00000000
    820, //  00101100 11000000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    824, //  00011100 11000000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    5952, //  00000010 11101000 00000000 00000000
    1858, //  01000010 11100000 00000000 00000000
    1860, //  00100010 11100000 00000000 00000000
    838, //  01100010 11000000 00000000 00000000
    1864, //  00010010 11100000 00000000 00000000
    842, //  01010010 11000000 00000000 00000000
    844, //  00110010 11000000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    1872, //  00001010 11100000 00000000 00000000
    850, //  01001010 11000000 00000000 00000000
    852, //  00101010 11000000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    856, //  00011010 11000000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1888, //  00000110 11100000 00000000 00000000
    866, //  01000110 11000000 00000000 00000000
    868, //  00100110 11000000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    872, //  00010110 11000000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    880, //  00001110 11000000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6016, //  00000001 11101000 00000000 00000000
    1922, //  01000001 11100000 00000000 00000000
    1924, //  00100001 11100000 00000000 00000000
    902, //  01100001 11000000 00000000 00000000
    1928, //  00010001 11100000 00000000 00000000
    906, //  01010001 11000000 00000000 00000000
    908, //  00110001 11000000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    1936, //  00001001 11100000 00000000 00000000
    914, //  01001001 11000000 00000000 00000000
    916, //  00101001 11000000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    920, //  00011001 11000000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    1952, //  00000101 11100000 00000000 00000000
    930, //  01000101 11000000 00000000 00000000
    932, //  00100101 11000000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    936, //  00010101 11000000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    944, //  00001101 11000000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1984, //  00000011 11100000 00000000 00000000
    962, //  01000011 11000000 00000000 00000000
    964, //  00100011 11000000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    968, //  00010011 11000000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    976, //  00001011 11000000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    992, //  00000111 11000000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6144, //  00000000 00011000 00000000 00000000
    6146, //  01000000 00011000 00000000 00000000
    6148, //  00100000 00011000 00000000 00000000
    6150, //  01100000 00011000 00000000 00000000
    6152, //  00010000 00011000 00000000 00000000
    6154, //  01010000 00011000 00000000 00000000
    6156, //  00110000 00011000 00000000 00000000
    6158, //  01110000 00011000 00000000 00000000
    6160, //  00001000 00011000 00000000 00000000
    6162, //  01001000 00011000 00000000 00000000
    6164, //  00101000 00011000 00000000 00000000
    6166, //  01101000 00011000 00000000 00000000
    6168, //  00011000 00011000 00000000 00000000
    6170, //  01011000 00011000 00000000 00000000
    6172, //  00111000 00011000 00000000 00000000
    2078, //  01111000 00010000 00000000 00000000
    6176, //  00000100 00011000 00000000 00000000
    6178, //  01000100 00011000 00000000 00000000
    6180, //  00100100 00011000 00000000 00000000
    6182, //  01100100 00011000 00000000 00000000
    6184, //  00010100 00011000 00000000 00000000
    6186, //  01010100 00011000 00000000 00000000
    6188, //  00110100 00011000 00000000 00000000
    2094, //  01110100 00010000 00000000 00000000
    6192, //  00001100 00011000 00000000 00000000
    6194, //  01001100 00011000 00000000 00000000
    6196, //  00101100 00011000 00000000 00000000
    2102, //  01101100 00010000 00000000 00000000
    6200, //  00011100 00011000 00000000 00000000
    2106, //  01011100 00010000 00000000 00000000
    2108, //  00111100 00010000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6208, //  00000010 00011000 00000000 00000000
    6210, //  01000010 00011000 00000000 00000000
    6212, //  00100010 00011000 00000000 00000000
    6214, //  01100010 00011000 00000000 00000000
    6216, //  00010010 00011000 00000000 00000000
    6218, //  01010010 00011000 00000000 00000000
    6220, //  00110010 00011000 00000000 00000000
    2126, //  01110010 00010000 00000000 00000000
    6224, //  00001010 00011000 00000000 00000000
    6226, //  01001010 00011000 00000000 00000000
    6228, //  00101010 00011000 00000000 00000000
    2134, //  01101010 00010000 00000000 00000000
    6232, //  00011010 00011000 00000000 00000000
    2138, //  01011010 00010000 00000000 00000000
    2140, //  00111010 00010000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    6240, //  00000110 00011000 00000000 00000000
    6242, //  01000110 00011000 00000000 00000000
    6244, //  00100110 00011000 00000000 00000000
    2150, //  01100110 00010000 00000000 00000000
    6248, //  00010110 00011000 00000000 00000000
    2154, //  01010110 00010000 00000000 00000000
    2156, //  00110110 00010000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    6256, //  00001110 00011000 00000000 00000000
    2162, //  01001110 00010000 00000000 00000000
    2164, //  00101110 00010000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    2168, //  00011110 00010000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6272, //  00000001 00011000 00000000 00000000
    6274, //  01000001 00011000 00000000 00000000
    6276, //  00100001 00011000 00000000 00000000
    6278, //  01100001 00011000 00000000 00000000
    6280, //  00010001 00011000 00000000 00000000
    6282, //  01010001 00011000 00000000 00000000
    6284, //  00110001 00011000 00000000 00000000
    2190, //  01110001 00010000 00000000 00000000
    6288, //  00001001 00011000 00000000 00000000
    6290, //  01001001 00011000 00000000 00000000
    6292, //  00101001 00011000 00000000 00000000
    2198, //  01101001 00010000 00000000 00000000
    6296, //  00011001 00011000 00000000 00000000
    2202, //  01011001 00010000 00000000 00000000
    2204, //  00111001 00010000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    6304, //  00000101 00011000 00000000 00000000
    6306, //  01000101 00011000 00000000 00000000
    6308, //  00100101 00011000 00000000 00000000
    2214, //  01100101 00010000 00000000 00000000
    6312, //  00010101 00011000 00000000 00000000
    2218, //  01010101 00010000 00000000 00000000
    2220, //  00110101 00010000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    6320, //  00001101 00011000 00000000 00000000
    2226, //  01001101 00010000 00000000 00000000
    2228, //  00101101 00010000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    2232, //  00011101 00010000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6336, //  00000011 00011000 00000000 00000000
    6338, //  01000011 00011000 00000000 00000000
    6340, //  00100011 00011000 00000000 00000000
    2246, //  01100011 00010000 00000000 00000000
    6344, //  00010011 00011000 00000000 00000000
    2250, //  01010011 00010000 00000000 00000000
    2252, //  00110011 00010000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    6352, //  00001011 00011000 00000000 00000000
    2258, //  01001011 00010000 00000000 00000000
    2260, //  00101011 00010000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    2264, //  00011011 00010000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    6368, //  00000111 00011000 00000000 00000000
    2274, //  01000111 00010000 00000000 00000000
    2276, //  00100111 00010000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    2280, //  00010111 00010000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    2288, //  00001111 00010000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6400, //  00000000 10011000 00000000 00000000
    6402, //  01000000 10011000 00000000 00000000
    6404, //  00100000 10011000 00000000 00000000
    6406, //  01100000 10011000 00000000 00000000
    6408, //  00010000 10011000 00000000 00000000
    6410, //  01010000 10011000 00000000 00000000
    6412, //  00110000 10011000 00000000 00000000
    2318, //  01110000 10010000 00000000 00000000
    6416, //  00001000 10011000 00000000 00000000
    6418, //  01001000 10011000 00000000 00000000
    6420, //  00101000 10011000 00000000 00000000
    2326, //  01101000 10010000 00000000 00000000
    6424, //  00011000 10011000 00000000 00000000
    2330, //  01011000 10010000 00000000 00000000
    2332, //  00111000 10010000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    6432, //  00000100 10011000 00000000 00000000
    6434, //  01000100 10011000 00000000 00000000
    6436, //  00100100 10011000 00000000 00000000
    2342, //  01100100 10010000 00000000 00000000
    6440, //  00010100 10011000 00000000 00000000
    2346, //  01010100 10010000 00000000 00000000
    2348, //  00110100 10010000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    6448, //  00001100 10011000 00000000 00000000
    2354, //  01001100 10010000 00000000 00000000
    2356, //  00101100 10010000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    2360, //  00011100 10010000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6464, //  00000010 10011000 00000000 00000000
    6466, //  01000010 10011000 00000000 00000000
    6468, //  00100010 10011000 00000000 00000000
    2374, //  01100010 10010000 00000000 00000000
    6472, //  00010010 10011000 00000000 00000000
    2378, //  01010010 10010000 00000000 00000000
    2380, //  00110010 10010000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    6480, //  00001010 10011000 00000000 00000000
    2386, //  01001010 10010000 00000000 00000000
    2388, //  00101010 10010000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    2392, //  00011010 10010000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    6496, //  00000110 10011000 00000000 00000000
    2402, //  01000110 10010000 00000000 00000000
    2404, //  00100110 10010000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    2408, //  00010110 10010000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    2416, //  00001110 10010000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6528, //  00000001 10011000 00000000 00000000
    6530, //  01000001 10011000 00000000 00000000
    6532, //  00100001 10011000 00000000 00000000
    2438, //  01100001 10010000 00000000 00000000
    6536, //  00010001 10011000 00000000 00000000
    2442, //  01010001 10010000 00000000 00000000
    2444, //  00110001 10010000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    6544, //  00001001 10011000 00000000 00000000
    2450, //  01001001 10010000 00000000 00000000
    2452, //  00101001 10010000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    2456, //  00011001 10010000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    6560, //  00000101 10011000 00000000 00000000
    2466, //  01000101 10010000 00000000 00000000
    2468, //  00100101 10010000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    2472, //  00010101 10010000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    2480, //  00001101 10010000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6592, //  00000011 10011000 00000000 00000000
    2498, //  01000011 10010000 00000000 00000000
    2500, //  00100011 10010000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    2504, //  00010011 10010000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    2512, //  00001011 10010000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    2528, //  00000111 10010000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6656, //  00000000 01011000 00000000 00000000
    6658, //  01000000 01011000 00000000 00000000
    6660, //  00100000 01011000 00000000 00000000
    6662, //  01100000 01011000 00000000 00000000
    6664, //  00010000 01011000 00000000 00000000
    6666, //  01010000 01011000 00000000 00000000
    6668, //  00110000 01011000 00000000 00000000
    2574, //  01110000 01010000 00000000 00000000
    6672, //  00001000 01011000 00000000 00000000
    6674, //  01001000 01011000 00000000 00000000
    6676, //  00101000 01011000 00000000 00000000
    2582, //  01101000 01010000 00000000 00000000
    6680, //  00011000 01011000 00000000 00000000
    2586, //  01011000 01010000 00000000 00000000
    2588, //  00111000 01010000 00000000 00000000
    542, //  01111000 01000000 00000000 00000000
    6688, //  00000100 01011000 00000000 00000000
    6690, //  01000100 01011000 00000000 00000000
    6692, //  00100100 01011000 00000000 00000000
    2598, //  01100100 01010000 00000000 00000000
    6696, //  00010100 01011000 00000000 00000000
    2602, //  01010100 01010000 00000000 00000000
    2604, //  00110100 01010000 00000000 00000000
    558, //  01110100 01000000 00000000 00000000
    6704, //  00001100 01011000 00000000 00000000
    2610, //  01001100 01010000 00000000 00000000
    2612, //  00101100 01010000 00000000 00000000
    566, //  01101100 01000000 00000000 00000000
    2616, //  00011100 01010000 00000000 00000000
    570, //  01011100 01000000 00000000 00000000
    572, //  00111100 01000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6720, //  00000010 01011000 00000000 00000000
    6722, //  01000010 01011000 00000000 00000000
    6724, //  00100010 01011000 00000000 00000000
    2630, //  01100010 01010000 00000000 00000000
    6728, //  00010010 01011000 00000000 00000000
    2634, //  01010010 01010000 00000000 00000000
    2636, //  00110010 01010000 00000000 00000000
    590, //  01110010 01000000 00000000 00000000
    6736, //  00001010 01011000 00000000 00000000
    2642, //  01001010 01010000 00000000 00000000
    2644, //  00101010 01010000 00000000 00000000
    598, //  01101010 01000000 00000000 00000000
    2648, //  00011010 01010000 00000000 00000000
    602, //  01011010 01000000 00000000 00000000
    604, //  00111010 01000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    6752, //  00000110 01011000 00000000 00000000
    2658, //  01000110 01010000 00000000 00000000
    2660, //  00100110 01010000 00000000 00000000
    614, //  01100110 01000000 00000000 00000000
    2664, //  00010110 01010000 00000000 00000000
    618, //  01010110 01000000 00000000 00000000
    620, //  00110110 01000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    2672, //  00001110 01010000 00000000 00000000
    626, //  01001110 01000000 00000000 00000000
    628, //  00101110 01000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    632, //  00011110 01000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6784, //  00000001 01011000 00000000 00000000
    6786, //  01000001 01011000 00000000 00000000
    6788, //  00100001 01011000 00000000 00000000
    2694, //  01100001 01010000 00000000 00000000
    6792, //  00010001 01011000 00000000 00000000
    2698, //  01010001 01010000 00000000 00000000
    2700, //  00110001 01010000 00000000 00000000
    654, //  01110001 01000000 00000000 00000000
    6800, //  00001001 01011000 00000000 00000000
    2706, //  01001001 01010000 00000000 00000000
    2708, //  00101001 01010000 00000000 00000000
    662, //  01101001 01000000 00000000 00000000
    2712, //  00011001 01010000 00000000 00000000
    666, //  01011001 01000000 00000000 00000000
    668, //  00111001 01000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    6816, //  00000101 01011000 00000000 00000000
    2722, //  01000101 01010000 00000000 00000000
    2724, //  00100101 01010000 00000000 00000000
    678, //  01100101 01000000 00000000 00000000
    2728, //  00010101 01010000 00000000 00000000
    682, //  01010101 01000000 00000000 00000000
    684, //  00110101 01000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    2736, //  00001101 01010000 00000000 00000000
    690, //  01001101 01000000 00000000 00000000
    692, //  00101101 01000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    696, //  00011101 01000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6848, //  00000011 01011000 00000000 00000000
    2754, //  01000011 01010000 00000000 00000000
    2756, //  00100011 01010000 00000000 00000000
    710, //  01100011 01000000 00000000 00000000
    2760, //  00010011 01010000 00000000 00000000
    714, //  01010011 01000000 00000000 00000000
    716, //  00110011 01000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    2768, //  00001011 01010000 00000000 00000000
    722, //  01001011 01000000 00000000 00000000
    724, //  00101011 01000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    728, //  00011011 01000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    2784, //  00000111 01010000 00000000 00000000
    738, //  01000111 01000000 00000000 00000000
    740, //  00100111 01000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    744, //  00010111 01000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    752, //  00001111 01000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6912, //  00000000 11011000 00000000 00000000
    6914, //  01000000 11011000 00000000 00000000
    6916, //  00100000 11011000 00000000 00000000
    2822, //  01100000 11010000 00000000 00000000
    6920, //  00010000 11011000 00000000 00000000
    2826, //  01010000 11010000 00000000 00000000
    2828, //  00110000 11010000 00000000 00000000
    782, //  01110000 11000000 00000000 00000000
    6928, //  00001000 11011000 00000000 00000000
    2834, //  01001000 11010000 00000000 00000000
    2836, //  00101000 11010000 00000000 00000000
    790, //  01101000 11000000 00000000 00000000
    2840, //  00011000 11010000 00000000 00000000
    794, //  01011000 11000000 00000000 00000000
    796, //  00111000 11000000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    6944, //  00000100 11011000 00000000 00000000
    2850, //  01000100 11010000 00000000 00000000
    2852, //  00100100 11010000 00000000 00000000
    806, //  01100100 11000000 00000000 00000000
    2856, //  00010100 11010000 00000000 00000000
    810, //  01010100 11000000 00000000 00000000
    812, //  00110100 11000000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    2864, //  00001100 11010000 00000000 00000000
    818, //  01001100 11000000 00000000 00000000
    820, //  00101100 11000000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    824, //  00011100 11000000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    6976, //  00000010 11011000 00000000 00000000
    2882, //  01000010 11010000 00000000 00000000
    2884, //  00100010 11010000 00000000 00000000
    838, //  01100010 11000000 00000000 00000000
    2888, //  00010010 11010000 00000000 00000000
    842, //  01010010 11000000 00000000 00000000
    844, //  00110010 11000000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    2896, //  00001010 11010000 00000000 00000000
    850, //  01001010 11000000 00000000 00000000
    852, //  00101010 11000000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    856, //  00011010 11000000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    2912, //  00000110 11010000 00000000 00000000
    866, //  01000110 11000000 00000000 00000000
    868, //  00100110 11000000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    872, //  00010110 11000000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    880, //  00001110 11000000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7040, //  00000001 11011000 00000000 00000000
    2946, //  01000001 11010000 00000000 00000000
    2948, //  00100001 11010000 00000000 00000000
    902, //  01100001 11000000 00000000 00000000
    2952, //  00010001 11010000 00000000 00000000
    906, //  01010001 11000000 00000000 00000000
    908, //  00110001 11000000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    2960, //  00001001 11010000 00000000 00000000
    914, //  01001001 11000000 00000000 00000000
    916, //  00101001 11000000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    920, //  00011001 11000000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    2976, //  00000101 11010000 00000000 00000000
    930, //  01000101 11000000 00000000 00000000
    932, //  00100101 11000000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    936, //  00010101 11000000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    944, //  00001101 11000000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3008, //  00000011 11010000 00000000 00000000
    962, //  01000011 11000000 00000000 00000000
    964, //  00100011 11000000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    968, //  00010011 11000000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    976, //  00001011 11000000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    992, //  00000111 11000000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7168, //  00000000 00111000 00000000 00000000
    7170, //  01000000 00111000 00000000 00000000
    7172, //  00100000 00111000 00000000 00000000
    7174, //  01100000 00111000 00000000 00000000
    7176, //  00010000 00111000 00000000 00000000
    7178, //  01010000 00111000 00000000 00000000
    7180, //  00110000 00111000 00000000 00000000
    3086, //  01110000 00110000 00000000 00000000
    7184, //  00001000 00111000 00000000 00000000
    7186, //  01001000 00111000 00000000 00000000
    7188, //  00101000 00111000 00000000 00000000
    3094, //  01101000 00110000 00000000 00000000
    7192, //  00011000 00111000 00000000 00000000
    3098, //  01011000 00110000 00000000 00000000
    3100, //  00111000 00110000 00000000 00000000
    1054, //  01111000 00100000 00000000 00000000
    7200, //  00000100 00111000 00000000 00000000
    7202, //  01000100 00111000 00000000 00000000
    7204, //  00100100 00111000 00000000 00000000
    3110, //  01100100 00110000 00000000 00000000
    7208, //  00010100 00111000 00000000 00000000
    3114, //  01010100 00110000 00000000 00000000
    3116, //  00110100 00110000 00000000 00000000
    1070, //  01110100 00100000 00000000 00000000
    7216, //  00001100 00111000 00000000 00000000
    3122, //  01001100 00110000 00000000 00000000
    3124, //  00101100 00110000 00000000 00000000
    1078, //  01101100 00100000 00000000 00000000
    3128, //  00011100 00110000 00000000 00000000
    1082, //  01011100 00100000 00000000 00000000
    1084, //  00111100 00100000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7232, //  00000010 00111000 00000000 00000000
    7234, //  01000010 00111000 00000000 00000000
    7236, //  00100010 00111000 00000000 00000000
    3142, //  01100010 00110000 00000000 00000000
    7240, //  00010010 00111000 00000000 00000000
    3146, //  01010010 00110000 00000000 00000000
    3148, //  00110010 00110000 00000000 00000000
    1102, //  01110010 00100000 00000000 00000000
    7248, //  00001010 00111000 00000000 00000000
    3154, //  01001010 00110000 00000000 00000000
    3156, //  00101010 00110000 00000000 00000000
    1110, //  01101010 00100000 00000000 00000000
    3160, //  00011010 00110000 00000000 00000000
    1114, //  01011010 00100000 00000000 00000000
    1116, //  00111010 00100000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    7264, //  00000110 00111000 00000000 00000000
    3170, //  01000110 00110000 00000000 00000000
    3172, //  00100110 00110000 00000000 00000000
    1126, //  01100110 00100000 00000000 00000000
    3176, //  00010110 00110000 00000000 00000000
    1130, //  01010110 00100000 00000000 00000000
    1132, //  00110110 00100000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    3184, //  00001110 00110000 00000000 00000000
    1138, //  01001110 00100000 00000000 00000000
    1140, //  00101110 00100000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    1144, //  00011110 00100000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7296, //  00000001 00111000 00000000 00000000
    7298, //  01000001 00111000 00000000 00000000
    7300, //  00100001 00111000 00000000 00000000
    3206, //  01100001 00110000 00000000 00000000
    7304, //  00010001 00111000 00000000 00000000
    3210, //  01010001 00110000 00000000 00000000
    3212, //  00110001 00110000 00000000 00000000
    1166, //  01110001 00100000 00000000 00000000
    7312, //  00001001 00111000 00000000 00000000
    3218, //  01001001 00110000 00000000 00000000
    3220, //  00101001 00110000 00000000 00000000
    1174, //  01101001 00100000 00000000 00000000
    3224, //  00011001 00110000 00000000 00000000
    1178, //  01011001 00100000 00000000 00000000
    1180, //  00111001 00100000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    7328, //  00000101 00111000 00000000 00000000
    3234, //  01000101 00110000 00000000 00000000
    3236, //  00100101 00110000 00000000 00000000
    1190, //  01100101 00100000 00000000 00000000
    3240, //  00010101 00110000 00000000 00000000
    1194, //  01010101 00100000 00000000 00000000
    1196, //  00110101 00100000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    3248, //  00001101 00110000 00000000 00000000
    1202, //  01001101 00100000 00000000 00000000
    1204, //  00101101 00100000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    1208, //  00011101 00100000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7360, //  00000011 00111000 00000000 00000000
    3266, //  01000011 00110000 00000000 00000000
    3268, //  00100011 00110000 00000000 00000000
    1222, //  01100011 00100000 00000000 00000000
    3272, //  00010011 00110000 00000000 00000000
    1226, //  01010011 00100000 00000000 00000000
    1228, //  00110011 00100000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    3280, //  00001011 00110000 00000000 00000000
    1234, //  01001011 00100000 00000000 00000000
    1236, //  00101011 00100000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    1240, //  00011011 00100000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    3296, //  00000111 00110000 00000000 00000000
    1250, //  01000111 00100000 00000000 00000000
    1252, //  00100111 00100000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    1256, //  00010111 00100000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1264, //  00001111 00100000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7424, //  00000000 10111000 00000000 00000000
    7426, //  01000000 10111000 00000000 00000000
    7428, //  00100000 10111000 00000000 00000000
    3334, //  01100000 10110000 00000000 00000000
    7432, //  00010000 10111000 00000000 00000000
    3338, //  01010000 10110000 00000000 00000000
    3340, //  00110000 10110000 00000000 00000000
    1294, //  01110000 10100000 00000000 00000000
    7440, //  00001000 10111000 00000000 00000000
    3346, //  01001000 10110000 00000000 00000000
    3348, //  00101000 10110000 00000000 00000000
    1302, //  01101000 10100000 00000000 00000000
    3352, //  00011000 10110000 00000000 00000000
    1306, //  01011000 10100000 00000000 00000000
    1308, //  00111000 10100000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    7456, //  00000100 10111000 00000000 00000000
    3362, //  01000100 10110000 00000000 00000000
    3364, //  00100100 10110000 00000000 00000000
    1318, //  01100100 10100000 00000000 00000000
    3368, //  00010100 10110000 00000000 00000000
    1322, //  01010100 10100000 00000000 00000000
    1324, //  00110100 10100000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    3376, //  00001100 10110000 00000000 00000000
    1330, //  01001100 10100000 00000000 00000000
    1332, //  00101100 10100000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    1336, //  00011100 10100000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7488, //  00000010 10111000 00000000 00000000
    3394, //  01000010 10110000 00000000 00000000
    3396, //  00100010 10110000 00000000 00000000
    1350, //  01100010 10100000 00000000 00000000
    3400, //  00010010 10110000 00000000 00000000
    1354, //  01010010 10100000 00000000 00000000
    1356, //  00110010 10100000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    3408, //  00001010 10110000 00000000 00000000
    1362, //  01001010 10100000 00000000 00000000
    1364, //  00101010 10100000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    1368, //  00011010 10100000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    3424, //  00000110 10110000 00000000 00000000
    1378, //  01000110 10100000 00000000 00000000
    1380, //  00100110 10100000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    1384, //  00010110 10100000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1392, //  00001110 10100000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7552, //  00000001 10111000 00000000 00000000
    3458, //  01000001 10110000 00000000 00000000
    3460, //  00100001 10110000 00000000 00000000
    1414, //  01100001 10100000 00000000 00000000
    3464, //  00010001 10110000 00000000 00000000
    1418, //  01010001 10100000 00000000 00000000
    1420, //  00110001 10100000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    3472, //  00001001 10110000 00000000 00000000
    1426, //  01001001 10100000 00000000 00000000
    1428, //  00101001 10100000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    1432, //  00011001 10100000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    3488, //  00000101 10110000 00000000 00000000
    1442, //  01000101 10100000 00000000 00000000
    1444, //  00100101 10100000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    1448, //  00010101 10100000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    1456, //  00001101 10100000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3520, //  00000011 10110000 00000000 00000000
    1474, //  01000011 10100000 00000000 00000000
    1476, //  00100011 10100000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    1480, //  00010011 10100000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    1488, //  00001011 10100000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1504, //  00000111 10100000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7680, //  00000000 01111000 00000000 00000000
    7682, //  01000000 01111000 00000000 00000000
    7684, //  00100000 01111000 00000000 00000000
    3590, //  01100000 01110000 00000000 00000000
    7688, //  00010000 01111000 00000000 00000000
    3594, //  01010000 01110000 00000000 00000000
    3596, //  00110000 01110000 00000000 00000000
    1550, //  01110000 01100000 00000000 00000000
    7696, //  00001000 01111000 00000000 00000000
    3602, //  01001000 01110000 00000000 00000000
    3604, //  00101000 01110000 00000000 00000000
    1558, //  01101000 01100000 00000000 00000000
    3608, //  00011000 01110000 00000000 00000000
    1562, //  01011000 01100000 00000000 00000000
    1564, //  00111000 01100000 00000000 00000000
    542, //  01111000 01000000 00000000 00000000
    7712, //  00000100 01111000 00000000 00000000
    3618, //  01000100 01110000 00000000 00000000
    3620, //  00100100 01110000 00000000 00000000
    1574, //  01100100 01100000 00000000 00000000
    3624, //  00010100 01110000 00000000 00000000
    1578, //  01010100 01100000 00000000 00000000
    1580, //  00110100 01100000 00000000 00000000
    558, //  01110100 01000000 00000000 00000000
    3632, //  00001100 01110000 00000000 00000000
    1586, //  01001100 01100000 00000000 00000000
    1588, //  00101100 01100000 00000000 00000000
    566, //  01101100 01000000 00000000 00000000
    1592, //  00011100 01100000 00000000 00000000
    570, //  01011100 01000000 00000000 00000000
    572, //  00111100 01000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7744, //  00000010 01111000 00000000 00000000
    3650, //  01000010 01110000 00000000 00000000
    3652, //  00100010 01110000 00000000 00000000
    1606, //  01100010 01100000 00000000 00000000
    3656, //  00010010 01110000 00000000 00000000
    1610, //  01010010 01100000 00000000 00000000
    1612, //  00110010 01100000 00000000 00000000
    590, //  01110010 01000000 00000000 00000000
    3664, //  00001010 01110000 00000000 00000000
    1618, //  01001010 01100000 00000000 00000000
    1620, //  00101010 01100000 00000000 00000000
    598, //  01101010 01000000 00000000 00000000
    1624, //  00011010 01100000 00000000 00000000
    602, //  01011010 01000000 00000000 00000000
    604, //  00111010 01000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    3680, //  00000110 01110000 00000000 00000000
    1634, //  01000110 01100000 00000000 00000000
    1636, //  00100110 01100000 00000000 00000000
    614, //  01100110 01000000 00000000 00000000
    1640, //  00010110 01100000 00000000 00000000
    618, //  01010110 01000000 00000000 00000000
    620, //  00110110 01000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    1648, //  00001110 01100000 00000000 00000000
    626, //  01001110 01000000 00000000 00000000
    628, //  00101110 01000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    632, //  00011110 01000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7808, //  00000001 01111000 00000000 00000000
    3714, //  01000001 01110000 00000000 00000000
    3716, //  00100001 01110000 00000000 00000000
    1670, //  01100001 01100000 00000000 00000000
    3720, //  00010001 01110000 00000000 00000000
    1674, //  01010001 01100000 00000000 00000000
    1676, //  00110001 01100000 00000000 00000000
    654, //  01110001 01000000 00000000 00000000
    3728, //  00001001 01110000 00000000 00000000
    1682, //  01001001 01100000 00000000 00000000
    1684, //  00101001 01100000 00000000 00000000
    662, //  01101001 01000000 00000000 00000000
    1688, //  00011001 01100000 00000000 00000000
    666, //  01011001 01000000 00000000 00000000
    668, //  00111001 01000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    3744, //  00000101 01110000 00000000 00000000
    1698, //  01000101 01100000 00000000 00000000
    1700, //  00100101 01100000 00000000 00000000
    678, //  01100101 01000000 00000000 00000000
    1704, //  00010101 01100000 00000000 00000000
    682, //  01010101 01000000 00000000 00000000
    684, //  00110101 01000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    1712, //  00001101 01100000 00000000 00000000
    690, //  01001101 01000000 00000000 00000000
    692, //  00101101 01000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    696, //  00011101 01000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3776, //  00000011 01110000 00000000 00000000
    1730, //  01000011 01100000 00000000 00000000
    1732, //  00100011 01100000 00000000 00000000
    710, //  01100011 01000000 00000000 00000000
    1736, //  00010011 01100000 00000000 00000000
    714, //  01010011 01000000 00000000 00000000
    716, //  00110011 01000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    1744, //  00001011 01100000 00000000 00000000
    722, //  01001011 01000000 00000000 00000000
    724, //  00101011 01000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    728, //  00011011 01000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1760, //  00000111 01100000 00000000 00000000
    738, //  01000111 01000000 00000000 00000000
    740, //  00100111 01000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    744, //  00010111 01000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    752, //  00001111 01000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    7936, //  00000000 11111000 00000000 00000000
    3842, //  01000000 11110000 00000000 00000000
    3844, //  00100000 11110000 00000000 00000000
    1798, //  01100000 11100000 00000000 00000000
    3848, //  00010000 11110000 00000000 00000000
    1802, //  01010000 11100000 00000000 00000000
    1804, //  00110000 11100000 00000000 00000000
    782, //  01110000 11000000 00000000 00000000
    3856, //  00001000 11110000 00000000 00000000
    1810, //  01001000 11100000 00000000 00000000
    1812, //  00101000 11100000 00000000 00000000
    790, //  01101000 11000000 00000000 00000000
    1816, //  00011000 11100000 00000000 00000000
    794, //  01011000 11000000 00000000 00000000
    796, //  00111000 11000000 00000000 00000000
    286, //  01111000 10000000 00000000 00000000
    3872, //  00000100 11110000 00000000 00000000
    1826, //  01000100 11100000 00000000 00000000
    1828, //  00100100 11100000 00000000 00000000
    806, //  01100100 11000000 00000000 00000000
    1832, //  00010100 11100000 00000000 00000000
    810, //  01010100 11000000 00000000 00000000
    812, //  00110100 11000000 00000000 00000000
    302, //  01110100 10000000 00000000 00000000
    1840, //  00001100 11100000 00000000 00000000
    818, //  01001100 11000000 00000000 00000000
    820, //  00101100 11000000 00000000 00000000
    310, //  01101100 10000000 00000000 00000000
    824, //  00011100 11000000 00000000 00000000
    314, //  01011100 10000000 00000000 00000000
    316, //  00111100 10000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3904, //  00000010 11110000 00000000 00000000
    1858, //  01000010 11100000 00000000 00000000
    1860, //  00100010 11100000 00000000 00000000
    838, //  01100010 11000000 00000000 00000000
    1864, //  00010010 11100000 00000000 00000000
    842, //  01010010 11000000 00000000 00000000
    844, //  00110010 11000000 00000000 00000000
    334, //  01110010 10000000 00000000 00000000
    1872, //  00001010 11100000 00000000 00000000
    850, //  01001010 11000000 00000000 00000000
    852, //  00101010 11000000 00000000 00000000
    342, //  01101010 10000000 00000000 00000000
    856, //  00011010 11000000 00000000 00000000
    346, //  01011010 10000000 00000000 00000000
    348, //  00111010 10000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    1888, //  00000110 11100000 00000000 00000000
    866, //  01000110 11000000 00000000 00000000
    868, //  00100110 11000000 00000000 00000000
    358, //  01100110 10000000 00000000 00000000
    872, //  00010110 11000000 00000000 00000000
    362, //  01010110 10000000 00000000 00000000
    364, //  00110110 10000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    880, //  00001110 11000000 00000000 00000000
    370, //  01001110 10000000 00000000 00000000
    372, //  00101110 10000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    376, //  00011110 10000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    3968, //  00000001 11110000 00000000 00000000
    1922, //  01000001 11100000 00000000 00000000
    1924, //  00100001 11100000 00000000 00000000
    902, //  01100001 11000000 00000000 00000000
    1928, //  00010001 11100000 00000000 00000000
    906, //  01010001 11000000 00000000 00000000
    908, //  00110001 11000000 00000000 00000000
    398, //  01110001 10000000 00000000 00000000
    1936, //  00001001 11100000 00000000 00000000
    914, //  01001001 11000000 00000000 00000000
    916, //  00101001 11000000 00000000 00000000
    406, //  01101001 10000000 00000000 00000000
    920, //  00011001 11000000 00000000 00000000
    410, //  01011001 10000000 00000000 00000000
    412, //  00111001 10000000 00000000 00000000
    158, //  01111001 00000000 00000000 00000000
    1952, //  00000101 11100000 00000000 00000000
    930, //  01000101 11000000 00000000 00000000
    932, //  00100101 11000000 00000000 00000000
    422, //  01100101 10000000 00000000 00000000
    936, //  00010101 11000000 00000000 00000000
    426, //  01010101 10000000 00000000 00000000
    428, //  00110101 10000000 00000000 00000000
    174, //  01110101 00000000 00000000 00000000
    944, //  00001101 11000000 00000000 00000000
    434, //  01001101 10000000 00000000 00000000
    436, //  00101101 10000000 00000000 00000000
    182, //  01101101 00000000 00000000 00000000
    440, //  00011101 10000000 00000000 00000000
    186, //  01011101 00000000 00000000 00000000
    188, //  00111101 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1984, //  00000011 11100000 00000000 00000000
    962, //  01000011 11000000 00000000 00000000
    964, //  00100011 11000000 00000000 00000000
    454, //  01100011 10000000 00000000 00000000
    968, //  00010011 11000000 00000000 00000000
    458, //  01010011 10000000 00000000 00000000
    460, //  00110011 10000000 00000000 00000000
    206, //  01110011 00000000 00000000 00000000
    976, //  00001011 11000000 00000000 00000000
    466, //  01001011 10000000 00000000 00000000
    468, //  00101011 10000000 00000000 00000000
    214, //  01101011 00000000 00000000 00000000
    472, //  00011011 10000000 00000000 00000000
    218, //  01011011 00000000 00000000 00000000
    220, //  00111011 00000000 00000000 00000000
    94, //  01111010 00000000 00000000 00000000
    992, //  00000111 11000000 00000000 00000000
    482, //  01000111 10000000 00000000 00000000
    484, //  00100111 10000000 00000000 00000000
    230, //  01100111 00000000 00000000 00000000
    488, //  00010111 10000000 00000000 00000000
    234, //  01010111 00000000 00000000 00000000
    236, //  00110111 00000000 00000000 00000000
    110, //  01110110 00000000 00000000 00000000
    496, //  00001111 10000000 00000000 00000000
    242, //  01001111 00000000 00000000 00000000
    244, //  00101111 00000000 00000000 00000000
    118, //  01101110 00000000 00000000 00000000
    248, //  00011111 00000000 00000000 00000000
    122, //  01011110 00000000 00000000 00000000
    124, //  00111110 00000000 00000000 00000000
    62, //  01111100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    3, //  11000000 00000000 00000000 00000000
    5, //  10100000 00000000 00000000 00000000
    7, //  11100000 00000000 00000000 00000000
    9, //  10010000 00000000 00000000 00000000
    11, //  11010000 00000000 00000000 00000000
    13, //  10110000 00000000 00000000 00000000
    15, //  11110000 00000000 00000000 00000000
    17, //  10001000 00000000 00000000 00000000
    19, //  11001000 00000000 00000000 00000000
    21, //  10101000 00000000 00000000 00000000
    23, //  11101000 00000000 00000000 00000000
    25, //  10011000 00000000 00000000 00000000
    27, //  11011000 00000000 00000000 00000000
    29, //  10111000 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    33, //  10000100 00000000 00000000 00000000
    35, //  11000100 00000000 00000000 00000000
    37, //  10100100 00000000 00000000 00000000
    39, //  11100100 00000000 00000000 00000000
    41, //  10010100 00000000 00000000 00000000
    43, //  11010100 00000000 00000000 00000000
    45, //  10110100 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    49, //  10001100 00000000 00000000 00000000
    51, //  11001100 00000000 00000000 00000000
    53, //  10101100 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    57, //  10011100 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    65, //  10000010 00000000 00000000 00000000
    67, //  11000010 00000000 00000000 00000000
    69, //  10100010 00000000 00000000 00000000
    71, //  11100010 00000000 00000000 00000000
    73, //  10010010 00000000 00000000 00000000
    75, //  11010010 00000000 00000000 00000000
    77, //  10110010 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    81, //  10001010 00000000 00000000 00000000
    83, //  11001010 00000000 00000000 00000000
    85, //  10101010 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    89, //  10011010 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    97, //  10000110 00000000 00000000 00000000
    99, //  11000110 00000000 00000000 00000000
    101, //  10100110 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    105, //  10010110 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    113, //  10001110 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    129, //  10000001 00000000 00000000 00000000
    131, //  11000001 00000000 00000000 00000000
    133, //  10100001 00000000 00000000 00000000
    135, //  11100001 00000000 00000000 00000000
    137, //  10010001 00000000 00000000 00000000
    139, //  11010001 00000000 00000000 00000000
    141, //  10110001 00000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    145, //  10001001 00000000 00000000 00000000
    147, //  11001001 00000000 00000000 00000000
    149, //  10101001 00000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    153, //  10011001 00000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    161, //  10000101 00000000 00000000 00000000
    163, //  11000101 00000000 00000000 00000000
    165, //  10100101 00000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    169, //  10010101 00000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    177, //  10001101 00000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    193, //  10000011 00000000 00000000 00000000
    195, //  11000011 00000000 00000000 00000000
    197, //  10100011 00000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    201, //  10010011 00000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    209, //  10001011 00000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    225, //  10000111 00000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    257, //  10000000 10000000 00000000 00000000
    259, //  11000000 10000000 00000000 00000000
    261, //  10100000 10000000 00000000 00000000
    263, //  11100000 10000000 00000000 00000000
    265, //  10010000 10000000 00000000 00000000
    267, //  11010000 10000000 00000000 00000000
    269, //  10110000 10000000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    273, //  10001000 10000000 00000000 00000000
    275, //  11001000 10000000 00000000 00000000
    277, //  10101000 10000000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    281, //  10011000 10000000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    289, //  10000100 10000000 00000000 00000000
    291, //  11000100 10000000 00000000 00000000
    293, //  10100100 10000000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    297, //  10010100 10000000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    305, //  10001100 10000000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    321, //  10000010 10000000 00000000 00000000
    323, //  11000010 10000000 00000000 00000000
    325, //  10100010 10000000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    329, //  10010010 10000000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    337, //  10001010 10000000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    353, //  10000110 10000000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    385, //  10000001 10000000 00000000 00000000
    387, //  11000001 10000000 00000000 00000000
    389, //  10100001 10000000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    393, //  10010001 10000000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    401, //  10001001 10000000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    417, //  10000101 10000000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    449, //  10000011 10000000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    513, //  10000000 01000000 00000000 00000000
    515, //  11000000 01000000 00000000 00000000
    517, //  10100000 01000000 00000000 00000000
    519, //  11100000 01000000 00000000 00000000
    521, //  10010000 01000000 00000000 00000000
    523, //  11010000 01000000 00000000 00000000
    525, //  10110000 01000000 00000000 00000000
    527, //  11110000 01000000 00000000 00000000
    529, //  10001000 01000000 00000000 00000000
    531, //  11001000 01000000 00000000 00000000
    533, //  10101000 01000000 00000000 00000000
    535, //  11101000 01000000 00000000 00000000
    537, //  10011000 01000000 00000000 00000000
    539, //  11011000 01000000 00000000 00000000
    541, //  10111000 01000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    545, //  10000100 01000000 00000000 00000000
    547, //  11000100 01000000 00000000 00000000
    549, //  10100100 01000000 00000000 00000000
    551, //  11100100 01000000 00000000 00000000
    553, //  10010100 01000000 00000000 00000000
    555, //  11010100 01000000 00000000 00000000
    557, //  10110100 01000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    561, //  10001100 01000000 00000000 00000000
    563, //  11001100 01000000 00000000 00000000
    565, //  10101100 01000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    569, //  10011100 01000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    577, //  10000010 01000000 00000000 00000000
    579, //  11000010 01000000 00000000 00000000
    581, //  10100010 01000000 00000000 00000000
    583, //  11100010 01000000 00000000 00000000
    585, //  10010010 01000000 00000000 00000000
    587, //  11010010 01000000 00000000 00000000
    589, //  10110010 01000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    593, //  10001010 01000000 00000000 00000000
    595, //  11001010 01000000 00000000 00000000
    597, //  10101010 01000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    601, //  10011010 01000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    609, //  10000110 01000000 00000000 00000000
    611, //  11000110 01000000 00000000 00000000
    613, //  10100110 01000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    617, //  10010110 01000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    625, //  10001110 01000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    641, //  10000001 01000000 00000000 00000000
    643, //  11000001 01000000 00000000 00000000
    645, //  10100001 01000000 00000000 00000000
    647, //  11100001 01000000 00000000 00000000
    649, //  10010001 01000000 00000000 00000000
    651, //  11010001 01000000 00000000 00000000
    653, //  10110001 01000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    657, //  10001001 01000000 00000000 00000000
    659, //  11001001 01000000 00000000 00000000
    661, //  10101001 01000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    665, //  10011001 01000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    673, //  10000101 01000000 00000000 00000000
    675, //  11000101 01000000 00000000 00000000
    677, //  10100101 01000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    681, //  10010101 01000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    689, //  10001101 01000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    705, //  10000011 01000000 00000000 00000000
    707, //  11000011 01000000 00000000 00000000
    709, //  10100011 01000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    713, //  10010011 01000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    721, //  10001011 01000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    737, //  10000111 01000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    769, //  10000000 11000000 00000000 00000000
    771, //  11000000 11000000 00000000 00000000
    773, //  10100000 11000000 00000000 00000000
    775, //  11100000 11000000 00000000 00000000
    777, //  10010000 11000000 00000000 00000000
    779, //  11010000 11000000 00000000 00000000
    781, //  10110000 11000000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    785, //  10001000 11000000 00000000 00000000
    787, //  11001000 11000000 00000000 00000000
    789, //  10101000 11000000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    793, //  10011000 11000000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    801, //  10000100 11000000 00000000 00000000
    803, //  11000100 11000000 00000000 00000000
    805, //  10100100 11000000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    809, //  10010100 11000000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    817, //  10001100 11000000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    833, //  10000010 11000000 00000000 00000000
    835, //  11000010 11000000 00000000 00000000
    837, //  10100010 11000000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    841, //  10010010 11000000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    849, //  10001010 11000000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    865, //  10000110 11000000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    897, //  10000001 11000000 00000000 00000000
    899, //  11000001 11000000 00000000 00000000
    901, //  10100001 11000000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    905, //  10010001 11000000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    913, //  10001001 11000000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    929, //  10000101 11000000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    961, //  10000011 11000000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1025, //  10000000 00100000 00000000 00000000
    1027, //  11000000 00100000 00000000 00000000
    1029, //  10100000 00100000 00000000 00000000
    1031, //  11100000 00100000 00000000 00000000
    1033, //  10010000 00100000 00000000 00000000
    1035, //  11010000 00100000 00000000 00000000
    1037, //  10110000 00100000 00000000 00000000
    1039, //  11110000 00100000 00000000 00000000
    1041, //  10001000 00100000 00000000 00000000
    1043, //  11001000 00100000 00000000 00000000
    1045, //  10101000 00100000 00000000 00000000
    1047, //  11101000 00100000 00000000 00000000
    1049, //  10011000 00100000 00000000 00000000
    1051, //  11011000 00100000 00000000 00000000
    1053, //  10111000 00100000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1057, //  10000100 00100000 00000000 00000000
    1059, //  11000100 00100000 00000000 00000000
    1061, //  10100100 00100000 00000000 00000000
    1063, //  11100100 00100000 00000000 00000000
    1065, //  10010100 00100000 00000000 00000000
    1067, //  11010100 00100000 00000000 00000000
    1069, //  10110100 00100000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1073, //  10001100 00100000 00000000 00000000
    1075, //  11001100 00100000 00000000 00000000
    1077, //  10101100 00100000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    1081, //  10011100 00100000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1089, //  10000010 00100000 00000000 00000000
    1091, //  11000010 00100000 00000000 00000000
    1093, //  10100010 00100000 00000000 00000000
    1095, //  11100010 00100000 00000000 00000000
    1097, //  10010010 00100000 00000000 00000000
    1099, //  11010010 00100000 00000000 00000000
    1101, //  10110010 00100000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1105, //  10001010 00100000 00000000 00000000
    1107, //  11001010 00100000 00000000 00000000
    1109, //  10101010 00100000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    1113, //  10011010 00100000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1121, //  10000110 00100000 00000000 00000000
    1123, //  11000110 00100000 00000000 00000000
    1125, //  10100110 00100000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    1129, //  10010110 00100000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1137, //  10001110 00100000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1153, //  10000001 00100000 00000000 00000000
    1155, //  11000001 00100000 00000000 00000000
    1157, //  10100001 00100000 00000000 00000000
    1159, //  11100001 00100000 00000000 00000000
    1161, //  10010001 00100000 00000000 00000000
    1163, //  11010001 00100000 00000000 00000000
    1165, //  10110001 00100000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    1169, //  10001001 00100000 00000000 00000000
    1171, //  11001001 00100000 00000000 00000000
    1173, //  10101001 00100000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    1177, //  10011001 00100000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1185, //  10000101 00100000 00000000 00000000
    1187, //  11000101 00100000 00000000 00000000
    1189, //  10100101 00100000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    1193, //  10010101 00100000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1201, //  10001101 00100000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1217, //  10000011 00100000 00000000 00000000
    1219, //  11000011 00100000 00000000 00000000
    1221, //  10100011 00100000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    1225, //  10010011 00100000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1233, //  10001011 00100000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1249, //  10000111 00100000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1281, //  10000000 10100000 00000000 00000000
    1283, //  11000000 10100000 00000000 00000000
    1285, //  10100000 10100000 00000000 00000000
    1287, //  11100000 10100000 00000000 00000000
    1289, //  10010000 10100000 00000000 00000000
    1291, //  11010000 10100000 00000000 00000000
    1293, //  10110000 10100000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    1297, //  10001000 10100000 00000000 00000000
    1299, //  11001000 10100000 00000000 00000000
    1301, //  10101000 10100000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    1305, //  10011000 10100000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1313, //  10000100 10100000 00000000 00000000
    1315, //  11000100 10100000 00000000 00000000
    1317, //  10100100 10100000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    1321, //  10010100 10100000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1329, //  10001100 10100000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1345, //  10000010 10100000 00000000 00000000
    1347, //  11000010 10100000 00000000 00000000
    1349, //  10100010 10100000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    1353, //  10010010 10100000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1361, //  10001010 10100000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1377, //  10000110 10100000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1409, //  10000001 10100000 00000000 00000000
    1411, //  11000001 10100000 00000000 00000000
    1413, //  10100001 10100000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    1417, //  10010001 10100000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    1425, //  10001001 10100000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1441, //  10000101 10100000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1473, //  10000011 10100000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1537, //  10000000 01100000 00000000 00000000
    1539, //  11000000 01100000 00000000 00000000
    1541, //  10100000 01100000 00000000 00000000
    1543, //  11100000 01100000 00000000 00000000
    1545, //  10010000 01100000 00000000 00000000
    1547, //  11010000 01100000 00000000 00000000
    1549, //  10110000 01100000 00000000 00000000
    527, //  11110000 01000000 00000000 00000000
    1553, //  10001000 01100000 00000000 00000000
    1555, //  11001000 01100000 00000000 00000000
    1557, //  10101000 01100000 00000000 00000000
    535, //  11101000 01000000 00000000 00000000
    1561, //  10011000 01100000 00000000 00000000
    539, //  11011000 01000000 00000000 00000000
    541, //  10111000 01000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1569, //  10000100 01100000 00000000 00000000
    1571, //  11000100 01100000 00000000 00000000
    1573, //  10100100 01100000 00000000 00000000
    551, //  11100100 01000000 00000000 00000000
    1577, //  10010100 01100000 00000000 00000000
    555, //  11010100 01000000 00000000 00000000
    557, //  10110100 01000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1585, //  10001100 01100000 00000000 00000000
    563, //  11001100 01000000 00000000 00000000
    565, //  10101100 01000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    569, //  10011100 01000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1601, //  10000010 01100000 00000000 00000000
    1603, //  11000010 01100000 00000000 00000000
    1605, //  10100010 01100000 00000000 00000000
    583, //  11100010 01000000 00000000 00000000
    1609, //  10010010 01100000 00000000 00000000
    587, //  11010010 01000000 00000000 00000000
    589, //  10110010 01000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1617, //  10001010 01100000 00000000 00000000
    595, //  11001010 01000000 00000000 00000000
    597, //  10101010 01000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    601, //  10011010 01000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1633, //  10000110 01100000 00000000 00000000
    611, //  11000110 01000000 00000000 00000000
    613, //  10100110 01000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    617, //  10010110 01000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    625, //  10001110 01000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1665, //  10000001 01100000 00000000 00000000
    1667, //  11000001 01100000 00000000 00000000
    1669, //  10100001 01100000 00000000 00000000
    647, //  11100001 01000000 00000000 00000000
    1673, //  10010001 01100000 00000000 00000000
    651, //  11010001 01000000 00000000 00000000
    653, //  10110001 01000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    1681, //  10001001 01100000 00000000 00000000
    659, //  11001001 01000000 00000000 00000000
    661, //  10101001 01000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    665, //  10011001 01000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1697, //  10000101 01100000 00000000 00000000
    675, //  11000101 01000000 00000000 00000000
    677, //  10100101 01000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    681, //  10010101 01000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    689, //  10001101 01000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1729, //  10000011 01100000 00000000 00000000
    707, //  11000011 01000000 00000000 00000000
    709, //  10100011 01000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    713, //  10010011 01000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    721, //  10001011 01000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    737, //  10000111 01000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1793, //  10000000 11100000 00000000 00000000
    1795, //  11000000 11100000 00000000 00000000
    1797, //  10100000 11100000 00000000 00000000
    775, //  11100000 11000000 00000000 00000000
    1801, //  10010000 11100000 00000000 00000000
    779, //  11010000 11000000 00000000 00000000
    781, //  10110000 11000000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    1809, //  10001000 11100000 00000000 00000000
    787, //  11001000 11000000 00000000 00000000
    789, //  10101000 11000000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    793, //  10011000 11000000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1825, //  10000100 11100000 00000000 00000000
    803, //  11000100 11000000 00000000 00000000
    805, //  10100100 11000000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    809, //  10010100 11000000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    817, //  10001100 11000000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1857, //  10000010 11100000 00000000 00000000
    835, //  11000010 11000000 00000000 00000000
    837, //  10100010 11000000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    841, //  10010010 11000000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    849, //  10001010 11000000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    865, //  10000110 11000000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1921, //  10000001 11100000 00000000 00000000
    899, //  11000001 11000000 00000000 00000000
    901, //  10100001 11000000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    905, //  10010001 11000000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    913, //  10001001 11000000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    929, //  10000101 11000000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    961, //  10000011 11000000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2049, //  10000000 00010000 00000000 00000000
    2051, //  11000000 00010000 00000000 00000000
    2053, //  10100000 00010000 00000000 00000000
    2055, //  11100000 00010000 00000000 00000000
    2057, //  10010000 00010000 00000000 00000000
    2059, //  11010000 00010000 00000000 00000000
    2061, //  10110000 00010000 00000000 00000000
    2063, //  11110000 00010000 00000000 00000000
    2065, //  10001000 00010000 00000000 00000000
    2067, //  11001000 00010000 00000000 00000000
    2069, //  10101000 00010000 00000000 00000000
    2071, //  11101000 00010000 00000000 00000000
    2073, //  10011000 00010000 00000000 00000000
    2075, //  11011000 00010000 00000000 00000000
    2077, //  10111000 00010000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2081, //  10000100 00010000 00000000 00000000
    2083, //  11000100 00010000 00000000 00000000
    2085, //  10100100 00010000 00000000 00000000
    2087, //  11100100 00010000 00000000 00000000
    2089, //  10010100 00010000 00000000 00000000
    2091, //  11010100 00010000 00000000 00000000
    2093, //  10110100 00010000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    2097, //  10001100 00010000 00000000 00000000
    2099, //  11001100 00010000 00000000 00000000
    2101, //  10101100 00010000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    2105, //  10011100 00010000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2113, //  10000010 00010000 00000000 00000000
    2115, //  11000010 00010000 00000000 00000000
    2117, //  10100010 00010000 00000000 00000000
    2119, //  11100010 00010000 00000000 00000000
    2121, //  10010010 00010000 00000000 00000000
    2123, //  11010010 00010000 00000000 00000000
    2125, //  10110010 00010000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    2129, //  10001010 00010000 00000000 00000000
    2131, //  11001010 00010000 00000000 00000000
    2133, //  10101010 00010000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    2137, //  10011010 00010000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2145, //  10000110 00010000 00000000 00000000
    2147, //  11000110 00010000 00000000 00000000
    2149, //  10100110 00010000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    2153, //  10010110 00010000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    2161, //  10001110 00010000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2177, //  10000001 00010000 00000000 00000000
    2179, //  11000001 00010000 00000000 00000000
    2181, //  10100001 00010000 00000000 00000000
    2183, //  11100001 00010000 00000000 00000000
    2185, //  10010001 00010000 00000000 00000000
    2187, //  11010001 00010000 00000000 00000000
    2189, //  10110001 00010000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    2193, //  10001001 00010000 00000000 00000000
    2195, //  11001001 00010000 00000000 00000000
    2197, //  10101001 00010000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    2201, //  10011001 00010000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2209, //  10000101 00010000 00000000 00000000
    2211, //  11000101 00010000 00000000 00000000
    2213, //  10100101 00010000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    2217, //  10010101 00010000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    2225, //  10001101 00010000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2241, //  10000011 00010000 00000000 00000000
    2243, //  11000011 00010000 00000000 00000000
    2245, //  10100011 00010000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    2249, //  10010011 00010000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    2257, //  10001011 00010000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2273, //  10000111 00010000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2305, //  10000000 10010000 00000000 00000000
    2307, //  11000000 10010000 00000000 00000000
    2309, //  10100000 10010000 00000000 00000000
    2311, //  11100000 10010000 00000000 00000000
    2313, //  10010000 10010000 00000000 00000000
    2315, //  11010000 10010000 00000000 00000000
    2317, //  10110000 10010000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    2321, //  10001000 10010000 00000000 00000000
    2323, //  11001000 10010000 00000000 00000000
    2325, //  10101000 10010000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    2329, //  10011000 10010000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2337, //  10000100 10010000 00000000 00000000
    2339, //  11000100 10010000 00000000 00000000
    2341, //  10100100 10010000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    2345, //  10010100 10010000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    2353, //  10001100 10010000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2369, //  10000010 10010000 00000000 00000000
    2371, //  11000010 10010000 00000000 00000000
    2373, //  10100010 10010000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    2377, //  10010010 10010000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    2385, //  10001010 10010000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2401, //  10000110 10010000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2433, //  10000001 10010000 00000000 00000000
    2435, //  11000001 10010000 00000000 00000000
    2437, //  10100001 10010000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    2441, //  10010001 10010000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    2449, //  10001001 10010000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2465, //  10000101 10010000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2497, //  10000011 10010000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2561, //  10000000 01010000 00000000 00000000
    2563, //  11000000 01010000 00000000 00000000
    2565, //  10100000 01010000 00000000 00000000
    2567, //  11100000 01010000 00000000 00000000
    2569, //  10010000 01010000 00000000 00000000
    2571, //  11010000 01010000 00000000 00000000
    2573, //  10110000 01010000 00000000 00000000
    527, //  11110000 01000000 00000000 00000000
    2577, //  10001000 01010000 00000000 00000000
    2579, //  11001000 01010000 00000000 00000000
    2581, //  10101000 01010000 00000000 00000000
    535, //  11101000 01000000 00000000 00000000
    2585, //  10011000 01010000 00000000 00000000
    539, //  11011000 01000000 00000000 00000000
    541, //  10111000 01000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2593, //  10000100 01010000 00000000 00000000
    2595, //  11000100 01010000 00000000 00000000
    2597, //  10100100 01010000 00000000 00000000
    551, //  11100100 01000000 00000000 00000000
    2601, //  10010100 01010000 00000000 00000000
    555, //  11010100 01000000 00000000 00000000
    557, //  10110100 01000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    2609, //  10001100 01010000 00000000 00000000
    563, //  11001100 01000000 00000000 00000000
    565, //  10101100 01000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    569, //  10011100 01000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2625, //  10000010 01010000 00000000 00000000
    2627, //  11000010 01010000 00000000 00000000
    2629, //  10100010 01010000 00000000 00000000
    583, //  11100010 01000000 00000000 00000000
    2633, //  10010010 01010000 00000000 00000000
    587, //  11010010 01000000 00000000 00000000
    589, //  10110010 01000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    2641, //  10001010 01010000 00000000 00000000
    595, //  11001010 01000000 00000000 00000000
    597, //  10101010 01000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    601, //  10011010 01000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2657, //  10000110 01010000 00000000 00000000
    611, //  11000110 01000000 00000000 00000000
    613, //  10100110 01000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    617, //  10010110 01000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    625, //  10001110 01000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2689, //  10000001 01010000 00000000 00000000
    2691, //  11000001 01010000 00000000 00000000
    2693, //  10100001 01010000 00000000 00000000
    647, //  11100001 01000000 00000000 00000000
    2697, //  10010001 01010000 00000000 00000000
    651, //  11010001 01000000 00000000 00000000
    653, //  10110001 01000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    2705, //  10001001 01010000 00000000 00000000
    659, //  11001001 01000000 00000000 00000000
    661, //  10101001 01000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    665, //  10011001 01000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2721, //  10000101 01010000 00000000 00000000
    675, //  11000101 01000000 00000000 00000000
    677, //  10100101 01000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    681, //  10010101 01000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    689, //  10001101 01000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2753, //  10000011 01010000 00000000 00000000
    707, //  11000011 01000000 00000000 00000000
    709, //  10100011 01000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    713, //  10010011 01000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    721, //  10001011 01000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    737, //  10000111 01000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2817, //  10000000 11010000 00000000 00000000
    2819, //  11000000 11010000 00000000 00000000
    2821, //  10100000 11010000 00000000 00000000
    775, //  11100000 11000000 00000000 00000000
    2825, //  10010000 11010000 00000000 00000000
    779, //  11010000 11000000 00000000 00000000
    781, //  10110000 11000000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    2833, //  10001000 11010000 00000000 00000000
    787, //  11001000 11000000 00000000 00000000
    789, //  10101000 11000000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    793, //  10011000 11000000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2849, //  10000100 11010000 00000000 00000000
    803, //  11000100 11000000 00000000 00000000
    805, //  10100100 11000000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    809, //  10010100 11000000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    817, //  10001100 11000000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2881, //  10000010 11010000 00000000 00000000
    835, //  11000010 11000000 00000000 00000000
    837, //  10100010 11000000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    841, //  10010010 11000000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    849, //  10001010 11000000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    865, //  10000110 11000000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2945, //  10000001 11010000 00000000 00000000
    899, //  11000001 11000000 00000000 00000000
    901, //  10100001 11000000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    905, //  10010001 11000000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    913, //  10001001 11000000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    929, //  10000101 11000000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    961, //  10000011 11000000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3073, //  10000000 00110000 00000000 00000000
    3075, //  11000000 00110000 00000000 00000000
    3077, //  10100000 00110000 00000000 00000000
    3079, //  11100000 00110000 00000000 00000000
    3081, //  10010000 00110000 00000000 00000000
    3083, //  11010000 00110000 00000000 00000000
    3085, //  10110000 00110000 00000000 00000000
    1039, //  11110000 00100000 00000000 00000000
    3089, //  10001000 00110000 00000000 00000000
    3091, //  11001000 00110000 00000000 00000000
    3093, //  10101000 00110000 00000000 00000000
    1047, //  11101000 00100000 00000000 00000000
    3097, //  10011000 00110000 00000000 00000000
    1051, //  11011000 00100000 00000000 00000000
    1053, //  10111000 00100000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3105, //  10000100 00110000 00000000 00000000
    3107, //  11000100 00110000 00000000 00000000
    3109, //  10100100 00110000 00000000 00000000
    1063, //  11100100 00100000 00000000 00000000
    3113, //  10010100 00110000 00000000 00000000
    1067, //  11010100 00100000 00000000 00000000
    1069, //  10110100 00100000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    3121, //  10001100 00110000 00000000 00000000
    1075, //  11001100 00100000 00000000 00000000
    1077, //  10101100 00100000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    1081, //  10011100 00100000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3137, //  10000010 00110000 00000000 00000000
    3139, //  11000010 00110000 00000000 00000000
    3141, //  10100010 00110000 00000000 00000000
    1095, //  11100010 00100000 00000000 00000000
    3145, //  10010010 00110000 00000000 00000000
    1099, //  11010010 00100000 00000000 00000000
    1101, //  10110010 00100000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    3153, //  10001010 00110000 00000000 00000000
    1107, //  11001010 00100000 00000000 00000000
    1109, //  10101010 00100000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    1113, //  10011010 00100000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3169, //  10000110 00110000 00000000 00000000
    1123, //  11000110 00100000 00000000 00000000
    1125, //  10100110 00100000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    1129, //  10010110 00100000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1137, //  10001110 00100000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3201, //  10000001 00110000 00000000 00000000
    3203, //  11000001 00110000 00000000 00000000
    3205, //  10100001 00110000 00000000 00000000
    1159, //  11100001 00100000 00000000 00000000
    3209, //  10010001 00110000 00000000 00000000
    1163, //  11010001 00100000 00000000 00000000
    1165, //  10110001 00100000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    3217, //  10001001 00110000 00000000 00000000
    1171, //  11001001 00100000 00000000 00000000
    1173, //  10101001 00100000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    1177, //  10011001 00100000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3233, //  10000101 00110000 00000000 00000000
    1187, //  11000101 00100000 00000000 00000000
    1189, //  10100101 00100000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    1193, //  10010101 00100000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1201, //  10001101 00100000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3265, //  10000011 00110000 00000000 00000000
    1219, //  11000011 00100000 00000000 00000000
    1221, //  10100011 00100000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    1225, //  10010011 00100000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1233, //  10001011 00100000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1249, //  10000111 00100000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3329, //  10000000 10110000 00000000 00000000
    3331, //  11000000 10110000 00000000 00000000
    3333, //  10100000 10110000 00000000 00000000
    1287, //  11100000 10100000 00000000 00000000
    3337, //  10010000 10110000 00000000 00000000
    1291, //  11010000 10100000 00000000 00000000
    1293, //  10110000 10100000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    3345, //  10001000 10110000 00000000 00000000
    1299, //  11001000 10100000 00000000 00000000
    1301, //  10101000 10100000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    1305, //  10011000 10100000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3361, //  10000100 10110000 00000000 00000000
    1315, //  11000100 10100000 00000000 00000000
    1317, //  10100100 10100000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    1321, //  10010100 10100000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1329, //  10001100 10100000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3393, //  10000010 10110000 00000000 00000000
    1347, //  11000010 10100000 00000000 00000000
    1349, //  10100010 10100000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    1353, //  10010010 10100000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1361, //  10001010 10100000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1377, //  10000110 10100000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3457, //  10000001 10110000 00000000 00000000
    1411, //  11000001 10100000 00000000 00000000
    1413, //  10100001 10100000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    1417, //  10010001 10100000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    1425, //  10001001 10100000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1441, //  10000101 10100000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1473, //  10000011 10100000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3585, //  10000000 01110000 00000000 00000000
    3587, //  11000000 01110000 00000000 00000000
    3589, //  10100000 01110000 00000000 00000000
    1543, //  11100000 01100000 00000000 00000000
    3593, //  10010000 01110000 00000000 00000000
    1547, //  11010000 01100000 00000000 00000000
    1549, //  10110000 01100000 00000000 00000000
    527, //  11110000 01000000 00000000 00000000
    3601, //  10001000 01110000 00000000 00000000
    1555, //  11001000 01100000 00000000 00000000
    1557, //  10101000 01100000 00000000 00000000
    535, //  11101000 01000000 00000000 00000000
    1561, //  10011000 01100000 00000000 00000000
    539, //  11011000 01000000 00000000 00000000
    541, //  10111000 01000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3617, //  10000100 01110000 00000000 00000000
    1571, //  11000100 01100000 00000000 00000000
    1573, //  10100100 01100000 00000000 00000000
    551, //  11100100 01000000 00000000 00000000
    1577, //  10010100 01100000 00000000 00000000
    555, //  11010100 01000000 00000000 00000000
    557, //  10110100 01000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1585, //  10001100 01100000 00000000 00000000
    563, //  11001100 01000000 00000000 00000000
    565, //  10101100 01000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    569, //  10011100 01000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3649, //  10000010 01110000 00000000 00000000
    1603, //  11000010 01100000 00000000 00000000
    1605, //  10100010 01100000 00000000 00000000
    583, //  11100010 01000000 00000000 00000000
    1609, //  10010010 01100000 00000000 00000000
    587, //  11010010 01000000 00000000 00000000
    589, //  10110010 01000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1617, //  10001010 01100000 00000000 00000000
    595, //  11001010 01000000 00000000 00000000
    597, //  10101010 01000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    601, //  10011010 01000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1633, //  10000110 01100000 00000000 00000000
    611, //  11000110 01000000 00000000 00000000
    613, //  10100110 01000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    617, //  10010110 01000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    625, //  10001110 01000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3713, //  10000001 01110000 00000000 00000000
    1667, //  11000001 01100000 00000000 00000000
    1669, //  10100001 01100000 00000000 00000000
    647, //  11100001 01000000 00000000 00000000
    1673, //  10010001 01100000 00000000 00000000
    651, //  11010001 01000000 00000000 00000000
    653, //  10110001 01000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    1681, //  10001001 01100000 00000000 00000000
    659, //  11001001 01000000 00000000 00000000
    661, //  10101001 01000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    665, //  10011001 01000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1697, //  10000101 01100000 00000000 00000000
    675, //  11000101 01000000 00000000 00000000
    677, //  10100101 01000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    681, //  10010101 01000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    689, //  10001101 01000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1729, //  10000011 01100000 00000000 00000000
    707, //  11000011 01000000 00000000 00000000
    709, //  10100011 01000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    713, //  10010011 01000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    721, //  10001011 01000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    737, //  10000111 01000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3841, //  10000000 11110000 00000000 00000000
    1795, //  11000000 11100000 00000000 00000000
    1797, //  10100000 11100000 00000000 00000000
    775, //  11100000 11000000 00000000 00000000
    1801, //  10010000 11100000 00000000 00000000
    779, //  11010000 11000000 00000000 00000000
    781, //  10110000 11000000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    1809, //  10001000 11100000 00000000 00000000
    787, //  11001000 11000000 00000000 00000000
    789, //  10101000 11000000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    793, //  10011000 11000000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1825, //  10000100 11100000 00000000 00000000
    803, //  11000100 11000000 00000000 00000000
    805, //  10100100 11000000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    809, //  10010100 11000000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    817, //  10001100 11000000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1857, //  10000010 11100000 00000000 00000000
    835, //  11000010 11000000 00000000 00000000
    837, //  10100010 11000000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    841, //  10010010 11000000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    849, //  10001010 11000000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    865, //  10000110 11000000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1921, //  10000001 11100000 00000000 00000000
    899, //  11000001 11000000 00000000 00000000
    901, //  10100001 11000000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    905, //  10010001 11000000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    913, //  10001001 11000000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    929, //  10000101 11000000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    961, //  10000011 11000000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4097, //  10000000 00001000 00000000 00000000
    4099, //  11000000 00001000 00000000 00000000
    4101, //  10100000 00001000 00000000 00000000
    4103, //  11100000 00001000 00000000 00000000
    4105, //  10010000 00001000 00000000 00000000
    4107, //  11010000 00001000 00000000 00000000
    4109, //  10110000 00001000 00000000 00000000
    4111, //  11110000 00001000 00000000 00000000
    4113, //  10001000 00001000 00000000 00000000
    4115, //  11001000 00001000 00000000 00000000
    4117, //  10101000 00001000 00000000 00000000
    4119, //  11101000 00001000 00000000 00000000
    4121, //  10011000 00001000 00000000 00000000
    4123, //  11011000 00001000 00000000 00000000
    4125, //  10111000 00001000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4129, //  10000100 00001000 00000000 00000000
    4131, //  11000100 00001000 00000000 00000000
    4133, //  10100100 00001000 00000000 00000000
    4135, //  11100100 00001000 00000000 00000000
    4137, //  10010100 00001000 00000000 00000000
    4139, //  11010100 00001000 00000000 00000000
    4141, //  10110100 00001000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    4145, //  10001100 00001000 00000000 00000000
    4147, //  11001100 00001000 00000000 00000000
    4149, //  10101100 00001000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    4153, //  10011100 00001000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4161, //  10000010 00001000 00000000 00000000
    4163, //  11000010 00001000 00000000 00000000
    4165, //  10100010 00001000 00000000 00000000
    4167, //  11100010 00001000 00000000 00000000
    4169, //  10010010 00001000 00000000 00000000
    4171, //  11010010 00001000 00000000 00000000
    4173, //  10110010 00001000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    4177, //  10001010 00001000 00000000 00000000
    4179, //  11001010 00001000 00000000 00000000
    4181, //  10101010 00001000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    4185, //  10011010 00001000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4193, //  10000110 00001000 00000000 00000000
    4195, //  11000110 00001000 00000000 00000000
    4197, //  10100110 00001000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    4201, //  10010110 00001000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    4209, //  10001110 00001000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4225, //  10000001 00001000 00000000 00000000
    4227, //  11000001 00001000 00000000 00000000
    4229, //  10100001 00001000 00000000 00000000
    4231, //  11100001 00001000 00000000 00000000
    4233, //  10010001 00001000 00000000 00000000
    4235, //  11010001 00001000 00000000 00000000
    4237, //  10110001 00001000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    4241, //  10001001 00001000 00000000 00000000
    4243, //  11001001 00001000 00000000 00000000
    4245, //  10101001 00001000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    4249, //  10011001 00001000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4257, //  10000101 00001000 00000000 00000000
    4259, //  11000101 00001000 00000000 00000000
    4261, //  10100101 00001000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    4265, //  10010101 00001000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    4273, //  10001101 00001000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4289, //  10000011 00001000 00000000 00000000
    4291, //  11000011 00001000 00000000 00000000
    4293, //  10100011 00001000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    4297, //  10010011 00001000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    4305, //  10001011 00001000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4321, //  10000111 00001000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4353, //  10000000 10001000 00000000 00000000
    4355, //  11000000 10001000 00000000 00000000
    4357, //  10100000 10001000 00000000 00000000
    4359, //  11100000 10001000 00000000 00000000
    4361, //  10010000 10001000 00000000 00000000
    4363, //  11010000 10001000 00000000 00000000
    4365, //  10110000 10001000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    4369, //  10001000 10001000 00000000 00000000
    4371, //  11001000 10001000 00000000 00000000
    4373, //  10101000 10001000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    4377, //  10011000 10001000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4385, //  10000100 10001000 00000000 00000000
    4387, //  11000100 10001000 00000000 00000000
    4389, //  10100100 10001000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    4393, //  10010100 10001000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    4401, //  10001100 10001000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4417, //  10000010 10001000 00000000 00000000
    4419, //  11000010 10001000 00000000 00000000
    4421, //  10100010 10001000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    4425, //  10010010 10001000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    4433, //  10001010 10001000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4449, //  10000110 10001000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4481, //  10000001 10001000 00000000 00000000
    4483, //  11000001 10001000 00000000 00000000
    4485, //  10100001 10001000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    4489, //  10010001 10001000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    4497, //  10001001 10001000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4513, //  10000101 10001000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4545, //  10000011 10001000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4609, //  10000000 01001000 00000000 00000000
    4611, //  11000000 01001000 00000000 00000000
    4613, //  10100000 01001000 00000000 00000000
    4615, //  11100000 01001000 00000000 00000000
    4617, //  10010000 01001000 00000000 00000000
    4619, //  11010000 01001000 00000000 00000000
    4621, //  10110000 01001000 00000000 00000000
    527, //  11110000 01000000 00000000 00000000
    4625, //  10001000 01001000 00000000 00000000
    4627, //  11001000 01001000 00000000 00000000
    4629, //  10101000 01001000 00000000 00000000
    535, //  11101000 01000000 00000000 00000000
    4633, //  10011000 01001000 00000000 00000000
    539, //  11011000 01000000 00000000 00000000
    541, //  10111000 01000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4641, //  10000100 01001000 00000000 00000000
    4643, //  11000100 01001000 00000000 00000000
    4645, //  10100100 01001000 00000000 00000000
    551, //  11100100 01000000 00000000 00000000
    4649, //  10010100 01001000 00000000 00000000
    555, //  11010100 01000000 00000000 00000000
    557, //  10110100 01000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    4657, //  10001100 01001000 00000000 00000000
    563, //  11001100 01000000 00000000 00000000
    565, //  10101100 01000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    569, //  10011100 01000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4673, //  10000010 01001000 00000000 00000000
    4675, //  11000010 01001000 00000000 00000000
    4677, //  10100010 01001000 00000000 00000000
    583, //  11100010 01000000 00000000 00000000
    4681, //  10010010 01001000 00000000 00000000
    587, //  11010010 01000000 00000000 00000000
    589, //  10110010 01000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    4689, //  10001010 01001000 00000000 00000000
    595, //  11001010 01000000 00000000 00000000
    597, //  10101010 01000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    601, //  10011010 01000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4705, //  10000110 01001000 00000000 00000000
    611, //  11000110 01000000 00000000 00000000
    613, //  10100110 01000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    617, //  10010110 01000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    625, //  10001110 01000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4737, //  10000001 01001000 00000000 00000000
    4739, //  11000001 01001000 00000000 00000000
    4741, //  10100001 01001000 00000000 00000000
    647, //  11100001 01000000 00000000 00000000
    4745, //  10010001 01001000 00000000 00000000
    651, //  11010001 01000000 00000000 00000000
    653, //  10110001 01000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    4753, //  10001001 01001000 00000000 00000000
    659, //  11001001 01000000 00000000 00000000
    661, //  10101001 01000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    665, //  10011001 01000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4769, //  10000101 01001000 00000000 00000000
    675, //  11000101 01000000 00000000 00000000
    677, //  10100101 01000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    681, //  10010101 01000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    689, //  10001101 01000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4801, //  10000011 01001000 00000000 00000000
    707, //  11000011 01000000 00000000 00000000
    709, //  10100011 01000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    713, //  10010011 01000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    721, //  10001011 01000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    737, //  10000111 01000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4865, //  10000000 11001000 00000000 00000000
    4867, //  11000000 11001000 00000000 00000000
    4869, //  10100000 11001000 00000000 00000000
    775, //  11100000 11000000 00000000 00000000
    4873, //  10010000 11001000 00000000 00000000
    779, //  11010000 11000000 00000000 00000000
    781, //  10110000 11000000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    4881, //  10001000 11001000 00000000 00000000
    787, //  11001000 11000000 00000000 00000000
    789, //  10101000 11000000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    793, //  10011000 11000000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4897, //  10000100 11001000 00000000 00000000
    803, //  11000100 11000000 00000000 00000000
    805, //  10100100 11000000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    809, //  10010100 11000000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    817, //  10001100 11000000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4929, //  10000010 11001000 00000000 00000000
    835, //  11000010 11000000 00000000 00000000
    837, //  10100010 11000000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    841, //  10010010 11000000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    849, //  10001010 11000000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    865, //  10000110 11000000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    4993, //  10000001 11001000 00000000 00000000
    899, //  11000001 11000000 00000000 00000000
    901, //  10100001 11000000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    905, //  10010001 11000000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    913, //  10001001 11000000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    929, //  10000101 11000000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    961, //  10000011 11000000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5121, //  10000000 00101000 00000000 00000000
    5123, //  11000000 00101000 00000000 00000000
    5125, //  10100000 00101000 00000000 00000000
    5127, //  11100000 00101000 00000000 00000000
    5129, //  10010000 00101000 00000000 00000000
    5131, //  11010000 00101000 00000000 00000000
    5133, //  10110000 00101000 00000000 00000000
    1039, //  11110000 00100000 00000000 00000000
    5137, //  10001000 00101000 00000000 00000000
    5139, //  11001000 00101000 00000000 00000000
    5141, //  10101000 00101000 00000000 00000000
    1047, //  11101000 00100000 00000000 00000000
    5145, //  10011000 00101000 00000000 00000000
    1051, //  11011000 00100000 00000000 00000000
    1053, //  10111000 00100000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5153, //  10000100 00101000 00000000 00000000
    5155, //  11000100 00101000 00000000 00000000
    5157, //  10100100 00101000 00000000 00000000
    1063, //  11100100 00100000 00000000 00000000
    5161, //  10010100 00101000 00000000 00000000
    1067, //  11010100 00100000 00000000 00000000
    1069, //  10110100 00100000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    5169, //  10001100 00101000 00000000 00000000
    1075, //  11001100 00100000 00000000 00000000
    1077, //  10101100 00100000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    1081, //  10011100 00100000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5185, //  10000010 00101000 00000000 00000000
    5187, //  11000010 00101000 00000000 00000000
    5189, //  10100010 00101000 00000000 00000000
    1095, //  11100010 00100000 00000000 00000000
    5193, //  10010010 00101000 00000000 00000000
    1099, //  11010010 00100000 00000000 00000000
    1101, //  10110010 00100000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    5201, //  10001010 00101000 00000000 00000000
    1107, //  11001010 00100000 00000000 00000000
    1109, //  10101010 00100000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    1113, //  10011010 00100000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5217, //  10000110 00101000 00000000 00000000
    1123, //  11000110 00100000 00000000 00000000
    1125, //  10100110 00100000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    1129, //  10010110 00100000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1137, //  10001110 00100000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5249, //  10000001 00101000 00000000 00000000
    5251, //  11000001 00101000 00000000 00000000
    5253, //  10100001 00101000 00000000 00000000
    1159, //  11100001 00100000 00000000 00000000
    5257, //  10010001 00101000 00000000 00000000
    1163, //  11010001 00100000 00000000 00000000
    1165, //  10110001 00100000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    5265, //  10001001 00101000 00000000 00000000
    1171, //  11001001 00100000 00000000 00000000
    1173, //  10101001 00100000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    1177, //  10011001 00100000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5281, //  10000101 00101000 00000000 00000000
    1187, //  11000101 00100000 00000000 00000000
    1189, //  10100101 00100000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    1193, //  10010101 00100000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1201, //  10001101 00100000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5313, //  10000011 00101000 00000000 00000000
    1219, //  11000011 00100000 00000000 00000000
    1221, //  10100011 00100000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    1225, //  10010011 00100000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1233, //  10001011 00100000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1249, //  10000111 00100000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5377, //  10000000 10101000 00000000 00000000
    5379, //  11000000 10101000 00000000 00000000
    5381, //  10100000 10101000 00000000 00000000
    1287, //  11100000 10100000 00000000 00000000
    5385, //  10010000 10101000 00000000 00000000
    1291, //  11010000 10100000 00000000 00000000
    1293, //  10110000 10100000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    5393, //  10001000 10101000 00000000 00000000
    1299, //  11001000 10100000 00000000 00000000
    1301, //  10101000 10100000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    1305, //  10011000 10100000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5409, //  10000100 10101000 00000000 00000000
    1315, //  11000100 10100000 00000000 00000000
    1317, //  10100100 10100000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    1321, //  10010100 10100000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1329, //  10001100 10100000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5441, //  10000010 10101000 00000000 00000000
    1347, //  11000010 10100000 00000000 00000000
    1349, //  10100010 10100000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    1353, //  10010010 10100000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1361, //  10001010 10100000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1377, //  10000110 10100000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5505, //  10000001 10101000 00000000 00000000
    1411, //  11000001 10100000 00000000 00000000
    1413, //  10100001 10100000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    1417, //  10010001 10100000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    1425, //  10001001 10100000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1441, //  10000101 10100000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1473, //  10000011 10100000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5633, //  10000000 01101000 00000000 00000000
    5635, //  11000000 01101000 00000000 00000000
    5637, //  10100000 01101000 00000000 00000000
    1543, //  11100000 01100000 00000000 00000000
    5641, //  10010000 01101000 00000000 00000000
    1547, //  11010000 01100000 00000000 00000000
    1549, //  10110000 01100000 00000000 00000000
    527, //  11110000 01000000 00000000 00000000
    5649, //  10001000 01101000 00000000 00000000
    1555, //  11001000 01100000 00000000 00000000
    1557, //  10101000 01100000 00000000 00000000
    535, //  11101000 01000000 00000000 00000000
    1561, //  10011000 01100000 00000000 00000000
    539, //  11011000 01000000 00000000 00000000
    541, //  10111000 01000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5665, //  10000100 01101000 00000000 00000000
    1571, //  11000100 01100000 00000000 00000000
    1573, //  10100100 01100000 00000000 00000000
    551, //  11100100 01000000 00000000 00000000
    1577, //  10010100 01100000 00000000 00000000
    555, //  11010100 01000000 00000000 00000000
    557, //  10110100 01000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1585, //  10001100 01100000 00000000 00000000
    563, //  11001100 01000000 00000000 00000000
    565, //  10101100 01000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    569, //  10011100 01000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5697, //  10000010 01101000 00000000 00000000
    1603, //  11000010 01100000 00000000 00000000
    1605, //  10100010 01100000 00000000 00000000
    583, //  11100010 01000000 00000000 00000000
    1609, //  10010010 01100000 00000000 00000000
    587, //  11010010 01000000 00000000 00000000
    589, //  10110010 01000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1617, //  10001010 01100000 00000000 00000000
    595, //  11001010 01000000 00000000 00000000
    597, //  10101010 01000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    601, //  10011010 01000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1633, //  10000110 01100000 00000000 00000000
    611, //  11000110 01000000 00000000 00000000
    613, //  10100110 01000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    617, //  10010110 01000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    625, //  10001110 01000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5761, //  10000001 01101000 00000000 00000000
    1667, //  11000001 01100000 00000000 00000000
    1669, //  10100001 01100000 00000000 00000000
    647, //  11100001 01000000 00000000 00000000
    1673, //  10010001 01100000 00000000 00000000
    651, //  11010001 01000000 00000000 00000000
    653, //  10110001 01000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    1681, //  10001001 01100000 00000000 00000000
    659, //  11001001 01000000 00000000 00000000
    661, //  10101001 01000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    665, //  10011001 01000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1697, //  10000101 01100000 00000000 00000000
    675, //  11000101 01000000 00000000 00000000
    677, //  10100101 01000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    681, //  10010101 01000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    689, //  10001101 01000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1729, //  10000011 01100000 00000000 00000000
    707, //  11000011 01000000 00000000 00000000
    709, //  10100011 01000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    713, //  10010011 01000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    721, //  10001011 01000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    737, //  10000111 01000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    5889, //  10000000 11101000 00000000 00000000
    1795, //  11000000 11100000 00000000 00000000
    1797, //  10100000 11100000 00000000 00000000
    775, //  11100000 11000000 00000000 00000000
    1801, //  10010000 11100000 00000000 00000000
    779, //  11010000 11000000 00000000 00000000
    781, //  10110000 11000000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    1809, //  10001000 11100000 00000000 00000000
    787, //  11001000 11000000 00000000 00000000
    789, //  10101000 11000000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    793, //  10011000 11000000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1825, //  10000100 11100000 00000000 00000000
    803, //  11000100 11000000 00000000 00000000
    805, //  10100100 11000000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    809, //  10010100 11000000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    817, //  10001100 11000000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1857, //  10000010 11100000 00000000 00000000
    835, //  11000010 11000000 00000000 00000000
    837, //  10100010 11000000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    841, //  10010010 11000000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    849, //  10001010 11000000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    865, //  10000110 11000000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1921, //  10000001 11100000 00000000 00000000
    899, //  11000001 11000000 00000000 00000000
    901, //  10100001 11000000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    905, //  10010001 11000000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    913, //  10001001 11000000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    929, //  10000101 11000000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    961, //  10000011 11000000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6145, //  10000000 00011000 00000000 00000000
    6147, //  11000000 00011000 00000000 00000000
    6149, //  10100000 00011000 00000000 00000000
    6151, //  11100000 00011000 00000000 00000000
    6153, //  10010000 00011000 00000000 00000000
    6155, //  11010000 00011000 00000000 00000000
    6157, //  10110000 00011000 00000000 00000000
    2063, //  11110000 00010000 00000000 00000000
    6161, //  10001000 00011000 00000000 00000000
    6163, //  11001000 00011000 00000000 00000000
    6165, //  10101000 00011000 00000000 00000000
    2071, //  11101000 00010000 00000000 00000000
    6169, //  10011000 00011000 00000000 00000000
    2075, //  11011000 00010000 00000000 00000000
    2077, //  10111000 00010000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6177, //  10000100 00011000 00000000 00000000
    6179, //  11000100 00011000 00000000 00000000
    6181, //  10100100 00011000 00000000 00000000
    2087, //  11100100 00010000 00000000 00000000
    6185, //  10010100 00011000 00000000 00000000
    2091, //  11010100 00010000 00000000 00000000
    2093, //  10110100 00010000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    6193, //  10001100 00011000 00000000 00000000
    2099, //  11001100 00010000 00000000 00000000
    2101, //  10101100 00010000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    2105, //  10011100 00010000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6209, //  10000010 00011000 00000000 00000000
    6211, //  11000010 00011000 00000000 00000000
    6213, //  10100010 00011000 00000000 00000000
    2119, //  11100010 00010000 00000000 00000000
    6217, //  10010010 00011000 00000000 00000000
    2123, //  11010010 00010000 00000000 00000000
    2125, //  10110010 00010000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    6225, //  10001010 00011000 00000000 00000000
    2131, //  11001010 00010000 00000000 00000000
    2133, //  10101010 00010000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    2137, //  10011010 00010000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6241, //  10000110 00011000 00000000 00000000
    2147, //  11000110 00010000 00000000 00000000
    2149, //  10100110 00010000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    2153, //  10010110 00010000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    2161, //  10001110 00010000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6273, //  10000001 00011000 00000000 00000000
    6275, //  11000001 00011000 00000000 00000000
    6277, //  10100001 00011000 00000000 00000000
    2183, //  11100001 00010000 00000000 00000000
    6281, //  10010001 00011000 00000000 00000000
    2187, //  11010001 00010000 00000000 00000000
    2189, //  10110001 00010000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    6289, //  10001001 00011000 00000000 00000000
    2195, //  11001001 00010000 00000000 00000000
    2197, //  10101001 00010000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    2201, //  10011001 00010000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6305, //  10000101 00011000 00000000 00000000
    2211, //  11000101 00010000 00000000 00000000
    2213, //  10100101 00010000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    2217, //  10010101 00010000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    2225, //  10001101 00010000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6337, //  10000011 00011000 00000000 00000000
    2243, //  11000011 00010000 00000000 00000000
    2245, //  10100011 00010000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    2249, //  10010011 00010000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    2257, //  10001011 00010000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2273, //  10000111 00010000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6401, //  10000000 10011000 00000000 00000000
    6403, //  11000000 10011000 00000000 00000000
    6405, //  10100000 10011000 00000000 00000000
    2311, //  11100000 10010000 00000000 00000000
    6409, //  10010000 10011000 00000000 00000000
    2315, //  11010000 10010000 00000000 00000000
    2317, //  10110000 10010000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    6417, //  10001000 10011000 00000000 00000000
    2323, //  11001000 10010000 00000000 00000000
    2325, //  10101000 10010000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    2329, //  10011000 10010000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6433, //  10000100 10011000 00000000 00000000
    2339, //  11000100 10010000 00000000 00000000
    2341, //  10100100 10010000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    2345, //  10010100 10010000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    2353, //  10001100 10010000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6465, //  10000010 10011000 00000000 00000000
    2371, //  11000010 10010000 00000000 00000000
    2373, //  10100010 10010000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    2377, //  10010010 10010000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    2385, //  10001010 10010000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2401, //  10000110 10010000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6529, //  10000001 10011000 00000000 00000000
    2435, //  11000001 10010000 00000000 00000000
    2437, //  10100001 10010000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    2441, //  10010001 10010000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    2449, //  10001001 10010000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2465, //  10000101 10010000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2497, //  10000011 10010000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6657, //  10000000 01011000 00000000 00000000
    6659, //  11000000 01011000 00000000 00000000
    6661, //  10100000 01011000 00000000 00000000
    2567, //  11100000 01010000 00000000 00000000
    6665, //  10010000 01011000 00000000 00000000
    2571, //  11010000 01010000 00000000 00000000
    2573, //  10110000 01010000 00000000 00000000
    527, //  11110000 01000000 00000000 00000000
    6673, //  10001000 01011000 00000000 00000000
    2579, //  11001000 01010000 00000000 00000000
    2581, //  10101000 01010000 00000000 00000000
    535, //  11101000 01000000 00000000 00000000
    2585, //  10011000 01010000 00000000 00000000
    539, //  11011000 01000000 00000000 00000000
    541, //  10111000 01000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6689, //  10000100 01011000 00000000 00000000
    2595, //  11000100 01010000 00000000 00000000
    2597, //  10100100 01010000 00000000 00000000
    551, //  11100100 01000000 00000000 00000000
    2601, //  10010100 01010000 00000000 00000000
    555, //  11010100 01000000 00000000 00000000
    557, //  10110100 01000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    2609, //  10001100 01010000 00000000 00000000
    563, //  11001100 01000000 00000000 00000000
    565, //  10101100 01000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    569, //  10011100 01000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6721, //  10000010 01011000 00000000 00000000
    2627, //  11000010 01010000 00000000 00000000
    2629, //  10100010 01010000 00000000 00000000
    583, //  11100010 01000000 00000000 00000000
    2633, //  10010010 01010000 00000000 00000000
    587, //  11010010 01000000 00000000 00000000
    589, //  10110010 01000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    2641, //  10001010 01010000 00000000 00000000
    595, //  11001010 01000000 00000000 00000000
    597, //  10101010 01000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    601, //  10011010 01000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2657, //  10000110 01010000 00000000 00000000
    611, //  11000110 01000000 00000000 00000000
    613, //  10100110 01000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    617, //  10010110 01000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    625, //  10001110 01000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6785, //  10000001 01011000 00000000 00000000
    2691, //  11000001 01010000 00000000 00000000
    2693, //  10100001 01010000 00000000 00000000
    647, //  11100001 01000000 00000000 00000000
    2697, //  10010001 01010000 00000000 00000000
    651, //  11010001 01000000 00000000 00000000
    653, //  10110001 01000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    2705, //  10001001 01010000 00000000 00000000
    659, //  11001001 01000000 00000000 00000000
    661, //  10101001 01000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    665, //  10011001 01000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2721, //  10000101 01010000 00000000 00000000
    675, //  11000101 01000000 00000000 00000000
    677, //  10100101 01000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    681, //  10010101 01000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    689, //  10001101 01000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2753, //  10000011 01010000 00000000 00000000
    707, //  11000011 01000000 00000000 00000000
    709, //  10100011 01000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    713, //  10010011 01000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    721, //  10001011 01000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    737, //  10000111 01000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    6913, //  10000000 11011000 00000000 00000000
    2819, //  11000000 11010000 00000000 00000000
    2821, //  10100000 11010000 00000000 00000000
    775, //  11100000 11000000 00000000 00000000
    2825, //  10010000 11010000 00000000 00000000
    779, //  11010000 11000000 00000000 00000000
    781, //  10110000 11000000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    2833, //  10001000 11010000 00000000 00000000
    787, //  11001000 11000000 00000000 00000000
    789, //  10101000 11000000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    793, //  10011000 11000000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2849, //  10000100 11010000 00000000 00000000
    803, //  11000100 11000000 00000000 00000000
    805, //  10100100 11000000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    809, //  10010100 11000000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    817, //  10001100 11000000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2881, //  10000010 11010000 00000000 00000000
    835, //  11000010 11000000 00000000 00000000
    837, //  10100010 11000000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    841, //  10010010 11000000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    849, //  10001010 11000000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    865, //  10000110 11000000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    2945, //  10000001 11010000 00000000 00000000
    899, //  11000001 11000000 00000000 00000000
    901, //  10100001 11000000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    905, //  10010001 11000000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    913, //  10001001 11000000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    929, //  10000101 11000000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    961, //  10000011 11000000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    7169, //  10000000 00111000 00000000 00000000
    7171, //  11000000 00111000 00000000 00000000
    7173, //  10100000 00111000 00000000 00000000
    3079, //  11100000 00110000 00000000 00000000
    7177, //  10010000 00111000 00000000 00000000
    3083, //  11010000 00110000 00000000 00000000
    3085, //  10110000 00110000 00000000 00000000
    1039, //  11110000 00100000 00000000 00000000
    7185, //  10001000 00111000 00000000 00000000
    3091, //  11001000 00110000 00000000 00000000
    3093, //  10101000 00110000 00000000 00000000
    1047, //  11101000 00100000 00000000 00000000
    3097, //  10011000 00110000 00000000 00000000
    1051, //  11011000 00100000 00000000 00000000
    1053, //  10111000 00100000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    7201, //  10000100 00111000 00000000 00000000
    3107, //  11000100 00110000 00000000 00000000
    3109, //  10100100 00110000 00000000 00000000
    1063, //  11100100 00100000 00000000 00000000
    3113, //  10010100 00110000 00000000 00000000
    1067, //  11010100 00100000 00000000 00000000
    1069, //  10110100 00100000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    3121, //  10001100 00110000 00000000 00000000
    1075, //  11001100 00100000 00000000 00000000
    1077, //  10101100 00100000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    1081, //  10011100 00100000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    7233, //  10000010 00111000 00000000 00000000
    3139, //  11000010 00110000 00000000 00000000
    3141, //  10100010 00110000 00000000 00000000
    1095, //  11100010 00100000 00000000 00000000
    3145, //  10010010 00110000 00000000 00000000
    1099, //  11010010 00100000 00000000 00000000
    1101, //  10110010 00100000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    3153, //  10001010 00110000 00000000 00000000
    1107, //  11001010 00100000 00000000 00000000
    1109, //  10101010 00100000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    1113, //  10011010 00100000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3169, //  10000110 00110000 00000000 00000000
    1123, //  11000110 00100000 00000000 00000000
    1125, //  10100110 00100000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    1129, //  10010110 00100000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1137, //  10001110 00100000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    7297, //  10000001 00111000 00000000 00000000
    3203, //  11000001 00110000 00000000 00000000
    3205, //  10100001 00110000 00000000 00000000
    1159, //  11100001 00100000 00000000 00000000
    3209, //  10010001 00110000 00000000 00000000
    1163, //  11010001 00100000 00000000 00000000
    1165, //  10110001 00100000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    3217, //  10001001 00110000 00000000 00000000
    1171, //  11001001 00100000 00000000 00000000
    1173, //  10101001 00100000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    1177, //  10011001 00100000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3233, //  10000101 00110000 00000000 00000000
    1187, //  11000101 00100000 00000000 00000000
    1189, //  10100101 00100000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    1193, //  10010101 00100000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1201, //  10001101 00100000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3265, //  10000011 00110000 00000000 00000000
    1219, //  11000011 00100000 00000000 00000000
    1221, //  10100011 00100000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    1225, //  10010011 00100000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1233, //  10001011 00100000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1249, //  10000111 00100000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    7425, //  10000000 10111000 00000000 00000000
    3331, //  11000000 10110000 00000000 00000000
    3333, //  10100000 10110000 00000000 00000000
    1287, //  11100000 10100000 00000000 00000000
    3337, //  10010000 10110000 00000000 00000000
    1291, //  11010000 10100000 00000000 00000000
    1293, //  10110000 10100000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    3345, //  10001000 10110000 00000000 00000000
    1299, //  11001000 10100000 00000000 00000000
    1301, //  10101000 10100000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    1305, //  10011000 10100000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3361, //  10000100 10110000 00000000 00000000
    1315, //  11000100 10100000 00000000 00000000
    1317, //  10100100 10100000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    1321, //  10010100 10100000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1329, //  10001100 10100000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3393, //  10000010 10110000 00000000 00000000
    1347, //  11000010 10100000 00000000 00000000
    1349, //  10100010 10100000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    1353, //  10010010 10100000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1361, //  10001010 10100000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1377, //  10000110 10100000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3457, //  10000001 10110000 00000000 00000000
    1411, //  11000001 10100000 00000000 00000000
    1413, //  10100001 10100000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    1417, //  10010001 10100000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    1425, //  10001001 10100000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1441, //  10000101 10100000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1473, //  10000011 10100000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    7681, //  10000000 01111000 00000000 00000000
    3587, //  11000000 01110000 00000000 00000000
    3589, //  10100000 01110000 00000000 00000000
    1543, //  11100000 01100000 00000000 00000000
    3593, //  10010000 01110000 00000000 00000000
    1547, //  11010000 01100000 00000000 00000000
    1549, //  10110000 01100000 00000000 00000000
    527, //  11110000 01000000 00000000 00000000
    3601, //  10001000 01110000 00000000 00000000
    1555, //  11001000 01100000 00000000 00000000
    1557, //  10101000 01100000 00000000 00000000
    535, //  11101000 01000000 00000000 00000000
    1561, //  10011000 01100000 00000000 00000000
    539, //  11011000 01000000 00000000 00000000
    541, //  10111000 01000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3617, //  10000100 01110000 00000000 00000000
    1571, //  11000100 01100000 00000000 00000000
    1573, //  10100100 01100000 00000000 00000000
    551, //  11100100 01000000 00000000 00000000
    1577, //  10010100 01100000 00000000 00000000
    555, //  11010100 01000000 00000000 00000000
    557, //  10110100 01000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    1585, //  10001100 01100000 00000000 00000000
    563, //  11001100 01000000 00000000 00000000
    565, //  10101100 01000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    569, //  10011100 01000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3649, //  10000010 01110000 00000000 00000000
    1603, //  11000010 01100000 00000000 00000000
    1605, //  10100010 01100000 00000000 00000000
    583, //  11100010 01000000 00000000 00000000
    1609, //  10010010 01100000 00000000 00000000
    587, //  11010010 01000000 00000000 00000000
    589, //  10110010 01000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    1617, //  10001010 01100000 00000000 00000000
    595, //  11001010 01000000 00000000 00000000
    597, //  10101010 01000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    601, //  10011010 01000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1633, //  10000110 01100000 00000000 00000000
    611, //  11000110 01000000 00000000 00000000
    613, //  10100110 01000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    617, //  10010110 01000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    625, //  10001110 01000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3713, //  10000001 01110000 00000000 00000000
    1667, //  11000001 01100000 00000000 00000000
    1669, //  10100001 01100000 00000000 00000000
    647, //  11100001 01000000 00000000 00000000
    1673, //  10010001 01100000 00000000 00000000
    651, //  11010001 01000000 00000000 00000000
    653, //  10110001 01000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    1681, //  10001001 01100000 00000000 00000000
    659, //  11001001 01000000 00000000 00000000
    661, //  10101001 01000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    665, //  10011001 01000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1697, //  10000101 01100000 00000000 00000000
    675, //  11000101 01000000 00000000 00000000
    677, //  10100101 01000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    681, //  10010101 01000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    689, //  10001101 01000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1729, //  10000011 01100000 00000000 00000000
    707, //  11000011 01000000 00000000 00000000
    709, //  10100011 01000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    713, //  10010011 01000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    721, //  10001011 01000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    737, //  10000111 01000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    3841, //  10000000 11110000 00000000 00000000
    1795, //  11000000 11100000 00000000 00000000
    1797, //  10100000 11100000 00000000 00000000
    775, //  11100000 11000000 00000000 00000000
    1801, //  10010000 11100000 00000000 00000000
    779, //  11010000 11000000 00000000 00000000
    781, //  10110000 11000000 00000000 00000000
    271, //  11110000 10000000 00000000 00000000
    1809, //  10001000 11100000 00000000 00000000
    787, //  11001000 11000000 00000000 00000000
    789, //  10101000 11000000 00000000 00000000
    279, //  11101000 10000000 00000000 00000000
    793, //  10011000 11000000 00000000 00000000
    283, //  11011000 10000000 00000000 00000000
    285, //  10111000 10000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1825, //  10000100 11100000 00000000 00000000
    803, //  11000100 11000000 00000000 00000000
    805, //  10100100 11000000 00000000 00000000
    295, //  11100100 10000000 00000000 00000000
    809, //  10010100 11000000 00000000 00000000
    299, //  11010100 10000000 00000000 00000000
    301, //  10110100 10000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    817, //  10001100 11000000 00000000 00000000
    307, //  11001100 10000000 00000000 00000000
    309, //  10101100 10000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    313, //  10011100 10000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1857, //  10000010 11100000 00000000 00000000
    835, //  11000010 11000000 00000000 00000000
    837, //  10100010 11000000 00000000 00000000
    327, //  11100010 10000000 00000000 00000000
    841, //  10010010 11000000 00000000 00000000
    331, //  11010010 10000000 00000000 00000000
    333, //  10110010 10000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    849, //  10001010 11000000 00000000 00000000
    339, //  11001010 10000000 00000000 00000000
    341, //  10101010 10000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    345, //  10011010 10000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    865, //  10000110 11000000 00000000 00000000
    355, //  11000110 10000000 00000000 00000000
    357, //  10100110 10000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    361, //  10010110 10000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    369, //  10001110 10000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    1921, //  10000001 11100000 00000000 00000000
    899, //  11000001 11000000 00000000 00000000
    901, //  10100001 11000000 00000000 00000000
    391, //  11100001 10000000 00000000 00000000
    905, //  10010001 11000000 00000000 00000000
    395, //  11010001 10000000 00000000 00000000
    397, //  10110001 10000000 00000000 00000000
    143, //  11110001 00000000 00000000 00000000
    913, //  10001001 11000000 00000000 00000000
    403, //  11001001 10000000 00000000 00000000
    405, //  10101001 10000000 00000000 00000000
    151, //  11101001 00000000 00000000 00000000
    409, //  10011001 10000000 00000000 00000000
    155, //  11011001 00000000 00000000 00000000
    157, //  10111001 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    929, //  10000101 11000000 00000000 00000000
    419, //  11000101 10000000 00000000 00000000
    421, //  10100101 10000000 00000000 00000000
    167, //  11100101 00000000 00000000 00000000
    425, //  10010101 10000000 00000000 00000000
    171, //  11010101 00000000 00000000 00000000
    173, //  10110101 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    433, //  10001101 10000000 00000000 00000000
    179, //  11001101 00000000 00000000 00000000
    181, //  10101101 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    185, //  10011101 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    961, //  10000011 11000000 00000000 00000000
    451, //  11000011 10000000 00000000 00000000
    453, //  10100011 10000000 00000000 00000000
    199, //  11100011 00000000 00000000 00000000
    457, //  10010011 10000000 00000000 00000000
    203, //  11010011 00000000 00000000 00000000
    205, //  10110011 00000000 00000000 00000000
    79, //  11110010 00000000 00000000 00000000
    465, //  10001011 10000000 00000000 00000000
    211, //  11001011 00000000 00000000 00000000
    213, //  10101011 00000000 00000000 00000000
    87, //  11101010 00000000 00000000 00000000
    217, //  10011011 00000000 00000000 00000000
    91, //  11011010 00000000 00000000 00000000
    93, //  10111010 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
    481, //  10000111 10000000 00000000 00000000
    227, //  11000111 00000000 00000000 00000000
    229, //  10100111 00000000 00000000 00000000
    103, //  11100110 00000000 00000000 00000000
    233, //  10010111 00000000 00000000 00000000
    107, //  11010110 00000000 00000000 00000000
    109, //  10110110 00000000 00000000 00000000
    47, //  11110100 00000000 00000000 00000000
    241, //  10001111 00000000 00000000 00000000
    115, //  11001110 00000000 00000000 00000000
    117, //  10101110 00000000 00000000 00000000
    55, //  11101100 00000000 00000000 00000000
    121, //  10011110 00000000 00000000 00000000
    59, //  11011100 00000000 00000000 00000000
    61, //  10111100 00000000 00000000 00000000
    31, //  11111000 00000000 00000000 00000000
};

const int bottomRankMask[] =
{
    0, //  00000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    512, //  00000000 01000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    1024, //  00000000 00100000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    512, //  00000000 01000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2048, //  00000000 00010000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    512, //  00000000 01000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    1024, //  00000000 00100000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    512, //  00000000 01000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4096, //  00000000 00001000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    512, //  00000000 01000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    1024, //  00000000 00100000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    512, //  00000000 01000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2048, //  00000000 00010000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    512, //  00000000 01000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    1024, //  00000000 00100000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    512, //  00000000 01000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    256, //  00000000 10000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    128, //  00000001 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    64, //  00000010 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    32, //  00000100 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    16, //  00001000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    8, //  00010000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    4, //  00100000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
    2, //  01000000 00000000 00000000 00000000
    1, //  10000000 00000000 00000000 00000000
};

}

#endif  // PEVAL_POKEREVALUATIONTABLES_H_
